#include <math.h>

#include "wsat.h"
extern PREFS prefs;
#define RAND 0
#define LONOFFSET 180
#define LATOFFSET 90
#define LON2X(l) ((int)((LONOFFSET+l-ox)*fact*zx)+RAND)
#define LAT2Y(l) ((int)((LATOFFSET-l-oy)*fact*zy)+RAND)
#define X2LON(x) ((float)((x-RAND)/(fact*zx)-LONOFFSET+ox))
#define Y2LAT(y) ((float)(LATOFFSET-ox-(y-RAND)/(fact*zy)))
#define RASCOL 0
#define RASCOL2 1


static int ox,oy;
static float zx,zy,fact;
#define LAB_TRACKCANVAS "trackcanvas"
void calc_x2satobs(KEPLER *kepler,ORBIT *orbit,
                   EPOINT *pos_sati,EPOINT *pos_sato,
                   int width_original,float max_sens_angle,
                   int x);

#define W_CHAR 8
void draw_rast(GtkWidget *wnd)
{
  int lon,lat;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor grey,white,num;
  grey.red=grey.green=grey.blue=0xa0;
  white.red=white.green=white.blue=0xf0;
  num.red=num.green=0x00; num.blue=0x00;
  char tmp[10];
  for (lon=-180; lon<=180; lon+=15)
  {
    draw_rgbline(rgbi,&grey,LON2X(lon),LAT2Y(-90),LON2X(lon),LAT2Y(90));
    sprintf(tmp,"%4d",lon);
    if ((ABS(lon)<180) && (!(lon%30)))
      draw_rgbstring(rgbi,&num,LON2X(lon)-2*W_CHAR,4,tmp);
  }
  draw_rgbline(rgbi,&white,LON2X(0),LAT2Y(-90),LON2X(0),LAT2Y(90));

  for (lat=-90; lat<=90; lat+=15)
  {
    draw_rgbline(rgbi,&grey,LON2X(-180),LAT2Y(lat),LON2X(180),LAT2Y(lat));
    sprintf(tmp,"%3d",lat);
    if ((ABS(lat)<90) && (!(lat%30)))
      draw_rgbstring(rgbi,&num,4,LAT2Y(lat)-W_CHAR,tmp);
  }
  draw_rgbline(rgbi,&white,LON2X(-180),LAT2Y(0),LON2X(180),LAT2Y(0));
}

void draw_back(GtkWidget *wnd)
{
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr;
  FILE *fp;
  FILE_TYPE ftype;
  GIF_PICINFO *gif_pci;
  CLRMAP clrmap;
  int width,height;
  int picwidth,picheight;
  int picyp=-1;
  guint16 *line_r, *line_g, *line_b;
  int x,y;
  if (!drawing_area) return;
  if (!rgbi) return;
  gif_pci=calloc(sizeof(GIF_PICINFO),1);
  width=drawing_area->allocation.width;
  height=drawing_area->allocation.height;
ox=oy=0;
zx=zy=1.;
fact=1.; // 360x180
  fact=MIN(width/360.,height/180.);
strcpy(prefs.map_file,"earthmap.gif");
  ftype=detect_file(prefs.map_file);
  if (!(fp=fopen(prefs.map_file,"rb"))) return;
  switch(ftype)
  {
#ifdef TIFFSIGN
    case FT_TIFF:
      read_tiff_header(fp,&picwidth,&picheight);
    break;
#endif
#ifdef JPEGSIGN
    case FT_JPEG:
      guchar *istr;
      jpg2str_1b(fp,&istr,&picwidth,&picheight);
    break;
#endif
#ifdef GIFSIGN
    case FT_GIF:
      if (read_gif_header1(fp, &clrmap, gif_pci)) 
      {
        free(gif_pci);
        return;
      }
      picwidth=gif_pci->width;
      picheight=gif_pci->height;
    break;
#endif
    default:
      printf("Unknown filetype of %s\n",prefs.map_file);
      return;
    break;
  }

  line_r=malloc(3*picwidth*sizeof(guint16));
  line_g=malloc(3*picwidth*sizeof(guint16));
  line_b=malloc(3*picwidth*sizeof(guint16));

  for (y=0; y<LAT2Y(-90)-LAT2Y(90); y++)
  {
    int picy=(picheight*y)/(LAT2Y(-90)-LAT2Y(90));
    if (y+LAT2Y(90)>=height-RAND) continue;
    if (y+LAT2Y(90)<RAND) continue;
    g_main_iteration(FALSE);            /* Handle GTK actions now and then */
    if (y+LAT2Y(90)>=height-RAND) continue;
    if (y+LAT2Y(90)<RAND) continue;
//    if (db->redraw) break;
    if (picy!=picyp)
    {
      switch(ftype)
      {
#ifdef TIFFSIGN
        case FT_TIFF:
          read_tiff_line(fp,picwidth, picy,line_r,line_g,line_b);
        break;
#endif
#ifdef JPEGSIGN
        case FT_JPEG:
          guchar *istr;
          read_jpg_line(istr,picwidth, picy,line_r,line_g,line_b);
        break;
#endif
#ifdef GIFSIGN
        case FT_GIF:
          read_gif_line1(fp,gif_pci,picy,line_r);
        break;
#endif
        default: break;
      }
    }
    picyp=picy;

    for (x=0; x<=LON2X(180)-LON2X(-180); x++)
    {
      int picx=(picwidth*x)/(LON2X(180)-LON2X(-180));
      if (x+LON2X(-180)>=width-RAND) continue;
      if (x+LON2X(-180)<RAND) continue;
      switch(ftype)
      {
        case FT_TIFF:
          clr.red=line_r[picx];
          clr.green=line_g[picx];
          clr.blue=line_b[picx];
        break;
        case FT_JPEG:
          clr.red=line_r[picx];
          clr.green=line_g[picx];
          clr.blue=line_b[picx];
        break;
        case FT_GIF:
        {
          int n=*(line_r+picx);
          n=MIN(MAX(n,0),255);
          if (n<0) n=0;
          clr.red=clrmap.red[n];
          clr.green=clrmap.green[n];
          clr.blue=clrmap.blue[n];
        }
        break;
        default: break;
       }
       draw_rgbpoint(rgbi,&clr,x+LON2X(-180),y+LAT2Y(90));
    }
    Refresh_Rect(drawing_area,0,y+LAT2Y(90),drawing_area->allocation.width-1,1);

  }
  free(gif_pci);
  free(line_r);
  free(line_g);
  free(line_b);
  fclose(fp);
}


#define SIZEREF 8
void draw_ref(GtkWidget *wnd,float lon,float lat)
{
  int ilon,ilat;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  int x1,y1,x2,y2;
  clr1.red=0xff;
  clr1.green=0x00;
  clr1.blue=0xff;
  ilon=R2D(lon);
  ilat=R2D(lat);
  x1=LON2X(ilon)-SIZEREF/2;
  x2=LON2X(ilon)+SIZEREF/2;
  y1=LAT2Y(ilat)-SIZEREF/2;
  y2=LAT2Y(ilat)+SIZEREF/2;
//    clr1=db->clrs.ref_pnt;
  draw_rgbline(rgbi,&clr1,x1,y1,x1,y2);
  draw_rgbline(rgbi,&clr1,x1,y2,x2,y2);
  draw_rgbline(rgbi,&clr1,x2,y2,x2,y1);
  draw_rgbline(rgbi,&clr1,x2,y1,x1,y1);
}

static void pijlrgb(RGBI *rgbi,GdkColor *clr,int x,int y,int r)
{
  int dx1,dy1,dx2,dy2;

  dx1=10*cos(D2R(r-30));
  dy1=10*sin(D2R(r-30));
  dx2=10*cos(D2R(r+30));
  dy2=10*sin(D2R(r+30));
  draw_rgbline(rgbi,clr,x+dx1,y+dy1,x,y);
  draw_rgbline(rgbi,clr,x+dx2,y+dy2,x,y);
}

static void draw_sattrack(GtkWidget *wnd,EPOINT p1,EPOINT p2,gboolean rgbm,int p)
{
  int x1,y1,x2,y2;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  clr1.red=0x00;
  clr1.green=0x00;
  clr1.blue=0x00;

  x1=LON2X(R2D(p1.lon));
  x2=LON2X(R2D(p2.lon));
  y1=LAT2Y(R2D(p1.lat));
  y2=LAT2Y(R2D(p2.lat));

  if (ABS(x2-x1)>90) return;
//    clr1=db->clrs.ssat_track;
    draw_rgbline(rgbi,&clr1,x1,y1,x2,y2);
    if (p)
    {
      pijlrgb(rgbi,&clr1,x1,y1,(int)R2D(atan2(y2-y1,x2-x1)));
    }
}
#define SCANCOL 3
#define SCANCOL2 4
static void draw_sensor(GtkWidget *wnd,SAT *sat,
                                 EPOINT *pos_sat,EPOINT *pos_earth,gboolean rgbm)
{
  EPOINT pos_subsat;
  int x;
  float lon,lat;
  float lonp,latp;
  EPOINT pos_sat2;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  if (!sat->orbit.max_sens_angle) return;
  clr1.red=0x00;
  clr1.green=0x00;
  clr1.blue=0xff;

  for (x=0; x<2048; x+=128)
  {
    calc_x2satobs(&sat->kepler,&sat->orbit,
                  pos_sat,&pos_sat2,
                  sat->orbit.width_original,R2D(sat->orbit.max_sens_angle),
                  x);
    calcposrel(&sat->kepler,&sat->orbit,&pos_sat2,pos_earth,&pos_subsat);
    pos_subsat.lon-=D2R(sat->orbit.offset_lon);
    lon=R2D(pos_subsat.lon);
    lat=R2D(pos_subsat.lat);
    {
//      clr1=db->clrs.ssat_sens;

      if ((x>0) && (ABS(lon-lonp) <30))
      {
        draw_rgbline(rgbi,&clr1,LON2X(lonp),LAT2Y(latp),LON2X(lon),LAT2Y(lat));
      }
      else
        draw_rgbpoint(rgbi,&clr1,LON2X(lon),LAT2Y(lat));
    }
    lonp=lon; latp=lat;
  }
}

#define SIZESAT 8
static void draw_sat(GtkWidget *wnd,SAT *sat,float lon,float lat)
{
  int x1,y1,x2,y2;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  GdkColor clr1;
  clr1.red=0xff;
  clr1.green=0x00;
  clr1.blue=0x00;
  lon=R2D(lon);
  lat=R2D(lat);
  x1=LON2X(lon)-SIZESAT/2;
  x2=LON2X(lon)+SIZESAT/2;
  y1=LAT2Y(lat)-SIZESAT/2;
  y2=LAT2Y(lat)+SIZESAT/2;
  draw_rgbline(rgbi,&clr1,x1,y1,x1,y2);
  draw_rgbline(rgbi,&clr1,x1,y2,x2,y2);
  draw_rgbline(rgbi,&clr1,x2,y2,x2,y1);
  draw_rgbline(rgbi,&clr1,x2,y1,x1,y1);
  draw_rgbstring(rgbi,&clr1,x1+2*W_CHAR,y1+1*W_CHAR,sat->satname);
}

// calc and draw piece of track between tma and tmb
static void draw_track(GtkWidget *wnd,SAT *sat,struct tm tma,struct tm tmb)
{
  EPOINT pos_subsat1,pos_subsat2;
  EPOINT pos_sat;
  EPOINT pos_earth;
  int i=0;
  struct tm tm;
  for (tm=tma; mktime_ntz(&tm) < mktime_ntz(&tmb); tm.tm_min++)
  {
    calc_onepos(sat,tm,0,&pos_earth,&pos_sat,&pos_subsat1);
    if (i) draw_sattrack(wnd,pos_subsat1,pos_subsat2,TRUE,(i==1));
    pos_subsat2=pos_subsat1;
    i++;
  }
  calc_onepos(sat,tmb,0,&pos_earth,&pos_sat,&pos_subsat1);
  draw_sattrack(wnd,pos_subsat1,pos_subsat2,TRUE,TRUE);
}
// calc and draw scanline
static void calcdraw_sensor(struct tm cur_tm,GtkWidget *wnd,SAT *sat)
{
  EPOINT pos_subsat;
  EPOINT pos_sat;
  EPOINT pos_earth;
  calc_onepos(sat,cur_tm,0,&pos_earth,&pos_sat,&pos_subsat);
  draw_sensor(wnd,sat,&pos_sat,&pos_earth,TRUE);
}

// calc and draw piece of track
static void draw_trackpiece(GtkWidget *wnd,SAT *sat,struct tm cur_tm)
{
  struct tm tma;
  struct tm tmb;
  int strtmin=-5;
  int stopmin=+40;

  tma=cur_tm; tma.tm_min+=strtmin;
  tmb=cur_tm; tmb.tm_min+=stopmin;
  draw_track(wnd,sat,tma,tmb);
}

// draw visibility line around sat
static void draw_vis(GtkWidget *wnd,SAT *sat,EPOINT *refpos,GdkColor clr1)
{
  float d;
  float elev=0;
  float c;
  float lon,lat;
  float lonp=0,latp=0;
  gboolean first=TRUE;
  gboolean normal=TRUE;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  if (!sat) return;
  d=acos(Rearth/(sat->orbit.height+Rearth)*cos(elev))-elev;
  for (c=0; c<=360; c+=1)
  {
    float cc=D2R(c);
    lat=asin(sin(refpos->lat)*cos(d) + cos(refpos->lat)*sin(d)*cos(cc));
    lon=asin(sin(cc)*sin(d)/cos(lat));
    // volgende OK voor sats, maar refpos=observer en sat=geo geeft rare knik.
    // Met test <PI/3. is dat weg, maar dan elders fout.
    // --> Waarom deze test zo?
    if (((ABS(refpos->lat+d*cos(cc))) < PI/2.))
    {
      lon=R2D(lon);
      lat=R2D(lat);
      normal=TRUE;
    }
    else
    {
      if (lon>0) lon=180-R2D(lon); else lon=-180-R2D(lon);
      lat=R2D(lat);
      normal=FALSE;
    }
    lon+=R2D(refpos->lon);
    if (lon>180) lon-=360;
    if (lon<-180) lon+=360;
    {
      if ((!first) && (ABS(lonp-lon) < 60))
      {
        draw_rgbline(rgbi,&clr1,LON2X(lonp),LAT2Y(latp),LON2X(lon),LAT2Y(lat));
      }
      else
      {
        draw_rgbpoint(rgbi,&clr1,LON2X(lon),LAT2Y(lat));
      }
    }
    first=FALSE;
    lonp=lon; latp=lat;
  }
}
// draw one sat on it's position
void draw_onepos(struct tm cur_tm,GtkWidget *wnd,SAT *sat,EPOINT pos_sat,EPOINT pos_earth)
{
#ifdef XXX
  EPOINT pos_subsat=sat->pos;
  DIRECTION satdir=sat->dir;
#endif
  GdkColor clr_s,clr_u,clr1;
  draw_sat(wnd,sat,sat->pos.lon,sat->pos.lat);
  calcdraw_sensor(cur_tm,wnd,sat);

  clr_s.red=0xff; clr_s.green=0; clr_s.blue=0xff;
  clr_u.red=0xff; clr_u.green=0; clr_u.blue=0;
//  if (sat->selected) clr1=db->clrs.ssat_vis; else clr1=db->clrs.usat_vis;
clr1=clr_s;
  draw_vis(wnd,sat,&sat->pos,clr1);

  if (sat->selected)
    draw_trackpiece(wnd,sat,cur_tm);

  if (sat->selected)
  {
#ifdef XXX
    static int do_predict;
    if (satdir.elev>db->elev_hor) do_predict=0;  

    if (do_predict)
    {
      if (db->next_sat) db->sat_sel=db->next_sat;
      select_this_sat(db->sat_sel);
      report_nextpass(wnd,sat);
    }
    if (satdir.elev<db->elev_hor) do_predict=1;    // next time predict will be done
                                                   // (1 step delay for detection going down correctly)
#endif
  }
}
