/**************************************************
 * RCSId: $Id: file_save.c,v 1.10 2017/09/05 19:07:16 ralblas Exp $
 *
 * File-save related functions 
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: file_save.c,v $
 * Revision 1.10  2017/09/05 19:07:16  ralblas
 * _
 *
 * Revision 1.9  2017/05/27 11:07:32  ralblas
 * _
 *
 * Revision 1.8  2017/01/11 18:45:08  ralblas
 * _
 *
 * Revision 1.7  2016/12/24 11:00:45  ralblas
 * _
 *
 * Revision 1.6  2016/12/09 13:53:45  ralblas
 * _
 *
 * Revision 1.5  2016/12/08 20:38:30  ralblas
 * _
 *
 * Revision 1.4  2016/12/03 19:10:59  ralblas
 * _
 *
 * Revision 1.3  2016/11/18 23:02:58  ralblas
 * _
 *
 * Revision 1.2  2016/09/30 07:26:32  ralblas
 * _
 *
 * Revision 1.1  2016/03/24 22:05:38  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "wsat.h"
#include "menu_wsat.h"
#include "hrpt_funcs.h"
#include "xtrack_basefunc.h"

// #undef _MHRPT1_

extern PREFS prefs;

#if _SUN_
#  include "sun.h"
#endif
#if _TIFF_
#  include "tiff.h"
#endif
#if _PPM_
#  include "ppm.h"
#endif
#if _GIF_
#  include "gif.h"
#endif
#if _HRPT1_
#  include "hrpt1.h"
#endif
#if _HRI1_
#  include "hri1.h"
#endif

#define FILENAME_PREFIX "dump"

#define LAB_PPM_MCH "Multi-channel"

#define LAB_CH " "             /* prefix number */
#define LAB_CHSUF   " "        /* suffix number */
#define LAB_CH1     " 1 "
#define LAB_CH2     " 2 "
#define LAB_CH3     " 3 "
#define LAB_CH4     " 4 "
#define LAB_CH5     " 5 "
#define LAB_CH6     " 6 "
#define LAB_CH7     " 7 "
#define LAB_CH8     " 8 "
#define LAB_CH9     " 9 "
#define LAB_CH10    " 10 "
#define LAB_CHALL   "All"
#define LAB_FILENAME "^Save to"
#define LAB_STATUS "Status"
#define LAB_BITSHIFT "^Bitshift"

#define LAB_Genfn "New name"
#define LAB_Save "Save"
#define LAB_Exit "Close"
#define LAB_HRPT "(C)HRPT"
#define LAB_SUN  "Sun"
#define LAB_TIFF "Tiff"
#define LAB_PPM  "PPM"
#define LAB_PGM  "PGM"
#define LAB_PGM1 "PGM1"
#define LAB_GIF  "Gif"
#define LAB_JPEG "Jpeg"
#define LAB_MHRPT  "MHRPT"
#define LAB_HPT   "(C)HPT"

#define FN_EXT(t) (t==FT_SUN?      "sun" : \
                   t==FT_TIFF?     "tif" : \
                   t==FT_PPM?      "ppm" : \
                   t==FT_PGM?      "pgm" : \
                   t==FT_PGM1?     "pgm" : \
                   t==FT_GIF?      "gif" : \
                   t==FT_JPEG?     "jpg" : \
                   t==FT_HRPT?     "10b" : \
                   t==FT_CHRPT?    "10b" : \
                   t==FT_SEAWIF?   "10b" : \
                   t==FT_METOP?    "10b" : \
                   t==FT_HRI?      "hri" : \
                   t==FT_HPT?      "hpt" : \
                   t==FT_CHPT?     "C10" : \
                                   "dat")

/*************************************
 * Write file header.  
 *************************************/
int write_header(PICINFO *pci)
{
  int err=0;
  switch(pci->file_type)
  {
#if _SUN_
    case FT_SUN:
      err=write_sun_header(pci);
    break;
#endif
#if _TIFF_
    case FT_TIFF:
      err=write_tiff_header(pci);
    break;
#endif
#if _PPM_
    case FT_PPM: case FT_PGM: case FT_PGM1:
      err=write_ppm_header(pci);
    break;
#endif
#if _GIF_
    case FT_GIF:
      err=write_gif_header(pci);
    break;
#endif
#if _JPEG_
    case FT_JPEG:
      err=write_jpg_header(pci);
    break;
#endif
#if _HRPT1_
    case FT_HRPT: case FT_CHRPT: case FT_SEAWIF: case FT_MHRPT1:
      err=write_hrpt1_hdr(pci);
    break;
#endif
#if _MHxRPT1_
    case FT_MHRPT1:
      err=write_mhrpt1_hdr(pci);
    break;
#endif
    case FT_HPT: case FT_CHPT:
      pci->fri=calc_hrptframeinfo(pci->file_type,pci->channels_recorded,pci->width);
    break;
    // case FT_METOP and FT_FY3 not supported
    default: break;
  }
  return err;
}

/*************************************
 * Finish writing if needed.
 *************************************/
void write_finish(PICINFO *pci)
{
#if _TIFF_
  if (pci->file_type==FT_TIFF)
    write_tiff_trailer(pci);
#endif
#if _GIF_
  if (pci->file_type==FT_GIF)
    write_gif_end(&pci->gif_pci, pci->fp);
#endif
}


/*************************************
 * Write a line.
 *************************************/
static int write_line(guint16 *l,              /* B/W or red */
               guint16 *l_g,            /* green */
               guint16 *l_b,            /* blue */
               PICINFO *pci,
               int     dec_status)      /* sync status decoder cur. line */
{
  switch(pci->file_type)
  {
#if _SUN_
    case FT_SUN:
      pci->fri.bytes_per_line=pci->width;
      write_sun_line(pci,l);
    break;
#endif
#if _TIFF_
    case FT_TIFF:
      pci->fri.bytes_per_line=pci->width;
      write_tiff_line(pci,l,l_g,l_b);
    break;
#endif
#if _PPM_
    case FT_PPM: case FT_PGM: case FT_PGM1:
      pci->fri.bytes_per_line=pci->width;
      write_ppm_line(pci,l,l_g,l_b);
    break;
#endif
#if _GIF_
    case FT_GIF:
      write_gif_line(pci,l);
    break;
#endif
#if _JPEG_
    case FT_JPEG:
      write_jpg_line(pci,l,l_g,l_b);
    break;
#endif
#if _HRPT1_
    case FT_HRPT: case FT_CHRPT: case FT_SEAWIF:
      write_hrpt1_line(pci,l,dec_status);
    break;
#endif
#if _MHRPT1_
    case FT_MHRPT1:
      write_mhrpt1_line(pci,l);
    break;
#endif
    case FT_HPT:
      write_hpt_line(pci,l);
    break;
    case FT_CHPT:
      write_hpt_line(pci,l);
    break;
    // case FT_METOP and FT_FY3 not supported
    default:
    break;
  }
  return 0;
}

/*************************************
 * Determine channel(s) to write.
 *************************************/
int load_chan(PICINFO *pci_r,VIEWINFO *vwi_w,int *chan_arr)
{
  int nr_rch=0;
  int nr_wch=0;
  int ch;
  for (ch=1; ch<NR_CHANNELS_MAX; ch++)
  {
    if (pci_r->channels_recorded[ch])
    {
      if (vwi_w->channel_selected[ch])
      {
        chan_arr[nr_wch]=nr_rch;
        nr_wch++;
      }
      nr_rch++;
    }
  }
  return nr_wch;
}

static int prepare_save(GtkWidget *window,PICINFO *pci_r,PICINFO *pci_w,
                                      VIEWINFO *vwi_r,VIEWINFO *vwi_w,
                                      int *picstart_r,int *picstart_w,int *chan_arr)
{
  int depth=256;
  int bitshift=0;
  int ch;
  int nr_wch=0;

  *picstart_r=*picstart_w=0;
  vwi_w->lin_xtbl=NULL;
  vwi_w->revlin_xtbl=NULL;

  // Determine where first pixel is in line
  if (pci_w->channels_recorded[0])                // apparently write in 'a' hrpt-format, containing ALL received bits
  {
    if (pci_w->file_type==FT_CHPT)
      pci_w->fri.header_len=750*2;                // totlen=0x6c50=27728 = 25600+2128
    else
      pci_w->fri.header_len=750;                  // ?? niet gezet als read=METOP / write=HRPT
    *picstart_w=pci_w->fri.header_len;
  }
  if (pci_r->channels_recorded[0])                // apparently read from file containing ALL received bits
    *picstart_r=pci_r->fri.header_len;

  if (IS_HRPT_FAM(pci_w->file_type))              // To a HRPT format, including 10b, hpt, C10 etc.; (metop/fy3 not supported for write)
  {
    pci_w->no_xymapping=TRUE;                     // Don't do any mapping; no processing

    if (!vwi_w->scan_ver)                         // hm, needed? I said: No processing...
    {
      vwi_w->scan_hor=W_E;
      vwi_w->scan_ver=N_S;
    }

    if (IS_HRPT_METOP_FAM(pci_r->file_type))      // From a HRPT format, including 10b, hpt, C10, metop/fy3
      depth=1024;
    else
      bitshift=2;
  }
  else                                            // To non-HRPT format: gif, jpeg etc.
  {
    if ((IS_HRPT_FAM(pci_r->file_type)) ||        // From HRPT or
        (pci_r->file_type==FT_PGM))               // from 16-bits PGM
    {
      if (pci_w->file_type==FT_PGM)               // To 2-bytes PGM */
        bitshift=0;
      else                                        // To 1-byte PGM format
      {
        if (vwi_r->lum_max<256)
          bitshift=0;
        else if (vwi_r->lum_max<512)
          bitshift=-1;
        else
          bitshift=-2;
      }
    }
  }
  Set_Entry(window,LAB_BITSHIFT,"%d",bitshift);

  // Determine channels to write
  for (ch=1; ch<NR_CHANNELS_MAX; ch++)
    vwi_w->channel_selected[ch]=pci_w->channels_recorded[ch];
  nr_wch=load_chan(pci_r,vwi_w,chan_arr);

  // Determine sizes of zoomed picture to save
  //   Determine width
  if (vwi_r->zval.zx>=1)
  {
    pci_w->width=pci_r->width/vwi_r->zval.zx;
  }
  else
  {
    pci_w->width=pci_r->width;
    vwi_w->zval.zx=1;
  }

  if (vwi_w->linearize)
  { 

    pci_w->width*=2.; // see EXPAND_L, mappings.c
    pci_w->fri.words_per_line*=2;
  }

//   Determine height
  if (vwi_r->zval.zy>=1)
  {
    pci_w->height=pci_r->height/vwi_r->zval.zy;
  }
  else
  {
    pci_w->height=pci_r->height;
    vwi_w->zval.zy=1;
  }

// depth 
  pci_w->clrmap.size=pci_r->clrmap.size;
  if (pci_w->file_type==FT_PGM1)
  {
    pci_w->clrmap.size=256; 
    pci_w->depth=8;
  }

  pci_w->scan_ver=vwi_w->scan_ver;
  pci_w->scan_hor=vwi_w->scan_hor;

/* Write the file header */
  write_header(pci_w);
  return nr_wch;
}

/*************************************
 * Save.  
 * 
 *************************************/
void Save_it(GtkWidget *window,
             PICINFO    *pci_r,
             PICINFO    *pci_w,
             char       *file_name,
             VIEWINFO   *vwi_r)     /* only needed for HRI */
{
  GtkWidget *progress_bar;
  guint *xmap=NULL;
  guint *ymap=NULL;
  guint *zmap=NULL;
  VIEWINFO vwi_w=*vwi_r;
  VIEWINFO vwi_ri;
  guint16 **lines;
  guint16 **linesrgb=NULL;
  guint16 *line_from;
  guint16 *line_to,*line_to_g,*line_to_b;
  gboolean *line_grid=NULL;
  int x,y,ch;
  int nr_wch=0;
  int nr_ilines;
  int dec_status;
  int chan_arr[NR_CHANNELS_MAX];
  int picstart_r,picstart_w;
  int bitshift=0;
  int depth=256;
  guint16 br,bg,bb;
  gboolean cancelled=FALSE;
  gboolean grid;

  guint16 sync[6]={0x284,0x16f,0x35c,0x19d,0x20f,0x095};
/* Open file for write */
  if (!(pci_w->fp=fopen(file_name,"wb")))
  {
    Create_Message("Error","Can't open file %s.",file_name);
    return;
  }
    // case FT_METOP and FT_FY3 not supported

  nr_wch=prepare_save(window,pci_r,pci_w,vwi_r,&vwi_w,&picstart_r,&picstart_w,chan_arr);

/* Generate mapping arrays xmap and ymap */
  {
    PICINFO pci2=*pci_r;
    if (vwi_r->process_data)
    {
      pci2.fri.inter_chan=1;
      pci2.fri.header_len=0;
    }
    map_xy(pci_w->width,pci_w->height,pci_w->no_xymapping,FALSE,&pci2,&vwi_w,&xmap,&ymap);
    map_z(vwi_r->lum_min,vwi_r->lum_max,vwi_r->gamma,vwi_r->invert,depth,pci_r->clrmap.size,&zmap);
  }

  // Deterine how many lines used to output 1 line
  nr_ilines=vwi_r->max_vshift-vwi_r->min_vshift+1;
  if (vwi_r->despeckle) nr_ilines*=3;

  // Allocate memory for read and write lines
  lines=(guint16 **)alloc_lines(pci_r->fri.words_per_line,nr_ilines);
  line_from=lines[0];                                   /* direct from readed */
  if (vwi_r->process_data)
    linesrgb=(guint16 **)alloc_lines(pci_r->fri.words_per_line,3); /* translated */

  line_to=calloc(pci_w->fri.words_per_line,sizeof(guint16));   /* red, B/W */
  line_to_g=calloc(pci_w->fri.words_per_line,sizeof(guint16)); /* green */
  line_to_b=calloc(pci_w->fri.words_per_line,sizeof(guint16)); /* blue */
  if (vwi_r->grid)
    line_grid=(gboolean *)calloc(pci_w->fri.words_per_line,sizeof(*line_grid));

  // Start progress bar
  progress_bar=Create_Progress(window,"Saving...",TRUE);

  // If source is GIF then rewind it
  if (pci_r->file_type==FT_GIF) rewind_gif(pci_r);

  // Save the picture
  vwi_ri=*vwi_r;
  vwi_ri.lin_xtbl=NULL; // separate alloc for lintbl here
  vwi_ri.zval.dx=1;          // needed to force 1:1 translation
  for (y=0; y<pci_w->height; y++)
  {
    int x_w=0;
    // Read 1 or more lines needed to produce 1 output line

    dec_status=read_lines(vwi_r,pci_r,ymap,y,nr_ilines,
                                            lines,NULL,NULL,line_grid);

    // Copy HRPT header to line_to if HRPT --> HRPT
    // NOTE: Received FT_METOP has reconstructed hrpt-header, including sync.
    if ((pci_r->channels_recorded[0]) && (pci_w->channels_recorded[0]))
    {
      for (x=0; x<picstart_r; x++)
      {
        *(line_to+x)=*(line_from+x);
        *(line_to_g+x)=*(line_from+x);
        *(line_to_b+x)=*(line_from+x);
      }
    }
    else if (pci_w->channels_recorded[0])   // If no pci_r->channels_recorded[0]: add hrpt-sync anyway.
    {                                       //   No calibration data etc.
      for (x=0; x<6; x++)                   //     Add sync
      {
        *(line_to+x)=sync[x];
      }
    }

    if (vwi_r->process_data)
    {
/* Translate input line to linesrgb */
      Translate_Line(&vwi_ri,pci_r,0,NULL,lines,nr_ilines,
                      linesrgb[0],linesrgb[1],linesrgb[2]);
      nr_wch=1;
    }

/* Add all pixels of all channels to save to "line_to" */
    for (x=0; x<pci_w->width; x++)          // for all pixels in a line */
    {
      int xoffs=*(xmap+x);                  // offset in src; hrpt: start at 750, incr. = 5
      for (ch=0; ch<nr_wch; ch++)           // for all channels to save */
      {
/* Get pixel taking into account x-offset and flip */
        if ((xoffs>=0) && (xoffs<pci_r->fri.words_per_line))
        {
          if (vwi_r->process_data)
          {
            br=*(linesrgb[0]+xoffs);
            bg=*(linesrgb[1]+xoffs);
            bb=*(linesrgb[2]+xoffs);
          }
          else
          {
            br=*(line_from+xoffs+chan_arr[ch]-chan_arr[0]); // full hrpt: chan_arr[ch]=ch
            bg=br;
            bb=br;
          }
        }
        else
        {
          br=bg=bb=0;
        }
/* Get HRI grid-info. */
        if (line_grid) grid=*(line_grid+*(xmap+x)+chan_arr[ch]-chan_arr[0]);

/* Translate pix between 10bits <--> 8bits words */
        if (!IS_HRPT_FAM(pci_w->file_type))
        {
          br=zmap[br];
          bg=zmap[bg];
          bb=zmap[bb];
        }
        else
        {
          if (bitshift<0)
          {
            br>>=(bitshift*-1);
            bg>>=(bitshift*-1);
            bb>>=(bitshift*-1);
          }
          else
          {
            br<<=bitshift;
            bg<<=bitshift;
            bb<<=bitshift;
          }
        }
/* Replace pix by grid, if requested and gridflag is true. 
   (line_grid=true if vwi->grid=true) 
*/
        if ((line_grid) && (grid))
        {
          if ((pci_w->file_type==FT_TIFF) || (pci_w->file_type==FT_PPM))
            /* use colored grid. */
            add_gridpix(vwi_r,&br,&bg,&bb);
          else                        /* For bw format use white grid. */
            add_gridpix(NULL,&br,&bg,&bb);
        }

        if (x_w+picstart_w < pci_w->fri.words_per_line)
        {
          *(line_to + x_w + picstart_w)   = br;
          *(line_to_g + x_w + picstart_w) = bg;
          *(line_to_b + x_w + picstart_w) = bb;
        }

/* Increment write-X. Different from x if multi-channels are saved. */
        x_w++;
      }
    }

    // Copy HRPT trailer to line_to if HRPT --> HRPT. Not done if reeceived=FT_METOP/FT_FY3
    if ((pci_r->file_type!=FT_METOP) && (pci_r->file_type!=FT_FY3))
    {
      if ((pci_r->channels_recorded[0]) && (pci_w->channels_recorded[0]))
      {
        int picend;
        picend=pci_r->fri.words_per_frame-100;
        for (x=picend; x<(picend+100); x++)
        {
          *(line_to + x_w + picstart_w)  =*(line_from+x);
          *(line_to_g + x_w + picstart_w)=*(line_from+x);
          *(line_to_b + x_w + picstart_w)=*(line_from+x);
          x_w++;
        }
      }
    }

/* Update progress bar and detect if cancelled */
    if (Update_Progress(progress_bar,y,pci_w->height))
    {
      cancelled=TRUE;
      break;
    }

/* Write line */
    write_line(line_to,line_to_g,line_to_b,pci_w,dec_status);
  }

/* Close progress bar */
  Close_Window(gtk_widget_get_toplevel(progress_bar));

  write_finish(pci_w);

  if (vwi_ri.lin_xtbl) free(vwi_ri.lin_xtbl); vwi_ri.lin_xtbl=NULL;
  free_lines(lines,nr_ilines);
  if (linesrgb) free_lines(linesrgb,3);
  free(line_to);
  free(line_to_g);
  free(line_to_b);
  if (line_grid) free(line_grid);
  free(xmap);
  free(ymap);
  free(zmap);
  fclose(pci_w->fp);

  if (cancelled)
    Create_Message("Info","Cancelled; %d of %d lines saved in %s.",
               y,pci_w->height,file_name);
  else
    Set_Entry(window,LAB_STATUS,"Saved.");
//    Create_Message("Info","Saved in %s; bitshift=%d.",
//                                           file_name,bitshift);
}



/*
%s		grp->sat_src
%c		pc.chan
%t		timestr
%n		channelnumber
%y/%M/%d/%h/%m	tijd
Example:
"%p%S_%c_%t%r"
*/

#ifdef WEG
static int replace_str(char *l,char *s1,char *s2)
{
  char *p,*p1;
  int n1=strlen(s1);
  int n2=strlen(s2);
  if ((p=strstr(l,s1)))
  {
    if (n1 >= n2)
    {
      strcpy(p+n2,p+n1);
      strncpy(p,s2,n2);
    }
    else
    {
      for (p1=l+strlen(l)+(n2-n1); p1>p; p1--) *p1=*(p1-(n2-n1));
      strncpy(p,s2,n2);
    
    }
    if (strstr(s2,s1)) return 0;
    return 1;
  }
  return 0;
}

static char *mk_filename(char *sat,char *ch,struct tm *tm,char *frmt)
{
  static char fno[200];
  char timestr[200];
  /* Gen timestring for filename generation */
  *timestr=0;
  if (tm)
    strftime(timestr,20,"%y%m%d_%H%M",tm);

  if (frmt)
  {
    strcpy(fno,frmt);

    replace_str(fno,"%s",sat);
    replace_str(fno,"%c",ch);
    replace_str(fno,"%t",timestr);
    if (*timestr)
    {
      strftime(timestr,200,fno,tm);
      strcpy(fno,timestr);
    }
  }
  else
  {
    strcpy(fno,FILENAME_PREFIX);
  }
  return fno;
}
#endif

// Get filename from pci->filename: fn
//   Add prefix/suffix to fn:
//   ch=string: add first char in filename (=channel number, or nothing if > 1 selected)
//   sattype: extension (.hpt etc.)
//   add pathname if pci->worklocation != NULL
//   if file exist: add number such that generated filename doesn't exist
static int create_new_filename2(char *fn,PICINFO *pci,char *ch,char *sattype)
{
  FILE *fp=NULL;
  int nr=0;
  char *p;
  do
  {
    if (fp) fclose(fp); fp=NULL;
    if (nr>1000) return 0;

    // create filename
    if (pci->work_location)
      strcpy(fn,pci->work_location);
    else
      strcpy(fn,"");
    if ((p=strrchr(pci->file_name,DIR_SEPARATOR)))
      strcat(fn,p+1);
    else
      strcat(fn,pci->file_name);

    // remove extension
    if ((p=strrchr(fn,'.'))) *p=0;

    // complete filename
    if (nr)
    {
      if ((ch) && (*ch))
        sprintf(fn,"%s_c%s_%02d.%s",fn,ch,nr,sattype);
      else
        sprintf(fn,"%s_%02d.%s",fn,nr,sattype);
    }
    else
    {
      if ((ch) && (*ch))
        sprintf(fn,"%s_c%s.%s",fn,ch,sattype);
      else
        sprintf(fn,"%s.%s",fn,sattype);
    }
    nr++;
  } while ((fp=fopen(fn,"rb")));                    /* Exit if file doesn't exist. */
  return 1;
}

/*************************************
 * Manage buttons.  
 *************************************/
void save_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *save_window=gtk_widget_get_toplevel(widget);
  char *name=(char *)data;
  char fn[200],*pfn;
  char sattype[100];
  char channels[100];
  struct tm tm;
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  static int busy;
  int i;
  if (busy) return;
  busy=1;
  Set_Entry(widget,LAB_STATUS,"");
  tm=mom_tm0(prefs.utc_offset);
  strncpy(fn,Get_Entry(save_window,LAB_FILENAME),150);
  strcpy(sattype,(pci->sat_type==HRPT?  "hrpt"  : 
                  pci->sat_type==CHRPT? "chrpt" : 
                  pci->sat_type==M1HRPT? "m1hrpt":
                  pci->sat_type==M2HRPT? "m2hrpt":
                  pci->sat_type==METOP? "metop" : FILENAME_PREFIX));

  // if 1 channel selected: channels="<nr>"
  // otherwise: channels=""
  *channels=0;
  for (i=1; i<NR_CHANNELS_MAX; i++)
  {
    char tmp[20];
    sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);           /* Construct button name */
    if (Get_Button(save_window,tmp))
    {
      if (*channels) { *channels=0; break; }
      sprintf(channels,"%s%d",channels,i);
    }
  }

/* Exit button */
  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(save_window);
  }
  
/* Catch selected format. */
  if ((!strcmp(name,LAB_Genfn)) ||
      (!strcmp(name,LAB_HRPT))  ||
      (!strcmp(name,LAB_HPT))   ||
      (!strcmp(name,LAB_PPM))   ||
      (!strcmp(name,LAB_PGM))   ||
      (!strcmp(name,LAB_PGM1))  ||
      (!strcmp(name,LAB_GIF))   ||
      (!strcmp(name,LAB_JPEG)))
  {
    FILE_TYPE file_type=0;
    if (Get_Button(save_window,LAB_SUN))  file_type=FT_SUN;
    if (Get_Button(save_window,LAB_TIFF)) file_type=FT_TIFF;
    if (Get_Button(save_window,LAB_PPM))  file_type=FT_PPM;
    if (Get_Button(save_window,LAB_PGM))  file_type=FT_PGM;
    if (Get_Button(save_window,LAB_PGM1)) file_type=FT_PGM1;
    if (Get_Button(save_window,LAB_GIF))  file_type=FT_GIF;
    if (Get_Button(save_window,LAB_JPEG)) file_type=FT_JPEG;
    if (Get_Button(save_window,LAB_HRPT)) file_type=FT_HRPT;
    if (Get_Button(save_window,LAB_HPT))  file_type=(pci->file_type==FT_FY3? FT_CHPT : FT_HPT);
    // fn gets new filename; 
    //   if working location defined: including path name
    //   and made unique
    create_new_filename2(fn,pci,channels,FN_EXT(file_type));
    if ((pfn=strrchr(fn,DIR_SEPARATOR))) pfn++; else pfn=fn;
    Set_Entry(save_window,LAB_FILENAME,"%s",pfn);

    if ((vwi->process_data) && 
        ((Get_Button(save_window,LAB_PPM)) || (Get_Button(save_window,LAB_JPEG))))
    {
      Set_Led(save_window,LAB_PPM_MCH,0xf00);
    }
    else
    {
      Set_Led(save_window,LAB_PPM_MCH,0x777);
    }
  }

  if ((!strcmp(name,LAB_CHALL))  && ((pci->file_type==FT_HRPT)  || 
      (pci->file_type==FT_METOP) || (pci->file_type==FT_FY3)   || 
      (pci->file_type==FT_HPT)   || (pci->file_type==FT_CHPT)))
  {
    int i;
    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      GtkWidget *b;
      char tmp[20];
      if ((pci) && (pci->channels_recorded[i]))      /* If channel recorded */
      {
        sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);           /* Construct button name */
        if ((b=Find_Widget(save_window,tmp)))
          Set_Button(save_window,tmp,TRUE);
      }
    }
  }

/* Channel button. If no HRPT selected then just 1 channel may be selected. */
  if ((!strncmp(name,LAB_CH,strlen(LAB_CH))) || (!strcmp(name,LAB_CHALL)) ||
      (!strcmp(name,LAB_PPM))                || (!strcmp(name,LAB_PGM))   ||
      (!strcmp(name,LAB_PGM1))               || (!strcmp(name,LAB_GIF))   ||
      (!strcmp(name,LAB_JPEG)))
  {
    if ((Get_Button(save_window,LAB_SUN))  ||
        (Get_Button(save_window,LAB_GIF))  ||
        (Get_Button(save_window,LAB_JPEG)) ||
        (Get_Button(save_window,LAB_PPM))  ||
        (Get_Button(save_window,LAB_PGM))  ||
        (Get_Button(save_window,LAB_PGM1)))
    {
      int i;
      for (i=1; i<NR_CHANNELS_MAX; i++)
      {
        GtkWidget *b;
        char tmp[20];
        sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);           /* Construct button name */
        if ((b=Find_Widget(save_window,tmp)))
          if (b!=widget)
            Set_Button(save_window,tmp,FALSE);
      }
    }
  }


/* Save button */
  if ((*fn) && (!strcmp(name,LAB_Save)))
  {
    FILE_TYPE file_type=0;
    PICINFO pci_w=*pci;
    char file_name[400];
    int i;
    int nr_chan=0;

/* Get choices */
    if (Get_Button(save_window,LAB_SUN))  file_type=FT_SUN;
    if (Get_Button(save_window,LAB_TIFF)) file_type=FT_TIFF;
    if (Get_Button(save_window,LAB_PPM))  file_type=FT_PPM;
    if (Get_Button(save_window,LAB_PGM))  file_type=FT_PGM;
    if (Get_Button(save_window,LAB_PGM1)) file_type=FT_PGM1;
    if (Get_Button(save_window,LAB_GIF))  file_type=FT_GIF;
    if (Get_Button(save_window,LAB_JPEG)) file_type=FT_JPEG;
    if (Get_Button(save_window,LAB_HRPT))
    {
      if (IS_HRPT_FAM(pci->file_type)) 
        file_type=pci->file_type;             /* Use same format as source */
      else
        file_type=FT_HRPT;
      if ((pci->file_type==FT_METOP) || (pci->file_type==FT_HPT)) pci_w.channels_recorded[0]=TRUE;
      if ((pci->file_type==FT_FY3) || (pci->file_type==FT_CHPT)) pci_w.channels_recorded[0]=TRUE;
    }
    if (Get_Button(save_window,LAB_MHRPT)) file_type=FT_MHRPT1;
    if (Get_Button(save_window,LAB_HPT))
    {
      if (pci->file_type==FT_FY3) file_type=FT_CHPT; else file_type=FT_HPT;
      pci_w.channels_recorded[0]=TRUE;
    }

    pci_w.file_type=file_type;

    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      GtkWidget *button;
      char tmp[20];
      pci_w.channels_recorded[i]=FALSE;
      sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);               /* Construct button name */
      if ((button=Find_Widget(save_window,tmp)))       /* Get button widget */
      {
        pci_w.channels_recorded[i]=Get_Button(save_window,tmp);
        if (pci_w.channels_recorded[i]) nr_chan++;
      }
    }
    if ((!nr_chan) && (!vwi->process_data))
    {
      Create_Message("Error","No channels selected.");
      busy=0;
      return;
    }

    if ((pci_w.file_type!=FT_HRPT) && (pci_w.file_type!=FT_CHRPT) && 
        (pci_w.file_type!=FT_MHRPT1) && (pci_w.file_type!=FT_MHRPT2) &&
        (pci_w.file_type!=FT_HPT) && (pci_w.file_type!=FT_CHPT))
      pci_w.channels_recorded[0]=FALSE;

    pci_w.fri.bytes_per_line=pci->width;
    pci_w.fri.words_per_line=pci->width;
    sprintf(file_name,"%s%s",pci_w.work_location,fn);

    if ((pci_w.fp=fopen(file_name,"rb")))
    {
      fclose(pci_w.fp);
      if ((Create_Choice("Warning",2,"Cancel","OK"," File %s exist. Overwrite?",
                                                   file_name))==1)
      {
        busy=0;
        return;
      }
    }
    Save_it(save_window,pci,&pci_w,file_name,vwi);
  }
  busy=0;
}

/*************************************
 * Set state of channel buttons 
 *************************************/
void Update_channel_save(GtkWidget *main_window,PICINFO *pci,VIEWINFO *vwi)
{
  GtkWidget *save_window=Find_Window(main_window,LAB_Save);
  if (vwi->process_data)
  {
    Update_channel_state(save_window,NULL,NULL,LAB_CH,LAB_CHSUF);
  }
  else
  {
    Update_channel_state(save_window,pci,vwi,LAB_CH,LAB_CHSUF);
  }

  if ((vwi->process_data) && (Get_Button(save_window,LAB_PPM)))
    Set_Led(save_window,LAB_PPM_MCH,0xf00);
  else
    Set_Led(save_window,LAB_PPM_MCH,0x777);
}

/*************************************
 * Save file 
 * Menu: File -> Save as
 *************************************/
void MenuSave(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  GtkWidget *save_window;
  GtkWidget *wa[4],*wb,*wc,*wd,*we,*wx,*wy,*wz;
  GtkWidget *w1,*w2,*w3;
  GtkWidget *wt[12];
  int i,n=0;
  char fn[200],*pfn,sattype[20];
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

  // name sat and extension
  strcpy(sattype,(pci->sat_type==HRPT?   "hrpt"  : 
                  pci->sat_type==CHRPT?  "chrpt" : 
                  pci->sat_type==M1HRPT? "m1hrpt":
                  pci->sat_type==M2HRPT? "m2hrpt":
                  pci->sat_type==METOP?  "metop" : FILENAME_PREFIX));

  if (!(save_window=Create_Window(main_window,0,0,LAB_Save,NULL))) return;

/*
// waarom was dit toegevoegd? Steelt toetsen weg, die komen dan iet meer in bv. entry.
//  gtk_signal_connect(GTK_OBJECT(save_window), "key_press_event",
//		      GTK_SIGNAL_FUNC(save_commands), NULL);
*/

  wa[1]=Create_Led(LAB_PPM_MCH,0x7ff);

  wa[2]=Create_ButtonArray(NULL,save_commands,5,
                             TOGGLE,LAB_CH1,FALSE,
                             TOGGLE,LAB_CH2,FALSE,
                             TOGGLE,LAB_CH3,FALSE,
                             TOGGLE,LAB_CH4,FALSE,
                             TOGGLE,LAB_CH5,FALSE,
                             TOGGLE,LAB_CH6,FALSE,
                             TOGGLE,LAB_CH7,FALSE,
                             TOGGLE,LAB_CH8,FALSE,
                             TOGGLE,LAB_CH9,FALSE,
                             TOGGLE,LAB_CH10,FALSE,
                             0
                          );

  {
    GdkColor startbut_clr,startbutsclr;
    GtkWidget *wbut;
    startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
    startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;
    for (i=1; i<=10; i++)
    {
      char tmp[10];
      sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);
      wbut=Find_Widget(wa[2],tmp);
      Set_Color(wbut,GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);
    }
  }
  wa[3]=Create_Button(LAB_CHALL,save_commands);
  wa[2]=Pack("Channel select",'v',wa[2],5,wa[3],5,NULL);

  wt[n++]=Create_Radio(LAB_HRPT,TRUE,save_commands);
  wt[n++]=Create_Radio(LAB_HPT,FALSE,save_commands);
#if _SUN_
  wt[n++]=Create_Radio(LAB_SUN,FALSE,save_commands);
#endif
#if _TIFF_
  wt[n++]=Create_Radio(LAB_TIFF,FALSE,save_commands);
#endif
#if _PPM_
  wt[n++]=Create_Radio(LAB_PPM,FALSE,save_commands);
  wt[n++]=Create_Radio(LAB_PGM,FALSE,save_commands);
  wt[n++]=Create_Radio(LAB_PGM1,FALSE,save_commands);
#endif
#if _GIF_
  wt[n++]=Create_Radio(LAB_GIF,FALSE,save_commands);
#endif
#if _JPEG_
  wt[n++]=Create_Radio(LAB_JPEG,FALSE,save_commands);
#endif
#if _MHRPT1_
  wt[n++]=Create_Radio(LAB_MHRPT,FALSE,save_commands);
#endif
  wb=Pack(NULL,'v',NULL);
  for (i=0; i<n; i++)
    wb=Pack(NULL,'v',wb,0,wt[i],0,NULL);
  wb=Pack("File type",'v',wb,0,NULL);

  wd=Create_Entry(LAB_FILENAME,save_commands,"%-28s","");
  we=Create_Entry(LAB_STATUS,NULL,"%-28s","");
  wd=Pack(NULL,'v',wd,1,we,1,NULL);
  we=Create_Entry(LAB_BITSHIFT,NULL,"%1d",0);

  w1=Create_Button(LAB_Genfn,save_commands);
  w2=Create_Button(LAB_Save,save_commands);
  w3=Create_Button(LAB_Exit,save_commands);
  wc=SPack("","h",w1,"5ef",w2,"5ef",w3,"5ef",NULL);

  wd=Pack(NULL,'h',wd,5,NULL);
  wx=SPack(NULL,"h",wa[1],"1",we,"e1",NULL);
  wx=Pack(NULL,'v',wx,1,wa[2],10,wd,5,NULL);
  wy=Pack(NULL,'h',wx,10,wb,10,NULL);
  wz=Pack(NULL,'v',wy,10,wc,10,NULL);

  gtk_container_add(GTK_CONTAINER(save_window),wz);
  gtk_widget_show(save_window);
  place_window(save_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);

  if (IS_HRPT_METOP_FAM(pci->file_type))
  {
    Set_Button(save_window,LAB_HRPT,TRUE);
  }
  else switch(pci->file_type)
  {
    case FT_PPM : Set_Button(save_window,LAB_PPM,TRUE);  break;
    case FT_PGM : Set_Button(save_window,LAB_PGM,TRUE);  break;
    case FT_PGM1: Set_Button(save_window,LAB_PGM1,TRUE); break;
    case FT_GIF : Set_Button(save_window,LAB_GIF,TRUE);  break;
    case FT_JPEG: Set_Button(save_window,LAB_JPEG,TRUE); break;
    case FT_HRI : Set_Button(save_window,LAB_GIF,TRUE);  break;
    default     : Set_Button(save_window,LAB_GIF,TRUE);  break;
  }
  Update_channel_save(main_window,pci,vwi);
  {
    char channels[10]; *channels=0;
  
    for (i=1; i<NR_CHANNELS_MAX; i++)
    {
      char tmp[20];
      sprintf(tmp,"%s%d%s",LAB_CH,i,LAB_CHSUF);           /* Construct button name */
      if (Get_Button(save_window,tmp))
      {
        if (*channels) { *channels=0; break; }
        sprintf(channels,"%s%d",channels,i);
      }
    }
    strcpy(fn,pci->file_name);
    create_new_filename2(fn,pci,channels,FN_EXT(pci->file_type));
    if ((pfn=strrchr(fn,DIR_SEPARATOR))) pfn++; else pfn=fn;
    Set_Entry(save_window,LAB_FILENAME,"%s",pfn);
  }
}

/*************************************
 * Rewrite file header using info in pci 
 *************************************/
int Rewrite_hdr(PICINFO *pci)
{
  int rslt=0;
  switch(pci->file_type)
  {
/* Just these formats; NO hri! */
    case FT_CHRPT: case FT_HRPT: case FT_SEAWIF: case FT_MHRPT1:
      if ((pci->fp=freopen(pci->file_name,"rb+",pci->fp)))
      {
        if (write_hrpt1_hdr(pci))
        {
          Create_Message("Error","Rewriting header in %s failed.",
                                                          pci->file_name);
          rslt=0;
        }
        else
        {
          Create_Message("Info","Rewriting header in %s succeeded.",
                                                          pci->file_name);
          rslt=1;
        }
        if (!(pci->fp=freopen(pci->file_name,"rb",pci->fp)))
        {
          Create_Message("Error","Can't reopen %s for reading",
                                                          pci->file_name);
          rslt=0;
        }
      }
      else
      {
        Create_Message("Error","Can't reopen %s for writing",pci->file_name);
        rslt=0;
      }
    break;
    default:
    break;
  }
  return rslt;
}
