/**************************************************
 * RCSId: $Id: gtk_inter.c,v 1.3 2017/07/07 21:56:27 ralblas Exp $
 *
 * Interfaces between GTK and functions
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: gtk_inter.c,v $
 * Revision 1.3  2017/07/07 21:56:27  ralblas
 * _
 *
 * Revision 1.2  2016/12/02 11:14:47  ralblas
 * _
 *
 * Revision 1.1  2011/02/28 21:23:01  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdlib.h>
#include "wsat.h"
#include "hrpt1.h"
#include "menu_wsat.h"


/*************************************
 *************************************
 * Interface functions for menu
 *************************************
 *************************************/


/*************************************
 * Load palette from file
 *************************************/
void load_palette(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  PICINFO *pci=Get_Picinfo_db(widget);

  if (!GTK_CHECK_MENU_ITEM(widget)->active) return;
  load_std_palette(main_window,pci);
  Update_Main(widget);
}

/*************************************
 * Generate grey palette
 *************************************/
void load_greyscale(GtkWidget *widget)
{
  GtkWidget *main_window=Find_Parent_Window(widget);
  if (!GTK_CHECK_MENU_ITEM(widget)->active) return;
  load_std_palette(main_window,NULL);
  Update_Main(widget);
}

/*************************************
 * Handle zoom actions
 *************************************/
void Menu_Zoombuts(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);

  char *name=(char *)data;
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  int wnd_width=drawing_area->allocation.width;  /* width window */
  int wnd_height=drawing_area->allocation.height; /* height window */


  if (!strcmp(name,MENU_FULL))
  {
      vwi->zval=zoom_full(0,0,0,0);
  }

  if (!strcmp(name,MENU_ZMIN))
  {
    vwi->zval.zx=vwi->zval.zx*vwi->fix_zoomfactor;
    vwi->zval.zy=vwi->zval.zy*vwi->fix_zoomfactor;
    vwi->zval.ox=vwi->zval.ox+
                 nint(pci->width/vwi->zval.zx*(vwi->fix_zoomfactor-1)/2);
    vwi->zval.oy=vwi->zval.oy+
                 nint(pci->height/vwi->zval.zy*(vwi->fix_zoomfactor-1)/2);
  }
  if (!strcmp(name,MENU_ZMOUT))
  {
      vwi->zval.ox=vwi->zval.ox-
                   nint(pci->width/vwi->zval.zx*(vwi->fix_zoomfactor-1)/2);
      vwi->zval.oy=vwi->zval.oy-
                   nint(pci->height/vwi->zval.zy*(vwi->fix_zoomfactor-1)/2);
      vwi->zval.zx=vwi->zval.zx/vwi->fix_zoomfactor;
      vwi->zval.zy=vwi->zval.zy/vwi->fix_zoomfactor;
  }
  if (!strcmp(name,MENU_ZMVAR))
  {
    Create_Cursor(drawing_area,GDK_PLUS);
    vwi->zooming=TRUE;
  }
  if (!strcmp(name,MENU_ZMFIX))
  {
    vwi->fixzooming=TRUE;
  }

  if (!strcmp(name,MENU_ZMLAST))
  {
    ZOOMVALS tzval;
    tzval=vwi->zval;
    vwi->zval=vwi->zval_pre;
    vwi->zval_pre=tzval;
  }

  if (!strcmp(name,MENU_ZMDXY))
  {
    vwi->zval.dx=vwi->zval.dy;
    vwi->zval.zx=vwi->zval.dx/((float)wnd_width/(float)pci->width);
  }
  if (!strcmp(name,MENU_ZMDYX))
  {
    vwi->zval.dy=vwi->zval.dx;
    vwi->zval.zy=vwi->zval.dy/((float)wnd_height/(float)pci->height);
  }
  if (!strcmp(name,MENU_ZMXY))
  {
    vwi->zval.zx=vwi->zval.zy;
  }
  if (!strcmp(name,MENU_ZMYX))
  {
    vwi->zval.zy=vwi->zval.zx;
  }

  if (!strcmp(name,MENU_ZVAL))
  {
    Menu_Zoom(widget);
  }

/* Check and correct zoom values */
  correct_zoomval(drawing_area,vwi,pci);

  Update_Zoom(main_window);
  Update_Main(widget);
}

/*************************************
 * View choices
 *************************************/
void Menu_Viewbuts(GtkWidget *widget,gpointer data)
{
  char *name=(char *)data;
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  int nstate;
  nstate=GTK_CHECK_MENU_ITEM(widget)->active;
/* Scan direction */
  if (!strcmp(name,MENU_SATDIR))
  {
    if (nstate==1)
    {
      vwi->scan_hor=E_W;
      vwi->scan_ver=S_N;
    }
    else
    {
      vwi->scan_hor=W_E;
      vwi->scan_ver=N_S;
    }
  }

/* Add grid */
  if (!strcmp(name,MENU_GRIDADD))
  {
    vwi->grid=(nstate? TRUE : FALSE);
  }

  if (!strcmp(name,MENU_POSXY))
  {
    vwi->coordxy=(nstate? TRUE : FALSE);
  }



/* Inverse mapping */
  if (!strcmp(name,MENU_INVERSE))
  {
    vwi->invert=(nstate? TRUE : FALSE);
  }

/* Map to temp. */
  if (!strcmp(name,MENU_TEMP))
  {
    vwi->map_temp=(nstate? TRUE : FALSE);
  }

/* HRPT Linearization */
  if (!strcmp(name,MENU_LIN))  
  {
    vwi->linearize=(nstate? TRUE : FALSE);
  }

/* Despeckle picture */
  if (!strcmp(name,MENU_DESPEC))  
  {
    vwi->despeckle=(nstate? TRUE : FALSE);
  }

/* destrip metop picture */
  if (!strcmp(name,MENU_DESTRIP))  
  {
    vwi->destrip=(nstate? TRUE : FALSE);
  }

  if (!strcmp(name,MENU_RS))  
  {
    vwi->reed_solomon=(nstate? TRUE : FALSE);
  }

/* Skip faulty lines */
  if (!strcmp(name,MENU_SKIP_FLTS))
  {
    vwi->skip_faulty_lines=(nstate? TRUE : FALSE);
  }

  if (!strcmp(name,MENU_SKIP_PRE))
  {
    vwi->mask_skip=(nstate? vwi->mask_skip|MASK_HRPT1_SYNC_OK_PRE :
                            vwi->mask_skip&(~MASK_HRPT1_SYNC_OK_PRE));
  }
  if (!strcmp(name,MENU_SKIP_POST))
  {
    vwi->mask_skip=(nstate? vwi->mask_skip|MASK_HRPT1_SYNC_OK_POST :
                            vwi->mask_skip&(~MASK_HRPT1_SYNC_OK_POST));
  }

  if (!strcmp(name,MENU_MARK_FLTS))
  {
    vwi->mark_faulty_lines=(nstate? TRUE : FALSE);
  }
  Update_Main(widget);
}

