#include "wsat.h"
#include "hrpt1.h"
#include "hrpt.h"
/*************************************
 * Read a quad word (5 bytes)
 *************************************/
static int read_hptqwords(FILE *fp,guint16 *pline,int nr_qwords)
{
  int i;
  guchar tmp[7];
  for (i=0; i<nr_qwords; i++)
  {
    if (!(fread(tmp,5,1,fp))) return 0;
    if (pline)
    {
      pline[0+i*4]=((tmp[0]<<2)&0x3fc)+((tmp[1]>>6)&0x03);
      pline[1+i*4]=((tmp[1]<<4)&0x3f0)+((tmp[2]>>4)&0x0f);
      pline[2+i*4]=((tmp[2]<<6)&0x3c0)+((tmp[3]>>2)&0x3f);
      pline[3+i*4]=((tmp[3]<<8)&0x300)+((tmp[4]>>0)&0xff);
    }
  }
  return 1;
}

/*************************************
 * Calculate # of lines in hrpt file
 *************************************/
static int hpt_lines_in_file(PICINFO *pci)
{
  int y=0;
  FILE *fp=pci->fp;
  int nr_qwords=pci->fri.words_per_frame/4;
  fseek(fp,pci->pic_strt,SEEK_SET);
  for (; ; y++)
  {
    if (!(read_hptqwords(fp,NULL,nr_qwords))) break;
  }
  return y;
}

// hpt format has 13864 bytes = 11091.2 words
// c10 format has 27728 bytes = 22182.4 words
int read_hpt_hdr(PICINFO *pci,int nrchan)
{
  int i;
  int wpf=2218*nrchan+(nrchan/5); // hpt: 11091, c10: 22182
  int pos_start_word=740; // moet eigenlijk 750 zijn (maar dat is niet deelbaar door (8/10))
  if (pci->file_type==FT_CHPT) pos_start_word+=60;

  pci->fri.words_per_frame=wpf;
  pci->fri.words_per_line=wpf;
  pci->pic_strt=(nrchan/5)*pos_start_word*10/8;        // zet op goede kanaaalnr., 740 moet eigenlijk 750 zijn (niet deelbaar door 8)
  pci->width=2048;
  pci->height=hpt_lines_in_file(pci);
  pci->depth=10;
  pci->clrmap.size=1024;
  for (i=0; i<pci->clrmap.size; i++)
  {
    pci->clrmap.red[i]=i;
    pci->clrmap.green[i]=i;
    pci->clrmap.blue[i]=i;
  }
  pci->fri.bytes_per_line=pci->fri.words_per_frame*10/8;

  if (pci->fri.words_per_frame*10./8. != pci->fri.bytes_per_line)
    pci->fri.bytes_per_line++;

  for (i=0; i<=10; i++)pci->channels_recorded[i]=FALSE;
  switch(pci->file_type)
  {
    case FT_HPT:  case FT_METOP:                   /* HRPT/METOP */
      for (i=0; i<=5; i++)
        pci->channels_recorded[i]=TRUE;
    break;
    case FT_CHPT: case FT_FY3:                     /* Fengyun */
      for (i=0; i<=10; i++)
        pci->channels_recorded[i]=TRUE;
    break;
    default:
    break;
  }


  for (i=0; i<=nrchan; i++)
    pci->channels_recorded[i]=TRUE;
  pci->width_original=2048;
  pci->max_sens_angle=DEF_NOAA_ALPHAMAX;
  pci->sat_type=HRPT;
  pci->fri.inter_chan=nrchan;
  strcpy(pci->sat_name,""); // unknown satellite; get from data?
  return 0;
}

/*************************************
 * Put (c)hrpt line number y from file into *line
 * Return: decoder frame error code
 *************************************/
int read_hpt_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line)
{
  FILE *fp=pci->fp;
  guint16 syncpost[8];
  int nr_qwords=pci->fri.words_per_frame/4;
  int rslt=0;
  do
  {
    rslt=MASK_HRPT1_READ_OK_PRE;

    fseek(fp,pci->pic_strt + y*pci->fri.bytes_per_line,SEEK_SET);

    read_hptqwords(fp,line,nr_qwords);
/* read next sync */
    read_hptqwords(fp,syncpost,2);

/* Test sync mode if sync-word is recorded */
    if ((line[0]==0x284) && (line[1]==0x16f) && (line[2]==0x35c) &&
        (line[3]==0x19d) && (line[4]==0x20f) && (line[5]==0x095))
      rslt|=MASK_HRPT1_SYNC_OK_PRE;

    if ((syncpost[0]==0x284) && (syncpost[1]==0x16f) && (syncpost[2]==0x35c) &&
        (syncpost[3]==0x19d) && (syncpost[4]==0x20f) && (syncpost[5]==0x095))
      rslt|=MASK_HRPT1_SYNC_OK_POST;

    y=y+(vwi->scan_ver==S_N? (-1) : (1));   /* incr or decr line nr. dep. on scan dir. */
    if (y<0) return 0;                      /* stop if BOF */

    if (!vwi->skip_faulty_lines) break;
    if ( ((rslt&MASK_HRPT1_SYNC_OK_PRE)  || (!(vwi->mask_skip&MASK_HRPT1_SYNC_OK_PRE))) &&
         ((rslt&MASK_HRPT1_SYNC_OK_POST) || (!(vwi->mask_skip&MASK_HRPT1_SYNC_OK_POST))) )
      break;

  } while (rslt);                           /* stop if EOF */

  return rslt;
}
