/**************************************************
 * RCSId: $Id: hrpt_calcs.c,v 1.1 2003/03/05 16:17:52 ralblas Exp $
 *
 * Conversion of pixel-value into temperature and related functions.
 * This file also contains the satellite constants (for each sat. different).
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: hrpt_calcs.c,v $
 * Revision 1.1  2003/03/05 16:17:52  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <ctype.h>
#include <math.h>
#include <stdlib.h>
#include "wsat.h"
#include "hrpt.h"
/*************************************
 * Load satellite dependent constants
 * for temperature calculation 
 *************************************/
NOAACAL load_cal(char *satname)
{
  int noaa=0;
  char *p;
  NOAACAL noaacal;
  for (p=satname; !isdigit(*p); p++);
  if (*p) noaa=atoi(p);

  switch(noaa)
  {
    case 14:
/* Table 1.4.10-4 */
      noaacal.d0[1]=276.597;
      noaacal.d0[2]=276.597;
      noaacal.d0[3]=276.597;
      noaacal.d0[4]=276.597;
      noaacal.d1[1]=0.051275;
      noaacal.d1[2]=0.051275;
      noaacal.d1[3]=0.051275;
      noaacal.d1[4]=0.051275;
      noaacal.d2[1]=1.363E-06;
      noaacal.d2[2]=1.363E-06;
      noaacal.d2[3]=1.363E-06;
      noaacal.d2[4]=1.363E-06;
      noaacal.d3[1]=0;
      noaacal.d3[2]=0;
      noaacal.d3[3]=0;
      noaacal.d3[4]=0;
      noaacal.d4[1]=0;
      noaacal.d4[2]=0;
      noaacal.d4[3]=0;
      noaacal.d4[4]=0;

      noaacal.c1=1.1910427E-05;
      noaacal.c2=1.4387752;

/* Table 1.4.10-1 */
/*  NOTE: nuc is taken for 230-270K! 
    a and b should be calculated from 4 numbers/channel in the table?
*/
      noaacal.nuc[3]=2642.807;
      noaacal.a[3]=0.0;
      noaacal.b[3]=1.0;
      noaacal.nuc[4]=928.8284;
      noaacal.a[4]=0.0;
      noaacal.b[4]=1.0;
      noaacal.nuc[5]=834.8066;
      noaacal.a[5]=0.0;
      noaacal.b[5]=1.0;


/* Table 1.4.10-3 */
      noaacal.Ns[3]=0.0069;   /* Rsp */
      noaacal.b0[3]=-0.0031;  /* D */
      noaacal.b1[3]=1.00359;  /* A */
      noaacal.b2[3]=0;        /* B */
      noaacal.Ns[4]=-4.05;
      noaacal.b0[4]=3.72;
      noaacal.b1[4]=0.92378;
      noaacal.b2[4]=0.0003822;
      noaacal.Ns[5]=-2.29;
      noaacal.b0[5]=2.00;
      noaacal.b1[5]=0.96194;
      noaacal.b2[5]=0.0001742;
    break;
    default:
      Create_Message("Error",
                     "No temperature mapping\n for %s available!\n Asume NOAA15...",
                      satname);
    case 15:
/* Table D.1-8 */
      noaacal.d0[1]=276.60157;
      noaacal.d0[2]=276.62531;
      noaacal.d0[3]=276.67413;
      noaacal.d0[4]=276.59258;
      noaacal.d1[1]=0.051045;
      noaacal.d1[2]=0.050909;
      noaacal.d1[3]=0.050907;
      noaacal.d1[4]=0.050966;
      noaacal.d2[1]=1.36328E-06;
      noaacal.d2[2]=1.47266E-06;
      noaacal.d2[3]=1.47656E-06;
      noaacal.d2[4]=1.47656E-06;
      noaacal.d3[1]=0;
      noaacal.d3[2]=0;
      noaacal.d3[3]=0;
      noaacal.d3[4]=0;
      noaacal.d4[1]=0;
      noaacal.d4[2]=0;
      noaacal.d4[3]=0;
      noaacal.d4[4]=0;
      noaacal.c1=1.1910427E-05;
      noaacal.c2=1.4387752;

/* Table D1.1-11 */
      noaacal.nuc[3]=2695.9743;
      noaacal.a[3]=1.621256;
      noaacal.b[3]=0.998015;
      noaacal.nuc[4]=925.4075;
      noaacal.a[4]=0.337810;
      noaacal.b[4]=0.998719;
      noaacal.nuc[5]=839.8979;
      noaacal.a[5]=0.304558;
      noaacal.b[5]=0.999024;

/* Table D.1-14 */
      noaacal.Ns[3]=0;
      noaacal.b0[3]=0;
      noaacal.b1[3]=1;
      noaacal.b2[3]=0;
      noaacal.Ns[4]=-4.50;
      noaacal.b0[4]=4.76;
      noaacal.b1[4]=-0.0932;
      noaacal.b2[4]=0.0004524;
      noaacal.Ns[5]=-3.61;
      noaacal.b0[5]=3.83;
      noaacal.b1[5]=-0.0659;
      noaacal.b2[5]=0.0002811;
    break;

    case 16:
/* Table D.2-9 */
      noaacal.d0[1]=276.355;
      noaacal.d0[2]=276.142;
      noaacal.d0[3]=275.996;
      noaacal.d0[4]=276.132;
      noaacal.d1[1]=5.562E-02;
      noaacal.d1[2]=5.605E-02;
      noaacal.d1[3]=5.486E-02;
      noaacal.d1[4]=5.494E-02;
      noaacal.d2[1]=-1.590E-05;
      noaacal.d2[2]=-1.707E-05;
      noaacal.d2[3]=-1.223E-05;
      noaacal.d2[4]=-1.344E-05;
      noaacal.d3[1]=2.486E-08;
      noaacal.d3[2]=2.595E-08;
      noaacal.d3[3]=1.862E-08;
      noaacal.d3[4]=2.112E-08;
      noaacal.d4[1]=-1.199E-11;
      noaacal.d4[2]=-1.224E-11;
      noaacal.d4[3]=-0.853E-11;
      noaacal.d4[4]=-1.001E-11;
      noaacal.c1=1.1910427E-05;
      noaacal.c2=1.4387752;

/* Table D.2-12 */
      noaacal.nuc[3]=2700.1148;
      noaacal.a[3]=1.592459;
      noaacal.b[3]=0.998147;
      noaacal.nuc[4]=917.2289;
      noaacal.a[4]=0.332380;
      noaacal.b[4]=0.998522;
      noaacal.nuc[5]=838.1255;
      noaacal.a[5]=0.674623;
      noaacal.b[5]=0.998363;

/* Table D.1-14 */
      noaacal.Ns[3]=0;
      noaacal.b0[3]=0;
      noaacal.b1[3]=1;
      noaacal.b2[3]=0;
      noaacal.Ns[4]=-2.467;
      noaacal.b0[4]=2.96;
      noaacal.b1[4]=-0.05411;
      noaacal.b2[4]=0.00024532;
      noaacal.Ns[5]=-2.009;
      noaacal.b0[5]=2.25;
      noaacal.b1[5]=-0.03665;
      noaacal.b2[5]=0.00014854;
    break;
  }
  return noaacal;
}

/*************************************
 * Translate pix-value into temperature.
 * At start: ls contains one line of pixels
 * At end:   channels 3..5 in ls (if present) 
 *           contain temperature*10 in Kelvin
 *************************************/
void map_noaatemp(guint16 **ls,NOAACAL *nc,PICINFO *pci,VIEWINFO *vwi)
{
  static int prtnum;
  static float Tprt[4][3];
  static float Nbb[6];
  float d1,d2,d3,d4;
  float Tbb,Tbbs;
  int Ce;
  float e=2.7182818;
  int ch,p;
  int i,n;
  guint16 *l=ls[0];

  if (!pci->channels_recorded[0]) return;
/* Determine temp from PRT (Platinum Resistance Thermometer) */
  for (ch=3; ch<=5; ch++)
  {
    prtnum=0;
    for (i=0; i<5; i++)
    {
      if ((ls[i][17]<5) && (ls[i][18]<5) && (ls[i][19]<5)) continue;
      if (prtnum>4) break;
      p=ch+14;
      d1=ls[i][p];
      d2=d1*d1;
      d3=d2*d1;
      d4=d3*d1;
      Tprt[prtnum][ch-3]=nc->d0[prtnum+1]    + 
                         nc->d1[prtnum+1]*d1 + 
                         nc->d2[prtnum+1]*d2 + 
                         nc->d3[prtnum+1]*d3 + 
                         nc->d4[prtnum+1]*d4;
      prtnum++;
    }
    Tbb=(Tprt[0][ch-3]+Tprt[1][ch-3]+Tprt[2][ch-3]+Tprt[3][ch-3])/4.;
    Tbbs=nc->a[ch]+nc->b[ch]*Tbb;
/* Blackbody radiance */
    Nbb[ch]=nc->c1*pow(nc->nuc[ch],3.)/(pow(e,(nc->c2*nc->nuc[ch]/Tbbs))-1);
  }

/* Average Cbb */
  for (ch=3; ch<=5; ch++)
  {
    int Cbb,Cs;
    float tmp;
    int chan_offset;
    if (!pci->channels_recorded[ch]) continue;
    chan_offset=0;
    for (i=1; i<=ch; i++)
    {
      if (pci->channels_recorded[i]) chan_offset++;
    }
    chan_offset--;

    Cbb=0; Cs=0;
    for (n=0; n<10; n++)
    {
      p=22+(ch-3)+3*n;
      Cbb=Cbb+l[p];

      p=51+(ch-1)+5*n;
      Cs=Cs+l[p];
    }
    Cbb/=10;
    Cs/=10;

    tmp=(Nbb[ch]-nc->Ns[ch])/(Cs-Cbb);
    for (i=0; i<pci->width; i++)
    {
      float Nlin,Ncor,Ne,Tes,Te;
      p=pci->fri.header_len+i*pci->fri.inter_chan+chan_offset;
      Ce=l[p];
      Nlin=nc->Ns[ch] + tmp*(Cs-Ce);

      Ncor=nc->b0[ch] + nc->b1[ch]*Nlin + nc->b2[ch]*Nlin*Nlin;
      Ne=Nlin+Ncor;
      Tes=nc->c2*nc->nuc[ch]/(log(1+(nc->c1*pow(nc->nuc[ch],3.)/Ne)));
      Te=(Tes-nc->a[ch])/nc->b[ch];
      l[p]=Te*10;
    }
  }
}

