/**************************************************
 * RCSId: $Id: hrpt_guictrl.c,v 1.5 2017/09/04 22:25:47 ralblas Exp $
 *
 * Old HRPT reader
 * Project: WSAT
 * Author: R. Alblas
 * NOTE: This function is not guaranteed portable between big/little endian
 *       machines!
 *
 * History: 
 * $Log: hrpt_guictrl.c,v $
 * Revision 1.5  2017/09/04 22:25:47  ralblas
 * _
 *
 * Revision 1.4  2017/07/02 10:38:33  ralblas
 * _
 *
 * Revision 1.3  2017/04/29 18:20:03  ralblas
 * _
 *
 * Revision 1.2  2016/12/30 11:03:09  ralblas
 * _
 *
 * Revision 1.1  2016/11/20 11:27:46  ralblas
 * Initial revision
 *
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include <stdlib.h>
#include "wsat.h"
#include "hrpt_funcs.h"
#include "hrpt_gui.h"
#include "menu_wsat.h"
#include "pp_dec_interf.h"

static GtkWidget *hide_but;
extern PREFS prefs;

typedef enum
{
  HRPT_SELNR=0,
  CHRPT_SELNR=1,
  M1HRPT_SELNR=2,
  M2HRPT_SELNR=3,
  METOP_SELNR=4,
  FENGYUN_SELNR=5,
  FENGYUNAB_SELNR=6
} SATSELNR;

void get_sat_set_but(RECORDINFO *rdi,GtkWidget *hrpt_window)
{
#ifdef __ADD_USB__
  if (rdi->port_type==usb)
  {
    int channr=get_sat(&rdi->usb);

    switch(channr)
    {
      case HRPT_SELNR   : Set_Button(hrpt_window,LAB_HRPT,1);   rdi->file_type=FT_HRPT;    break;
      case CHRPT_SELNR  : Set_Button(hrpt_window,LAB_CHRPT,1);  rdi->file_type=FT_CHRPT;   break;
      case M1HRPT_SELNR : Set_Button(hrpt_window,LAB_MHRPT,1);  rdi->file_type=FT_MHRPT1;  break;
      case M2HRPT_SELNR : Set_Button(hrpt_window,LAB_M2HRPT,1); rdi->file_type=FT_MHRPT2;  break;
      case METOP_SELNR  : Set_Button(hrpt_window,LAB_METOP,1);  rdi->file_type=FT_METOP;   break;
      case FENGYUN_SELNR: Set_Button(hrpt_window,LAB_FYC,1);    rdi->file_type=FT_FY3;     break;
      case FENGYUNAB_SELNR: Set_Button(hrpt_window,LAB_FYAB,1); rdi->file_type=FT_FY3;     break;
      default           : Set_Button(hrpt_window,LAB_OTHER,1);  rdi->sat_type=sat_unk;     break;
    }
    if ((channr==METOP_SELNR) || ( channr==FENGYUN_SELNR))
    {
      Sense_Button(hrpt_window,LAB_RS_REC,TRUE);
    }
    else
    {
      Sense_Button(hrpt_window,LAB_RS_REC,FALSE);
    }
  }
#endif
}
int no_live;

#define SWTYPE_STOPSTARTn
/*************************************
 * Get button signals for Record functions
 *************************************/
void Record_Hrpt_Commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *hrpt_window=Find_Window(main_window,Lab_Hrptwnd);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);

  int addr_portd;
  char *name;
  gboolean channelselect_changed=FALSE;
  static gboolean reprogramming_fpga=TRUE;
  gboolean chan[6];
  int i;

  if (!rdi) return;  /* create_options_menu activates this func before everything is settled! */

  name=(char *)data;
  addr_portd=rdi->parport_base;

/* Get status of channel buttons */
  if (Find_Widget(hrpt_window,LAB_CHA))
  {
    chan[0]=TRUE;
    chan[1]=Get_Button(hrpt_window,LAB_CH1);
    chan[2]=Get_Button(hrpt_window,LAB_CH2);
    chan[3]=Get_Button(hrpt_window,LAB_CH3);
    chan[4]=Get_Button(hrpt_window,LAB_CH4);
    chan[5]=Get_Button(hrpt_window,LAB_CH5);
  }
  else
  {
    for (i=0; i<=5; i++) chan[i]=TRUE;
  }

  if (!strncmp(name,LAB_CHX,strlen(LAB_CHX)))
  {
    channelselect_changed=TRUE;
  }

  if (!strcmp(name,LAB_CHA))
  {
    int nr_sel=0;
/* If all channels selected then deselect all, else select all. */
    gboolean select_val=FALSE;
    for (i=1; i<=5; i++) if (chan[i]) nr_sel++;
    if (nr_sel<5) select_val=TRUE;

/* Select or deselect all channels. */
    for (i=1; i<=5; i++) chan[i]= select_val;
    Set_Button(hrpt_window,LAB_CH1,select_val);
    Set_Button(hrpt_window,LAB_CH2,select_val);
    Set_Button(hrpt_window,LAB_CH3,select_val);
    Set_Button(hrpt_window,LAB_CH4,select_val);
    Set_Button(hrpt_window,LAB_CH5,select_val);

    channelselect_changed=TRUE;
  }

/* Show-during-record parameters */
  {
    char *ext;
    ext=soption_menu_get_history(Find_Widget(widget,LAB_VCHR));
    if ((ext) && (!strcmp(name,ext)))
    {
      rdi->sh_recchan=atoi(name+strlen(LAB_CHX_V));
    }
  }

  if (!strcmp(name,LAB_DIR_NS))
  {
    rdi->scan_ver=N_S;
    rdi->scan_hor=W_E;
  }
  if (!strcmp(name,LAB_DIR_SN))
  {
    rdi->scan_ver=S_N;
    rdi->scan_hor=E_W;
  }

  if (!strcmp(name,LAB_DIR_NSN))
  {
    if (Update_Togglelabel(widget))
    {
      rdi->scan_ver=S_N;
      rdi->scan_hor=E_W;
    }
    else
    {
      rdi->scan_ver=N_S;
      rdi->scan_hor=W_E;
    }
  }

  if (!strcmp(name,LAB_MAXY))
  {
    rdi->height=GTK_ADJUSTMENT(widget)->value;
  }


/* HRPT/CHRPT choice */
  if ((!strcmp(name,LAB_HRPT)) || 
      (!strcmp(name,LAB_CHRPT)) ||
      (!strcmp(name,LAB_MHRPT)) ||
      (!strcmp(name,LAB_M2HRPT)) ||
      (!strcmp(name,LAB_METOP)) ||
      (!strcmp(name,LAB_FYAB)) ||
      (!strcmp(name,LAB_FYC)))
  {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      GtkWidget *w;
#ifdef SWTYPE_STOPSTART
      gboolean running=FALSE;
      if (rdi->recording)
      {
        running=TRUE;
        Set_Button(widget,LAB_Start,FALSE);
        while (g_main_iteration(FALSE));
      }
#endif
      if (!strcmp(name,LAB_HRPT))
      {
	rdi->file_type=FT_HRPT;
	rdi->sat_type=HRPT;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,HRPT_SELNR); 
  #endif
      }
      else if (!strcmp(name,LAB_CHRPT))
      {
	rdi->file_type=FT_CHRPT;
	rdi->sat_type=CHRPT;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,CHRPT_SELNR); 
  #endif
      }
      else if (!strcmp(name,LAB_MHRPT))
      {
	rdi->file_type=FT_MHRPT1;
	rdi->sat_type=M1HRPT;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,M1HRPT_SELNR); 
  #endif
      }
      else if (!strcmp(name,LAB_M2HRPT))
      {
	rdi->file_type=FT_MHRPT2;
	rdi->sat_type=M2HRPT;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,M2HRPT_SELNR); 
  #endif
      }
      else if (!strcmp(name,LAB_METOP))
      {
	rdi->file_type=FT_METOP;
	rdi->sat_type=METOP;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,METOP_SELNR); 
  #endif
      }
      else if (!strcmp(name,LAB_FYC))
      {
	rdi->file_type=FT_FY3;
	rdi->sat_type=FY3;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,FENGYUN_SELNR); 
  #endif
      }
      else if (!strcmp(name,LAB_FYAB))
      {
	rdi->file_type=FT_FY3;
	rdi->sat_type=FY3;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,FENGYUNAB_SELNR); 
  #endif
      }
      else
      {
	rdi->file_type=FT_HRPT;
	rdi->sat_type=HRPT;
  #ifdef __ADD_USB__
	if (rdi->port_type==usb) set_sat(&rdi->usb,HRPT_SELNR); 
  #endif
      }
      if (rdi->hwsel_type)
      {
	reprogramming_fpga=TRUE;
	channelselect_changed=TRUE;
      }
      if (rdi->file_type == FT_CHRPT)
      {
	w=Find_Widget(hrpt_window,LAB_CH1);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH12);
	w=Find_Widget(hrpt_window,LAB_CH2);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH34);
	w=Find_Widget(hrpt_window,LAB_CH3);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH56);
	w=Find_Widget(hrpt_window,LAB_CH4);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH78);
	w=Find_Widget(hrpt_window,LAB_CH5);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH910);
      }
      else
      {
	w=Find_Widget(hrpt_window,LAB_CH1);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH1);
	w=Find_Widget(hrpt_window,LAB_CH2);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH2);
	w=Find_Widget(hrpt_window,LAB_CH3);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH3);
	w=Find_Widget(hrpt_window,LAB_CH4);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH4);
	w=Find_Widget(hrpt_window,LAB_CH5);
	if (w) gtk_label_set_text(GTK_LABEL(GTK_BIN(w)->child),LAB_CH5);

/*
Make in options menu not-active channels insensitive.


This works, for whole menu:
Sense_Button(hrpt_window,LAB_VCHR,FALSE);

This should work for 1 item of options menu
Sense_Button(hrpt_window,LAB_CH5_V,FALSE);
Or:
w=Find_Widget(hrpt_window,LAB_CH5_V);
gtk_widget_set_sensitive(w,FALSE);    

*/

// RS 
  if ((!strcmp(name,LAB_METOP)) || 
      (!strcmp(name,LAB_FYC)) ||
      (!strcmp(name,LAB_FYAB)))
 {
    Sense_Button(widget,LAB_RS_REC,TRUE);
  }
  else
  {
    Sense_Button(widget,LAB_RS_REC,FALSE);
  }

      }
#ifdef SWTYPE_STOPSTART
      if (running)
        Set_Button(widget,LAB_Start,TRUE);
#endif
    }
    Sense_Button(widget,LAB_Start,TRUE);
  }
  if (!strcmp(name,LAB_OTHER))
  {
    if (!Get_Button(widget,LAB_Start)) // don't 'unsense' if running
      Sense_Button(widget,LAB_Start,FALSE);
  }

/* Copy state of buttons to channels_recording. 
   This depends on channels selected and choice HRPT/CHRPT. 
*/
  for (i=1; i<NR_CHANNELS_MAX; i++)
    rdi->channels_recording[i]=FALSE;
  for (i=1; i<NR_CHANNELS_MAX; i++)
  {
    switch(rdi->file_type)
    {
      case FT_MHRPT1: case FT_MHRPT2:
        if (i<=6)
        {
          rdi->channels_recording[i]=TRUE;
        }
      break;
      case FT_CHRPT:
        if (i<=5)
        {
          rdi->channels_recording[i*2-1]=chan[i];
          rdi->channels_recording[i*2]=chan[i];
        }
      break;
      case FT_HRPT: case FT_METOP:
        if (i<=5)
        {
          rdi->channels_recording[i]=chan[i];
        }
      break;
      case FT_FY3:
        rdi->channels_recording[i]=TRUE;
      break;
      default:
      break;
    } 
  }

/* Decoder/generator choice */
  if (!strcmp(name,LAB_DEC))
  {
    rdi->gen1_dec0=(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))? FALSE : TRUE);
    reprogramming_fpga=TRUE;
    channelselect_changed=TRUE;
  }

  if (!strcmp(name,LAB_RS_REC))
  {
    rdi->reed_solomon=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
#define LAB_Live "No live"
  if (!strcmp(name,LAB_Live)) no_live=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

/* Recording to save or not to save... */
  if (!strcmp(name,LAB_Save))
  {
    rdi->save_flag=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }
  if (!strcmp(name,LAB_KEP))
  {
    rdi->auto_get_kepler=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_IGNSYNC))
  {
    rdi->ignore_sync=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_ADVANCED))
  {
    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      hide_but=Find_Widget(hrpt_window,LAB_ADV_SET);
      if (hide_but) gtk_widget_hide(hide_but); 
    }
    else
    {
      if (hide_but) gtk_widget_show(hide_but);
      hide_but=NULL;
    }
  }

/* Start record */
  if ((!strcmp(name,LAB_Start)) && (Update_Togglelabel(widget)))
  {
    if (rdi->sat_type!=sat_unk)
    {
      rdi->recording=TRUE;
  #ifdef __ADD_USB__
      if (rdi->port_type==usb)
      {
        ena_dat(&rdi->usb);
      }
  #endif
      Sense_Button(hrpt_window,LAB_Exit,FALSE);
      Sense_Button(hrpt_window,LAB_Save,FALSE);
  #ifndef SWTYPE_STOPSTART
      Sense_Button(hrpt_window,LAB_HRPT,FALSE);
      Sense_Button(hrpt_window,LAB_CHRPT,FALSE);
      Sense_Button(hrpt_window,LAB_MHRPT,FALSE);
      Sense_Button(hrpt_window,LAB_M2HRPT,FALSE);
      Sense_Button(hrpt_window,LAB_METOP,FALSE);
      Sense_Button(hrpt_window,LAB_FYAB,FALSE);
      Sense_Button(hrpt_window,LAB_FYC,FALSE);
  #endif
      Sense_Button(hrpt_window,LAB_CH1,FALSE);
      Sense_Button(hrpt_window,LAB_CH2,FALSE);
      Sense_Button(hrpt_window,LAB_CH3,FALSE);
      Sense_Button(hrpt_window,LAB_CH4,FALSE);
      Sense_Button(hrpt_window,LAB_CH5,FALSE);
      Sense_Button(hrpt_window,LAB_CHA,FALSE);

      Recordpp_Hrpt(hrpt_window,main_window,rdi,pci,vwi);

      Sense_Button(hrpt_window,LAB_Exit,TRUE);
      Sense_Button(hrpt_window,LAB_Save,TRUE);
      Sense_Button(hrpt_window,LAB_HRPT,TRUE);
      Sense_Button(hrpt_window,LAB_CHRPT,TRUE);
      Sense_Button(hrpt_window,LAB_MHRPT,TRUE);
      Sense_Button(hrpt_window,LAB_M2HRPT,TRUE);
      Sense_Button(hrpt_window,LAB_METOP,TRUE);
      Sense_Button(hrpt_window,LAB_FYAB,TRUE);
      Sense_Button(hrpt_window,LAB_FYC,TRUE);

      Sense_Button(hrpt_window,LAB_CH1,TRUE);
      Sense_Button(hrpt_window,LAB_CH2,TRUE);
      Sense_Button(hrpt_window,LAB_CH3,TRUE);
      Sense_Button(hrpt_window,LAB_CH4,TRUE);
      Sense_Button(hrpt_window,LAB_CH5,TRUE);
      Sense_Button(hrpt_window,LAB_CHA,TRUE);
      rdi->recording=FALSE;
  #ifdef __ADD_USB__
      dis_dat(&rdi->usb);
  #endif
      Set_Button(hrpt_window,LAB_Start,FALSE);
    }
    else
    {
      Set_Button(hrpt_window,LAB_Start,FALSE);
      Create_Message("Error","No (correct) satellite choosen!\n");
    }
  }

  if ((!strcmp(name,LAB_Start)) && (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))))
  {
    rdi->recording=FALSE;
  }

  if (!strcmp(name,LAB_Stop))
  {
    rdi->recording=FALSE;
  }

  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(hrpt_window);
    Sense_Button(main_window,MENU_HRI,TRUE);
    Sense_Button(main_window,MENU_LRIT,TRUE);
    Sense_Button(main_window,MENU_METOP,TRUE);
    Sense_Button(main_window,MENU_RAWD,TRUE);
    Sense_Button(main_window,MENU_IQTEST,TRUE);
  }

/* Write new select to decoder */
  if (rdi->hwsel_type)
    rdi->sel_hw=(rdi->file_type==FT_CHRPT? TRUE : FALSE);
  else
    rdi->sel_hw=rdi->gen1_dec0;

  if ((rdi->port_type==parport) && (channelselect_changed))
  {
/* Open port connection */
    if (!Open_HWConnect(rdi))
    {
      if (rdi->port_type==usb)            // NVT als geen kanaalselectie
      {
        if (main_window)
          Create_Message("Error","Can't open USB port.");
        else
          fprintf(stderr,"Can't open USB port.");
      }
      else if (rdi->port_type==ethernet) // NVT als geen kanaalselectie
      {
        if (main_window)
          Create_Message("Error","Can't open ethernet port.");
        else
          fprintf(stderr,"Can't open ethernet port.");
      }
      else
      {
        if (main_window)
          Create_Message("Error",PAR_ERR_MESSAGE,addr_portd);
        else
          fprintf(stderr,PAR_ERR_MESSAGE,addr_portd);
      }
      return;
    }
    select_channel(chan,rdi,reprogramming_fpga);
//    Close_HWConnect(rdi);
    reprogramming_fpga=FALSE;
  }
}

static void close_wnd(GtkWidget *wnd)
{
  GtkWidget *mwnd=Find_Window(gtk_widget_get_toplevel(wnd),Lab_Main);;
  Sense_Button(mwnd,MENU_HRI,TRUE);
  Sense_Button(mwnd,MENU_LRIT,TRUE);
  Sense_Button(mwnd,MENU_METOP,TRUE);
  Sense_Button(mwnd,MENU_RAWD,TRUE);
  Sense_Button(mwnd,MENU_IQTEST,TRUE);
  destroy_window(wnd);
}

/*************************************
 * Record window. 
 * This function is started from the main menu.
 *************************************/
void Menu_Rec_Hrpt(GtkWidget *widget)
{
  GtkWidget *w1,*w2,*w3,*w4,*w5,*w6,*w7,*w8;
  GtkWidget *wa,*wb,*wc,*wd,*we,*wf,*wg,*wh,*wq,*wx,*wy,*wz;
  GtkWidget *hrpt_window;
  GtkWidget *main_window=Find_Parent_Window(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  GdkColor startbut_clr,startbutsclr;
  startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
  startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;

  hide_but=NULL;
  if (!(hrpt_window=Create_Window(main_window,0,0,Lab_Hrptwnd,close_wnd))) return;

/* Set policy to 'auto-resize' */
  gtk_window_set_policy((GtkWindow *)hrpt_window,FALSE,FALSE,TRUE);

  rdi->sat_type=HRPT;
  rdi->file_type=FT_HRPT;

  if (rdi->port_type==usb)
  {
    int i;
    for (i=0; i<=10; i++) rdi->channels_recording[i]=TRUE;
  }
  else if (rdi->port_type==ethernet)
  {
    int i;
    for (i=0; i<=10; i++) rdi->channels_recording[i]=TRUE;
  }
  else
  {
    rdi->channels_recording[0]=TRUE;
  }
  if (!rdi->height) rdi->height=drawing_area->allocation.height;

/* ==================== Button groups ==================== */
/* -------------------- Channel choice ------------------- */
  if (rdi->port_type==parport)
  {
    wa=Create_ButtonArray("Channel selection",Record_Hrpt_Commands,3,
                          TOGGLE,LAB_CH1,rdi->channels_recording[1],
                          TOGGLE,LAB_CH2,rdi->channels_recording[2],
                          TOGGLE,LAB_CH3,rdi->channels_recording[3],
                          TOGGLE,LAB_CH4,rdi->channels_recording[4],
                          TOGGLE,LAB_CH5,rdi->channels_recording[5],
                          BUTTON,LAB_CHA,
                          0
                	 );
  }
  else
  {
//  wa=Create_Label("(No\nchannel selection\npossible)");
    wa=NULL;
  }		       

/* -------------------- HRPT/CHRPT  ------------------- */
  w1=Create_Radio(LAB_HRPT,TRUE,Record_Hrpt_Commands);
  w2=Create_Radio(LAB_CHRPT,FALSE,Record_Hrpt_Commands);
  w3=Create_Radio(LAB_MHRPT,FALSE,Record_Hrpt_Commands);
  w4=Create_Radio(LAB_M2HRPT,FALSE,Record_Hrpt_Commands);
  w5=Create_Radio(LAB_METOP,FALSE,Record_Hrpt_Commands);
  w6=Create_Radio(LAB_FYAB,FALSE,Record_Hrpt_Commands);
  w7=Create_Radio(LAB_FYC,FALSE,Record_Hrpt_Commands);
  w8=Create_Radio(LAB_OTHER,FALSE,Record_Hrpt_Commands);
  wb=Pack("Sat type",'v',w1,0,w2,0,w3,0,w4,0,w5,0,w6,0,w7,0,w8,0,NULL);

  w1=Create_Optionmenu(LAB_VCHR,Record_Hrpt_Commands,rdi->sh_recchan-1,
                       LAB_CH1_V,LAB_CH2_V,LAB_CH3_V,LAB_CH4_V,LAB_CH5_V,
                       LAB_CH6_V,LAB_CH7_V,LAB_CH8_V,LAB_CH9_V,LAB_CH10_V,
                       0);
  w1=Pack("Show channel",'h',w1,0,NULL);
  w2=Create_Check(LAB_Save,Record_Hrpt_Commands,rdi->save_flag);
no_live=0;
w4=NULL;
//  w4=Create_Check(LAB_Live,Record_Hrpt_Commands,no_live);
  rdi->reed_solomon=TRUE;
  w3=Create_Check(LAB_RS_REC,Record_Hrpt_Commands,rdi->reed_solomon);

  w1=Pack("During record",'v',w1,5,w2,5,w3,5,w4,5,NULL);

  wb=Pack("",'h',wb,5,w1,5,NULL);
  w1=Create_Check(LAB_Save,Record_Hrpt_Commands,rdi->save_flag);
/* --------------------START/STOP----------------------- */
  w1=Create_Toggle(LAB_Start,Record_Hrpt_Commands,FALSE);
  Set_Color(w1,GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);

/*
  w2=Create_Button(LAB_Stop,Record_Hrpt_Commands);
*/
  w2=Create_Toggle(LAB_DIR_NSN,Record_Hrpt_Commands,FALSE);
  wc=SPack("","hH",w1,"fe3",w2,"fe3",NULL);

/* --------------------- STATUS ------------------------- */
  w1=Create_Entry(LAB_Recording,NULL,"%-20s","Off");
  w2=Create_Entry(LAB_Satname  ,NULL,"%-20s","");
  w3=Create_Entry(LAB_RECLINES,NULL,"%4d",0);
  wd=Pack("Status",'v',w1,2,w2,2,w3,2,NULL);

/* ==================== Advanced Button groups ==================== */
/* --------------------- DEC/GEN ---------------------- */
  we=NULL;
  if ((!rdi->hwsel_type) && (rdi->port_type!=usb))
  {
    w1=Create_Radio(LAB_DEC,TRUE,Record_Hrpt_Commands);
    w2=Create_Radio(LAB_GEN,FALSE,Record_Hrpt_Commands);
    we=Pack("Hardware select",'h',w1,0,w2,0,NULL);
  }
  

/* --------------------- SHOW DURING REC -------------- */
//  w1=Create_Optionmenu(LAB_VCHR,Record_Hrpt_Commands,rdi->sh_recchan-1,
//                       LAB_CH1_V,LAB_CH2_V,LAB_CH3_V,LAB_CH4_V,LAB_CH5_V,
//                       LAB_CH6_V,LAB_CH7_V,LAB_CH8_V,LAB_CH9_V,LAB_CH10_V,
//                       0);


  w1=Create_Spin(LAB_MAXY,Record_Hrpt_Commands,"%d%d%d",rdi->height,1,10000);
w2=NULL;
  wf=Pack("Show during record",'h',w1,5,w2,1,NULL);


/* --------------------SAVE----------------------- */
//  w1=Create_Check(LAB_Save,Record_Hrpt_Commands,rdi->save_flag);
  w2=Create_Check(LAB_KEP,Record_Hrpt_Commands,rdi->auto_get_kepler);
  w3=Create_Check(LAB_IGNSYNC,Record_Hrpt_Commands,rdi->ignore_sync);
//  w4=Create_Check(LAB_RS,Record_Hrpt_Commands,TRUE);
  wg=Pack("",'h',w2,2,w3,2,NULL);


/* --------------------Advanced / Exit----------------------- */
  wh=Create_Button(LAB_Exit,Record_Hrpt_Commands);
  wq=Create_Toggle(LAB_ADVANCED,Record_Hrpt_Commands,FALSE);


/* --------------------PACK----------------------- */
  wx=SPack(NULL,"h",wb,"f2",wa,"e1",NULL);
  wx=Pack(NULL,'v',wx,5,wc,5,wd,1,NULL);

  if (we)
  {
    wy=Pack(LAB_ADV_SET,'v',we,5,wf,5,wg,1,NULL);
  }
  else
  {
    wy=Pack(LAB_ADV_SET,'v',wf,5,wg,1,NULL);
  }

  wz=SPack("","Hh",wq,"fe10",wh,"fe1",NULL);

  wz=Pack(NULL,'v',wx,5,wy,5,wz,1,NULL);

  gtk_container_add(GTK_CONTAINER(hrpt_window),wz);

  gtk_widget_show_all(hrpt_window);
  place_window(hrpt_window,0,0,prefs.sub_wnd_geom);

/*
  if ((rdi->port_type==usb) || (rdi->port_type==ethernet))
    gtk_widget_hide(Find_Widget(hrpt_window,"Channel selection"));
  else
    gtk_widget_show(Find_Widget(hrpt_window,"Channel selection"));
*/
  // Get channel info from decoder.
  // Currently only supported for parport.
  if (rdi->port_type==parport)
  {
    if (Open_HWConnect(rdi))
    {                                 // no err returned -> OK
      if (get_hwselected(rdi->parport_base,rdi))
	rdi->sel_hw=FALSE;
      else
	rdi->sel_hw=TRUE;

      if (rdi->hwsel_type)
	Set_Button(hrpt_window,LAB_CHRPT,rdi->sel_hw);
      else
	Set_Button(hrpt_window,LAB_GEN,rdi->sel_hw);

      Set_Button(hrpt_window,LAB_GEN,rdi->gen1_dec0);
//      Close_HWConnect(rdi);
    }
  }
  get_sat_set_but(rdi,hrpt_window);
  Sense_Button(main_window,MENU_HRI,FALSE);
  Sense_Button(main_window,MENU_LRIT,FALSE);
  Sense_Button(main_window,MENU_METOP,FALSE);
  Sense_Button(main_window,MENU_RAWD,FALSE);
  Sense_Button(main_window,MENU_IQTEST,FALSE);
  Sense_Button(hrpt_window,LAB_OTHER,FALSE);

/* Restore show-channel-during-record state */
  Set_Button(hrpt_window,LAB_VCHR,rdi->sh_recchan-1);

  Record_Hrpt_Commands(wq, LAB_ADVANCED);
  Set_Button(hrpt_window,LAB_DIR_NSN,(rdi->scan_ver==S_N? TRUE : FALSE));
}

/*************************************
 * Record auto. 
 *************************************/
void Recordpp_Hrpt_Auto(GtkWidget *main_window,char *chan,SAT_TYPE sattype)
{
  GtkWidget *recwnd;
  char labstr[20];
  Menu_Rec_Hrpt(main_window);
  recwnd=Find_Window(main_window,Lab_Hrptwnd);
  Set_Button(recwnd,LAB_DEC,1);

  if (strchr(chan,'c')) sattype=CHRPT;

  if (sattype==CHRPT)
  {
    Set_Button(recwnd,LAB_CHRPT,1);
  }

  for (; *chan; chan++)
  {
    if (strchr("1234567890",*chan))
    {
      int ichan=*chan-'0';
      if (!ichan) ichan+=10;
      if (sattype==CHRPT)
      {
        ichan=(ichan-1)/2 + 1;
      }
      if ((ichan>=1) && (ichan<=5))
      {
        sprintf(labstr,"%s%d  ",LAB_CHX,ichan);
        Set_Button(recwnd,labstr,1);
/* Toggle buttons: func NOT activated using Set_Button, so do it here! */
        Record_Hrpt_Commands(Find_Widget(recwnd,LAB_Start), labstr);
      }
    }
    else if (*chan=='a')
    {
/* Do 'all-channel select'. (Toggle-button; don't click it extra!) */
      Set_Button(recwnd,LAB_CHA,1);
    }
    else if (*chan=='g')
    {
      Set_Button(recwnd,LAB_GEN,1);
    }
    else
      continue;
  }

  Set_Button(recwnd,LAB_Start,1);
/* Toggle buttons: func NOT actibated using Set_Button, so do it here! */
  Record_Hrpt_Commands(Find_Widget(recwnd,LAB_Start), LAB_Start);
}
