#define TESTN /* enable different out-sync meth.; to be evaluated */
/**************************************************
 * RCSId: $Id: hrpt_rec_pp.c,v 1.1 2010/12/21 21:14:03 ralblas Exp $
 *
 * HRPT record functions.
 * Based on FPGA-decoder, connected to parallel port.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: hrpt_rec_pp.c,v $
 * Revision 1.1  2010/12/21 21:14:03  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifdef __ADD_PAR__ 

#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <stdlib.h>
#include "pp_dec_interf.h"
#include "wsat.h"
#include "menu_wsat.h"
#include "hrpt.h"

#if _HRPT1_
#  include "hrpt1.h"
#endif

extern PREFS prefs;


/*************************************
 * Get a (C)HRPT-frame from decoder.
 * Decoder is connected to parallel port.
 * Return:
 *   0 -> Signal lost.
 *   1 -> OK; decoder in-sync
 *   2 -> OK, but decoder out-sync
 *************************************/
int getpp_hrpt_frame(guint16    *frame,
                     RECORDINFO *rdi)
{
  int datstat;
  int words_remaining;
  gboolean start_frame_flag=FALSE;
#ifdef TEST
  gboolean out_sync=FALSE;
#endif
  int cnt_nosig=0;

  words_remaining=rdi->fri.words_per_frame;
  while (words_remaining > 0)              /* Read all words in a frame */
  {
/* Get data from parallel port */
    if (!(get_pp_data(rdi,&datstat)))
    {
      return 0;                            /* Signal lost! */
    }

/* Check if received word contains a frame sync pulse from decoder.
   If sync pulse received, this is the first word of a frame.
   (Normally, this is the case each time getpp_hri_frame is entered.)
   NOTE This is NOT the decoder in-sync flag!
*/
    if (STARTFRAME_PP(datstat))
    {
/* Now, enable to save remaining words of frame. */
      start_frame_flag=TRUE;
    }

/* Save remaining words of frame. */
    if (start_frame_flag)
    {
      *frame=((datstat&0xff) << 2) + ((datstat>>12)&0x3);
      frame++;
      words_remaining--;
      cnt_nosig=0;
#ifdef TEST
      if (!INSYNC_PP(datstat)) out_sync=TRUE;
#endif
    }
    else
    {
/* If no readyn: make sure that now and then gui is activated */
      cnt_nosig++;
      if (cnt_nosig>1000)
      {
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
        cnt_nosig=0;
      }
    }
  }
#ifdef TEST
  if (out_sync) return 2;
#else
  if (!INSYNC_PP(datstat)) return 2;        /* decoder not in-sync. */
#endif
  return 1;                                 /* decoder in-sync. */
}

#endif

