/**************************************************
 * RCSId: $Id: hrpt_rec_usb.c,v 1.9 2017/09/17 13:39:47 ralblas Exp $
 *
 * HRPT record functions.
 * Based on FPGA-decoder, connected to parallel port.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: hrpt_rec_usb.c,v $
 * Revision 1.9  2017/09/17 13:39:47  ralblas
 * _
 *
 * Revision 1.8  2017/07/05 20:18:03  ralblas
 * _
 *
 * Revision 1.7  2017/07/05 17:40:47  ralblas
 * _
 *
 * Revision 1.6  2017/05/28 14:36:50  ralblas
 * _
 *
 * Revision 1.5  2017/05/02 17:15:37  ralblas
 * _
 *
 * Revision 1.4  2017/01/04 16:24:57  ralblas
 * _
 *
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include <malloc.h>
#include "wsat.h"
#ifdef NAAR_GODILREGMAP
  #include "hrpt_funcs.h"
  #include "godil_regmap.h"
#endif

/*
#ifndef __ADD_USB__ 
int usb_list_ndevs(FT_DEVICE_LIST_INFO_NODE **devInfo)
{
  return 0;
}
#endif
*/
#ifdef __ADD_USB__ 

// size USB buffer
#define BUF_SIZE 1024

// multibyte per scan(s): 2, 4
#define MUXSELECTBIT 0x80
         
// multibyte=4
#define INSYNCBIT 0x0800
#define FRSTRTBIT 0x0400
#define FRAMESTART 0x40

// all modes
#define STARTFRAME_USB(reg)  ((reg & FRSTRTBIT))
#define INSYNC_USB(reg) ((reg & INSYNCBIT))

//??#define STARTSFRAME_USB(reg) ((reg & 0x1000))   // super-frame, MHRPT

int usb_list_ndevs(FT_DEVICE_LIST_INFO_NODE **devInfo)
{
  FT_STATUS ftStatus;
  FT_DEVICE_LIST_INFO_NODE *i_devInfo=NULL;
  DWORD numDevs;
#if __GTK_WIN32__ == 1
  ftStatus = ft_CreateDeviceInfoList(&numDevs);
#else
  ftStatus = FT_CreateDeviceInfoList(&numDevs);
#endif
  if (ftStatus != FT_OK) return -1;
  if (numDevs > 0)
  {
    // allocate storage for list based on numDevs
    i_devInfo = (FT_DEVICE_LIST_INFO_NODE*)calloc(sizeof(FT_DEVICE_LIST_INFO_NODE),numDevs);
    // get the device information list
#if __GTK_WIN32__ == 1
    ftStatus = ft_GetDeviceInfoList(i_devInfo,&numDevs);
#else
    ftStatus = FT_GetDeviceInfoList(i_devInfo,&numDevs);
#endif
    if(ftStatus != FT_OK)
    {
      free(i_devInfo);
      i_devInfo=NULL;
      *devInfo=NULL;
      return -1;
    }
  }
  *devInfo=i_devInfo;
  return numDevs;
}

// test: convert saved file to  'multi_byte==4' kind of data
static int getbyte_file(char *fn)
{
  static FILE *fp;
  static int idat;
  int dat;
  static int frmcnt=0;
  if (fn)
  {
    if (!fp) fp=fopen(fn,"rb");
  }
  else
  {
    if (fp) fclose(fp); fp=NULL;
    return 0;
  }
  // !! [7:6]: 10/11/01/10 or 10/00/11/10 or 10/11/10/00
  // Dus als [6] 2x 1 dan 2e maat [7]=0.
  // Dus NIET 11/11! Wel 11/01
  if (fp)
  {
    static int bytecnt;
    switch(bytecnt)
    {
      case 0: 
        idat=getc(fp);
        dat=(idat>>2)&0x3f;     // d[0][7:2]
      break;
      case 1:
        dat=(idat<<4)&0x30;
        idat=getc(fp);
        frmcnt=(frmcnt+1)%1024;
        dat+=((idat>>4)&0x0f); // d[0][1:0], d[1][7:4]
        dat+=0x80;
      break;
      case 2:
        dat=(idat<<2)&0x3c;
        idat=getc(fp);
        frmcnt=(frmcnt+1)%1024;
        dat+=((idat>>6)&0x03); // d[1][3:0], d[2][7:6]
        dat+=0x80;
      break;
      case 3:
        frmcnt=(frmcnt+1)%1024;
        dat=((idat)&0x3f);     // d[2][5:0]
        dat+=0x80;
      break;
    }
    if (frmcnt==1) dat+=0x40;
    bytecnt=(bytecnt+1)%4;

    if (idat==EOF)
    {
      rewind(fp);
      return -1;
    }
    else
    {
      return dat;
    }
  }
  else
  {
    return 0;
  }
}
// for hrpt(-like):
//  2 bytes per word
// [7]=1:
//   [6]   = pdat_tx[13]
//   [5:0] = pdat_tx[11:10] pdat_tx[7:4];
// [7]=0:
//   [6]   = pdat_tx[12]
//   [5:0] = pdat_tx[9:8] pdat_tx[3:0];
//
// for metop:
//
static int get_usb_data(RECORDINFO *rdi,int *word)
{
  static int nrbytes;
  static DWORD dwBytesRead;
  static unsigned char pcBufRead[BUF_SIZE+10],inp;
  int done;
  static int hword;
  static int multi_byte=0;
  int iword;
  static int bocnt,syncdet,insync;
  FT_STATUS ftStatus;
  FT_HANDLE ftHandle=rdi->usb.ftHandle;
  if (!word)
  {
    nrbytes=0;
    return 0;
  }
  if (rdi->usb.fast_ftd)
  {
    switch (rdi->sat_type)
    {
      case IQTEST: multi_byte=0; break; // 1 byte per scan
      default:     multi_byte=2; break; // 2 bytes per scan
    }
  }
  else
  {
    switch (rdi->sat_type)
    {
      case METOP : multi_byte=4; break; // 4 bytes per 3 scans
      case FY3   : multi_byte=4; break; // 4 bytes per 3 scans
      case IQTEST: multi_byte=0; break; // 1 byte per scan
      default:     multi_byte=2; break; // 2 bytes per scan
    }
  }

  done=0;
  while (!done)
  {
    if (!nrbytes)          // no bytes in buffer -> needs read
    {                      // check status
#if __GTK_WIN32__ == 1
      ftStatus = ft_Read(ftHandle, pcBufRead, BUF_SIZE, &dwBytesRead);
#else
      ftStatus = FT_Read(ftHandle, pcBufRead, BUF_SIZE, &dwBytesRead);
#endif
      if ((ftStatus != FT_OK) || (dwBytesRead<BUF_SIZE))
      { // buffer not full, so time to do 'something'
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
      }
      nrbytes=dwBytesRead;
    }
    if (nrbytes>0)
    { // data in USB-buffer -> read it
      if ((multi_byte==4) && (rdi->sim_record==1))
      {
        int stat=getbyte_file(rdi->sim_file);
        if (stat<0) return 0;
        inp=(unsigned char)(stat&0xff);
      }
      else
      {
        inp=pcBufRead[dwBytesRead-nrbytes];
      }

      if (multi_byte==0)                        // 1 byte per scan
      {
        iword=0x0800 + inp;
        done=1;
      }
      else if (multi_byte==2)                   // 2 bytes per scan
      {
        if (!(inp&MUXSELECTBIT))
        {
          hword=((inp&0x30)<<4) + (inp&0x0f);                // [9:8] [3:0]
	  iword=0;
        }
        else
        {
          iword=hword | ((inp&0x30)<<6) | ((inp&0x0f)<<4) ; // [11:10] [7:4]
          done=1;
        }
      }
      else //if (multi_byte==4)                 // 4 bytes per 3 scans
      {
        static unsigned char mem;
        int strtsync;
        if ((inp&MUXSELECTBIT) == 0)   // "in-sync bit = 0
        {                              // just 1 word sync=0: sync multi-byte
          bocnt=0;                     // sync. multibyte counter
          if (syncdet>0) syncdet--;    // if syncdet=0: really out-sync
        }
        else
        {
          bocnt=(bocnt+1) % 4;         // count 0...3 etc.
          if (syncdet<3) syncdet++;    // incr. syncdet up to 3
        }
        if (syncdet>1) insync=1; else insync=0; // actual sync detector

        strtsync=(inp&FRAMESTART? 1 :0);        // pdo(6)=1: start-of-frame
        // construct output byte: 3 bytes needs 4 input bytes
        //		sync	strtfr
        //      bocnt	7	6	5	4	3	2	1	0
        // byte 0:	0	?	d0[7]	d0[6]	d0[5]	d0[4]	d0[3]	d0[2] 
        // byte 1:	1	sfr0	d0[1]	d0[0]	d1[7]	d1[6]	d1[5]	d1[4]
        // byte 2:	1	sfr1	d1[3]	d1[2]	d1[1]	d1[0]	d2[7]	d2[6]
        // byte 3:	1	sfr2	d2[5]	d2[4]	d2[3]	d2[2]	d2[1]	d2[0]
        inp&=0x3f;
        switch (bocnt)
        {
          case 0: iword=inp<<2;                 // pd(7 downto 2) <=pdo(5 downto 0); // first 6 bits
                  done=0;                       // need more input to make a data-byte
          break;
          case 1: iword+=((inp>>4)&0x3);        // pd(1 downto 0) <=pdo(5 downto 4); // add 2 bits: fill to 8 bits
                  mem=(inp<<4)&0xf0;            // mem(7 downto 4)<=pdo(3 downto 0); // hold remaining 4 bits
                  done=1;			// data-byte complete
          break;
          case 2: iword=mem;                    // pd(7 downto 4) <=mem(7 downto 4); // get 4 bits from prev. catch
                  iword+=((inp>>2)&0xf);        // pd(3 downto 0) <=pdo(5 downto 2); // add 4 bits: fill to 8 bits
                  mem=(inp<<6)&0xc0;            // mem(7 downto 6)<=pdo(1 downto 0); // hold remaining 2 bits
                  done=1;			// data-byte complete

          break;
          case 3: iword=mem;                    // pd(7 downto 6)<=mem(7 downto 6); // get 2 bits from prev. catch
                  iword+=(inp&0x3f);            // pd(5 downto 0)<=pdo(5 downto 0); // add 6 bits: fill to 8 bits
                  done=1;			// data-byte complete
          break;
        }
        iword&=0xff;                            // remove flags (iword bouwt op!)
        if (strtsync) iword|=FRSTRTBIT;		// add start-of-frame
        if (insync)   iword|=INSYNCBIT;		// add reconstructed syncstate
      }
      nrbytes--;  				// count down available bytes
    }
  }
  *word=iword;					// output word: complete frame-byte + sync state + start-frame flag
  return 1;
}

gboolean usb_wait_for_insync(RECORDINFO *rdi,int threshold)
{
  int datstat;
  guint32 count=0;
  guint32 threshold_word;
  int n=0;
  int was_open=(int)rdi->usb.ftHandle;
  if (rdi->ignore_sync) return TRUE; // don't wait for in-sync
  if ((!was_open) && (usb_init_connect(&rdi->usb))) return FALSE; // error opening USB
  // words->frame: *11092 (HRPT) 
  // wait for in-sync: this is word-based (8 or 10 bits/word
  // hrpt  : 66.54 words/sec
  // meteor: 66.54 words/sec
  // metop : 3500 kb/s so 437.5 kbytes/sec -> 437.5/66.54=6.5x faster
  switch(rdi->sat_type)
  {
    case FY3   : threshold_word=threshold*11092*6.5; break;
    case METOP : threshold_word=threshold*11092*6.5; break;
    case M1HRPT: threshold_word=threshold*11092;     break;
    case M2HRPT: threshold_word=threshold*11092;     break;
    case CHRPT : threshold_word=threshold*11092*2;   break;
    default    : threshold_word=threshold*11092;     break;
  }
  do
  {
    if (!rdi->recording)
    {
//      if (!was_open) usb_close_connect(rdi);
      return FALSE;           /* Stop if recording aborted */
    }
    if ((get_usb_data(rdi,&datstat))) //returns if new data available
    {
      if (INSYNC_USB(datstat))
        count++;
      else if (rdi->sat_type!=LRIT)
        count=0;
      if ((rdi->sat_type==LRIT) ||  (rdi->sat_type==METOP) ||  (rdi->sat_type==FY3))
      {
        rdi->vitsync=((datstat&0x100)?1:0);
        rdi->inv_dat=((datstat&0x200)?1:0);
      }
    }
    else
    {
/*
   waited too long for new data; problem with readyn? 
   Try again,but first do g_main_iteration to catch e.g. 'Stop' event.
*/
    }
/* Evaluate gui-events now and then. (Not too often.) */
    if (n++>10000)
    {
      while (g_main_iteration(FALSE));
      n=0;
    }
  } while (count<threshold_word);
//  if (!was_open) usb_close_connect(rdi);
  return TRUE;
}

int usb_wait_for_outsync(RECORDINFO *rdi,int threshold)
{
  int datstat;
  guint32 count=0;
  guint32 threshold_word;
  int n=0;
//  int was_open=(int)rdi->ftHandle;

  if (usb_init_connect(&rdi->usb)) return FALSE;
/* Translate #frames-to-wait to #words-to-wait */
  threshold_word=threshold*rdi->fri.words_per_line;
  do
  {
    if (!rdi->recording)
    {
//      if (!was_open) usb_close_connect(rdi);
      return FALSE;           /* Stop if recording aborted */
    }
    if ((get_usb_data(rdi,&datstat)))
    {
      if (!INSYNC_USB(datstat))
        count++;
      else
        count=0;
    }
    else
    {
/*
   waited too long for new data; problem with readyn? 
   Try again,but first do g_main_iteration to catch e.g. 'Stop' event.
*/
    }

/* Evaluate gui-events now and then. (Not too often.) */
    if (n++>100)
    {
      while (g_main_iteration(FALSE));
      n=0;
    }
  } while (count<threshold_word);

//  if (!was_open) usb_close_connect(rdi);
  return TRUE;
}

// datstat: 12 bits
//   (C)HRPT: in_sync & strtdat & pd[9:0]
//   MHRPT  : in_sync & strtdat & '0' & '0'     & pd[7:0]
//   LRIT   : in_sync & strtdat & inv_dat & vitsync & pd[7:0]
//   METOP
//
// early sync: dump prev. (incomplete) frame, continue with current frame
//      x1 y2 x3
// late sync: dump after frame full, then wait until sync, some bytes lost 
//      x1 x2 -- x3 x4
//
int usb_get_hrpt_frame(guint16    *frame,RECORDINFO *rdi)
{
  static gboolean erl_frmflg;
  int words_remaining;
  int datstat;
  gboolean start_frame_flag;
  int cnt_nosig=0;
  if (!frame)
  {
    getbyte_file(NULL);
    return 0;
  }
  
  start_frame_flag=erl_frmflg;
  words_remaining=rdi->fri.words_per_frame;

  if (erl_frmflg)                                       // early frame: first byte was already taken
  {
    frame++;
    words_remaining--; 
  }
  erl_frmflg=FALSE;

  if (rdi->sat_type==IQTEST) start_frame_flag=TRUE;
  while (words_remaining > 0)              // Read all words in a frame 
  {
/* Get one data-word from port */
    if (!(get_usb_data(rdi,&datstat)))
    {
      return 0;                                           // Signal lost! 
    }

/* Check if received word contains a frame sync pulse from decoder.
   If sync pulse received, this is the first word of a frame.
   (Normally, this is the case each time usb_get_hrpt_frame is entered.)
   NOTE This is NOT the decoder in-sync flag!
*/
    if (STARTFRAME_USB(datstat))   // start-of-frame
    {
#ifdef XXX
// slaat op hol bij FY!
      if (!rdi->ignore_sync)
      {
        if (start_frame_flag)      // but prev. frame not finished!
        {
          erl_frmflg=TRUE;  // early sync; dump frame
          break;
        }
      }
#endif
/* Now, enable to save remaining words of frame. */
      start_frame_flag=TRUE;
    }

/* Save remaining words of frame. */
    if (rdi->ignore_sync)
      start_frame_flag=TRUE;

    if (start_frame_flag)
    {
      if ((rdi->sat_type==METOP) || (rdi->sat_type==FY3))
        *frame=datstat & 0x0ff;
      else
        *frame=datstat & 0x3ff;
      frame++;
      words_remaining--;
      cnt_nosig=0;
    }
    else
    {
/* If no readyn: make sure that now and then gui is activated */
      cnt_nosig++;
      if (cnt_nosig>1000)
      {
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
        cnt_nosig=0;
      }
    }
  }
  if (!INSYNC_USB(datstat)) return 2;        /* decoder not in-sync. */
//g_usleep(170000); // test buffer
  return 1;                                 /* decoder in-sync. */
}

// syncwoord: 0x284,0x16f,0x35c,0x19d,0x20f,0x095
// = 1010000100 0101101111 1101011100 0110011101 1000001111 0010010101

/*
int a_dat=0;       -- read: HRPT-data, write: stop data
int a_chn=1;
int a_sta=2;
int c_rd=0xc0;
int c_wr=0x40;
*/


#endif

