/**************************************************
 * RCSId: $Id: mhrpt2_read.c,v 1.3 2016/11/02 18:58:32 ralblas Exp $
 *
 * Meteor HRPT reader
 * Project: WSAT
 * Author: R. Alblas
 * NOTE: This function is not guaranteed portable between big/little endian
 *       machines!
 *
 * History: 
 * $Log: mhrpt2_read.c,v $
 * Revision 1.3  2016/11/02 18:58:32  ralblas
 * _
 *
 * Revision 1.2  2016/09/30 08:38:49  ralblas
 * _
 *
 * Revision 1.1  2015/02/01 22:39:27  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2011 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include "wsat.h"
#include "hrpt1.h"
#include "mhrpt.h"
#include <malloc.h>
extern PREFS prefs;
#define FRMLEN1 256
#define FRMLEN2 1024
#define TFRMLEN (FRMLEN1*50)
#define SYNC2a 0x0218a7a3
#define SYNC2b 0x92dd9abf

#define START_MSU_MR1 22
#define START_PICDATA1 50
#define END_MSU_MR1 253

#define START_MSU_MR2 22
#define START_PICDATA2 0 // alleen pic-data.
#define END_MSU_MR2 1021
// In MSU-MR:
//   bytes 1 chan: 1540*10/8=1925
//                             *6
//                          -----
//                          11550 bytes
// 
//   lsync                      8
//   tijd enz. byte 9...50     42
//                          -----
// Totaal MSU-MR            11600 bytes
//
// PEC's  : 50* 2             100
// headers: 50*22            1100
//                          -----
// total 1 line: 256*50 =   12800 bytes
//
// MHRPT-frame M1:
//   0...  3: SYNC
//   4... 21: info
//  22... 29: SYNC2
//  30... 71:
//  72...253: 182 bytes pic, in grp van 30 bytes: [4*C1,4*C2,...4*C6],... 4*6*10/8=30
// 254, 255: PEC
//-----------------
//  0...  3: SYNC
//  4... 21: info
// 22...253: 232 bytes pic, in grp van 30 bytes[4*C1,4*C2,...4*C6],...
// 254, 255: PEC
//-----------------
//...
// (tot 50*)
// Tot.: 182+49*232=11550 (385 sets of 30 bytes, 1 chan: width=385*4=1540
// N.B. GEEN geheel aantal sets per frame!
// ----------------------------------------------------------------------------------
// MHRPT-frame M2:
// frame 1:
//  0...   3: SYNC		2900
//  4...  21: info
// 22... 259: MSU-SR 
//                  n    ... n+  7: SYNC2
//                  n+ 8 ... n+ 35: time etc.?
//                  n+36 ... n+231: data: 196 bytes pic
// 260... 277: info
// 278... 515: MSU-SR    		  238 bytes pic (als sync hier niet in zit)
// 516... 533: info
// 534... 771: MSU-SR   		  238 bytes pic (als sync hier niet in zit)
// 772... 789: info
// 790...1023: MSU-SR   		  234 bytes pic (als sync hier niet in zit)
// 
// Total picdata: 196+238+238+234=906?
//
// frames 2...12 (11x): 
// 
//   0...   3: SYNC
//   4...  21: info
//  22... 259: MSU-SR   	 238 bytes pic
// 278... 515: MSU-SR   	 238 bytes pic
// 534... 771: MSU-SR   	 238 bytes pic
// 790...1023: MSU-SR   	 234 bytes pic
// 
// Total picdata: 238+238+238+234=948?
// 
// frame 13:
//   0...   3: SYNC
//   4...  21: info
//  22... 259: MSU-SR   	 238 bytes pic
// 260... 277: info
// 278... 515: MSU-SR   	 238 bytes pic
// 516... 533: info
//---------------------------------------------------
// # pic-bytes: 906+11*948+576=11910		verwacht: 11550, te veel: 388
//---------------------------------------------------
// 534... 537: extra bytes?
// 538... 545: SYNC2
// 546... 771: MSU-SR rest 	226 bytes
// 790...1023: MSU-SR		234 bytes pic
// 
// frames 14...25 (12x): 
// 
//   0...   3: SYNC
//   4...  21: info
//  22... 259: MSU-SR   	 238 bytes pic
// 278... 515: MSU-SR   	 238 bytes pic
// 534... 771: MSU-SR   	 238 bytes pic
// 790...1023: MSU-SR   	 234 bytes pic
//---------------------------------------------------
// # pic-bytes: 460+12*948=11836		verwacht: 11550, te veel: 286
//---------------------------------------------------
//
// kan. 1-3: 1572 pixels, 4-6: 1540 pixels ==> 1572*3+1540*3=9336*10/8=11670
// #msu tot.: 3*238+234 
// 

// tot: 934+11*948=11362
// Nog nodig: 188
// Er is nog: 538, dus teveel: 350?
//
// 

//-----------------
// (tot 25*)
// 
// 1e byte na sync2: 0x2924, (+12798) 0x5b22, (+12802) 0x8d24
#ifdef BESCHR
00002900  1a cf fc 1d 94 06 00 02  00 00 ff ff ff ff 00 00 
00002910  00 00 00 00 00 00 c4 c5  00 2b 76 88 02 18 a7 a3 <<<SYNC2
...
00005900  1a cf fc 1d 57 ff 3e 41  00 00 ff ff ff ff 00 00 
00005910  00 00 00 00 00 00 11 c6  31 f4 d5 2c 0b 02 c0 b0 
...
00005b00  c2 c3 c4 c5 00 00 54 41  78 7b ff ff ff ff 00 00 
00005b10  00 00 00 00 00 00 00 2b  76 89 02 18 a7 a3 92 dd <<<SYNC2
#endif


// read added file-header; N1 or N2
// return: record HRPTHDR with extracted info (allocated mem)
//         NULL if no header found at start-of-file
HRPTHDR *read_mhrpthdr(FILE *fp,FILE_TYPE file_type)
{
  HRPTHDR *hrpthdr;
  unsigned long size;
  int i;
  fseek(fp,0,SEEK_END);
  size=ftell(fp);
  fseek(fp,0,SEEK_SET);

/* allocate mem for hrpthdr struct */
  if ((hrpthdr=(HRPTHDR *)calloc(sizeof(HRPTHDR),1))==NULL) return NULL;

/* Read first part: sign ...  tm_comp_sat_vlg */
  if (!(fread(hrpthdr,131,1,fp)))
  { free(hrpthdr); puts("??"); return NULL; }

  if ((!strncmp(hrpthdr->sign,MHRPT1SIGN,4)) || (!strncmp(hrpthdr->sign,MHRPT2SIGN,4)))
  {

  /* Read second part: offset_lon ... channel */
    if (!(fread((char *)&hrpthdr->offset_lon,15,1,fp)))
    { free(hrpthdr);  return NULL; }

  /* Read fourth part: widthmax ... end */
    if (!(fread((char *)&hrpthdr->widthmax,6,1,fp)))
    { free(hrpthdr);  return NULL; }
  }
  else  // no MHRPT header; clean hrpthdr
  {
    memset(hrpthdr,0,131);
  }

  // volgende 2 regels weghalen als opgeslagen height=OK
  switch(file_type)
  {
    case FT_MHRPT2:
      hrpthdr->width=GUINT16_TO_LE(DEF_METEORN2_XRES);  // ch1-3: 1572
      hrpthdr->height=GUINT16_TO_LE(((size-256)/12800));
    break;
    default:
      hrpthdr->width=GUINT16_TO_LE(DEF_METEORN1_XRES);
      hrpthdr->height=GUINT16_TO_LE(((size-256)/12800));
    break;
  }
  for (i=0; i<=6; i++) hrpthdr->channel[i]=TRUE;
  return hrpthdr;
}

// calc. some mhrpt figures: # channels, bytes before pic-data etc.
// (N1 and N2)
FRAMEINFO calc_mhrptframeinfo(FILE_TYPE file_type,gboolean *channels,int width)
{
  FRAMEINFO fri;
  int i;

  fri.header_len=(file_type==FT_MHRPT2? START_PICDATA2: START_PICDATA1);
  fri.leader_len=0;
  fri.words_per_frame=0;
/* Contribution of actual picture data */
  fri.inter_chan=0;
  for (i=1; i<NR_CHANNELS_MAX; i++)
    if (channels[i]) fri.inter_chan++;

  fri.words_per_frame=width*fri.inter_chan;

/* Contribution of header and leader. 
   Leader contains extra bytes to make words_per_frame*10/8 integer.
*/
  if (channels[0])
  {
    fri.words_per_frame+=fri.header_len;
    fri.words_per_frame+=fri.leader_len;
  }
  fri.words_per_line=fri.words_per_frame;

/* Calculate file bytes per line */
  fri.bytes_per_line=12800;   // fri.words_per_line*10/8;

/* Add contribution of extra 'frame separation' bytes */
  fri.bytes_per_line+=0;

  return fri;
}

// compare 'nrb' bytes in b with 'l1/l2' (max. 8 bytes)
// return: # bit-mismatches
static int cmp_sync(unsigned char *b,unsigned long l1,unsigned long l2,int nrb)
{
  int i;
  unsigned char sb;
  int err=0;
  for (i=0; i<nrb; i++)
  {
    if (i<4)
    {
      sb=(l1>>((8*(3-i))&0xff));
      if (b[i]!=sb) err++;
    }
    else
    {
      sb=(l2>>((8*((3-i+4))&0xff)));
      if (b[i]!=sb) err++;
    }
  }
  
  return err;
}

// N1 only:
// Search start; set file pointer at:
//  0  1  2  3         22 23 24.....
// 1A CF FC 1D .....   02 18 A7.....
int search_startofline(FILE *fp,int n_nr)
{
  int ret=0;
  unsigned char l[FRMLEN1];
  if (!fp) return 0;
  rewind(fp);
  while ((fread(l,FRMLEN1,1,fp))) 
  {
    if ((cmp_sync(l,MHRPT_SYNC,0,4)) > 2); // err 
    
    if (n_nr==1) // N1 meteor
    {
      if ((cmp_sync(l+22,SYNC2a,SYNC2b,8)<4))
      {
        fseek(fp,-1*FRMLEN1,SEEK_CUR);
        ret=1;
        break;
      }
    }

//Niet gebruikt voor N2!
    if (n_nr==2) // N2 meteor
    {
ret=1; break;
/*
      if ((cmp_sync(l+28,SYNC2a,SYNC2b,8)<4))
      {
        fseek(fp,-1*FRMLEN1,SEEK_CUR);
        ret=1;
        break;
      }
*/
    }
  }
  return ret;
}

// N1:
// Vogende voor direct op scherm krijgen plaatje.
// line: vanaf (incl) SYNC2: (byte 22)
// <8 bytes SYNC2><42 bytes cal. etc><video-data>
void mn1frame2line(unsigned char *frame,int size,int np,guint16 *line)
{
  int fpos1,fpos;
  int bnr,q5nr,chnr,x,xp;

  if (np>49) return;

  if (np==0) fpos1=START_MSU_MR1+START_PICDATA1;
  else       fpos1=START_MSU_MR1;
  if (np==0) bnr=0; else bnr=182+(np-1)*232;

  if (np==0)
  {
    int n;
    for (n=0; n<START_PICDATA1; n++)
    {
      line[n]=frame[n+START_MSU_MR1];
    }
  }
  line+=START_PICDATA1;

  for (; fpos1<=END_MSU_MR1; fpos1++,bnr++)
  {
    fpos=fpos1*size;
    q5nr=bnr/5;
    chnr=q5nr%6;
    x=((q5nr/6)*4)+(bnr%5);
    xp=x-1;

    switch(bnr%5)
    {
      case 0:
        line[x*6+chnr]  =((frame[fpos]&0xff)<<2);
      break;
      case 1:
        line[xp*6+chnr]+=((frame[fpos]&0xc0)>>6);
        line[x*6+chnr]  =((frame[fpos]&0x3f)<<4);
      break;
      case 2:
        line[xp*6+chnr]+=((frame[fpos]&0xf0)>>4);
        line[x*6+chnr]  =((frame[fpos]&0x0f)<<6);
      break;
      case 3:
        line[xp*6+chnr]+=((frame[fpos]&0xfc)>>2);
        line[x*6+chnr]  =((frame[fpos]&0x03)<<8);
      break;
      case 4:
        line[xp*6+chnr]+=((frame[fpos]&0xff)>>0);
      break;
    }
  }
}

#ifdef MN2descr
name		   length	pos
------------------------------
SYNC (1A CF FC 1D)	4		
telemetry		2
PA BIS-M  DM		4
SSPD			4
MTVZA			8
MSU-MR		      238	 22
telemetry		2
PA BIS-M  DM		4
SSPD			4
MTVZA			8
MSU-MR		      238	278
telemetry		2
PA BIS-M  DM		4
SSPD			4
MTVZA			8
MSU-MR		      238	534
telemetry		2
PA BIS-M  DM            4
SSPD			4
MTVZA			8
MSU-MR		      234	790
#endif


static int pos_msu[4]={ 22,278,534,790};
static int len_msu[4]={238,238,238,234};
static int msu_syncpos[2];

// detect pos. 'p' in frame is inside one of 4 MSU blocks (N2)
int is_video(int p)
{
  int i;
  for (i=0; i<4; i++)
  {
    if (p<pos_msu[i]) return 0;
    if ((p>pos_msu[i]+len_msu[i]) && ((i==3) || (p<pos_msu[i+1])))  return 0;
  }
  return 1;
}

// voor direct afbeelden tijdens opname; zie hrpt_rec.c
// line: vanaf (incl) SYNC2: (byte 22)
// <8 bytes SYNC2><42 bytes cal. etc><video-data>
void mn2frame2line(unsigned char *frame,int size,int np,int xstrt,guint16 *line)
{
  int fpos1,fpos;
  static int bnr,q5nr,chnr,x,xp;
  int nr_msu;
  if (np>49) return;

  if (np==0)
  {
    bnr=0;
  }
  
  xstrt+=20;
  for (nr_msu=0; nr_msu<4; nr_msu++)
  {
    for (fpos1=pos_msu[nr_msu]; fpos1<pos_msu[nr_msu]+len_msu[nr_msu]; fpos1++,bnr++)
    {
      fpos=fpos1*size;
      if (np==0)
      {
        if (fpos1<=xstrt) bnr=0;
        if (fpos1<xstrt) continue;
      }
      q5nr=bnr/5;
      chnr=q5nr%6;
      x=((q5nr/6)*4)+(bnr%5);
      xp=x-1;
      switch(bnr%5)
      {
        case 0:
          line[x*6+chnr]  =((frame[fpos]&0xff)<<2);
        break;
        case 1:
          line[xp*6+chnr]+=((frame[fpos]&0xc0)>>6);
          line[x*6+chnr]  =((frame[fpos]&0x3f)<<4);
        break;
        case 2:
          line[xp*6+chnr]+=((frame[fpos]&0xf0)>>4);
          line[x*6+chnr]  =((frame[fpos]&0x0f)<<6);
        break;
        case 3:
          line[xp*6+chnr]+=((frame[fpos]&0xfc)>>2);
          line[x*6+chnr]  =((frame[fpos]&0x03)<<8);
        break;
        case 4:
          line[xp*6+chnr]+=((frame[fpos]&0xff)>>0);
        break;
      }
    }
  }
}

// voor direct afbeelden tijdens opname; zie hrpt_rec.c
int m1hrpt_is_startframe(guint16 *frame2)
{
  int i;
  unsigned char frame1[8];
  for (i=0; i<8; i++) frame1[i]=(frame2[START_MSU_MR1+i]);
  if ((cmp_sync(frame1,SYNC2a,SYNC2b,8)<4)) return 1;
  return 0;
}

int m2hrpt_is_startframe(guint16 *frame2)
{
  int i,j,nm;
  unsigned char frame1[8];
  for (nm=0; nm<4; nm++)
  {
    for (i=pos_msu[nm]; i<pos_msu[nm]+len_msu[nm]; i++)
    {
      for (j=0; j<8; j++) frame1[j]=(frame2[i+j]);
      if ((cmp_sync(frame1,SYNC2a,SYNC2b,8)<4))
      {
//printf(">>> %d  %d  %x  %x  %x  %x\n",nm,i,frame1[0],frame1[1],frame1[2],frame1[3]);
         return i;
      }
    }
  }
  return 0;
}

// extract pic-data from a MSU-block to line (interleaved channels)
// np==0: first msu of a line (contains sync2)
// pos: position of first pic-data in msu
static void msu2line(unsigned char *frame,int pos,int len_msu,int np,guint16 *line)
{
  int fpos1,fpos;
  int q5nr,chnr,x,xp;
  static int bnr;
  unsigned char *msu=frame+pos;   
  if (np==0) bnr=0;
  for (fpos1=0; fpos1<len_msu; fpos1++,bnr++)
  {
    fpos=fpos1; // *size;
    q5nr=bnr/5;
    chnr=q5nr%6;                  // SPECERR??? Correction '+5' (corrects channel shift)
    x=((q5nr/6)*4)+(bnr%5);
    xp=x-1;
    switch(bnr%5)
    {
      case 0:
        line[x*6+chnr]  =((msu[fpos]&0xff)<<2);
      break;
      case 1:
        if (xp>=0) line[xp*6+chnr]+=((msu[fpos]&0xc0)>>6);
        line[x*6+chnr]  =((msu[fpos]&0x3f)<<4);
      break;
      case 2:
        if (xp>=0) line[xp*6+chnr]+=((msu[fpos]&0xf0)>>4);
        line[x*6+chnr]  =((msu[fpos]&0x0f)<<6);
      break;
      case 3:
        if (xp>=0) line[xp*6+chnr]+=((msu[fpos]&0xfc)>>2);
        line[x*6+chnr]  =((msu[fpos]&0x03)<<8);
      break;
      case 4:
        if (xp>=0) line[xp*6+chnr]+=((msu[fpos]&0xff)>>0);
      break;
    }
  }
}



// N1: read MSU-MR's for 1 line (50 frames)
void read_mhrpt1_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line)
{
  int np=0;
  unsigned char frame[FRMLEN1];
  fseek(pci->fp,pci->pic_strt + y*pci->fri.bytes_per_line,SEEK_SET);
  while (fread(frame,FRMLEN1,1,pci->fp))
  {
    if ((cmp_sync(frame+START_MSU_MR1,SYNC2a,SYNC2b,8)<4)) np=0;
    mn1frame2line(frame, 1, np,line);
    np++;
    if (np>=50) break;
  }
}

// N2: 
// search sync2 in one of msu's in a frame
// store pos. SYNC2 for 'even' and 'odd' lines in 'msu_syncpos[2]' (global)
static int search_msu(FILE *fp)
{
  unsigned char frame[FRMLEN2];
  int i;
  int n=0;
  // Search for 2 syncs in 2 frames -> even and odd lines
  while (fread(frame,FRMLEN2,1,fp))
  {
    for (i=0; i<FRMLEN2; i++)
    {
      if ((cmp_sync(frame+i,SYNC2a,SYNC2b,8)<1))
      {
        msu_syncpos[n++]=i;
      }
      if (n>1) break;
    }
    if (i<FRMLEN2) break;
  }

  // msu-mr odd line starts earlier than msu-mr of even line
  //   -> swap if wrong order
  // ==> sync2pos[0]=even, sync2pos[1]=odd
  if (msu_syncpos[0]>msu_syncpos[1])
  {
    int tmp;
    tmp=msu_syncpos[1];
    msu_syncpos[1]=msu_syncpos[0];
    msu_syncpos[0]=tmp;
  }

  return (n==2? 1 : 0);
}



// search SYNC2 pos.
int get_msupos(FILE *fp)
{
  int ret=search_msu(fp);
  return ret;
}

// goto frame having 'sync'.
// at exit: 'frame' contains frame with msu-sync, so start-of-line
// return: index pos_msu containing pos. of start-of-msu in frame
static int gotosync(FILE *fp,unsigned char *frame)
{
  int nr_msu;
  while (fread(frame,FRMLEN2,1,fp))            // read 1 frame
  {
    // search even lines
    if ((cmp_sync(frame+msu_syncpos[0],SYNC2a,SYNC2b,8)<4))
    {
      nr_msu=(msu_syncpos[0]>=pos_msu[3]? 3: msu_syncpos[0]>=pos_msu[2]? 2: msu_syncpos[0]>=pos_msu[1]? 1: 0);

      return nr_msu;
    } 

    // search odd lines
    if ((cmp_sync(frame+msu_syncpos[1],SYNC2a,SYNC2b,8)<4))
    {
      nr_msu=(msu_syncpos[1]>=pos_msu[3]? 3: msu_syncpos[1]>=pos_msu[2]? 2: msu_syncpos[1]>=pos_msu[1]? 1: 0);
      return nr_msu+4;
    }
  }
  // no sync found; eof
  return -1;
}


//              
// ch1..3: width=1572, lowered to 1570 for simpl. reason (same width as ch4-6)
// ch4..6: width=1570
// read MSU-MR's for 1 line (50 frames)
#define OFFSET 50

// read MSU-MR's for 1 line (50 frames)
//   1 line has 11790 bytes (1572 pixels, 6 channels ==> 1572*10/8*6=11790)
//   Incl 50 bytes info at start, 10 spare at end: 11790+50+10=11850
//   1 frame has 3*238+234=948 bytes
//   ==> 1 line needs 11850/948=12.5 frames=12800 bytes
//   frame with start-of-line at alternating 12288 or 13312
void read_mhrpt2_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line)
{
  int nr_msu;
  int pos;
  int len;
  int np;
  unsigned char frame[FRMLEN2];
  int eo;
  int npos=y*12288+(y%2? 1024:0);

  // goto pos in file where frame starts with start-of-line 'y'
  fseek(pci->fp,pci->pic_strt+npos,SEEK_SET);
  // goto exact start-of-line
  if ((nr_msu=gotosync(pci->fp,frame))>=0)
  {
    if (nr_msu>=4) eo=2; else eo=1;
    nr_msu=nr_msu%4;
    for (np=0; np<13*4; np++)
    {
      if (eo==2)
      {
        if (np==0)
        {
          pos=msu_syncpos[1]+OFFSET;
          len=len_msu[nr_msu]-(msu_syncpos[1]-pos_msu[nr_msu])-OFFSET;
        }
        else
        {
          pos=pos_msu[nr_msu];
          len=len_msu[nr_msu];
        }
      }
      else
      {
        if (np==0)
        {
          pos=msu_syncpos[0]+OFFSET;
          len=len_msu[nr_msu]-(msu_syncpos[0]-pos_msu[nr_msu])-OFFSET;
        }
        else
        {
          pos=pos_msu[nr_msu];
          len=len_msu[nr_msu];
        }
      }

      msu2line(frame,pos,len,np,line);
      nr_msu++;
      if (nr_msu==4)
      {
        nr_msu=0;
        if (!(fread(frame,FRMLEN2,1,pci->fp))) break;
      }
    }
  }
}

