/**************************************************
 * RCSId: $Id: hri.h,v 1.1 2004/04/06 21:31:36 ralblas Exp $
 *
 * HRI header file
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: hri.h,v $
 * Revision 1.1  2004/04/06 21:31:36  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifndef HRI_HDR
#define HRI_HDR

/* ==== Some HRI frame specific characteristics ==== */
#define SYNC1 0x05
#define SYNC2 0x0c
#define SYNC3 0xdf

#define LEN_HRIFRAME 364
#define NR_HRI_CHAN 5
/* Max. length 1 line is 5000. Needs 5000/8=625 bytes for grid info. */
#define GRIDBYTES_LINE 650

/* Channel codes */
#define NO_SUBFRAME 0x00
#define SINGLE_SUBFRAME 0xff
#define FIRST_SUBFRAME 0xf0
#define SECOND_SUBFRAME 0x0f

/*************************************
 * Mapping HRI subframe.
 * Derived from Meteosat system guide.
 * NOTE: Size of elements important!
 *************************************/
typedef struct
{
  guint8 sync[3];
  guint8 id;
  gint16 frames_per_subframe;
  gint16 nr_subfr;
  gint16 cur_subfr;
  gint16 line_nr;
  gint32 image_nr;
  guint8 form_ind;
  guint8 spectr_ind[4];
  guint8 grid_ind;
  guint8 text_ind;
  guint8 scan_dir;
  guint8 spare1[4];
  guint8 sp1[8];
  gint16 sat_ind;
  gint16 year_im_acq;
  gint16 day_im_acq;
  gint16 image_time;
  guint8 label[12];
  guint8 spare2[12];
} HRI_SUBFRHDR;

/*************************************
 * HRI channel info.
 *************************************/
typedef struct
{
  int width,height;
  gboolean dbl_res;
} HRI_CHANNEL;

/*************************************
 * HRI picture info.
 * This is a HRI extension of PICINFO.
 * It holds some characteristics special for HRI.
 * E.g. channels in 1 transmission can have different sizes.
 *************************************/
typedef struct
{
  char satname[30];               /* name of the received satellite */
  HRI_CHANNEL channel_info[NR_HRI_CHAN];
  char format_ind;                /* A, B or X */
  gboolean encrypted;              /* transmission encrypted */
  int frm_p_subfrm;               /* frames per subframe: 4 or 8 */
  int subfrm_p_line;              /* subframnes per line (1 or 2) */
  int first_subfr;                /* number first recorded subframe */
  int first_line_nr;              /* number of first recorded line */
  int start_pic;                  /* start picture in first frame */
} HRI_PICINFO;

void hri_satind2satname(guint16, char *);
void hri_channelselect();
void add_hrigrid();

#endif
