/**************************************************
 * RCSId: $Id: hrpt.h,v 1.1 2014/12/09 19:17:32 ralblas Exp $
 *
 * Record HRPT using parallel port.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: hrpt.h,v $
 * Revision 1.1  2014/12/09 19:17:32  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifndef HRPT_REC_PP_HDR
#define HRPT_REC_PP_HDR
#include "wsat.h"

/* Define NOAA constants */
#define DEF_NOAA_ALPHAMAX 55.4   /* max. sensor swing in 1 dir. */
#define DEF_NOAA_XRES 2048

/* Define Fenyung constants */
#define DEF_FY_ALPHAMAX 55.4     /* max. sensor swing in 1 dir. */
#define DEF_FY_XRES 2048

/* Define Seastar constants */
#define DEF_SEAST_ALPHAMAX 58.3  /* max. sensor swing in 1 dir. */
#define DEF_SEAST_XRES 1288

/* Max. expected frame length. This is about 22000 for CHRPT. */
#define MAX_HRPT_FRAMELEN 30000


/* Parsed HRPT header results */
typedef struct hrpt_framehdr
{
  guint16 sync_word[6]; 
  gboolean avhrr_sync;
  guint16 minor_framenr;
  guint16 spacecraft_addr;
  gboolean resync;
  char chan3ab;
  guint16 day_count;
  guint32 msec_count;
  guint16 telemetry[5];
  guint16 target_temp[5];
  guint16 back_scan[10][5];
  guint16 space_scan[10][5];
  gboolean avhrr_synclate;
  guint16 period_998_khz;
  guchar ch3_patchtemp;
  char sat_name[20];
  struct tm sat_time;
  guint16 msec;
} HRPT_FRAMEHDR;

typedef struct noaacal
{
/* Table D.1-8 */
  float d0[5];
  float d1[5];
  float d2[5];
  float d3[5];
  float d4[5];
  float nuc[6];
  float a[6];
  float b[6];
  float Ns[6];
  float b0[6];
  float b1[6];
  float b2[6];
  float c1;
  float c2;
} NOAACAL;


FRAMEINFO calc_hrptframeinfo();
NOAACAL load_cal();
void map_noaatemp();
gint map_temp();
gint inv_maptemp();

#endif
