/**************************************************
 * RCSId: $Id: wsat.h,v 1.11 2017/07/20 21:35:53 ralblas Exp $
 *
 * WSAT header file 
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: wsat.h,v $
 * Revision 1.11  2017/07/20 21:35:53  ralblas
 * _
 *
 * Revision 1.10  2017/07/01 17:57:15  ralblas
 * _
 *
 * Revision 1.9  2017/05/28 14:30:22  ralblas
 * _
 *
 * Revision 1.8  2017/05/07 16:31:11  ralblas
 * _
 *
 * Revision 1.7  2017/01/02 15:59:47  ralblas
 * _
 *
 * Revision 1.6  2016/12/09 13:38:15  ralblas
 * _
 *
 * Revision 1.5  2016/12/05 22:03:34  ralblas
 * _
 *
 * Revision 1.4  2016/12/03 00:29:09  ralblas
 * _
 *
 * Revision 1.3  2016/11/18 10:05:07  ralblas
 * _
 *
 * Revision 1.2  2016/06/04 20:48:17  ralblas
 * _
 *
 * Revision 1.1  2016/02/11 22:35:23  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifndef WSAT_HDR
#define WSAT_HDR

/* Define if to compile for Windows.
   Needed because there are a few portability problems (yet).
*/
#ifndef __GTK_WIN32__
#define __GTK_WIN32__ 0
#endif

#include <stdio.h>
#include <time.h>
#include "sattrack.h"

#ifdef __ADD_ETH__
  #if __GTK_WIN32__
    #include <winsock2.h>
  #else
    #include <sys/types.h>
    #include <sys/socket.h>
    #include <linux/if_packet.h>
    #include <linux/if_ether.h>
    #include <linux/if_arp.h>
    #include <netinet/in.h>
    #define SOCKADDR_IN struct sockaddr_in
    #define SOCKET int
  #endif
#endif

#include "gtk/gtk.h"
#include "gdk/gdkkeysyms.h"
#include "sgtk.h"

/* Define PI if unknown */
#ifndef PI
#define PI 3.1415926
#endif

/* gonio functions/vars */
#define PIx2 (PI*2.)           /* not present in math.h */
#define D2R(g) ((g)*PI/180.)   /* degree --> radians */
#define R2D(g) ((g)*180./PI)   /* radians --> degree */

/* Define supported file formats */ 
#define _SUN_   0
#define _TIFF_  0
#define _PPM_  1
#define _GIF_   1
#ifndef _JPEG_
  #define _JPEG_  0
#endif
#ifndef _HRPT1_
  #define _HRPT1_ 0
#endif
#ifndef _HRI1_
  #define _HRI1_  0
#endif
#ifndef _RAWDT_
  #define _RAWDT_  0
#endif
#define _AVHRR1B_   0
#define _MHRPT1_ 1
#define _MHRPT2_ 1
#ifndef _LRIT1_
  #define _LRIT1_ 0
#endif
#ifndef _IQTEST1_
  #define _IQTEST1_ 0
#endif

#ifndef _TRACKER_
  #define _TRACKER_ 0
#endif

/*************************************
 * Satellite types and scan direction
 *************************************/
typedef enum {sat_unk=0,HRPT,CHRPT,M1HRPT,M2HRPT,SEASTAR,
              HRI,LRIT,FY3,METOP,IQTEST} SAT_TYPE;
#ifndef __SCANDIR__
#define  __SCANDIR__
typedef enum {scan_unk=0,N_S,S_N,W_E,E_W} SCANDIR;
#endif

/*************************************
 * File formats, channel types
 *************************************/
typedef enum { error_filetype=0,unknown_filetype,
               FT_SUN,FT_TIFF,FT_PPM,FT_PGM,FT_PGM1,FT_GIF,FT_JPEG,
               FT_HRPT,FT_CHRPT,FT_SEAWIF,FT_HRI,FT_HPT,FT_CHPT,
               FT_MHRPT1,FT_MHRPT1N,FT_MHRPT2,FT_MHRPT2N,  // N: no header found
               FT_METOP,FT_FY3,FT_LRIT,FT_IQTEST
             } FILE_TYPE;

#define IS_HRPT_FAM(t) (t==FT_HRPT   ? TRUE : \
                        t==FT_CHRPT  ? TRUE : \
                        t==FT_SEAWIF ? TRUE : \
                        t==FT_MHRPT1 ? TRUE : \
                        t==FT_MHRPT2 ? TRUE : \
                        t==FT_HPT    ? TRUE : \
                        t==FT_CHPT   ? TRUE : \
                                       FALSE)

#define IS_HRPT_METOP_FAM(t) (IS_HRPT_FAM(t)? TRUE : \
                              t==FT_METOP   ? TRUE : \
                              t==FT_FY3     ? TRUE : \
                                              FALSE)

#define IS_PRIM_DATA(t) (IS_HRPT_FAM(t) ? TRUE : \
                               t==FT_HRI ? TRUE : \
                                         FALSE)

#define IS_HRPTIR(vwi) (vwi->channel_selected[3]? TRUE: \
                        vwi->channel_selected[4]? TRUE: \
                        vwi->channel_selected[5]? TRUE: FALSE)

/*************************************
 * Color info
 *************************************/
#define MAX_PCICLR 4096
typedef struct clrmap
{
  int size;
  int red[MAX_PCICLR],green[MAX_PCICLR],blue[MAX_PCICLR];
} CLRMAP;

/* Mask of grid info */
#define GRID_MASK 0x8000

/* Fixed zoom factor */
#define FIX_ZOOM 2.0

/*************************************
 * Frame info
 *************************************/
typedef struct frameinfo
{
  int words_per_frame;     /* amount of words in a frame */
  int words_per_line;      /* total amount of saved data words per line */
  int bytes_per_line;      /* total amount of saved bytes per line */
  int header_len;          /* words before first picture-word */
  int leader_len;          /* words after last picture-word */
  int inter_chan;          /* interval between 2 words of a channel */
  int dist_chan;           /* interval between 2 channels */
} FRAMEINFO;


#define Lab_Main "Wsat"
#define Lab_Track "Tracker"
/*************************************
 * Picture info
 *************************************/
#if _GIF_
#  include "gif.h"
#endif

#include "hri.h"

#define NR_CHANNELS_MAX 11
#define LENSATNAME 40
#define LENFILENAME 200

#define USE_GTK
#include "sattrack.h"
#ifndef SATTRACK_HDR
typedef struct wpoint
{
  float x,y,z;
  float lon,lat;
} EPOINT;

typedef struct kepler
{
  gint32 epoch_year;
  float  epoch_day;
  float  decay_rate;
  float  inclination;
  float  raan;
  float  eccentricity;
  float  perigee;
  float  anomaly;
  float  motion;
  gint32 epoch_rev;
} KEPLER;

typedef struct orbit
{
  gboolean valid;                  /* flag: kepler data valid? */

/* Start time record (used) */
  struct tm start_tm;              /* record start time  */
  int       start_ms;              /* record start time msec */

/* Start time record (satellite) */
  struct tm start_tm_sat;          /* record start time  */
  int       start_ms_sat;          /* record start time msec */

/* Start time record (computer) */
  struct tm start_tm_comp;          /* record start time  */

/* Reference time Kepler data */
  gint32    ref_time;              /* total reference time  */
  struct tm ref_tm;                /* reference time year ... sec */
  int       ref_ms;                /* reference time msec */

  float height;                    /* sat height [meters] */ 
  float loop_time;                 /* time 1 loop [seconds] */
  float k2_n;
  float k2_w;
  float dh_w;

  gboolean use_sattime;
  int offset_gmt;
  int offset_sec;
  float offset_lon;
} ORBIT;
#endif

typedef struct picinfo
{
  char work_location[300];         /* location of working directory */
  char file_name[LENFILENAME];     /* name of current file */
  FILE *fp;                        /* filepointer; NULL if no file opened */
  FILE_TYPE file_type;             /* file format */
  SAT_TYPE sat_type;               /* satellite type (HRPT, CHRPT etc.) */
  char sat_name[LENSATNAME];       /* name satellite */

  int width,height,depth;          /* Size picture */
  gboolean no_xymapping;           /* if true: x->x and y->y, no flipping etc. */
  gboolean reed_solomon;           /* reed solomon done */
  int jpg_quality;                 /* quality jpeg in percent (0...100%) */
  float max_sens_angle;            /* satellite sensor angle */
  CLRMAP clrmap;                   /* */
  FRAMEINFO fri;                   /* Frame info */
  long pic_strt;                   /* pic starts at this byte number */

  int width_original;              /* Size original pic */
  int ox,oy;                       /* Offset to original pic */
  int lines_per_sec;               /* lines per second */

  SCANDIR scan_hor,scan_ver;       /* Movement of satellite */
  gboolean channels_recorded[NR_CHANNELS_MAX];  /* Channels recorded */
  struct tm picture_time;          /* Time of pic (not always time of record) */

  int syncsignal_del;              /* delay in-sync signal decoder */
#if _GIF_
  GIF_PICINFO gif_pci;             /* Extra pic info for GIF */
#endif

  HRI_PICINFO hri_pci;             /* Extra pic info for HRI */
  KEPLER kepler;                   /* Keplerian elements of sat orbit */
  ORBIT orbit;                     /* Orbit vars */

  char kep_location[300];           /* location Kepler files */

  char firmware[8];
} PICINFO;

typedef struct zoomvals
{
  float zx,zy;                     /* Zoom factors (>1 = zoomed in) */
  float dx,dy;                     /* pic_size/screen_size/zoomfactor */
  int ox,oy;                       /* offset */
} ZOOMVALS;

#define NR_PREFMAPS 2
typedef struct clrmapchan
{
  char name[20];
  int red_offset;
  int green_offset;
  int blue_offset;
  float red[NR_CHANNELS_MAX];
  float green[NR_CHANNELS_MAX];
  float blue[NR_CHANNELS_MAX];
  float gamma;
} CLRMAPCHAN;

/*************************************
 * View info
 *************************************/
typedef struct viewinfo
{
  gboolean redraw;                 /* To signal that picture should be redraw */
  struct zoomvals zval;            /* zoom parameters */
  struct zoomvals zval_pre;        /* previous zoom parameters */
  int lum_min;                     /* pixel with value <=lum_min -> black */
  int lum_max;                     /* pixel with value >=lum_max -> white */

  int *lin_xtbl;                   /* linearize table */
  int *revlin_xtbl;                /* reverse linearize table */

  gboolean coordxy;                /* show xy coord. instead of lonlat */
  long lumstat[MAX_PCICLR];        /* lumstat[l]=number pixels with val. l */
  gboolean invert;                 /* inverse mapping */
  float gamma;                     /* gamma value */

  SCANDIR scan_hor,scan_ver;       /* scan direction current view */
  gboolean channel_selected[NR_CHANNELS_MAX]; /* single-channel selected */
  gboolean linearize;              /* linearize HRPT */
  gboolean keep_xyrel;             /* keep x/y relation */
  gboolean despeckle;              /* despeckle */
  gboolean destrip;                /* destrip (multi-frame lines) */
  gboolean reed_solomon;           /* enable reed solomon */
  gboolean grid;                   /* Add grid HRI */
  gboolean fixzooming;             /* busy with fixed zooming (command 'Z') */
  gboolean zooming;                /* busy with zooming (command 'z') */
  gboolean skip_faulty_lines;      /* Don't plot lines with error codes */
  gboolean mark_faulty_lines;      /* Mark errored lines */
  int      mask_skip;              /* 01 -> use post-sync; 10 -> use pre-sync to skip */
  gboolean map_temp;               /* map to temperature */

  float fix_zoomfactor;            /* fixed zooming factor */
  gboolean refresh_pic;            /* refresh picture instead of after each line */
  long foffset;                    /* file offset, for multiple-pic files */
  gboolean slide_mode;             /* slide flag */
  char slide_file[LENFILENAME];    /* slide file */
  int slide_speed;                 /* speed slide */
/* Colors for grid */
  GdkColor grid_clr;

/* ====== Variables used to process data  ======= */
  gboolean process_data;           /* data is processed (e.g. colorized) */

/* Channels selected in process-data mode */
  gboolean channels_selected[NR_CHANNELS_MAX];

/* Next variables are used for colorizing pictures. */
  struct clrmapchan cmhc;

/* Next variables are used for aligning */
  int hshift[NR_CHANNELS_MAX];
  int vshift[NR_CHANNELS_MAX];
  int tilt[NR_CHANNELS_MAX];
  int min_vshift,max_vshift;

/* Next variables are used to despeckle */
  int sur_ulim,dsp_llim;
} VIEWINFO;

#ifdef __ADD_ETH__
typedef struct eth
{
  SOCKET s;
  unsigned char *pbuf;
  int plen;
  int prot;
  void *buffer;
#ifdef __GTK_WIN32__
  SOCKADDR_IN socket_address;
#else
  struct sockaddr_ll socket_address;
#endif
} ETH;
#endif


typedef enum { parport=0,ethernet,usb } PORT_TYPE;

/*************************************
 * Record info
 *************************************/
typedef struct rec
{
  gboolean recording;              /* busy-with-recording flag */
  gboolean one_record;             /* Stop after 1 record */
  gboolean ignore_sync;            /* capture data without waiting on framesync */
  char rec_chan[10];               /* string of chanels to record (auto)  */
  char work_location[300];         /* name of directory in which to save file */
  char file_name[100];             /* name file in which to save recording */
  char file_prefix[30];            /* prefix file for auto-name generation */
  char file_suffix[30];            /* suffix file for auto-name generation */
  char file_hrptbase[30];          /* basename for file HRPT */
  char file_hrptext[30];           /* extension for file HRPT */
  char file_chrptbase[30];         /* basename for file CHRPT */
  char file_chrptext[30];          /* extension for file CHRPT */
  char file_mhrptbase[30];         /* basename for file MHRPT */
  char file_mhrptext[30];          /* extension for file MHRPT */
  char file_metopbase[30];         /* basename for file METOP */
  char file_metopext[30];          /* extension for file METOP */
  char file_fy3base[30];           /* basename for file FY3 */
  char file_fy3ext[30];            /* extension for file FY3 */
  char file_hribase[30];           /* basename for file HRI */
  char file_hriext[30];            /* extension for file HRI */
  FILE *fp;                        /* filepointer to recording file */
  FILE_TYPE file_type;             /* type of recording file */
  SAT_TYPE sat_type;               /* satellite type (HRPT, CHRPT etc.) */
  SCANDIR scan_hor,scan_ver;       /* direction of satellite */
  gboolean channels_recording[NR_CHANNELS_MAX]; /* channels recording */
  int sh_recchan;                  /* channel shown during record */
  char sat_name[LENSATNAME];       /* name of satellite */
  int height;                      /* expected height of picture */
  gboolean hwsel_type;             /* select HW: 'dec/gen' or 'hrpt/chrpt*/
  gboolean sel_hw;                 /* HW selected */
  gboolean gen1_dec0;              /* selected HW: 'generator' or 'decoder' */
  gboolean save_flag;              /* if true, save the recording */
  gboolean encrypted;              /* flag: Data encrypted? */
  gboolean record_all;             /* flag: Record also encrypted data */
  gboolean reed_solomon;           /* enable reed solomon during record*/
  int hrpt_start_after_in_sync;    /* start hrpt-rec. after # frames in-sync */
  int hrpt_stop_after_out_sync;    /* stop hrpt-rec. after # frames out-sync */
  int hri_start_after_in_sync;     /* start hri-rec. after # frames in-sync */
  int hri_stop_after_out_sync;     /* stop hri-rec. after # frames out-sync */
  int metop_start_after_in_sync;   /* start metop-rec. after # frames in-sync */
  int metop_stop_after_out_sync;   /* stop metop-rec. after # frames out-sync */
  gboolean vitsync;                /* viterby sync-flip */

  gboolean ena_man_gen;
  gboolean ena_punc_gen;
  gboolean ena_rand_gen;

  gboolean ena_man_dec;
  gboolean ena_punc_dec;
  gboolean ena_rand_dec;
  int syncpunc_dec;

  gboolean inv_dat;                /* input data inv-flip */
  gboolean batch_mode;             /* flag: Batch mode. */
  gboolean auto_get_kepler;        /* flag: catch Kepler after record? */
  char kepler_file[300];           /* Kepler-file */

  PORT_TYPE port_type;
#ifdef __ADD_ETH__
  struct eth eth;
#endif
#ifdef __ADD_USB__
  USB_ITEMS usb;
#endif
/*
   If parallel port is used to connect decoder with PC then 
   next var. contains address of this port to which it is connected. 
*/
  int parport_base;                /* decoder connected to this par. port */ 
  gboolean fast_parportaccess;     /* if true: fastest parport access */

  int sim_record;                // 0=not, 1=per-frame, 2=at USB-interface
  char *sim_file;


  FRAMEINFO fri;                   /* some numbers of frame-format recording */
} RECORDINFO;

/*
typedef struct rotor
{
  gboolean x_at_disc;
  gboolean inv_x,inv_y;   // to correct "wrong" rotor direction
  int xy_rotorlim;
  int deg2step;
  gboolean x_west_is_0;   // def.: east=0
  gboolean y_south_is_0;  // def.: north=0
  gboolean use_xy;
  int storm_pos_x;
  int storm_pos_y;
  int storm_wait_x;
  int storm_wait_y;
//  DIRECTION storm;
} ROTOR;
*/

/*************************************
 * Preferences
 *************************************/
typedef struct prefs
{
  WND_POS main_wnd_geom,sub_wnd_geom;
  int main_wnd_x,main_wnd_y;
  int main_wnd_w,main_wnd_h;
  int canvas_w,canvas_h;
  int sub_wnd_x,sub_wnd_y;

  PORT_TYPE port_type;
  int parport_nr;                  /* parport number: 1, 2 */
  int parport_address;             /* parport address */
  int parport_base;                /* decoder connected to this par. port */ 
  gboolean fast_parportaccess;     /* if true: fastest parport access */

  char usbport_id[30];
  gboolean usb_use_id;             /* if true: use usbport_id */
  gboolean fast_ftd;               /* if true: use ft232h (faster than ft245) */
  gboolean metfy3x;
  gboolean hwsel_type;             /* if true: sel. hrpt/chrpt, else dec/gen */
  int hrpt_start_after_in_sync;    /* start hrpt-rec. after # frames in-sync */
  int hrpt_stop_after_out_sync;    /* stop hrpt-rec. after # frames out-sync */
  int metop_start_after_in_sync;   /* start metop-rec. after # pcts? in-sync */
  int metop_stop_after_out_sync;   /* stop metop-rec. after # frames out-sync */


  char file_hrptbase[30];          /* basename for file HRPT */
  char file_hrptext[30];           /* extension for file HRPT */
  char file_chrptbase[30];         /* basename for file CHRPT */
  char file_chrptext[30];          /* extension for file CHRPT */
  char file_mhrptbase[30];         /* basename for file MHRPT */
  char file_mhrptext[30];          /* extension for file MHRPT */
  char file_metopbase[30];         /* basename for file LRIT */
  char file_metopext[30];          /* extension for file LRIT */
  char file_fy3base[30];           /* basename for file FY3 */
  char file_fy3ext[30];            /* extension for file FY3 */
  char file_hribase[30];           /* basename for file HRI */
  char file_hriext[30];            /* extension for file HRI */

  char used_preffile[300];         /* preffile used to read from */
  char dir_name[300];
  char base_name[300];
  char kep_dirname[300];
  char kep_filename[300];
  char filename_template[300];     /* template for generating filenames */
  char map_file[300];

  int hri_start_after_in_sync;     /* start hri-rec. after # frames in-sync */
  int hri_stop_after_out_sync;     /* stop hri-rec. after # frames out-sync */

  int hrpt_lines_per_screen;       /* (c)hrpt lines per screen during record */
  SCANDIR hrpt_scan_ver;           /* direction of satellite during record */
  int sh_recchan;                  /* channel shown during record */

  gboolean save_flag;              /* if true, save the recording */
  gboolean auto_get_kepler;        /* flag: catch Kepler after record? */
  EPOINT refpos;                    /* for tracker: position observer */
  float elev_horiz;                /* hozizon at this elevation */
  float elev_det;                  /* follow if elevation will reach at least this value */

  struct rotor rotor;
  int font_size;
  GdkColor grid_clr;
  int sur_ulim,dsp_llim;
  struct clrmapchan cmhc[NR_PREFMAPS];
  float fix_zoomfactor;            /* fixed zooming factor */

  float utc_offset;
  struct tm_offset tm_off;

  gboolean ext_prog_on;
  char prog_trackup[100];
  char prog_trackdown[100];

  gboolean linearize;
  gboolean keep_xyrel;
  char satlist[100];
  
  char *cur_dir;
  char *home_dir;
  char *prog_dir;
  gboolean log;
  float reptime;
} PREFS;

/*************************************
 * Keys for wsat data
 * These keys are used to add and extract data to/from GTK.
 *************************************/
#define WSAT_PCI "wsat_pci"
#define WSAT_VWI "wsat_vwi"
#define WSAT_RDI "wsat_rdi"

#define LAB_ROTOR_ELEAZIM "!To rotor"
#define LAB_ROTOR_XY "!To rotor "

/* Function definition */
#include "wsat_functions.h"

#endif
