/*****************************************************
 * RCSId: $Id: wsat_functions.h,v 1.9 2017/01/06 17:57:14 ralblas Exp $
 *
 * Header file containing function defs
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: wsat_functions.h,v $
 * Revision 1.9  2017/01/06 17:57:14  ralblas
 * _
 *
 * Revision 1.8  2017/01/04 15:43:48  ralblas
 * _
 *
 * Revision 1.7  2016/12/09 14:00:26  ralblas
 * _
 *
 * Revision 1.6  2016/12/09 13:58:01  ralblas
 * _
 *
 * Revision 1.5  2016/12/03 22:44:23  ralblas
 * _
 *
 * Revision 1.4  2016/11/14 21:55:19  ralblas
 * _
 *
 * Revision 1.3  2016/10/04 19:55:10  ralblas
 * _
 *
 * Revision 1.2  2016/05/27 09:55:28  ralblas
 * _
 *
 * Revision 1.1  2015/09/20 11:21:12  ralblas
 * Initial revision
 *
 *****************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifndef WSAT_FUNCS
#define WSAT_FUNCS

int Read_Prefs();
void Load_Prefs();

int open_file();
void close_picfile();
void close_recfile();
int open_new_recfile();
char *strip_filename(char *);
FILE_TYPE detect_file(char *);

gboolean draw_pic();
void Channel_Select();
void init_menubuts();
int exist_file(char *);
void log_message(char *frmt,...);

ZOOMVALS zoom_full(int,int,int,int);
void correct_zoomval();
void Update_Zoom();
void Update_All();
void Update_Main();
void Update_Main_Wait();
void Update_Lum();
void Update_channel_save();

GtkWidget *Create_orbitedt_wnd(GtkWidget *,KEPLER *,ORBIT *);
GtkWidget *Create_kepleredt_wnd(GtkWidget *,KEPLER *,ORBIT *);
void Set_Keplervals(GtkWidget *,KEPLER *,ORBIT *);
int read_norad_next_keps(FILE *,char *,KEPLER *);
GtkWidget *get_trackerwindow(GtkWidget *w);

int get_satkeps(FILE *,char *,KEPLER *);
void Reload_Kepler(PICINFO *);

int Rewrite_hdr(PICINFO *);
float x2earthangle(float ,SCANDIR ,int ,int ,float );
int calc_orbitconst(KEPLER *,ORBIT *);
int xy2lonlat(int ,int ,PICINFO *,EPOINT *);
int draw_lonlat();

GdkCursor *create_cursor();
gboolean key_actions();
void mouse_actions();
void scroll_actions();

void load_std_palette();
void load_extra_palette();
void Repaint();
int map_pic();
int map_xy(int      wnd_width,
           int      wnd_height,
           gboolean no_xymapping,
           gboolean single_channel,
           PICINFO  *pci,
           VIEWINFO *vwi,
           guint    **xmap,
           guint    **ymap);
int map_z(int lum_min,int lum_max,float gamma,gboolean invert,int clrmap_size,int zmax,guint **zmap);

int scrx2picx(int ,VIEWINFO *,PICINFO *);
int picx2scrx(int ,VIEWINFO *,PICINFO *);
int scry2picy(int ,VIEWINFO *,PICINFO *);
int picy2scry(int ,VIEWINFO *,PICINFO *);

void Update_Coord(GtkWidget *,int ,int ,PICINFO *,VIEWINFO *);

void Translate_Pix();
void Translate_Line(VIEWINFO  *vwi,         /* view info */
                    PICINFO   *pci,         /* picture info */
                    int       bitshift,
                    guint     *zmap,        /* color mapping */
                    guint16   **lines,      /* Input line */
                    int       nrlines,      /* nr. Input line */
                    guint16   *line_r,      /* Output red line */
                    guint16   *line_g,      /* Output green line */
                    guint16   *line_b);     /* Output blue line */
void draw_a_line_gc();
void draw_a_line_bw(guint16    *line,        /* 1 line to draw */
                    int         wnd_width,   /* # pixels in line on canvas */
                    int         y,           /* y-position */
                    guint       *xmap,       /* hor. mapping */
                    guint       *zmap,       /* color mapping */
                    int         cmax,        /* Upper limit colormap */
                    RGBI        *rgbi,       /* RGB picture buffer */
                    PICINFO     *pci,
                    VIEWINFO    *vwi);       /* view info */

int KeyPressed();

void select_channel_pp();
int Open_HWConnect(RECORDINFO *rdi);
void Close_HWConnect(RECORDINFO *rdi);
void Update_channel_state();
gboolean get_pp_data();

gboolean wait_for_insync();
gboolean wait_for_outsync();

int get_ppreg();
int read_line();
int read_lines();
guint16 **alloc_lines();
void free_lines();
SAT *read_msat(FILE *fp,char *satnames_toload,SAT *sat);

int write_mhrpt1_line(PICINFO *pci,guint16 *line);
void read_mhrpt1_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line);
void read_mhrpt2_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *line);


void select_channel(gboolean   *chan,RECORDINFO *,gboolean);
int get_msupos(FILE *fp);
FRAMEINFO calc_hrptframeinfo();
FRAMEINFO calc_hriframeinfo();
int search_first_reliable_frame();
int interpret_framehdr();
char *xrptid2name(int ,SAT_TYPE);
void set_falseclr(VIEWINFO *vwi,PICINFO *pci);
void Reset_Mapchanclr(int set);

// std funcs
int read_ppm_header(PICINFO *pci);
#if _JPEG_ == 1
int write_jpg_header(PICINFO *pc);
void write_jpg_line(PICINFO *pc,guint16 *rl,guint16 *gl,guint16 *bl);
#endif

void add_gridpix();

int get_isuf();
int nint(float);

gboolean check_range(int,int ,int );
gboolean check_tm(struct tm *);
void yday2mday_mon(struct tm *);
gint32 mktime_ntz(struct tm *);
struct tm offset_tm(struct tm tm,float hoffset);


void fatal_error();
void error();
char *strcpya(char *);
int strcpyd(char **, char *);
int sprintfd(char **s,char *f,...);
void freed(char **s);
int replaced(char **l,char *s1,char *s2,...);

VIEWINFO *Get_Viewinfo_db();
PICINFO *Get_Picinfo_db();
RECORDINFO *Get_Recordinfo_db();
void sleep_ms(int);

void Recordpp_Hri_Auto(GtkWidget *, gboolean);
void Recordpp_Hri(GtkWidget *,RECORDINFO *,PICINFO *,VIEWINFO *);
void Recordpp_Hrpt_Auto(GtkWidget * ,char *, SAT_TYPE);
void Recordpp_Hrpt(GtkWidget *,GtkWidget *,RECORDINFO *,PICINFO *,VIEWINFO *);
                   
void calc_lum_thresholds(VIEWINFO *,int );
void do_slide(GtkWidget *widget,gboolean start);
int do_batch(RECORDINFO *rdi,PICINFO *pci,VIEWINFO *vwi);

int usb_list_ndevs(FT_DEVICE_LIST_INFO_NODE **devInfo);

void Menu_Testusb(GtkWidget *widget,gpointer data);
void calc_satpos(struct tm cur_tm,EPOINT *pos_subsat,DIRECTION *satdir,EPOINT *refpos,SAT *sat,ROTOR *rot);

void draw_rast(GtkWidget *wnd);
void draw_back(GtkWidget *wnd);
void draw_ref(GtkWidget *wnd,float lon,float lat);
void draw_onepos(struct tm cur_tm,GtkWidget *wnd,SAT *sat,EPOINT pos_sat,EPOINT pos_earth);
void calcposrel(KEPLER *kepler,ORBIT *orbit,
                EPOINT *pos_sat,EPOINT *pos_earth,EPOINT *pos_rel);
void calc_onepos(SAT *sat,struct tm cur_tm,int cur_ms,
                 EPOINT *pos_earth,EPOINT *pos_sat,EPOINT *pos_subsat);

int usb_write_reg(USB_ITEMS *usb,int addr,int dat);
int usb_init_connect(USB_ITEMS *usb);
void dis_dat(USB_ITEMS *usb);
void ena_dat(USB_ITEMS *usb);
void usb_close_connect(USB_ITEMS *usb);
void set_sat(USB_ITEMS *usb,int dat);
int get_sat(USB_ITEMS *usb);

int read_gif_header1(FILE *fp, CLRMAP *clrmap, GIF_PICINFO *gif_pci);
int read_gif_header(PICINFO *pci);
void read_gif_line1(FILE *fp,GIF_PICINFO *gif_pci,int y,guint16 *line);
void get_sat_set_but(RECORDINFO *rdi,GtkWidget *hrpt_window);

GtkWidget *nb_debug();

DISEQC val2diseqc(float val,int deg2step,gboolean inv,int limit);
int do_dqc(USB_ITEMS *usb,DISEQC diseqc,gboolean displ_show);
#endif
