/**************************************************
 * RCSId: $Id: map_chan.c,v 1.8 2017/05/27 11:03:49 ralblas Exp $
 *
 * Channel mapping on colors
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: map_chan.c,v $
 * Revision 1.8  2017/05/27 11:03:49  ralblas
 * _
 *
 * Revision 1.7  2017/05/07 15:09:30  ralblas
 * _
 *
 * Revision 1.6  2016/12/16 19:05:38  ralblas
 * _
 *
 * Revision 1.5  2016/12/09 09:21:20  ralblas
 * _
 *
 * Revision 1.4  2016/12/03 22:44:37  ralblas
 * _
 *
 * Revision 1.3  2016/12/03 12:04:25  ralblas
 * _
 *
 * Revision 1.2  2016/11/18 19:05:14  ralblas
 * _
 *
 * Revision 1.1  2015/09/20 11:33:04  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include "wsat.h"
#include "menu_wsat.h"

#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#define Lab_Mapclrwnd   "Map channels"
#define LAB_Exit        " Close "
#define LAB_Redraw      " Redraw "
#define LAB_Color       " Auto color "

#define LAB_SAVCLR1     " Save map 1"
#define LAB_RESCLR1     " Restore map 1"
#define LAB_SAVCLR2     " Save map 2"
#define LAB_RESCLR2     " Restore map 2"

#define LAB_Align       " Align settings "
#define LAB_Reset_Align " Reset align "
#define LAB_Auto_Align  " Auto align "
#define LAB_Alignled    "Aligned"

void channelalignbuttons_presence();

extern PREFS prefs;

/*
CHRPT spectra
Band No	Spectral Range (m)	Primary use of data:
1	0.58 - 0.68 (VIS)	Daytime clouds, ice and snow, vegetation
2	0.84 - 0.89 (VNIR)	Daytime clouds, vegetation, water
3	3.55 - 3.93 (MWIR)	Heat source, night cloud
4	10.3 - 11.3 (TIR)	SST, day/night clouds
5	11.5 - 12.5 (TIR)	SST, day/night clouds
6	1.58 - 1.64 (SWIR)	Soil humidity, provision of ice/snow cover distinguishing capability
7	0.43 - 0.48 (VIS)	Ocean color
8	0.48 - 0.53 (VIS)	Ocean color
9	0.53 - 0.58 (VIS)	Ocean color
10	0.90 - 0.965 (VNIR)	Water vapor
*/
static GtkWidget *map_window;
void Reset_Mapchanclr(int set)
{
  static int state;
  if (set>0) state|=set; // else if (set==0) state=0;

  if (state&1)
    Set_Widgetcolor(Find_Widget(map_window,LAB_SAVCLR1),'b',0xff,0x00,0x00);
  if (state&2)
    Set_Widgetcolor(Find_Widget(map_window,LAB_SAVCLR2),'b',0xff,0x00,0x00);
  if (set==0)
  {
    if (state&1) Set_Widgetcolor(Find_Widget(map_window,LAB_SAVCLR1),'b',0x00,0xff,0x00);
    if (state&2) Set_Widgetcolor(Find_Widget(map_window,LAB_SAVCLR2),'b',0x00,0xff,0x00);
    state=0;
  }
}

void set_falseclr(VIEWINFO *vwi,PICINFO *pci)
{
  int i;
  for (i=0; i<NR_CHANNELS_MAX; i++) vwi->channels_selected[i]=FALSE;
  vwi->cmhc.gamma=1.;
  switch(pci->file_type)
  {
    case FT_HRPT: case FT_HPT:
    {
      vwi->cmhc.gamma=1.4;
      if (TRUE)
      {
        vwi->channels_selected[1]=TRUE;
        vwi->channels_selected[2]=TRUE;
        vwi->channels_selected[3]=TRUE;
        vwi->cmhc.red[1]  =0.0;
        vwi->cmhc.green[1]=0.0;
        vwi->cmhc.blue[1] =1.4;
        vwi->cmhc.red[2]  =1.2;
        vwi->cmhc.green[2]=1.2;
        vwi->cmhc.blue[2] =0.0;
        vwi->cmhc.red[3]  =0.3;
        vwi->cmhc.green[3]=0.2;
        vwi->cmhc.blue[3] =0.0;
      }
      else
      {
        vwi->channels_selected[1]=TRUE;
        vwi->channels_selected[2]=TRUE;
        vwi->channels_selected[4]=TRUE;
        vwi->cmhc.red[1]  =0.0;
        vwi->cmhc.green[1]=0.0;
        vwi->cmhc.blue[1] =1.7;
        vwi->cmhc.red[2]  =1.8;
        vwi->cmhc.green[2]=1.8;
        vwi->cmhc.blue[2] =0.0;
      }
    }
    break;
    case FT_CHPT: case FT_FY3:
      vwi->cmhc.gamma=1.5;
      if (TRUE)
      {
        vwi->channels_selected[2]=TRUE;
        vwi->channels_selected[6]=TRUE;
        vwi->channels_selected[9]=TRUE;
        vwi->channels_selected[10]=TRUE;
        vwi->cmhc.red[2]   =0.8;
        vwi->cmhc.green[2] =1.8;
        vwi->cmhc.blue[2]  =0.0;
        vwi->cmhc.red[6]   =0.9;
        vwi->cmhc.green[6] =0.0;
        vwi->cmhc.blue[6]  =0.0;
        vwi->cmhc.red[9]   =0.0;
        vwi->cmhc.green[9] =0.0;
        vwi->cmhc.blue[9]  =0.8;
        vwi->cmhc.red[10]  =0.0;
        vwi->cmhc.green[10]=0.0;
        vwi->cmhc.blue[10] =0.3;
      }
      else
      {
        vwi->channels_selected[1]=TRUE;
        vwi->channels_selected[2]=TRUE;
        vwi->channels_selected[4]=TRUE;
        vwi->cmhc.red[1]  =0.0;
        vwi->cmhc.green[1]=0.0;
        vwi->cmhc.blue[1] =2.0;
        vwi->cmhc.red[2]  =2.0;
        vwi->cmhc.green[2]=2.0;
        vwi->cmhc.blue[2] =0.0;
        vwi->cmhc.red[4]  =0.0;
        vwi->cmhc.green[4]=0.0;
        vwi->cmhc.blue[4] =0.0;
      }
    break;
    case FT_MHRPT1: case FT_MHRPT2:
      vwi->channels_selected[1]=TRUE;
      vwi->channels_selected[2]=TRUE;
      vwi->channels_selected[3]=TRUE;
      vwi->cmhc.red[3]  =1.;
      vwi->cmhc.green[2]=1.;
      vwi->cmhc.blue[1] =1.;
    break;
    case FT_CHRPT:
      vwi->channels_selected[1]=TRUE;
      vwi->channels_selected[2]=TRUE;
      vwi->channels_selected[7]=TRUE;
      vwi->cmhc.red[1]  =1.;
      vwi->cmhc.green[2]=2.;
      vwi->cmhc.blue[7] =1.;
    break;
    case FT_SEAWIF: 
      vwi->channels_selected[6]=TRUE;
      vwi->channels_selected[5]=TRUE;
      vwi->channels_selected[2]=TRUE;
      vwi->cmhc.red[6]=1.;
      vwi->cmhc.green[5]=1.;
      vwi->cmhc.blue[2]=1.;
    break;
    case FT_METOP:
      vwi->channels_selected[1]=TRUE;
      vwi->channels_selected[2]=TRUE;
      vwi->channels_selected[3]=TRUE;
      vwi->cmhc.red_offset   =-10;
      vwi->cmhc.green_offset =  0;
      vwi->cmhc.blue_offset  =  0;
      vwi->cmhc.red[1]       =  0.0;   // 1.8;
      vwi->cmhc.green[1]     =  0.2;   // 0.8;
      vwi->cmhc.blue[1]      =  2.0;   // 0.0
      vwi->cmhc.red[2]       =  1.8;   // 0.5;
      vwi->cmhc.green[2]     =  0.8;   // 0.9;
      vwi->cmhc.blue[2]      =  0.0;
      vwi->cmhc.red[3]       =  0.5;
      vwi->cmhc.green[3]     =  0.9;
      vwi->cmhc.blue[3]      =  0.0;
      vwi->cmhc.red[5]       =  0.0; // 0.0;
      vwi->cmhc.green[5]     =  0.0; // 0.2;
      vwi->cmhc.blue[5]      =  0.0; // 2.0;
    break;
    default:
    break;
  }
}

static void set_clrbuttons(GtkWidget *mapchan_wnd,VIEWINFO *vwi)
{
  int c;
  Set_Adjust(mapchan_wnd,"ro","%d",vwi->cmhc.red_offset);
  Set_Adjust(mapchan_wnd,"go","%d",vwi->cmhc.green_offset);
  Set_Adjust(mapchan_wnd,"bo","%d",vwi->cmhc.blue_offset);
  for (c=1; c<NR_CHANNELS_MAX; c++)
  {
    char tmp[10];
    sprintf(tmp,"Ch%d",c);
    Set_Button(mapchan_wnd,tmp,vwi->channels_selected[c]);

    if ((vwi->cmhc.red[c]) || (vwi->cmhc.green[c]) || (vwi->cmhc.blue[c]))
    {
      vwi->channels_selected[c]=TRUE;
      sprintf(tmp,"Ch%d",c);
      Set_Button(mapchan_wnd,tmp,vwi->channels_selected[c]);
    }
    sprintf(tmp,"r%d",c);
    Set_Adjust(mapchan_wnd,tmp,"%f",vwi->cmhc.red[c]);
    sprintf(tmp,"g%d",c);
    Set_Adjust(mapchan_wnd,tmp,"%f",vwi->cmhc.green[c]);
    sprintf(tmp,"b%d",c);
    Set_Adjust(mapchan_wnd,tmp,"%f",vwi->cmhc.blue[c]);
  }
}
/*************************************
 * Manage buttons etc.
 * 
 *************************************/
void mapchan_cmds(GtkWidget *widget, gpointer data)
{
  GtkWidget *mapchan_wnd=NULL;
  GtkAdjustment *adj=NULL;
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

/* NOTE: Set_Button causes this function to be called again.  
This results as if channel-button is activated but for some reason OLD state of button is transferred. This forces channel_selected to FALSE instead of TRUE.
Variable "button_set" is a work-around for this problem. This has to be investigated!
*/
  static gboolean button_set;    /* flag button-set by program, not by user */

  char *name=(char *)data;
  int c;

/* Manage adjustment widgets */
  if (GTK_IS_ADJUSTMENT(widget))
  {
    adj=(GtkAdjustment *)widget;

    if (!strcmp(name,"ro"))     vwi->cmhc.red_offset=adj->value;
    if (!strcmp(name,"go"))     vwi->cmhc.green_offset=adj->value;
    if (!strcmp(name,"bo"))     vwi->cmhc.blue_offset=adj->value;
    if ((c=get_isuf(name,"r"))) vwi->cmhc.red[c]=adj->value;
    if ((c=get_isuf(name,"g"))) vwi->cmhc.green[c]=adj->value;
    if ((c=get_isuf(name,"b"))) vwi->cmhc.blue[c]=adj->value;
    if ((c=get_isuf(name,"h"))) vwi->hshift[c]=adj->value;
    if ((c=get_isuf(name,"v"))) vwi->vshift[c]=adj->value;
    if ((c=get_isuf(name,"tilt"))) vwi->tilt[c]=adj->value;
  }

/* Manage button widgets */
  if (GTK_IS_WIDGET(widget))
  {
    GtkWidget *main_window;
    gboolean state;

    if (GTK_IS_TOGGLE_BUTTON(widget))
      state=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

    mapchan_wnd=gtk_widget_get_toplevel(widget);
    main_window=Find_Window(mapchan_wnd,Lab_Main);

    if (!button_set)           /* Only set channel-flag if button is clicked */
      if ((c=get_isuf(name,"Ch"))) vwi->channels_selected[c]=state;

/* Exit this window */
    if (!strcmp(name,LAB_Exit))
    {
      Close_Window(mapchan_wnd); mapchan_wnd=NULL;
/* Enable channel choice buttons (menu); keep false colour mode */
      Sense_Button(main_window,"Channel",TRUE);
      Update_channel_save(main_window,pci,vwi);
    }

/* Redraw picture */
    if (!strcmp(name,LAB_Redraw))
    {
      Update_Main(mapchan_wnd);
    }

/* Init True color values */
    if (!strcmp(name,LAB_Color))
    {
/* Reset offset-values */
      for (c=1; c<NR_CHANNELS_MAX; c++)
      {
        vwi->cmhc.red[c]=0;
        vwi->cmhc.green[c]=0;
        vwi->cmhc.blue[c]=0;
      }

/* Enable channels and init values to combine them */
      set_falseclr(vwi,pci);

/* Set buttons and RGB values of channels activated */
      button_set=TRUE;
      set_clrbuttons(mapchan_wnd,vwi);
      Set_Adjust(main_window,LAB_GAMMAVAL,"%f",vwi->gamma);

      button_set=FALSE;

/* Redraw picture */
      Update_Main(mapchan_wnd);
    }

    if (!strcmp(name,LAB_SAVCLR1))
    {
      prefs.cmhc[0]=vwi->cmhc;
      Reset_Mapchanclr(1);
    }

    if (!strcmp(name,LAB_RESCLR1))
    {
      vwi->cmhc=prefs.cmhc[0];
      set_clrbuttons(mapchan_wnd,vwi);
    }

    if (!strcmp(name,LAB_SAVCLR2))
    {
      prefs.cmhc[1]=vwi->cmhc;
      Reset_Mapchanclr(2);
      Set_Widgetcolor(widget,'b',0xff,0,0);
    }

    if (!strcmp(name,LAB_RESCLR2))
    {
      vwi->cmhc=prefs.cmhc[1];
      set_clrbuttons(mapchan_wnd,vwi);
    }

/* Toggle the presence of the align buttons. */
    if (!strcmp(name,LAB_Align))
    {
      channelalignbuttons_presence(NULL,NULL,
                    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
    }

/* Reset Align channels */
    if (!strcmp(name,LAB_Reset_Align))
    {
      Set_Led(widget,LAB_Alignled,0x888);
      for (c=1; c<NR_CHANNELS_MAX; c++)
      {
        char tmp[10];
        vwi->hshift[c]=0;
        vwi->vshift[c]=0;
        vwi->tilt[c]=0;

        sprintf(tmp,"h%d",c);
        Set_Adjust(mapchan_wnd,tmp,"%d",vwi->hshift[c]);
        sprintf(tmp,"v%d",c);
        Set_Adjust(mapchan_wnd,tmp,"%d",vwi->vshift[c]);
        sprintf(tmp,"tilt%d",c);
        Set_Entry(mapchan_wnd,tmp,"%d",vwi->tilt[c]);
      }
      Update_Main(mapchan_wnd);
    }

/* Auto Align channels (CHRPT) */
    if (!strcmp(name,LAB_Auto_Align))
    {
      Set_Led(widget,LAB_Alignled,0xf00);
      for (c=1; c<NR_CHANNELS_MAX; c++)
      {
        vwi->hshift[c]=0;
        vwi->vshift[c]=0;
        vwi->tilt[c]=0;
      }

      switch(pci->file_type)
      {
        case FT_CHRPT:
          vwi->hshift[7]=-3;
          vwi->hshift[9]=-3;
          vwi->hshift[10]=-2;
          vwi->tilt[7]=4;
        break;
        case FT_MHRPT1:
         vwi->vshift[3]=-2;
         vwi->vshift[2]=-1;
       break;
        default: break;
      }

      for (c=1; c<NR_CHANNELS_MAX; c++)
      {
        char tmp[10];
        sprintf(tmp,"h%d",c);
        Set_Adjust(mapchan_wnd,tmp,"%d",vwi->hshift[c]);
        sprintf(tmp,"v%d",c);
        Set_Adjust(mapchan_wnd,tmp,"%d",vwi->vshift[c]);
        sprintf(tmp,"tilt%d",c);
        Set_Entry(mapchan_wnd,tmp,"%d",vwi->tilt[c]);
      }

      Update_Main(mapchan_wnd);
    }
  }

  for (c=1; c<NR_CHANNELS_MAX; c++)
  {
    char tmp[10];
    sprintf(tmp,"r%d",c);
    Sense_Button(mapchan_wnd,tmp,vwi->channels_selected[c]);
    sprintf(tmp,"g%d",c);
    Sense_Button(mapchan_wnd,tmp,vwi->channels_selected[c]);
    sprintf(tmp,"b%d",c);
    Sense_Button(mapchan_wnd,tmp,vwi->channels_selected[c]);
  }
}

/*************************************
 * Create channel map window
 * Activated by menu: View->map_chan
 *************************************/

/* Macro to add or not add buttons */
#define SPIN_E(c)   (pci->channels_recorded[c]? SPIN   : NSPIN)
#define TOGGLE_E(c) (pci->channels_recorded[c]? TOGGLE : NTOGGLE)

/* Formats in spin-buttons */
#define FRMT_RGB "%2.1f%2.1f%2.1f"
#define FRMT_RGBO "%2d%2d%2d"

void Menu_Mapchan(GtkWidget *widget)
{
  GtkWidget *wa1,*wb1,*wb2,*wb3;
  GtkWidget *wa,*wb,*wc[8],*wz;
//  GtkWidget *map_window;
  GtkWidget *main_window;
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

  main_window=Find_Parent_Window(widget);
  if (!(map_window=Create_Window(main_window,0,0,Lab_Mapclrwnd,NULL))) return;

/* Make window auto-shrink. */
  gtk_window_set_policy((GtkWindow *)map_window,FALSE,FALSE,TRUE);

  wa1=Create_ButtonArray("colors",mapchan_cmds,11,
                           LABEL,"",
                           TOGGLE_E(1),"Ch1",vwi->channels_selected[1],
                           TOGGLE_E(2),"Ch2",vwi->channels_selected[2],
                           TOGGLE_E(3),"Ch3",vwi->channels_selected[3],
                           TOGGLE_E(4),"Ch4",vwi->channels_selected[4],
                           TOGGLE_E(5),"Ch5",vwi->channels_selected[5],
                           TOGGLE_E(6),"Ch6",vwi->channels_selected[6],
                           TOGGLE_E(7),"Ch7",vwi->channels_selected[7],
                           TOGGLE_E(8),"Ch8",vwi->channels_selected[8],
                           TOGGLE_E(9),"Ch9",vwi->channels_selected[9],
                           TOGGLE_E(10),"Ch10",vwi->channels_selected[10],

                           SPIN,"ro",FRMT_RGBO,vwi->cmhc.red_offset,-255,255,
                           SPIN_E(1),"r1",FRMT_RGB,vwi->cmhc.red[1],-1.,9.,
                           SPIN_E(2),"r2",FRMT_RGB,vwi->cmhc.red[2],-1.,9.,
                           SPIN_E(3),"r3",FRMT_RGB,vwi->cmhc.red[3],-1.,9.,
                           SPIN_E(4),"r4",FRMT_RGB,vwi->cmhc.red[4],-1.,9.,
                           SPIN_E(5),"r5",FRMT_RGB,vwi->cmhc.red[5],-1.,9.,
                           SPIN_E(6),"r6",FRMT_RGB,vwi->cmhc.red[6],-1.,9.,
                           SPIN_E(7),"r7",FRMT_RGB,vwi->cmhc.red[7],-1.,9.,
                           SPIN_E(8),"r8",FRMT_RGB,vwi->cmhc.red[8],-1.,9.,
                           SPIN_E(9),"r9",FRMT_RGB,vwi->cmhc.red[9],-1.,9.,
                           SPIN_E(10),"r10",FRMT_RGB,vwi->cmhc.red[10],-1.,9.,
                           SPIN,"go",FRMT_RGBO,vwi->cmhc.green_offset,-255,255,
                           SPIN_E(1),"g1",FRMT_RGB,vwi->cmhc.green[1],-1.,9.,
                           SPIN_E(2),"g2",FRMT_RGB,vwi->cmhc.green[2],-1.,9.,
                           SPIN_E(3),"g3",FRMT_RGB,vwi->cmhc.green[3],-1.,9.,
                           SPIN_E(4),"g4",FRMT_RGB,vwi->cmhc.green[4],-1.,9.,
                           SPIN_E(5),"g5",FRMT_RGB,vwi->cmhc.green[5],-1.,9.,
                           SPIN_E(6),"g6",FRMT_RGB,vwi->cmhc.green[6],-1.,9.,
                           SPIN_E(7),"g7",FRMT_RGB,vwi->cmhc.green[7],-1.,9.,
                           SPIN_E(8),"g8",FRMT_RGB,vwi->cmhc.green[8],-1.,9.,
                           SPIN_E(9),"g9",FRMT_RGB,vwi->cmhc.green[9],-1.,9.,
                           SPIN_E(10),"g10",FRMT_RGB,vwi->cmhc.green[10],-1.,9.,
                           SPIN,"bo",FRMT_RGBO,vwi->cmhc.blue_offset,-255,255,
                           SPIN_E(1),"b1",FRMT_RGB,vwi->cmhc.blue[1],-1.,9.,
                           SPIN_E(2),"b2",FRMT_RGB,vwi->cmhc.blue[2],-1.,9.,
                           SPIN_E(3),"b3",FRMT_RGB,vwi->cmhc.blue[3],-1.,9.,
                           SPIN_E(4),"b4",FRMT_RGB,vwi->cmhc.blue[4],-1.,9.,
                           SPIN_E(5),"b5",FRMT_RGB,vwi->cmhc.blue[5],-1.,9.,
                           SPIN_E(6),"b6",FRMT_RGB,vwi->cmhc.blue[6],-1.,9.,
                           SPIN_E(7),"b7",FRMT_RGB,vwi->cmhc.blue[7],-1.,9.,
                           SPIN_E(8),"b8",FRMT_RGB,vwi->cmhc.blue[8],-1.,9.,
                           SPIN_E(9),"b9",FRMT_RGB,vwi->cmhc.blue[9],-1.,9.,
                           SPIN_E(10),"b10",FRMT_RGB,vwi->cmhc.blue[10],-1.,9.,
                           NULL);


  wb1=Create_ButtonArray("HShift",mapchan_cmds,10,
                           SPIN_E(1),"h1",FRMT_RGBO,vwi->hshift[1],-20,20,
                           SPIN_E(2),"h2",FRMT_RGBO,vwi->hshift[2],-20,20,
                           SPIN_E(3),"h3",FRMT_RGBO,vwi->hshift[3],-20,20,
                           SPIN_E(4),"h4",FRMT_RGBO,vwi->hshift[4],-20,20,
                           SPIN_E(5),"h5",FRMT_RGBO,vwi->hshift[5],-20,20,
                           SPIN_E(6),"h6",FRMT_RGBO,vwi->hshift[6],-20,20,
                           SPIN_E(7),"h7",FRMT_RGBO,vwi->hshift[7],-20,20,
                           SPIN_E(8),"h8",FRMT_RGBO,vwi->hshift[8],-20,20,
                           SPIN_E(9),"h9",FRMT_RGBO,vwi->hshift[9],-20,20,
                           SPIN_E(10),"h10",FRMT_RGBO,vwi->hshift[10],-20,20,
                           NULL);

  wb2=Create_ButtonArray("VShift",mapchan_cmds,10,
                           SPIN_E(1),"v1",FRMT_RGBO,vwi->vshift[1],-20,20,
                           SPIN_E(2),"v2",FRMT_RGBO,vwi->vshift[2],-20,20,
                           SPIN_E(3),"v3",FRMT_RGBO,vwi->vshift[3],-20,20,
                           SPIN_E(4),"v4",FRMT_RGBO,vwi->vshift[4],-20,20,
                           SPIN_E(5),"v5",FRMT_RGBO,vwi->vshift[5],-20,20,
                           SPIN_E(6),"v6",FRMT_RGBO,vwi->vshift[6],-20,20,
                           SPIN_E(7),"v7",FRMT_RGBO,vwi->vshift[7],-20,20,
                           SPIN_E(8),"v8",FRMT_RGBO,vwi->vshift[8],-20,20,
                           SPIN_E(9),"v9",FRMT_RGBO,vwi->vshift[9],-20,20,
                           SPIN_E(10),"v10",FRMT_RGBO,vwi->vshift[10],-20,20,
                           NULL);

  wb3=Create_ButtonArray("TILT",mapchan_cmds,10,
                    SPIN_E(1),"tilt1",FRMT_RGBO,vwi->tilt[1],-10,10,
                    SPIN_E(2),"tilt2",FRMT_RGBO,vwi->tilt[2],-10,10,
                    SPIN_E(3),"tilt3",FRMT_RGBO,vwi->tilt[3],-10,10,
                    SPIN_E(4),"tilt4",FRMT_RGBO,vwi->tilt[4],-10,10,
                    SPIN_E(5),"tilt5",FRMT_RGBO,vwi->tilt[5],-10,10,
                    SPIN_E(6),"tilt6",FRMT_RGBO,vwi->tilt[6],-10,10,
                    SPIN_E(7),"tilt7",FRMT_RGBO,vwi->tilt[7],-10,10,
                    SPIN_E(8),"tilt8",FRMT_RGBO,vwi->tilt[8],-10,10,
                    SPIN_E(9),"tilt9",FRMT_RGBO,vwi->tilt[9],-10,10,
                    SPIN_E(10),"tilt10",FRMT_RGBO,vwi->tilt[10],-10,10,
                           NULL);


  wc[1]=Create_Button(LAB_Redraw,mapchan_cmds);
  wc[2]=Create_Button(LAB_Color,mapchan_cmds);
  wc[3]=Create_ButtonArray("save/restore clrmap",mapchan_cmds,2,
                            BUTTON,LAB_SAVCLR1,
                            BUTTON,LAB_RESCLR1,
                            BUTTON,LAB_SAVCLR2,
                            BUTTON,LAB_RESCLR2,
                            0
                          );
  wc[2]=Pack(NULL,'h',wc[2],1,wc[3],1,NULL);
  wc[3]=Create_Button(LAB_Auto_Align,mapchan_cmds);
  wc[4]=Create_Led(LAB_Alignled,0x888);
  wc[5]=Create_Button(LAB_Reset_Align,mapchan_cmds);
  wc[6]=Create_Toggle(LAB_Align,mapchan_cmds,FALSE);

  wc[7]=Create_Button(LAB_Exit,mapchan_cmds);
  wa=Pack("",'v',wa1,10,NULL);
  wb=Pack("",'v',wb1,10,wb2,10,wb3,10,NULL);
  wc[3]=Pack("Align",'h',wc[3],1,wc[4],10,wc[5],1,NULL);
  wc[0]=Pack("",'h',wc[1],10,wc[2],10,wc[3],10,wc[6],1,wc[7],10,NULL);
  wz=Pack("",'v',wa,10,wb,10,wc[0],10,NULL);
  gtk_container_add(GTK_CONTAINER(map_window),wz);

  if (pci->file_type!=FT_CHRPT)
  {
    gtk_widget_hide(wc[3]);
    gtk_widget_hide(wc[6]);  
  }
  channelalignbuttons_presence(map_window,wb,FALSE);
  gtk_widget_show(map_window);
  place_window(map_window,prefs.sub_wnd_x,prefs.sub_wnd_y,prefs.sub_wnd_geom);

  // Set to 'false colour' (this will make  vwi->process_data=TRUE)
  Set_Button(main_window,MENU_FC,TRUE);

/* Disable channel choice buttons */
  Sense_Button(main_window,"Channel",FALSE);
  Update_channel_save(main_window,pci,vwi);
  Reset_Mapchanclr(-1);
}


/*************************************
 * Add/remove channel align buttons.
 * Must be called after creating window, to define 
 * internal static widgets.
 * This call will also make align-buttons hidden, 
 * which is the desired default.
 * If arguments are "NULL,NULL" the internal statics will be used.
 * (Needed if function is called at a place where these widgets 
 * are not available.)
 *************************************/
void channelalignbuttons_presence(GtkWidget *imap_window,
                                  GtkWidget *iw_align,
                                  gboolean active)
{
  static GtkWidget *w_align,*map_window;

/* Args are non-zero; save them into static widgets. */
  if (iw_align) w_align=iw_align;
  if (imap_window) map_window=imap_window;

/* Make align biuttons "visible" or "hide". */
  if (active)
  {
    gtk_widget_show (w_align);
  }
  else
  {
    gtk_widget_hide (w_align);
  }
}
