/*****************************************************
 * RCSId: $Id: fy3_read.c,v 1.5 2017/09/17 13:25:17 ralblas Exp $
 *
 * Header file containing function defs
 * Project: WSAT
 * Author: R. Alblas
 * Content:
 *   int read_fy31_hdr(PICINFO *pci)
 *   void read_fy3_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *l)
 *   int fy3frame2line(guint16 *frame,guint16 *line,gboolean destrip)
 *
 * History: 
 * $Log: fy3_read.c,v $
 * Revision 1.5  2017/09/17 13:25:17  ralblas
 * _
 *
 * Revision 1.4  2017/07/03 18:34:52  ralblas
 * _
 *
 * Revision 1.3  2017/06/10 21:58:58  ralblas
 * _
 *
 * Revision 1.2  2017/05/30 17:57:34  ralblas
 * _
 *
 * Revision 1.1  2017/05/28 15:01:56  ralblas
 * Initial revision
 *
 *
 *****************************************************/
/*******************************************************************
 * Copyright (C) 2000 WSAT. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/


#include "wsat.h"
#include "metop.h"
#include "hrpt.h"
#define LENPCKT 30000             // max. packet length > 25600
#define SYNCWORD 0x1acffc1d       // sync word AHRPT
#define MARK_OUTSYNC 0
#define INNER_SYNC 0x845bf5
// Databytes in frame: frame[14] ... frame[895]
#define DATA_BYTES_FRAME 882
#define DATA_BYTES_FRM_STRT 14
#define DATA_BYTES_FRM_END DATA_BYTES_FRM_STRT+DATA_BYTES_FRAME-1

#define is_metop(id) (!(id&0xf0? 1 : 0))
#define unsegmented(vcdu) (vcdu.ccsds.hdr.seq_flg==3)
#define apid_is_avhrr(vcdu) ((vcdu.ccsds.hdr.apid==103) || (vcdu.ccsds.hdr.apid==104))
#define vcid_is_avhrr_hr(vcdu) (vcdu.vc_id==0x9)

//#define vcid_is_fy_vc1(vcdu)   (vcdu.vc_id==0x3)   // MERSI
//#define vcid_is_fy_vc2(vcdu)   (vcdu.vc_id==0x5)   // VIRR day
//#define vcid_is_fy_vc3(vcdu)   (vcdu.vc_id==0x9)   // VIRR night
//#define vcid_is_fy_vc4(vcdu)   (vcdu.vc_id==0xa)   // VIRR night
//#define vcid_is_fy_vc5(vcdu)   (vcdu.vc_id==0xc)   // VIRR night

typedef enum vcid
{
  mtp_avhrr       = 0x09,
  fy3_mersi       = 0x13,
  fy3_virr_day    = 0x15,
  fy3_virr_night1 = 0x19,
  fy3_virr_night2 = 0x1a,
  fy3_virr_night3 = 0x1c
} VCID;

// store ccsds prim. header
static void read_ccsds(VCDU *vcdu)
{
  CCSDS_PRIM_HDR *hdr=&vcdu->ccsds.hdr;
  CCSDS_DATA_FLD1 *sec_hdr=&vcdu->ccsds.ccsds_data_fld;
  unsigned char *ccsds_data=vcdu->m_pdu.data+vcdu->m_pdu.fhp;
  hdr->vers       = (ccsds_data[0]&0xe0)>>5;
  hdr->type       = (ccsds_data[0]&0x10)>>4;
  hdr->sec_hdrflg = (ccsds_data[0]&0x08)>>3;
  hdr->apid       =((ccsds_data[0]&0x07)<<8)+ccsds_data[1];
  hdr->seq_flg    = (ccsds_data[2]&0xc0)>>6;
  hdr->pckt_seqcnt=((ccsds_data[2]&0x3f)<<8)+ccsds_data[3];
  hdr->pckt_len   =((ccsds_data[4])<<8)+ccsds_data[5];

  sec_hdr->sec_hdr_day  = ((ccsds_data[6])<<8)+ccsds_data[7];
  sec_hdr->sec_hdr_msec = ((ccsds_data[8])<<24)+((ccsds_data[9])<<16)+((ccsds_data[10])<<8)+ccsds_data[11];
  sec_hdr->sec_hdr_usec = ((ccsds_data[12])<<8)+ccsds_data[13];
}

static void read_vcduhdr(VCDU *vcdu,unsigned char *cvcdu)
{
  vcdu->version = (cvcdu[0]>>6)&0x03;
  vcdu->sc_id   = ((cvcdu[0]<<2)&0xfc) + ((cvcdu[1]>>6)&0x3);
  vcdu->vc_id   = (cvcdu[1]&0x3f);
  vcdu->cnt     = (cvcdu[2]<<16) + (cvcdu[3]<<8) + cvcdu[4];
  vcdu->replay  = (cvcdu[5]>>7)&0x01;
//vcdu->insert  = (cvcdu[6]<<8) + cvcdu[7];
  vcdu->data    = cvcdu+8;  // pos. 4+8=12: start M-PDU header

  vcdu->m_pdu.fhp=((vcdu->data[0]&0x7)<<8)+vcdu->data[1];  // ptr start 1e M_SDU
  vcdu->m_pdu.data=vcdu->data+2;               // start 882-bytes part
}

static int search_inner_sync(unsigned char *frame)
{
  int n,m;
  static unsigned int s=0;
  int is_pos=0;
  for (n=DATA_BYTES_FRM_STRT; n<=DATA_BYTES_FRM_END; n++)
  {
    for (m=7; m>=0; m--)
    {
      s=(s<<1)+(frame[n]&(1<<m)? 1 : 0);
      if ((s&0xffffff)==INNER_SYNC)
      {
        is_pos= (n-2)*8-m;
        break;
      }
    }
    if (is_pos) break;
  }
  if ((is_pos) && (is_pos<DATA_BYTES_FRM_STRT*8)) is_pos+=(DATA_BYTES_FRAME*8);
  return is_pos;
}

static int extr_word(char *ip,int bitshift)
{
  static char *p;
  static int b;
  int word,n;
  if (ip) { p=ip; b=7-bitshift; return 0; }
  word=0;
  for (n=0; n<10; n++)
  {
    word<<=1; word+=((*p)>>b)&1; b--;
    if (b<0) { p++; b=7; }
  }
  return word;
}

#define DO_NOAAFORMAT TRUE
#define WIDTH 2048
// Translate pixs into line.
// For the moment just pickup pixel-data.
// Start-of-data in returned 'line' at 1600 (is that OK?).
// bit-shift done using bitshft, according to shift inner-sync
static void pckt2line(CCSDS *ccsds,CCSDS_APPL_DATA *appl,
                      int bitshft,char *pckt,guint16 *line)
{
  gboolean use_noaa_format=DO_NOAAFORMAT;
  int i,j;
  int pixoffset=0;
  int hrptsync[]={0x284,0x16f,0x35c,0x19d,0x20f,0x095};

 // VIRR: pckt[22] ... pckt[...]
  if (!line) return;
  if (use_noaa_format) pixoffset=1600;

  extr_word(pckt+22+400,bitshft); // 400: start OK

  // 204800 bits detector earth view
  for (i=0; i<WIDTH; i++)
  {
    for (j=0; j<10; j++)
    {
      line[pixoffset+i*10+j]=extr_word(NULL,0);
    }
  }

  if (use_noaa_format)
  {
    for (i=0; i<6; i++) line[i]=hrptsync[i];
    // add converted calibration data
  }
}


// read info frame, detect if avhrr, and if first/last part
// return: 1: start/end avhrr packet part
//         0: in-betweeen avhrr packet part
//        -1: no AVHRR
static int get_frame_id(unsigned char *frame,int *cnt,VCID vcid)
{
  VCDU vcdu;
  int eol=-1;
  read_vcduhdr(&vcdu,(unsigned char *)(frame+4));

  if (vcdu.vc_id == (vcid&0xf))
  {
    *cnt=vcdu.cnt;
    if (vcdu.m_pdu.fhp < DATA_BYTES_FRAME)
    {
      read_ccsds(&vcdu);
      if (((apid_is_avhrr(vcdu)) && (unsegmented(vcdu))) || (!is_metop(vcid)))
      {
        eol=1;
      }
    }
    else if (is_metop(vcid))
    {
      eol=0;
    }
    else
    {
      int n;
      if ((n=search_inner_sync(frame)))
      {
        eol=n/8;
      }
    }
  }
  return eol;
}

// open new file: create table to convert y to position in file
static int fill_tbl(FILE *fp,int *tbl[2],int lentbl,VCID vcid)
{
  int curfrm;
  int height;
  int pcktcnt;
  unsigned char frame[1024];
  int eol;
  fseek(fp,0,SEEK_SET);
  curfrm=0;
  height=0;
  while (fread(frame,1024,1,fp))
  {
    if ((eol=get_frame_id(frame,&pcktcnt,vcid))>0)
    {
      if (height<lentbl)
      {
        tbl[0][height]=curfrm;
        if (1)
        {
          int sync1=(frame[0]<<24)+(frame[1]<<16)+(frame[2]<<8)+frame[3];
          unsigned int syncerr,i;
          int nrerr=0;
          syncerr=sync1^SYNCWORD;
          for (i=0; i<32; i++) if ((1<<i)&syncerr) nrerr++;
          tbl[1][height]=nrerr;
        }
      }
      height++;
    }
    curfrm++;
  }
  return height;
}

// table: create or read
static int manage_tbl(FILE *fp,int y,int *oerr,VCID vcid)
{
  static int *tbl[2];
  static int height;
  int nrfrm;

  if (fp)
  {
    if (tbl[0]) free(tbl[0]);
    if (tbl[1]) free(tbl[1]);
    fseek(fp,0,SEEK_END);
    nrfrm=ftell(fp)/1024;
    tbl[0]=calloc(nrfrm,sizeof(tbl[0]));
    tbl[1]=calloc(nrfrm,sizeof(tbl[1]));

    height=fill_tbl(fp,tbl,nrfrm,vcid);

    return height;
  }
  else
  {
    if (tbl[0]) return tbl[0][y];
  }
  return 0;
}

// create new table
static int create_tbl(FILE *fp,VCID vcid)
{
  return manage_tbl(fp,0,NULL,vcid);
}

// convert y (AVHRR) to frame number
static int get_framepos(int y,int *oerr)
{
  return manage_tbl(NULL,y,oerr,0);
}
 
void pri_vcid(FILE *fp)
{
  unsigned char frame[1024];
  VCDU vcdu;
  int idtbl[20],i,ok=0;
  int scid,p_scid;
  if (!fp) return;
  rewind(fp);
  memset(idtbl,0,sizeof(idtbl));
  while (fread(frame,1024,1,fp))
  {
    read_vcduhdr(&vcdu,(frame+4));
    if (vcdu.vc_id < 20) idtbl[vcdu.vc_id]++;
    if (p_scid==vcdu.sc_id)
    {
      ok++;
      if (ok>10) scid=vcdu.sc_id;
    }
    else
    {
      ok=0;
    }
    p_scid=vcdu.sc_id ;
      
  }
  printf("scid=%d\n",scid);
  for (i=0; i<20; i++)
  {
    if (idtbl[i])
    {
      printf("VCID %2d: %5d frames",i,idtbl[i]);
      if (scid==12)
      {
        if (i==9) printf("  (AVHRR)");
      }
      if (scid==50)
      {
        if (i==3) printf("  (MERSI)");
        if (i==5) printf("  (VIRR day)");
        if (i==9) printf("  (VIRR night)");
      }
      printf("\n");
    }
  }
}

// fhp=position inner-sync, rounded down
// pntr=pos. where to start (starts at 0)
// pckt: gets data-part frame at new position
// m_pdu_data points to start-of-datapart in frame
// return: incremented pointer (points to next pos. to add new data)
int copy_packet(int fhp,int pntr,char *pckt,unsigned char *m_pdu_data)
{
  int nradd=fhp;
  if (pntr+nradd > LENPCKT) nradd=LENPCKT-pntr;
  if ((pntr>=0) && (pntr+nradd<LENPCKT))
  {
    if (nradd>0) memcpy(pckt+pntr,m_pdu_data,nradd);
  }
  pntr+=nradd;
  return pntr;
}

// convert frame to avhrr 10 chan line
//  destrip=TRUE: try correction if missed frame
//  (frame in bytes)
// Choose which data; for the moment fixed to VIRR day (VCID=5).
//
// Data part in frame: 14...895 (counting from 0)
// Start-of-data: after inner_sync or at pos. 14 of frame (vcdu.m_pdu.data)
// End-of-data in frame = 14+882-1=895
static int fy3_frame2line(unsigned char *frame,guint16 *line,gboolean destrip)
{
  static char *pckt;
  static int pntr;
  int pcktlen=0;
  int eol=0;
  static int prevpcktcnt;
  VCDU vcdu;
VCID vcid=0x5; // VIRR day
  if (!pckt) pckt=malloc(LENPCKT);

  read_vcduhdr(&vcdu,(unsigned char *)(frame+4));

  if (vcdu.vc_id == (vcid&0xf))
  {
    int fhp,bitshft;
    if ((fhp=search_inner_sync(frame))) // fhp=pos. first bit(!) inner-sync
    { // inner sync in this frame
      // First complete last part of previous line
      bitshft=fhp%8;
      fhp/=8;                      // to byte-position
      prevpcktcnt=vcdu.cnt;

      pntr=copy_packet(fhp,pntr,pckt,vcdu.m_pdu.data);

      read_ccsds(&vcdu);
      pckt2line(&vcdu.ccsds,&vcdu.ccsds.ccsds_appl_data,bitshft,pckt,line);

      // Now start with new line.
      eol=1;
      pntr=0;
      pcktlen=DATA_BYTES_FRAME-fhp;
      pntr=copy_packet(pcktlen,pntr,pckt,vcdu.m_pdu.data+fhp);
    }
    else
    {
      if (destrip)
      {
        pntr+=(DATA_BYTES_FRAME*(vcdu.cnt-prevpcktcnt-1));
        prevpcktcnt=vcdu.cnt;
      }

      pntr=copy_packet(DATA_BYTES_FRAME,pntr,pckt,vcdu.m_pdu.data);
    }
  }

  return eol;
}

// allocated: pci->fri.words_per_line
// Read one line.
// Used in read_line()
void read_fy3_line(PICINFO *pci,VIEWINFO *vwi,int y,guint16 *l)
{
  char unsigned frame[1024];
  int nrfrm;
  int nrerr,rdy;
  if (!pci->fp) return;
  nrfrm=get_framepos(y,&nrerr);
  if ((y==0) || (vwi->zval.dy!=1.))
  {
    fseek(pci->fp,nrfrm*1024,SEEK_SET); // seek to about position start line
    // goto start-of-frame; mtp_avhrr_frame2line returns 0: ready
    do
    {
      if (!(fread(frame,1024,1,pci->fp))) break;

    } while (!fy3_frame2line(frame,l,vwi->destrip));
  }

  // catch actual line
  do
  {
    if (!(fread(frame,1024,1,pci->fp))) break;
    if (vwi->reed_solomon) rs_decode_frm(frame);
    rdy=fy3_frame2line(frame,l,vwi->destrip);
  } while (!rdy);

  if (nrerr)
  {
  }
}
// read FY header; set parameters; determine # lines
// Used in open_file()
int read_fy31_hdr(PICINFO *pci)
{
  int i;
  gboolean use_noaa_format=DO_NOAAFORMAT;
  pci->width=2048;
  pci->height=create_tbl(pci->fp,fy3_virr_day);
  pci->depth=10;
  pci->clrmap.size=1024;
  for (i=0; i<pci->clrmap.size; i++)
  {
    pci->clrmap.red[i]=i;
    pci->clrmap.green[i]=i;
    pci->clrmap.blue[i]=i;
  }
  for (i=0; i<NR_CHANNELS_MAX; i++) pci->channels_recorded[i]=FALSE;
  for (i=1; i<=10; i++)
    pci->channels_recorded[i]=TRUE;
  pci->width_original=2048;
  pci->max_sens_angle=DEF_NOAA_ALPHAMAX;
  pci->sat_type=FY3;

  if (use_noaa_format)
  {
    pci->channels_recorded[0]=TRUE;
    pci->fri.words_per_line=2048*10 + 1600;
    pci->fri.header_len=1600;
  }
  else
  {
    pci->fri.words_per_line=2048*10;
  }
  pci->fri.bytes_per_line=2048*10*10/8;
  pci->fri.inter_chan=10;
  strcpy(pci->sat_name,"Fengyun-3");
  return 0;
}

// convert frame to avhrr line
// Used for 'live' picture during record.
// (frame in guint16)
//  destrip=TRUE: try correction if missed frame
int fy3frame2line(guint16 *frame,guint16 *line,gboolean destrip)
{
  unsigned char bframe[1024];
  int i;
  for (i=0; i<1024; i++) bframe[i]=frame[i]&0xff;
  return fy3_frame2line(bframe,line,destrip);
}
