#include <stdio.h>
#include "reedsol.h"


//---------------------------------------------------------------------------
// Do the funky Reed-Solomon decode thing.
// Use offset=4 to skip ASM data if it is left in the datastream.
// Use mode==1 to select Meteor style R-S. Mode=0 is for LRIT.
// The RS function returns both the number of errors it corrected
// and the number of erros it found but could not correct. These
// values are returned as byte zero and byte one of the return integer.
//---------------------------------------------------------------------------
//
static int Decode_RS(unsigned char linebuf[], int offset, int mode)
{
   int i, x;
   int ret;
   int block;
   unsigned char rs_buf[255];


   // Initialise the return value to zero.
   ret = 0;

   //-- Reed Solomon 255,223 i=4 --//
   // 255, Code block (the unit of data that is processed) is 255 bytes.
   // 223 means 223 byte of data and 32 bytes of R-S symbols at end.
   // i=4 the interleaving depth is 4. Code block bytes are spread out
   // evenly throughout the whole line.

   // Run the RS decoder on each the 4 codeblocks in the 1020 byte line.
   for(block=0;block<4;block++) {

      // Build a RS 255 byte codeblock from CADU.
      for(i=0;i<255;i++) {
         rs_buf[i] = linebuf[4*i+block+offset];
      }

      // Perform the RS decode - ca the Phil Karn routine.
      if(mode) x = decode_rs_8(rs_buf, NULL, 0, 0);   // Meteor LRPT
      else x = decode_rs_ccsds(rs_buf, NULL, 0, 0);   // Mtsat LRIT

      // Count corrected bytes for information/statistics etc.
      if(x>0) ret = ret + x;

      // Count uncorrectable blocks for stats.
      if(x<0) ret = ret + 256;

      // Re-insert the corrected data back into main buffer.
      for(i=0;i<223;i++) {
         linebuf[4*i+block+offset] = rs_buf[i];
      }

   }

   // Return the error performance stats.
   return ret;
}

void rs_decode_frm(unsigned char *data)
{
  Decode_RS(data, 4, 0);
}

#include <gtk/gtk.h>
void rs_decode_frm2(guint16 *data)
{
  int i;
  unsigned char frm[1024];
  for (i=0; i<1024; i++) frm[i]=data[i];
  Decode_RS(frm, 4, 0);
  for (i=0; i<1024; i++) data[i]=frm[i];
}

