/**************************************************
 * RCSId: $Id: mouse_actions.c,v 1.1 2011/10/24 21:37:43 ralblas Exp $
 *
 * Mouse actions.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: mouse_actions.c,v $
 * Revision 1.1  2011/10/24 21:37:43  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include "wsat.h"



/* Translate window position to picture position */
#define W2P_X(wx) (wx*pci->width/drawing_area->allocation.width)
#define W2P_Y(wy) (wy*pci->height/drawing_area->allocation.height)

void swap(int *a,int *b)
{
  int t;
  t=*a; *a=*b; *b=t;
}

void redraw_zoomrect(GtkWidget *drawing_area,GdkGC **gc,guchar *rgbbuf,
                int px1,int py1,int px2,int py2,
                int  x1,int  y1,int  x2,int  y2)
{
  if (px1>px2) swap(&px1,&px2); 
  if (py1>py2) swap(&py1,&py2); 
  if (x1>x2) swap(&x1,&x2); 
  if (y1>y2) swap(&y1,&y2); 

  px1=MAX(px1,0);
  py1=MAX(py1,0);
  px2=MIN(px2,drawing_area->allocation.width-1);
  py2=MIN(py2,drawing_area->allocation.height-1);
  x1=MAX(x1,0);
  y1=MAX(y1,0);
  x2=MIN(x2,drawing_area->allocation.width-1);
  y2=MIN(y2,drawing_area->allocation.height-1);
  draw_rgbmap(drawing_area,rgbbuf,px1,py1,px2-px1+1,1);
  draw_rgbmap(drawing_area,rgbbuf,px1,py1,1        ,py2-py1+1);
  draw_rgbmap(drawing_area,rgbbuf,px2,py1,1        ,py2-py1+1);
  draw_rgbmap(drawing_area,rgbbuf,px1,py2,px2-px1+1,1);

/* Draw new rectangle */
  gdk_draw_rectangle(drawing_area->window,
           gc[4],FALSE,
           x1,y1,x2-x1,y2-y1);
}

/*************************************
 * Handle mouse actions
 * Connected to canvas 
 *************************************/
void mouse_actions(GtkWidget *widget, GdkEventMotion *event)
{
  GtkWidget *main_window=NULL;
  RGBI *rgbi=NULL;
  guchar *rgbbuf=NULL;
  GdkGC **gc=NULL;
  PICINFO *pci=NULL;
  VIEWINFO *vwi=NULL;

  static int zoom_state;
  static gboolean button1_pressed;

  int x,y;
  static int xstrt,ystrt;
  GdkModifierType state;
  GtkWidget *drawing_area=NULL;
  static int xp,yp;

  int xminp=MIN(xstrt,xp);
  int yminp=MIN(ystrt,yp);
  int dx=ABS(xstrt-xp);
  int dy=ABS(ystrt-yp);

  int zmrctsizex;
  int zmrctsizey;

  if (!(GTK_IS_WIDGET(widget))) return;  /* no mouseaction to handle... */


  main_window=gtk_widget_get_toplevel(widget);
  drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  rgbi=Get_RGBI(drawing_area);
  gc=Get_Gc(main_window);
  pci=Get_Picinfo_db(widget);
  vwi=Get_Viewinfo_db(widget);
  if (rgbi) rgbbuf=rgbi->rgbbuf;

  zmrctsizex=drawing_area->allocation.width/(vwi->fix_zoomfactor);
  zmrctsizey=drawing_area->allocation.height/(vwi->fix_zoomfactor);
/* Get mouse position */
  if (event->is_hint)
  {
    gdk_window_get_pointer(event->window,&x,&y,&state);
  }
  else
  {
    x=event->x;
    y=event->y;
    state=event->state;
  }
  Update_Coord(main_window,x,y,pci,vwi);

/* If mouse button 1 pressed */
  if (state & GDK_BUTTON1_MASK)
  {
    if (!button1_pressed)
    {
      button1_pressed=TRUE;

/* Handle mouse-zoom */
      if (vwi->zooming)
      {
        switch(zoom_state)
        {
          case 0:
            zoom_state=1;
            xstrt=x;
            ystrt=y;
          break;
          case 1:
            Create_Cursor(widget,GDK_TOP_LEFT_ARROW);
            zoom_state=2;
          break;
          default:
            zoom_state=0;
          break;
        }
      }

      if (vwi->fixzooming)
      {
        Create_Cursor(widget,GDK_TOP_LEFT_ARROW);
        vwi->fixzooming=FALSE;
        zoom_state=2;
        x=xstrt+zmrctsizex; y=ystrt+zmrctsizey;
      }
    }
  }
  else
    button1_pressed=FALSE;

/* Cancel zooming */
  if (state & GDK_BUTTON3_MASK)
  {
/* cancel zoom */
    zoom_state=0;
    vwi->zooming=FALSE;
    vwi->fixzooming=FALSE;
    Create_Cursor(widget,GDK_TOP_LEFT_ARROW);

/* Redraw to remove old rectangle */
    draw_rgbmap(drawing_area,rgbbuf,
                0,0,
                drawing_area->allocation.width,
                drawing_area->allocation.height);

  }

  switch(zoom_state)
  {
/* State 0: No zoom */
    case 0:
    break;

/* State 1: 1 point defined */
    case 1:
      redraw_zoomrect(drawing_area,gc,rgbbuf,
                      xminp,yminp, xminp+dx, yminp+dy,
                      xstrt,ystrt,x,y);
/* Save this rect position */
      xp=x;
      yp=y;
    break;
/* State 2: 2 points defined; redraw */
    case 2:
    {
      int xmin,ymin;
        
/* Stop zooming */
      vwi->zooming=FALSE;
      zoom_state=0;

      if (vwi->scan_hor==E_W)
        xmin=drawing_area->allocation.width-MAX(xstrt,x);
      else
        xmin=MIN(xstrt,x);

      if (vwi->scan_ver==S_N)
        ymin=drawing_area->allocation.height-MAX(ystrt,y);
      else
        ymin=MIN(ystrt,y);


/* Determine new zoom factors */
      vwi->zval.ox+=(W2P_X(xmin)/vwi->zval.zx+1);
      vwi->zval.oy+=(W2P_Y(ymin)/vwi->zval.zy+1);

      vwi->zval.zx*=ABS(drawing_area->allocation.width/(float)(x-xstrt));
      vwi->zval.zy*=ABS(drawing_area->allocation.height/(float)(y-ystrt));

      correct_zoomval(drawing_area,vwi,pci);

/* Update window and zoom buttons */
      Update_Main(widget,pci,vwi);
      Update_Zoom(main_window);
    }
    break;

  }

  if (vwi->zooming)
  {
    if (zoom_state==0)
    {
      Create_Cursor(widget,GDK_PLUS);
    }
  }
  if (vwi->fixzooming)
  {
    if (zoom_state==0)
    {
      Create_Cursor(widget,GDK_PLUS);
      zoom_state=11;
    }
/* Remove old rectangle by redrawing the pixels under the rectangle */
    dx=zmrctsizex; dy=zmrctsizey;
    xstrt=x-dx/2;
    ystrt=y-dy/2;
    xp=xstrt; yp=ystrt;
    redraw_zoomrect(drawing_area,gc,rgbbuf,
                      xminp,yminp, xminp+dx, yminp+dy,
                      xstrt,ystrt,xstrt+dx,ystrt+dy);
  }
}

/*************************************
 * Handle scrollbar actions
 *  
 *************************************/
void scroll_actions(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *drawing_area=Find_Widget(widget,"GTK_DRAWING_AREA");
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);

  char *name=(char *)data;
  float value,lower,upper;

  value=GTK_ADJUSTMENT(widget)->value;
  upper=GTK_ADJUSTMENT(widget)->upper;
  lower=GTK_ADJUSTMENT(widget)->lower;

  if (!strcmp(name,"hbar"))
  {
    if (vwi->scan_hor==E_W)
      vwi->zval.ox=(upper-lower)*(1.-1./vwi->zval.zx)-(value-lower);
    else
      vwi->zval.ox=value-lower;
    Update_Zoom(main_window);
  }

  if (!strcmp(name,"vbar"))
  {
    if (vwi->scan_ver==S_N)
      vwi->zval.oy=(upper-lower)*(1.-1./vwi->zval.zy)-(value-lower);
    else
      vwi->zval.oy=value-lower;
    Update_Zoom(main_window);
  }

  correct_zoomval(drawing_area,vwi,pci);
  Update_Main(widget);
}
