/**************************************************
 * RCSId: $Id: prefer.c,v 1.8 2017/07/01 21:49:18 ralblas Exp $
 *
 * Preferences
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: prefer.c,v $
 * Revision 1.8  2017/07/01 21:49:18  ralblas
 * _
 *
 * Revision 1.7  2017/05/07 16:18:54  ralblas
 * _
 *
 * Revision 1.6  2017/02/16 20:05:57  ralblas
 * _
 *
 * Revision 1.5  2017/01/01 15:02:42  ralblas
 * _
 *
 * Revision 1.4  2016/12/09 09:45:18  ralblas
 * _
 *
 * Revision 1.3  2016/12/03 22:42:11  ralblas
 * _
 *
 * Revision 1.2  2016/06/05 10:37:36  ralblas
 * _
 *
 * Revision 1.1  2016/02/15 20:14:37  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "wsat.h"
#include "menu_wsat.h"
#define METOPFRM_PER_LINE 15

/* Define button labels */
#define Lab_Prefwnd "Preferences"
#define LAB_Exit   " Close "
#define LAB_Use    " Use  "
#define LAB_Save   " Save "
#define LAB_LOPREF "Local preference file"
#define LAB_GLPREF "Used preference file"

#define LAB_WNDPOS "^Window positions"
#define LAB_WNDSIZE "Main window size"
#define LAB_WNDPOS_DFLT "Default"
#define LAB_WNDPOS_SMART "Smart"
#define LAB_WND_WIDTH "^Width"
#define LAB_WND_HEIGHT "^Height"

#define LAB_INTERF "Decoder interface"
#define LAB_PAR "Parallel"
#define LAB_ETH "Ethernet"
#define LAB_USB "USB"

#define LAB_USBPORT "USB Port"
#define LAB_USBFAST "USB fast"
#define LAB_USBCHOICE "Use ID or first found:"
#define LAB_TESTUSB "Search FTD devices"
#define LAB_USB_USEFIRST "First found"
#define LAB_USB_USEID "Use this: "

#define LAB_USBID "USB ID"

#define LAB_PARPORT "Parallel port"
#define LAB_P1 "Port 1"
#define LAB_P2 "Port 2"
#define LAB_PN "Port n"
#define LAB_PE "Address Port n"
#define LAB_FASTPPACC "Fast parport access"

#define LAB_SEL_HWTYPE "Select HW:"
#define LAB_SEL_DECGEN "decoder/generator"
#define LAB_SEL_DECTYPE "decoder type"

#define LAB_LINEARIZE "X-Linearize"
#define LAB_XYREL "Keep XY-relation"

#define LAB_DESPEC_LIM "Despeckle limits"
#define LAB_SPECK_SUL "^Sur. U-limit"
#define LAB_SPECK_SPL "^Pix. L-limit"
#define LAB_FIXZOOM "^Fixed zoom fact"

#define LAB_M2HRPT_CORR "Correct faulty pixels MHRPT-N2"
#define LAB_ODD_ONOFF "Add odd lines"
#define LAB_COR_ONOFF "Correct odd lines"

#define LAB_HRPTRECTIMES "Record times (unit 1/6 sec)"
#define LAB_HRPT_START "^Start after."
#define LAB_HRPT_STOP "^Stop after."

#define LAB_HRPTFILENAMES "File names"

#define LAB_HRPTFILE_BASE  "^Basename"
#define LAB_HRPTFILE_EXT   "^Extension"

#define LAB_MHRPTFILE_BASE "^Basename "
#define LAB_MHRPTFILE_EXT  "^Extension "

#define LAB_METOPFILE_BASE "^Basename  "
#define LAB_METOPFILE_EXT  "^Extension  "

#define LAB_FY3FILE_BASE   "^Basename   "
#define LAB_FY3FILE_EXT    "^Extension   "

#define LAB_METFY3X "Use MetFY3X file format"

#define LAB_LPS "^Lines/screen"
#define LAB_SCANDIR "^Scan direction"
#define LAB_DIR_NSN "South bound/North bound"

#define LAB_VCHR "!Show channel"
#define LAB_CHX_V   "Chan"
#define LAB_CH1_V   "Chan 1"
#define LAB_CH2_V   "Chan 2"
#define LAB_CH3_V   "Chan 3"
#define LAB_CH4_V   "Chan 4"
#define LAB_CH5_V   "Chan 5"
#define LAB_CH6_V   "Chan 6"
#define LAB_CH7_V   "Chan 7"
#define LAB_CH8_V   "Chan 8"
#define LAB_CH9_V   "Chan 9"
#define LAB_CH10_V  "Chan 10"

#define LAB_SAVREC   "Save record"
#define LAB_KEPREC   "Load kepler"


#define LAB_HRIRECTIMES "Record times (1/6 sec)"
#define LAB_HRIFILENAMES "File names"
#define LAB_HRI_START "^Start after"
#define LAB_HRI_STOP "^Stop after"
#define LAB_HRIFILE_BASE "^Basename"
#define LAB_HRIFILE_EXT "^Extension"

/*
#define LAB_METOPRECTIMES "Record times (1/6 sec)"
#define LAB_METOPFILENAMES "File names"
#define LAB_METOP_START "^Start after"
#define LAB_METOP_STOP "^Stop after"
#define LAB_METOPFILE_BASE "^Basename "
#define LAB_METOPFILE_EXT "^Extension "
*/

#define LAB_DIR_SAVE "Working directory"
#define LAB_FNTMPL "filename basename"

#define LAB_DIR_KEP  "Dir.   "
#define LAB_FILE_KEP "Name"

#define LAB_EXTPROGON "Enable ext. progs"
#define LAB_EXTPROGUP "Run going up"
#define LAB_EXTPROGDN "Run going down"

#define LAB_FSIZE "Font size"
#define LAB_GRIDCLR "Grid colour"
#define LAB_GRID_RED "^Red"
#define LAB_GRID_GREEN "^Green"
#define LAB_GRID_BLUE "^Blue"

#define LAB_REFPOS "Reference Position"
#define LAB_ELEVHOR "!Start tracking at "
#define LAB_ELEVDET "!Only track if reaches "
#define LAB_REFLON "Longitude"
#define LAB_REFLAT "Latitude"
#define LAB_ROTORSYS "Elev_Azim/  X_Y  "
#define LAB_XY_DISH  "   Y     /   X   "
#define LAB_X_DIR   "east/west"
#define LAB_Y_DIR   "north/south"

#define LAB_STORMX    "!X pos:"
#define LAB_STORMY    "!Y pos:"
#define LAB_STORMXW   "!Wait X:"
#define LAB_STORMYW   "!Wait Y:"

#define LAB_STORME    "!E pos:"
#define LAB_STORMA    "!A pos:"


//#define LAB_STORM_X  "!Storm pos. X"
//#define LAB_STORM_Y  "!Storm pos. Y"
//#define LAB_STORM_WX  "!Storm wait. X"
//#define LAB_STORM_WY  "!Storm wait. Y"

#define LAB_DEG2STEP "Steps / degr."
#define LAB_DEQCINVX "Flip X"
#define LAB_DEQCINVY "Flip Y"
#define LAB_DEQCLIMIT "!Limit"
#define LAB_DEQCLIMITDEG "!Limit deg"
#define LAB_SETLIM "Set lim HH390"

#define LAB_OFFSETTIME "!Offset UTC time (hours)"
#define LAB_OFFSETTIMESEC "!Offset UTC time (secs)"
#define LAB_OFFSETUTC "!Presented time: offset UTC"
#define LAB_SATLIST "^Satellite list"

#define SEPARATE " \n	="        /* parse word separator */

/* Defaults */
#define PREF_FILE_OLD ".wsatrc"
#define PREF_FILE "wsat.ini"
#define PARPORT_NUMBER 2          /* Default par port! */
#define GRID_CLR 0xff0000         /* Default grid color: red */

#define DEF_HRPTBASE "hrpt"
#define DEF_HRPTEXT ".dat"

#define DEF_CHRPTBASE "chrpt"
#define DEF_CHRPTEXT ".dat"

#define DEF_MHRPTBASE "meteor"
#define DEF_MHRPTEXT ".dat"

// MetFY3x: Metop<ABC>_YYYY-MM-DD_hhmm.dat
#define DEF_METOPBASE "metop"
#define DEF_METOPEXT ".dat"

// MetFY3x: FY3<ABC>_YYYY-MM-DD_hhmm.dat
#define DEF_FY3BASE "fy3"
#define DEF_FY3EXT ".dat"

#define DEF_HRIBASE "hri"
#define DEF_HRIEXT ".dat"

#define DEF_RECDIR ""
#define DEF_BASENAME "%s_ch%c"

#define PREF_HRPTFILEBASE   "Hrpt_File_Base"
#define PREF_HRPTFILEEXT    "Hrpt_File_Extension"
#define PREF_METEORFILEBASE "Meteor_File_Base"
#define PREF_METEORFILEEXT  "Meteor_File_Extension"
#define PREF_METOPFILEBASE  "Metop_File_Base"
#define PREF_METOPFILEEXT   "Metop_File_Extension"
#define PREF_FY3FILEBASE    "FY3_File_Base"
#define PREF_FY3FILEEXT     "FY3_File_Extension"
#define PREF_METFY3X        "METFY3X_format"
extern PREFS prefs;


/*************************************
 * Translate parport number into address  
 *************************************/
int ppnr_to_ppaddr(int parport_nr)
{
  int parport_base=0;
  switch(parport_nr)
  {
    case 1 : parport_base=0x378;      break;
    case 2 : parport_base=0x278;      break;
    default: parport_base=parport_nr; break;
  }
  return parport_base;
}

/*************************************
 * Copy some preferences
 *************************************/
void Load_Prefs(RECORDINFO *rdi,VIEWINFO *vwi,PICINFO *pci)
{
  prefs.parport_base=ppnr_to_ppaddr(prefs.parport_nr);
  if (rdi)
  {
    rdi->port_type=prefs.port_type;
    rdi->parport_base=prefs.parport_base;
    rdi->fast_parportaccess=prefs.fast_parportaccess;
    rdi->hwsel_type=prefs.hwsel_type;

    rdi->usb.usb_use_id=prefs.usb_use_id;
    strncpy(rdi->usb.usbport_id,prefs.usbport_id,30);
    rdi->usb.fast_ftd=prefs.fast_ftd;

    rdi->hrpt_start_after_in_sync=prefs.hrpt_start_after_in_sync;
    rdi->hrpt_stop_after_out_sync=prefs.hrpt_stop_after_out_sync;
    rdi->hri_start_after_in_sync=prefs.hri_start_after_in_sync;
    rdi->hri_stop_after_out_sync=prefs.hri_stop_after_out_sync;
    rdi->metop_start_after_in_sync=prefs.metop_start_after_in_sync*METOPFRM_PER_LINE;
    rdi->metop_stop_after_out_sync=prefs.metop_stop_after_out_sync*METOPFRM_PER_LINE;

    rdi->height=prefs.hrpt_lines_per_screen;
    rdi->scan_ver=prefs.hrpt_scan_ver;
    rdi->scan_hor=(prefs.hrpt_scan_ver==N_S? W_E: E_W);

    strcpy(rdi->file_hrptbase,prefs.file_hrptbase);
    strcpy(rdi->file_hrptext,prefs.file_hrptext);
    strcpy(rdi->file_chrptbase,prefs.file_chrptbase);
    strcpy(rdi->file_chrptext,prefs.file_chrptext);
    strcpy(rdi->file_mhrptbase,prefs.file_mhrptbase);
    strcpy(rdi->file_mhrptext,prefs.file_mhrptext);
    strcpy(rdi->file_metopbase,prefs.file_metopbase);
    strcpy(rdi->file_metopext,prefs.file_metopext);
    strcpy(rdi->file_fy3base,prefs.file_fy3base);
    strcpy(rdi->file_fy3ext,prefs.file_fy3ext);

    strcpy(rdi->file_hribase,prefs.file_hribase);
    strcpy(rdi->file_hriext,prefs.file_hriext);
    strcpy(rdi->work_location,prefs.dir_name);

    sprintf(rdi->kepler_file,"%s%s",prefs.kep_dirname,prefs.kep_filename);
    rdi->sh_recchan=prefs.sh_recchan;
    rdi->save_flag=prefs.save_flag;
    rdi->auto_get_kepler=prefs.auto_get_kepler;
  }

  if (vwi)
  {
    vwi->grid_clr.red=prefs.grid_clr.red;
    vwi->grid_clr.green=prefs.grid_clr.green;
    vwi->grid_clr.blue=prefs.grid_clr.blue;
    vwi->sur_ulim=prefs.sur_ulim;
    vwi->dsp_llim=prefs.dsp_llim;
    vwi->linearize=prefs.linearize;
    vwi->keep_xyrel=prefs.keep_xyrel;
    vwi->fix_zoomfactor=prefs.fix_zoomfactor;
  }

  if (pci)
  {
    strcpy(pci->work_location,prefs.dir_name);
    strcpy(pci->kep_location,prefs.kep_dirname);
    pci->syncsignal_del=-1;
    pci->jpg_quality=90;
  }

}

void Load_Prefdef()
{
  prefs.parport_nr=PARPORT_NUMBER;
  prefs.fast_parportaccess=FALSE;
  prefs.hwsel_type=FALSE;
  prefs.reptime=1.0;

  prefs.hrpt_start_after_in_sync=6;
  prefs.hrpt_stop_after_out_sync=90;

  prefs.metop_start_after_in_sync=6;
  prefs.metop_stop_after_out_sync=90;

  prefs.hri_start_after_in_sync=20;
  prefs.hri_stop_after_out_sync=20;

  prefs.hrpt_lines_per_screen=0;
  prefs.hrpt_scan_ver=N_S;
  prefs.sh_recchan=2;
  prefs.save_flag=TRUE;
  prefs.auto_get_kepler=FALSE;

  prefs.metfy3x=FALSE;
 
  strcpy(prefs.file_hrptbase,DEF_HRPTBASE);
  strcpy(prefs.file_hrptext,DEF_HRPTEXT);
  strcpy(prefs.file_chrptbase,DEF_CHRPTBASE);
  strcpy(prefs.file_chrptext,DEF_CHRPTEXT);
  strcpy(prefs.file_mhrptbase,DEF_MHRPTBASE);
  strcpy(prefs.file_mhrptext,DEF_MHRPTEXT);

  strcpy(prefs.file_metopbase,DEF_METOPBASE);
  strcpy(prefs.file_metopext,DEF_METOPEXT);

  strcpy(prefs.file_fy3base,DEF_FY3BASE);
  strcpy(prefs.file_fy3ext,DEF_FY3EXT);

  strcpy(prefs.file_hribase,DEF_HRIBASE);
  strcpy(prefs.file_hriext,DEF_HRIEXT);

  strcpy(prefs.dir_name,DEF_RECDIR);
  strcpy(prefs.base_name,DEF_BASENAME);

  prefs.font_size=0;
  prefs.grid_clr.red=(GRID_CLR>>16)&0xff;
  prefs.grid_clr.green=(GRID_CLR>>8)&0xff;
  prefs.grid_clr.blue=(GRID_CLR>>0)&0xff;

  prefs.sur_ulim=20;
  prefs.dsp_llim=10;
  prefs.fix_zoomfactor=FIX_ZOOM;
  prefs.linearize=FALSE;
  prefs.keep_xyrel=FALSE;

  prefs.refpos.lon=D2R(5);
  prefs.refpos.lat=D2R(52);
  prefs.elev_horiz=0.;
  prefs.elev_det=0.;

/* Let window system determine where to place the windows */
  prefs.main_wnd_geom=deflt_wndpos;
  prefs.sub_wnd_geom=deflt_wndpos;
  prefs.rotor.storm.x=91.;
  prefs.rotor.storm.y=91.;
  prefs.rotor.storm_wait_x=0;
  prefs.rotor.storm_wait_y=0;
  prefs.rotor.storm.elev=90.;
  prefs.rotor.storm.azim=0.;
  prefs.rotor.deg2step=8;
  prefs.rotor.inv_x=FALSE;
  prefs.rotor.inv_y=FALSE;
  prefs.rotor.xy_rotorlim=0;
  prefs.main_wnd_x=0;
  prefs.main_wnd_y=0;
  prefs.main_wnd_w=0;
  prefs.main_wnd_h=0;
}

void get_dirs(char *progname,char **cur_dir,char **prog_dir,char **home_dir)
{
  if (*cur_dir)  free(*cur_dir);  *cur_dir=NULL;
  if (*prog_dir) free(*prog_dir); *prog_dir=NULL;
  if (*home_dir) free(*home_dir); *home_dir=NULL;

  *cur_dir=g_get_current_dir();
  *prog_dir=get_path(progname);
  *home_dir=(char *)g_get_home_dir();
  #if __GTK_WIN32__ == 1
  {
    if (!(*home_dir))
    {
      if (!strncmp(*cur_dir+1,":\\",2))
      {
        *home_dir=malloc(10);
        strncpy(*home_dir,*cur_dir,2);
        *((*home_dir)+2)=0;
      }
    }
  }
  #endif
}

void search_file(char *rootfile,char *file,
                 char *cur_dir,char *home_dir,char *prog_dir)
{
/* Search pref-file */
  if (cur_dir)
    sprintf(file,"%s%c%s",cur_dir,DIR_SEPARATOR,rootfile);
  if (!(exist_file(file)))
  {
    *file=0;
    if (home_dir)
      sprintf(file,"%s%c%s",home_dir,DIR_SEPARATOR,rootfile);

    if (!(exist_file(file)))
    {
      *file=0;
      if (prog_dir)
        sprintf(file,"%s%c%s",prog_dir,DIR_SEPARATOR,rootfile);

      if (!(exist_file(file)))
      {
        *file=0;
      }
    }
  }
  return;
}

// change def. porttype to something what exists
static void reset_porttype()
{
  if (prefs.port_type==parport)
  {
    #ifndef __ADD_PAR__
      prefs.port_type=usb;
    #endif
  }
  if (prefs.port_type==ethernet)
  {
    #ifndef __ADD_ETH__
      prefs.port_type=usb;
    #endif
  }
  if (prefs.port_type==usb)
  {
    #ifndef __ADD_USB__
      prefs.port_type=parport;
    #endif
  }
}
/*************************************
 * Read preferences from file; first load defaults 
 *************************************/
#define LINELEN 300
int Read_Prefs(char *progname)
{
  FILE *fp;
  char l[LINELEN],*w1,*w2,*w3;

/* Load directory names: current, program location etc. */
  get_dirs(progname,&prefs.cur_dir,&prefs.prog_dir,&prefs.home_dir);

/* Load defaults */
  Load_Prefdef();
  reset_porttype();

  *prefs.used_preffile=0;
  search_file(PREF_FILE,prefs.used_preffile,
              prefs.cur_dir,prefs.home_dir,prefs.prog_dir);

// if wsat.ini not found: try old .wsatrc
  if (!*prefs.used_preffile)
  {
    *prefs.used_preffile=0;
    search_file(PREF_FILE_OLD,prefs.used_preffile,
                prefs.cur_dir,prefs.home_dir,prefs.prog_dir);
  }

/* Open preference file */
  if (!(fp=fopen(prefs.used_preffile,"r")))
  {
    return 0;
  }

/* Start reading/parsing preferences file */
  while (fgets(l,LINELEN,fp))
  {
    if (!strncmp(l,"Sat_list",strlen("Sat_list")))
    {
      if (!(w1=strtok(l,"="))) continue;
      if (!(w2=strtok(NULL,"\n"))) continue;
      while (*w2==' ') w2++;
      strcpy(prefs.satlist,w2);
    }
    else
    {
      if (!(w1=strtok(l,SEPARATE))) continue;
      if (!(w2=strtok(NULL,SEPARATE))) continue;
    }

    if (!strcmp(w1,"main_window_size"))
    {
      if ((w3=strtok(w2,"x")))
      {
        prefs.main_wnd_w=atoi(w3);
        if ((w3=strtok(NULL,"x\n"))) prefs.main_wnd_h=atoi(w3);
      }
    }

    if (!strcmp(w1,"main_window_position"))
    {
      if ((w3=strtok(w2,"@")))
      {
        if (!strcmp(w3,"abs_wndpos")) prefs.main_wnd_geom=abs_wndpos;
        if (!strcmp(w3,"smart_wndpos")) prefs.main_wnd_geom=smart_wndpos;
        if (!strcmp(w3,"center_wndpos")) prefs.main_wnd_geom=center_wndpos;
        if ((w3=strtok(NULL,"x\n"))) prefs.main_wnd_x=atoi(w3);
        if ((w3=strtok(NULL,"x\n"))) prefs.main_wnd_y=atoi(w3);
      }
    }

    if (!strcmp(w1,"sub_window_position"))
    {
      if ((w3=strtok(w2,"@")))
      {
        if (!strcmp(w3,"abs_wndpos")) prefs.sub_wnd_geom=abs_wndpos;
        if (!strcmp(w3,"smart_wndpos")) prefs.sub_wnd_geom=smart_wndpos;
        if (!strcmp(w3,"center_wndpos")) prefs.sub_wnd_geom=center_wndpos;
        if ((w3=strtok(NULL,"x\n"))) prefs.sub_wnd_x=atoi(w3);
        if ((w3=strtok(NULL,"x\n"))) prefs.sub_wnd_y=atoi(w3);
      }
    }

    if (!strcmp(w1,"Port_type"))
    {
      if (!strcmp(w2,"usb"))      prefs.port_type=usb;
      else if (!strcmp(w2,"eth")) prefs.port_type=ethernet;
      else                        prefs.port_type=parport;
    }

    reset_porttype();

    if (!strcmp(w1,"Parallel_port"))
      prefs.parport_nr=atoi(w2);

    if (!strcmp(w1,"Parallel_port_address"))
      prefs.parport_address=strtol(w2,NULL,16);

    if (!strcmp(w1,"Fast_Parportaccess"))
      prefs.fast_parportaccess=(*w2=='y'? TRUE : FALSE); 

    if (!strcmp(w1,"Hwtype_select"))
      prefs.hwsel_type=(!strcmp(w2,"dec_type")? TRUE : FALSE); 

    if (!strcmp(w1,"use_usbid"))
      prefs.usb_use_id=(*w2=='y'? TRUE : FALSE); 

    if (!strcmp(w1,"usb_id"))
      strcpy(prefs.usbport_id,w2);

    if (!strcmp(w1,"usb_fast_ftd"))
      prefs.fast_ftd=(*w2=='y'? TRUE : FALSE);

    if (!strcmp(w1,"Font_size"))
    {
      prefs.font_size= atoi(w2);
    }

    if (!strcmp(w1,"Grid_color"))
    {
      prefs.grid_clr.red  =(strtol(w2,NULL,16)>>16)&0xff;
      prefs.grid_clr.green=(strtol(w2,NULL,16)>>8)&0xff;
      prefs.grid_clr.blue =(strtol(w2,NULL,16)>>0)&0xff;
    }

    if (!strcmp(w1,"Linearize"))
    {
      prefs.linearize=(!strcmp(w2,"on")? TRUE : FALSE);
    }

    if (!strcmp(w1,"Keep_xy_relation"))
    {
      prefs.keep_xyrel=(!strcmp(w2,"on")? TRUE : FALSE);
    }

    if (!strcmp(w1,"Despeckle_limits"))
    {
      prefs.sur_ulim=atoi(w2);
      if (!(w2=strtok(NULL,SEPARATE))) continue;
      prefs.dsp_llim=atoi(w2);
    }


    if (!strcmp(w1,"Fixed_zoomfactor"))
    {
      prefs.fix_zoomfactor=atof(w2);
    }

    if (!strcmp(w1,"Hrpt_Record_Start_Stop_Times"))
    {
      prefs.hrpt_start_after_in_sync=atoi(w2);
      if (!(w2=strtok(NULL,SEPARATE))) continue;
      prefs.hrpt_stop_after_out_sync=atoi(w2);
    }
    if (!strcmp(w1,"Hri_Record_Start_Stop_Times"))
    {
      prefs.hri_start_after_in_sync=atoi(w2);
      if (!(w2=strtok(NULL,SEPARATE))) continue;
      prefs.hri_stop_after_out_sync=atoi(w2);
    }

    if (!strcmp(w1,"Hrpt_Record_Lines_per_screen"))
    {
      prefs.hrpt_lines_per_screen=atoi(w2);
    }
    if (!strcmp(w1,"Hrpt_Record_Scandirection"))
    {
      prefs.hrpt_scan_ver=(!strcmp(w2,"N_S")? N_S : 
                           !strcmp(w2,"S_N")? S_N :
                           scan_unk);
    }

    if (!strcmp(w1,"Hrpt_Record_Showchan"))
    {
      prefs.sh_recchan=atoi(w2);
    }

    if (!strcmp(w1,"Hrptrec_Save_rec"))
      prefs.save_flag=(*w2=='y'? TRUE : FALSE); 
    if (!strcmp(w1,"Hrptrec_Auto_getkepler"))
      prefs.auto_get_kepler=(*w2=='y'? TRUE : FALSE); 


    if (!strcmp(w1,PREF_HRPTFILEBASE))
    {
      strcpy(prefs.file_hrptbase,w2);
    }
    if (!strcmp(w1,PREF_HRPTFILEEXT))
    {
      strcpy(prefs.file_hrptext,w2);
    }

    if (!strcmp(w1,"Hri_File_Base"))
    {
      strcpy(prefs.file_hribase,w2);
    }
    if (!strcmp(w1,"Hri_File_Extension"))
    {
      strcpy(prefs.file_hriext,w2);
    }
    
    if (!strcmp(w1,PREF_METEORFILEBASE))
    {
      strcpy(prefs.file_mhrptbase,w2);
    }
    if (!strcmp(w1,PREF_METEORFILEEXT))
    {
      strcpy(prefs.file_mhrptext,w2);
    }

    if (!strcmp(w1,PREF_METOPFILEBASE))
    {
      strcpy(prefs.file_metopbase,w2);
    }
    if (!strcmp(w1,PREF_METOPFILEEXT))
    {
      strcpy(prefs.file_metopext,w2);
    }

    if (!strcmp(w1,PREF_FY3FILEBASE))
    {
      strcpy(prefs.file_fy3base,w2);
    }
    if (!strcmp(w1,PREF_FY3FILEEXT))
    {
      strcpy(prefs.file_fy3ext,w2);
    }

    if (!strcmp(w1,PREF_METFY3X))
    {
      prefs.metfy3x=(*w2=='y'? TRUE : FALSE); 
    }

    if (!strcmp(w1,"Working_Dir"))
    {
      strcpy(prefs.dir_name,w2);
      if (*(prefs.dir_name+strlen(prefs.dir_name)-1) != DIR_SEPARATOR)
        sprintf(prefs.dir_name,"%s%c",prefs.dir_name,DIR_SEPARATOR);

    }
    if (!strcmp(w1,"file_basename"))
    {
      strcpy(prefs.base_name,w2);
    }
    if (!strcmp(w1,"Kepler_File_Dir"))
    {
      strcpy(prefs.kep_dirname,w2);
      if (*(prefs.kep_dirname+strlen(prefs.kep_dirname)-1) != DIR_SEPARATOR)
        sprintf(prefs.kep_dirname,"%s%c",prefs.kep_dirname,DIR_SEPARATOR);
    }


    if (!strcmp(w1,"track.Prog_ud_on"))
    {
      prefs.ext_prog_on=(*w2=='y'? TRUE : FALSE);
    }
    if (!strcmp(w1,"track.Prog_up"))
    {
      w3=strtok(NULL,"\n");
      sprintf(prefs.prog_trackup,"%s %s",w2,(w3? w3 : ""));
    }
    if (!strcmp(w1,"track.Prog_down"))
    {
      w3=strtok(NULL,"\n");
      sprintf(prefs.prog_trackdown,"%s %s",w2,(w3? w3 : ""));
    }


    if (!strcmp(w1,"Observer_Lon"))
    {
      prefs.refpos.lon=D2R(atof(w2));
    }
    if (!strcmp(w1,"Observer_Lat"))
    {
      prefs.refpos.lat=D2R(atof(w2));
    }
    if (!strcmp(w1,"Elev_horizon"))
    {
      prefs.elev_horiz=D2R(atof(w2));
    }
    if (!strcmp(w1,"Elev_detect"))
    {
      prefs.elev_det=D2R(atof(w2));
    }

    if (!strcmp(w1,"Rotortype"))
    {
      prefs.rotor.use_xy=(!strcmp(w2,"X-Y")? TRUE : FALSE);
    }
    if (!strcmp(w1,"X_at_Disc"))
    {
      prefs.rotor.x_at_disc=atoi(w2);
    }
    if (!strcmp(w1,"Storm_pos_x"))
    {
      prefs.rotor.storm.x=atoi(w2);
    }
    if (!strcmp(w1,"X_Dir_0"))
    {
       prefs.rotor.x_west_is_0=(*w2=='w'? TRUE : FALSE);
    }
    if (!strcmp(w1,"Y_Dir_0"))
    {
       prefs.rotor.y_south_is_0=(*w2=='s'? TRUE : FALSE);
    }

    if (!strcmp(w1,"Storm_wait_x"))
    {
      prefs.rotor.storm_wait_x=atoi(w2);
    }
    if (!strcmp(w1,"Storm_pos_y"))
    {
      prefs.rotor.storm.y=atoi(w2);
    }
    if (!strcmp(w1,"Storm_wait_y"))
    {
      prefs.rotor.storm_wait_y=atoi(w2);
    }
    if (!strcmp(w1,"Flip_X"))
    {
      prefs.rotor.inv_x=atoi(w2);
    }
    if (!strcmp(w1,"Flip_Y"))
    {
      prefs.rotor.inv_y=atoi(w2);
    }
    if (!strcmp(w1,"DiSEqC_Deg2step"))
    {
      prefs.rotor.deg2step=atoi(w2);
    }
    if (!strcmp(w1,"DiSEqC_Limit"))
    {
      prefs.rotor.xy_rotorlim=atoi(w2);
    }
    if (!strcmp(w1,"Localtime_UTCOffset"))
    {
      prefs.utc_offset=atof(w2);
    }
    if (!strcmp(w1,"Hour_Offset"))
    {
      prefs.tm_off.tz_offset=atof(w2);
    }
    if (!strcmp(w1,"Sec_Offset"))
    {
      prefs.tm_off.sec_offset=atof(w2);
    }
    if (!strcmp(w1,"ClrMap"))      // M1 "xx" offs(0,0,-32) c1:[1.500,0.000,0.000] c2:[.....
    {
      int i;
      {
        char *p;
        if (*w2!='M') break;
        if ((i=atoi(w2+1))) //  && (n==i+1))
        {
          i--;
          if (!(w2=strtok(NULL," \n"))) continue;   // "naam"
          if (!(w2=strtok(NULL," \n"))) continue;   // offs(0,0,-32)
          if (!strncmp(w2,"offs[",5))
          {
            if ((p=strchr(w2,'['))) prefs.cmhc[i].red_offset=atoi(p+1);
            if ((p=strchr(p+1,',')))  prefs.cmhc[i].green_offset=atoi(p+1);
            if ((p=strchr(p+1,',')))  prefs.cmhc[i].blue_offset=atoi(p+1);

          }
          while ((w2=strtok(NULL," \n")))           // c1:[1.500,0.000,0.000]
          {
            if ((p=strchr(w2,'[')))
            {
              int n;
              *p=0;
              n=atoi(w2+1);
              if ((n<0) || (n>=NR_CHANNELS_MAX)) continue;
              prefs.cmhc[i].red[n]=atof(p+1);
              if ((p=strchr(p+1,',')))  prefs.cmhc[i].green[n]=atof(p+1);
              if ((p=strchr(p+1,',')))  prefs.cmhc[i].blue[n]=atof(p+1);
            }
          }
        }
      }
    }

  }
  fclose(fp);
  fprintf(stderr,"Preference file %s found.\n",prefs.used_preffile);

  return 1;
}

void pri_prefline(FILE *fp,char *key,char *frmt,...)
{
  va_list arg;
  va_start(arg,frmt);
  fprintf(fp,"%-30s = ",key);
  vfprintf(fp,frmt,arg);
  va_end(arg);
}

/*************************************
 * Save preferences into pref. file
 *************************************/
void Save_Prefs()
{
  FILE *fp;
  int i,c;
  if (!(fp=fopen(PREF_FILE,"w")))
  {
    Create_Message("Error","Can't open %s!",PREF_FILE);
    return;
  }

  pri_prefline(fp,"main_window_size","%dx%d\n",prefs.main_wnd_w,prefs.main_wnd_h);

  pri_prefline(fp,"main_window_position","");
  switch (prefs.main_wnd_geom)
  {
    case abs_wndpos:
      fprintf(fp,"abs_wndpos@%dx%d\n",prefs.main_wnd_x,prefs.main_wnd_y);
    break;
    case center_wndpos:
      fprintf(fp,"center_wndpos@%dx%d\n",prefs.main_wnd_x,prefs.main_wnd_y);
    break;
    default:
      fprintf(fp,"default\n");
    break;
  }

  pri_prefline(fp,"sub_window_position","");
  switch (prefs.sub_wnd_geom)
  {
    case abs_wndpos:
      fprintf(fp,"abs_wndpos@%dx%d\n",prefs.sub_wnd_x,prefs.sub_wnd_y);
    break;
    case center_wndpos:
      fprintf(fp,"center_wndpos@%dx%d\n",prefs.sub_wnd_x,prefs.sub_wnd_y);
    break;
    case smart_wndpos:
      fprintf(fp,"smart_wndpos@%dx%d\n",prefs.sub_wnd_x,prefs.sub_wnd_y);
    break;
    default:
      fprintf(fp,"default\n");
    break;
  }

  pri_prefline(fp,"Port_type","%s\n",
                     (prefs.port_type==usb?      "usb":
                      prefs.port_type==ethernet? "eth" :
                                                 "par"));
  pri_prefline(fp,"Parallel_port","%x\n",prefs.parport_nr);
  pri_prefline(fp,"Parallel_port_address","%x\n",prefs.parport_address);
  pri_prefline(fp,"Fast_Parportaccess","%s\n",
                     (prefs.fast_parportaccess? "yes" : "no"));

  pri_prefline(fp,"Hwtype_select","%s\n",
                     (prefs.hwsel_type? "dec_type" : "dec_gen"));

  pri_prefline(fp,"use_usbid","%s\n",(prefs.usb_use_id? "yes" : "no"));
  pri_prefline(fp,"usb_id","%s\n",prefs.usbport_id);
  pri_prefline(fp,"usb_fast_ftd","%s\n",(prefs.fast_ftd? "yes" : "no"));

  pri_prefline(fp,"Font_size","%d\n",prefs.font_size);

  pri_prefline(fp,"Grid_color","0x%x\n",(prefs.grid_clr.red<<16)+
                                     (prefs.grid_clr.green<<8)+
                                     (prefs.grid_clr.blue) );

  for (i=0; i<NR_PREFMAPS; i++)
  {
    pri_prefline(fp,"ClrMap","M%d \"%s\" offs[%d,%d,%d] ",
       i+1,prefs.cmhc[i].name,
      prefs.cmhc[i].red_offset,prefs.cmhc[i].green_offset,prefs.cmhc[i].blue_offset
    );
    for (c=0; c<NR_CHANNELS_MAX; c++)
    {
      if ((prefs.cmhc[i].red[c])||(prefs.cmhc[i].green[c])||(prefs.cmhc[i].blue[c]))
        fprintf(fp,"c%d[%.2f,%.2f,%.2f] ",c,prefs.cmhc[i].red[c],prefs.cmhc[i].green[c],prefs.cmhc[i].blue[c]);
    }
    fprintf(fp,"\n");
    
  }

  pri_prefline(fp,"Linearize","%s\n",prefs.linearize? "on" : "off");
  pri_prefline(fp,"Keep_xy_relation","%s\n",prefs.keep_xyrel? "on" : "off");
  pri_prefline(fp,"Despeckle_limits","%d %d\n",prefs.sur_ulim,prefs.dsp_llim);
  pri_prefline(fp,"Fixed_zoomfactor","%.2f\n",prefs.fix_zoomfactor);

  pri_prefline(fp,"Hrpt_Record_Start_Stop_Times","%d %d\n",
     prefs.hrpt_start_after_in_sync,prefs.hrpt_stop_after_out_sync);
  pri_prefline(fp,"Hri_Record_Start_Stop_Times","%d %d\n",
     prefs.hri_start_after_in_sync,prefs.hri_stop_after_out_sync);

  pri_prefline(fp,"Hrpt_Record_Lines_per_screen","%d\n",
    prefs.hrpt_lines_per_screen);
  pri_prefline(fp,"Hrpt_Record_Scandirection","%s\n",
    (prefs.hrpt_scan_ver==N_S? "N_S" : prefs.hrpt_scan_ver==S_N? "S_N" : "?"));
  pri_prefline(fp,"Hrpt_Record_Showchan","%d\n",prefs.sh_recchan);

  pri_prefline(fp,"Hrptrec_Save_rec","%s\n",
                     (prefs.save_flag? "yes" : "no"));
  pri_prefline(fp,"Hrptrec_Auto_getkepler","%s\n",
                     (prefs.auto_get_kepler? "yes" : "no"));

  pri_prefline(fp,PREF_HRPTFILEBASE,"%s\n",prefs.file_hrptbase);
  pri_prefline(fp,PREF_HRPTFILEEXT,"%s\n",prefs.file_hrptext);
  pri_prefline(fp,PREF_METEORFILEBASE,"%s\n",prefs.file_mhrptbase);
  pri_prefline(fp,PREF_METEORFILEEXT,"%s\n",prefs.file_mhrptext);
  pri_prefline(fp,PREF_METOPFILEBASE,"%s\n",prefs.file_metopbase);
  pri_prefline(fp,PREF_METOPFILEEXT,"%s\n",prefs.file_metopext);
  pri_prefline(fp,PREF_FY3FILEBASE,"%s\n",prefs.file_fy3base);
  pri_prefline(fp,PREF_FY3FILEEXT,"%s\n",prefs.file_fy3ext);

  pri_prefline(fp,PREF_METFY3X,"%s\n",(prefs.metfy3x? "yes" : "no"));

  pri_prefline(fp,"Hri_File_Base","%s\n",prefs.file_hribase);
  pri_prefline(fp,"Hri_File_Extension","%s\n",prefs.file_hriext);

  pri_prefline(fp,"Working_Dir","%s\n",prefs.dir_name);
  pri_prefline(fp,"file_basename","%s\n",prefs.base_name);
  pri_prefline(fp,"Kepler_File_Dir","%s\n",prefs.kep_dirname);
  pri_prefline(fp,"Kepler_File_name","%s\n",prefs.kep_filename);

  pri_prefline(fp,"track.Prog_ud_on","%s\n",(prefs.ext_prog_on? "yes" : "no"));
  pri_prefline(fp,"track.Prog_up","%s\n",prefs.prog_trackup);
  pri_prefline(fp,"track.Prog_down","%s\n",prefs.prog_trackdown);

//  pri_prefline(fp,"Observer_Name: %s\n",prefs.refpos_name);
  pri_prefline(fp,"Observer_Lon","%.3f\n",R2D(prefs.refpos.lon));
  pri_prefline(fp,"Observer_Lat","%.3f\n",R2D(prefs.refpos.lat));
  pri_prefline(fp,"Elev_horizon","%d\n",(int)R2D(prefs.elev_horiz));
  pri_prefline(fp,"Elev_detect","%d\n",(int)R2D(prefs.elev_det));

  pri_prefline(fp,"Rotortype","%s\n",(prefs.rotor.use_xy? "X-Y" : "Alev-Azim"));
  pri_prefline(fp,"X_at_Disc","%d\n",prefs.rotor.x_at_disc);
  pri_prefline(fp,"X_Dir_0","%s\n",prefs.rotor.x_west_is_0? "west" : "east");
  pri_prefline(fp,"Y_Dir_0","%s\n",prefs.rotor.y_south_is_0?"south":"north");


  pri_prefline(fp,"Storm_pos_x","%d\n",(int)prefs.rotor.storm.x);
  pri_prefline(fp,"Storm_wait_x","%d\n",prefs.rotor.storm_wait_x);
  pri_prefline(fp,"Storm_pos_y","%d\n",(int)prefs.rotor.storm.y);
  pri_prefline(fp,"Storm_wait_y","%d\n",prefs.rotor.storm_wait_y);
  pri_prefline(fp,"Flip_X","%d\n",prefs.rotor.inv_x);
  pri_prefline(fp,"Flip_Y","%d\n",prefs.rotor.inv_y);
  pri_prefline(fp,"DiSEqC_Deg2step","%d\n",prefs.rotor.deg2step);
  pri_prefline(fp,"DiSEqC_Limit","%d\n",prefs.rotor.xy_rotorlim);
  pri_prefline(fp,"Localtime_UTCOffset","%.2f\n",prefs.utc_offset);
  pri_prefline(fp,"Hour_Offset","%.2f\n",prefs.tm_off.tz_offset);
  pri_prefline(fp,"Sec_Offset","%.2f\n",prefs.tm_off.sec_offset);
  pri_prefline(fp,"Sat_list","%s\n",prefs.satlist);

  fclose(fp);
}

/*************************************
 * Get preferences window commands  
 *************************************/
void pref_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *pref_window=NULL;
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  char *name=(char *)data;

  if (GTK_IS_WIDGET(widget))
    pref_window=gtk_widget_get_toplevel(widget);

  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(pref_window);
  }

  if (!strcmp(name,LAB_Use))
  {
    Load_Prefs(rdi,vwi,pci);
    Set_Button(Find_Window(widget,"Record HRPT"),"Save",Get_Button(widget,LAB_SAVREC));
  }

  if (!strcmp(name,LAB_Save))
  {
    Save_Prefs();
    Reset_Mapchanclr(0);
  }

  {
    char *ext;
    ext=soption_menu_get_history(Find_Widget(widget,LAB_WNDPOS));
    if ((ext) && (!strcmp(name,ext)))
    {
      if (!strcmp(name,LAB_WNDPOS_DFLT))
      {
        prefs.main_wnd_geom=deflt_wndpos;
        prefs.sub_wnd_geom=deflt_wndpos;
      }
      if (!strcmp(name,LAB_WNDPOS_SMART))
      {
        prefs.main_wnd_geom=abs_wndpos;
        prefs.sub_wnd_geom=smart_wndpos;
      }
    }
  }

  if (!strcmp(name,LAB_WND_WIDTH))
  {
    prefs.main_wnd_w=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_WND_HEIGHT))
  {
    prefs.main_wnd_h=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_USB))
  {
    prefs.port_type=usb;
    Show_Button(widget,LAB_SEL_HWTYPE,prefs.port_type==parport);
    Show_Button(widget,LAB_PARPORT,prefs.port_type==parport);
    Show_Button(widget,LAB_USBPORT,prefs.port_type==usb);
  }
  if (!strcmp(name,LAB_ETH))
  {
    prefs.port_type=ethernet;
    Show_Button(widget,LAB_SEL_HWTYPE,prefs.port_type==parport);
    Show_Button(widget,LAB_PARPORT,prefs.port_type==parport);
    Show_Button(widget,LAB_USBPORT,prefs.port_type==usb);
  }
  if (!strcmp(name,LAB_PAR))
  {
    prefs.port_type=parport;
    Show_Button(widget,LAB_SEL_HWTYPE,prefs.port_type==parport);
    Show_Button(widget,LAB_PARPORT,prefs.port_type==parport);
    Show_Button(widget,LAB_USBPORT,prefs.port_type==usb);
  }

  if (!strcmp(name,LAB_P1))
  {
    prefs.parport_nr=1;
    rdi->parport_base=ppnr_to_ppaddr(prefs.parport_nr);
    Sense_Button(pref_window,LAB_PE,FALSE);
  }
  if (!strcmp(name,LAB_P2))
  {
    prefs.parport_nr=2;
    rdi->parport_base=ppnr_to_ppaddr(prefs.parport_nr);
    Sense_Button(pref_window,LAB_PE,FALSE);
  }
  if (!strcmp(name,LAB_PN))
  {
    prefs.parport_nr=0;
    rdi->parport_base=prefs.parport_address;
    Sense_Button(pref_window,LAB_PE,TRUE);
  }

  if (!strcmp(name,LAB_PE))
  {
    rdi->parport_base=strtol(Get_Entry(pref_window,LAB_PE),NULL,16);
    prefs.parport_address=rdi->parport_base;
  }

  if (!strcmp(name,LAB_SEL_DECGEN))
  {
    prefs.hwsel_type=FALSE;
    rdi->hwsel_type=prefs.hwsel_type;
  }
  if (!strcmp(name,LAB_SEL_DECTYPE))
  {
    prefs.hwsel_type=TRUE;
    rdi->hwsel_type=prefs.hwsel_type;
  }

  if (!strcmp(name,LAB_FASTPPACC))
  {
    prefs.fast_parportaccess=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
    rdi->fast_parportaccess=prefs.fast_parportaccess;
  }

  if (!strcmp(name,LAB_LINEARIZE))
  {
    prefs.linearize=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_XYREL))
  {
    prefs.keep_xyrel=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_SPECK_SUL))
  {
    prefs.sur_ulim=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_SPECK_SPL))
  {
    prefs.dsp_llim=GTK_ADJUSTMENT(widget)->value;
  }


  if (!strcmp(name,LAB_FIXZOOM))
  {
    prefs.fix_zoomfactor=GTK_ADJUSTMENT(widget)->value;
  }


  if (!strcmp(name,LAB_HRPT_START))
  {
    prefs.hrpt_start_after_in_sync=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_HRPT_STOP))
  {
    prefs.hrpt_stop_after_out_sync=GTK_ADJUSTMENT(widget)->value;
  }

/*
  if (!strcmp(name,LAB_METOP_START))
  {
    prefs.metop_start_after_in_sync=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_METOP_STOP))
  {
    prefs.metop_stop_after_out_sync=GTK_ADJUSTMENT(widget)->value;
  }
*/

  if (!strcmp(name,LAB_HRI_START))
  {
    prefs.hri_start_after_in_sync=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_HRI_STOP))
  {
    prefs.hri_stop_after_out_sync=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_LPS))
  {
    prefs.hrpt_lines_per_screen=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_DIR_NSN))
  {
    if (Update_Togglelabel(widget))
      prefs.hrpt_scan_ver=S_N;
    else
      prefs.hrpt_scan_ver=N_S;
  }

  if (!strcmp(name,LAB_SAVREC))
  {
    prefs.save_flag=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
    rdi->save_flag=prefs.save_flag;
  }
  if (!strcmp(name,LAB_KEPREC))
  {
    prefs.auto_get_kepler=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
    rdi->auto_get_kepler=prefs.auto_get_kepler;
  }

  {
    char *ext;
    ext=soption_menu_get_history(Find_Widget(widget,LAB_VCHR));
    if ((ext) && (!strcmp(name,ext)))
    {
      prefs.sh_recchan=atoi(name+strlen(LAB_CHX_V));
    }
  }

  if (!strcmp(name,LAB_HRPTFILE_BASE))
  {
    strcpy(prefs.file_hrptbase,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_HRPTFILE_BASE));
  }
  if (!strcmp(name,LAB_HRPTFILE_EXT))
  {
    strcpy(prefs.file_hrptext,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_HRPTFILE_EXT));
  }

  if (!strcmp(name,LAB_MHRPTFILE_BASE))
  {
    strcpy(prefs.file_mhrptbase,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_MHRPTFILE_BASE));
  }
  if (!strcmp(name,LAB_MHRPTFILE_EXT))
  {
    strcpy(prefs.file_mhrptext,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_MHRPTFILE_EXT));
  }

  if (!strcmp(name,LAB_METOPFILE_BASE))
  {
    strcpy(prefs.file_metopbase,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_METOPFILE_BASE));
  }
  if (!strcmp(name,LAB_METOPFILE_EXT))
  {
    strcpy(prefs.file_metopext,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_METOPFILE_EXT));
  }

  if (!strcmp(name,LAB_FY3FILE_BASE))
  {
    strcpy(prefs.file_fy3base,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_FY3FILE_BASE));
  }
  if (!strcmp(name,LAB_FY3FILE_EXT))
  {
    strcpy(prefs.file_fy3ext,
           Get_Entry(gtk_widget_get_toplevel(widget),
                                                   LAB_FY3FILE_EXT));
  }
  
  if (!strcmp(name,LAB_METFY3X))
  {
    prefs.metfy3x=Get_Button(widget,LAB_METFY3X);
    Sense_Button(widget,LAB_METOPFILE_BASE,!prefs.metfy3x);
    Sense_Button(widget,LAB_FY3FILE_BASE,!prefs.metfy3x);
  }

  if (!strcmp(name,LAB_HRIFILE_BASE))
  {
    strcpy(prefs.file_hribase,
        Get_Entry(gtk_widget_get_toplevel(widget),
                                                    LAB_HRIFILE_BASE));
  }
  if (!strcmp(name,LAB_HRIFILE_EXT))
  {
    strcpy(prefs.file_hriext,
        Get_Entry(gtk_widget_get_toplevel(widget),
                                                    LAB_HRIFILE_EXT));
  }

  if (!strcmp(name,LAB_DIR_SAVE))
  {
    strcpy(prefs.dir_name,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_DIR_SAVE));
  }
  if (!strcmp(name,LAB_FNTMPL))
  {
    strcpy(prefs.base_name,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_FNTMPL));
  }

  if (!strcmp(name,LAB_DIR_KEP))
  {
    strcpy(prefs.kep_dirname,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_DIR_KEP));
  }
  if (!strcmp(name,LAB_FILE_KEP))
  {
    strcpy(prefs.kep_filename,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_FILE_KEP));
  }

  if (!strcmp(name,LAB_EXTPROGON))
  {
    prefs.ext_prog_on=Get_Button(widget,LAB_EXTPROGON);
  }
  if (!strcmp(name,LAB_EXTPROGUP))
  {
    strcpy(prefs.prog_trackup,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_EXTPROGUP));
  }
  if (!strcmp(name,LAB_EXTPROGDN))
  {
    strcpy(prefs.prog_trackdown,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_EXTPROGDN));
  }

  if (!strcmp(name,LAB_FSIZE))
  {
    prefs.font_size=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_GRID_RED))
  {
    prefs.grid_clr.red=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_GRID_GREEN))
  {
    prefs.grid_clr.green=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_GRID_BLUE))
  {
    prefs.grid_clr.blue=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_REFLON))
  {
    prefs.refpos.lon=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_REFLAT))
  {
    prefs.refpos.lat=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_ELEVHOR))
  {
    prefs.elev_horiz=D2R(GTK_ADJUSTMENT(widget)->value);
  }
  if (!strcmp(name,LAB_ELEVDET))
  {
    prefs.elev_det=D2R(GTK_ADJUSTMENT(widget)->value);
  }

  if (!strcmp(name,LAB_ROTORSYS))
  {
    if (Update_Togglelabel(widget))
      prefs.rotor.use_xy=TRUE;
    else
      prefs.rotor.use_xy=FALSE;
    Sense_Button(widget,LAB_XY_DISH,prefs.rotor.use_xy);
    Sense_Button(widget,LAB_X_DIR,prefs.rotor.use_xy);
    Sense_Button(widget,LAB_Y_DIR,prefs.rotor.use_xy);
  }
  if (!strcmp(name,LAB_XY_DISH))
  {
    if (Update_Togglelabel(widget))
      prefs.rotor.x_at_disc=TRUE;
    else
      prefs.rotor.x_at_disc=FALSE;
  }
  if (!strcmp(name,LAB_X_DIR))
  {
    Update_Togglelabel(widget);
    prefs.rotor.x_west_is_0=Get_Button(widget,LAB_X_DIR);
  }
  if (!strcmp(name,LAB_Y_DIR))
  {
    Update_Togglelabel(widget);
    prefs.rotor.y_south_is_0=Get_Button(widget,LAB_Y_DIR);
  }


  if (!strcmp(name,LAB_STORMX))
  {
    prefs.rotor.storm.x=(float)GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_STORMXW))
  {
    prefs.rotor.storm_wait_x=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_STORMY))
  {
    prefs.rotor.storm.y=(float)GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_STORMYW))
  {
    prefs.rotor.storm_wait_y=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_STORME))
  {
    prefs.rotor.storm.elev=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_STORMA))
  {
    prefs.rotor.storm.azim=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_DEG2STEP))
  {
    prefs.rotor.deg2step=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_DEQCINVX))
  {
    prefs.rotor.inv_x=Get_Button(widget,name);
  }
  if (!strcmp(name,LAB_DEQCINVY))
  {
    prefs.rotor.inv_y=Get_Button(widget,name);
  }

  if (!strcmp(name,LAB_DEQCLIMIT))
  {
    prefs.rotor.xy_rotorlim=GTK_ADJUSTMENT(widget)->value;
    prefs.rotor.xy_rotorlim=MIN(prefs.rotor.xy_rotorlim,0x4ff);
    if (prefs.rotor.deg2step);
      Set_Adjust(widget,LAB_DEQCLIMITDEG,"%f",((float)prefs.rotor.xy_rotorlim/(float)prefs.rotor.deg2step)-0.);
  }
  if (!strcmp(name,LAB_DEQCLIMITDEG))
  {
    prefs.rotor.xy_rotorlim=(0.+GTK_ADJUSTMENT(widget)->value)*prefs.rotor.deg2step;
    prefs.rotor.xy_rotorlim=MIN(prefs.rotor.xy_rotorlim,0x4ff);
    Set_Adjust(widget,LAB_DEQCLIMIT,"%d",prefs.rotor.xy_rotorlim);
  }
  if (!strcmp(name,LAB_SETLIM))
  {
    prefs.rotor.xy_rotorlim=prefs.rotor.deg2step*91;
    prefs.rotor.xy_rotorlim=MIN(prefs.rotor.xy_rotorlim,0x4ff);
    Set_Adjust(widget,LAB_DEQCLIMIT,"%d",prefs.rotor.xy_rotorlim);
    if (prefs.rotor.deg2step)
      Set_Adjust(widget,LAB_DEQCLIMITDEG,"%f",((float)prefs.rotor.xy_rotorlim/(float)prefs.rotor.deg2step)-0.);
  }

  if (!strcmp(name,LAB_OFFSETUTC))
  {
    prefs.utc_offset=GTK_ADJUSTMENT(widget)->value;
  }
  if (!strcmp(name,LAB_OFFSETTIME))
  {
    prefs.tm_off.tz_offset=GTK_ADJUSTMENT(widget)->value;
    {
      float tmp=(prefs.tm_off.tz_offset-(int)prefs.tm_off.tz_offset)*4;
      if ((int)tmp!=(float)tmp)
      {      
        prefs.tm_off.tz_offset=((int)(prefs.tm_off.tz_offset*4))/4.;
        Set_Adjust(widget,LAB_OFFSETTIME,"%f",(float)prefs.tm_off.tz_offset);
      }
    }


  }
  if (!strcmp(name,LAB_OFFSETTIMESEC))
  {
    prefs.tm_off.sec_offset=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_SATLIST))
  {
    strcpy(prefs.satlist,Get_Entry(widget,LAB_SATLIST));
  }

 
  if (prefs.rotor.use_xy)
  {
    Show_Button(Find_Window(widget,Lab_Main),LAB_ROTOR_XY,TRUE);
    Show_Button(Find_Window(widget,Lab_Main),LAB_ROTOR_ELEAZIM,FALSE);
  }
  else
  {
    Show_Button(Find_Window(widget,Lab_Main),LAB_ROTOR_ELEAZIM,TRUE);
    Show_Button(Find_Window(widget,Lab_Main),LAB_ROTOR_XY,FALSE);
  }
}

int usb_list_nndevs();

void usb_commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *pref_window=NULL;
  char *name=(char *)data;
#ifdef __ADD_USB__ 

  if (GTK_IS_WIDGET(widget))
    pref_window=gtk_widget_get_toplevel(widget);

  if (!strcmp(name,LAB_USBID))
  {
    strcpy(prefs.usbport_id,
      Get_Entry(gtk_widget_get_toplevel(widget),LAB_USBID));
  }
  if (!strcmp(name,LAB_USB_USEID))
  {
    Sense_Button(widget,LAB_USBID,Get_Button(pref_window,LAB_USB_USEID));
    prefs.usb_use_id=TRUE;
  }
  if (!strcmp(name,LAB_USB_USEFIRST))
  {
    prefs.usb_use_id=FALSE;
  }

  if (!strcmp(name,LAB_USBFAST))
  {
    prefs.fast_ftd=Get_Button(widget,name);
  }

#define LAB_USBINFO "usblist"
  if (!strcmp(name,LAB_TESTUSB))
  {
    int n,nrdev=-1;
    GtkWidget *txt=Find_Widget(widget,LAB_USBINFO)->parent;
    FT_DEVICE_LIST_INFO_NODE *devInfo=NULL;
    nrdev=usb_list_ndevs(&devInfo);
    Clear_Text(txt);
    if (nrdev<-1)
    {
      Add_Text(txt,10,"%s","No USB port available.\n");
    }
    else if (nrdev==0)
    {
      Add_Text(txt,10,"%s","No FTD USB device connected.\n");
    }
    else
    {
      Add_Text(txt,10,"%-23s | %s\n","SerialNumber (USB ID)"  ,"Description");
      Add_Text(txt,10,"%-23s-+-%s\n","-----------------------","-----------");
      for (n=0; n<nrdev; n++)
      {
        char *sn=devInfo[n].SerialNumber;
        char *sd=devInfo[n].Description;

        if ((!*sn) && (!*sd))
          Add_Text(txt,12,"%-23s | %s\n","Unknown","No access to USB?");
        else
          Add_Text(txt,12,"%-23s | %s\n",sn,sd);
      }
    }
    if (devInfo) free(devInfo);
  }

#endif
}

GtkWidget *trackbook()
{
  GtkWidget *wg[6];

  wg[1]=Create_ButtonArray(LAB_REFPOS,pref_commands,1,
                        SPIN,LAB_REFLON,"%f%f%f",R2D(prefs.refpos.lon),0.,360.,
                        SPIN,LAB_REFLAT,"%f%f%f",R2D(prefs.refpos.lat),-90.,90.,
                         NULL
                        );

  wg[2]=Create_ButtonArray("Elevation levels",pref_commands,2,
                           LABEL,LAB_ELEVDET+1,
                           SPIN,LAB_ELEVDET,"%d%d%d",(int)R2D(prefs.elev_det),-10,90,
                           LABEL,LAB_ELEVHOR+1,
                           SPIN,LAB_ELEVHOR,"%d%d%d",(int)R2D(prefs.elev_horiz),-10,90,
                           NULL
                        );
  wg[3]=Create_ButtonArray("Time",pref_commands,2,
                           LABEL,"Time used for calc. is UTC.",
                           LABEL,"Time zone",
                           LABEL,LAB_OFFSETTIME+1,
                           SPIN,LAB_OFFSETTIME,"%3.2f%3.2f%f%f",0.25,prefs.tm_off.tz_offset,-23.,23.,
                           LABEL,"__________________________",
                           LABEL,"____________",

                           LABEL,"Compensate rotor delay.",
                           LABEL,"",
                           LABEL,LAB_OFFSETTIMESEC+1,
                           SPIN,LAB_OFFSETTIMESEC,"%3.2f%3.2f%f%f",0.1,prefs.tm_off.sec_offset,-60.,60.,
                           LABEL,"__________________________",
                           LABEL,"____________",

                           LABEL,LAB_OFFSETUTC+1,
                           SPIN,LAB_OFFSETUTC,"%3.2f%3.2f%f%f",0.25,prefs.utc_offset,-23.,23.,
                           0
                          );

//  wg[2]=Create_Spin(LAB_ELEVHOR,pref_commands,"%d%d%d",(int)R2D(prefs.elev_horiz),-10,90);
//  wg[2]=Pack("Elevation levels",'h',wg[2],1,NULL);
  wg[1]=Pack(NULL,'h',wg[1],1,wg[2],1,NULL);
  wg[1]=Pack(NULL,'v',wg[1],1,wg[3],1,NULL);
//------------------------------------------------------------------------------------------
  wg[2]=Create_ButtonArray("",pref_commands,-4,
                           LABEL,"Rotorsystem:",
                           TOGGLE,LAB_ROTORSYS,FALSE,
                           LABEL,"Dish at:",
                           TOGGLE,LAB_XY_DISH,FALSE,
                           LABEL,"  ",
                           LABEL,"  ",
                           LABEL,"  ",
                           LABEL,"  ",

                           LABEL,"X=0: ",
                           TOGGLE,LAB_X_DIR,FALSE,
                           LABEL,"Y=0: ",
                           TOGGLE,LAB_Y_DIR,FALSE,
                           0
                          );

  wg[3]=Create_ButtonArray("DisEqC config",pref_commands,6,
                           SPIN,LAB_DEG2STEP,"%d%d%d%d",8,prefs.rotor.deg2step,8,16,
                           LABEL,"Limit",
                           SPIN,LAB_DEQCLIMIT,"%d%d%d",prefs.rotor.xy_rotorlim,0,0xffff,
                           BUTTON,LAB_SETLIM,
                           LABEL,"    ", // LAB_DEQCINVX+1,
                           CHECKI,LAB_DEQCINVX,prefs.rotor.inv_x,

                           LABEL,"  ",
                           LABEL,"Limit deg.",
                           SPIN,LAB_DEQCLIMITDEG,"%.1f%.1f%.1f",
                             ((float)prefs.rotor.xy_rotorlim/(float)prefs.rotor.deg2step)-0.,-1.,91.,
                           LABEL,"    ",
                           LABEL,"    ",
//                           LABEL,LAB_DEQCINVY+1,
                           CHECKI,LAB_DEQCINVY,prefs.rotor.inv_y,

                           0
                          );
  wg[4]=Create_ButtonArray("Storm position",pref_commands,-4,
                           LABEL,LAB_STORMX+1,
                           SPIN,LAB_STORMX,"%d%d%d%d",1,(int)prefs.rotor.storm.x,0,180,
                           LABEL,LAB_STORMY+1,
                           SPIN,LAB_STORMY,"%d%d%d%d",1,(int)prefs.rotor.storm.y,0,180,

                           LABEL,LAB_STORMXW+1,
                           SPIN,LAB_STORMXW,"%d%d%d%d",1,(int)prefs.rotor.storm_wait_x,0,120,
                           LABEL,LAB_STORMYW+1,
                           SPIN,LAB_STORMYW,"%d%d%d%d",1,(int)prefs.rotor.storm_wait_y,0,120,

                           LABEL,LAB_STORME+1,
                           SPIN,LAB_STORME,"%f%f%f%f",1.,prefs.rotor.storm.elev,-30.,180.,
                           LABEL,LAB_STORMA+1,
                           SPIN,LAB_STORMA,"%d%d%d%d",1,(int)prefs.rotor.storm.azim,0,180,


/*
                           LABEL,"X:",
                           SPIN,LAB_STORM_X,"%d%d%d",(int)prefs.rotor.storm.x,0,180,
                           SPIN,LAB_STORM_WX,"%d%d%d%d",10,prefs.rotor.storm_wait_x,0,120,
                           LABEL,"Y:",
                           SPIN,LAB_STORM_Y,"%d%d%d",(int)prefs.rotor.storm.y,0,180,
                           SPIN,LAB_STORM_WY,"%d%d%d%d",10,prefs.rotor.storm_wait_y,0,120,
*/
                           0
                          );
wg[5]=NULL;/*
  wg[5]=Create_ButtonArray("Flip dir.",pref_commands,2,
                           LABEL,LAB_DEQCINVX+1+5,
                           CHECKI,LAB_DEQCINVX,prefs.inv_x,
                           LABEL,LAB_DEQCINVY+1+5,
                           CHECKI,LAB_DEQCINVY,prefs.inv_y,
                           0
                          );
*/

  wg[2]=Pack("rotorsystem",'v',wg[2],5,wg[3],5,wg[4],5,wg[5],5,NULL);
//------------------------------------------------------------------------------------------

  wg[4]=Create_Check(LAB_EXTPROGON,pref_commands,prefs.ext_prog_on); 
  wg[5]=Create_Entry(LAB_EXTPROGUP,pref_commands,"%-30s",prefs.prog_trackup); 
  wg[6]=Create_Entry(LAB_EXTPROGDN,pref_commands,"%-30s",prefs.prog_trackdown); 
  wg[3]=Pack("Ext. prog",'v',wg[4],3,wg[5],3,wg[6],3,NULL);
//------------------------------------------------------------------------------------------
  wg[4]=Create_Label("Use ';' as delimiter.\nDon't add leading/trailing spaces!\nExample: NOAA *;METOP-B");
  wg[4]=Pack(NULL,'h',wg[4],10,NULL);
  wg[5]=Create_Entry(LAB_SATLIST,pref_commands,"%-30s",prefs.satlist);
  wg[4]=Pack(NULL,'v',wg[4],10,wg[5],1,NULL);

  wg[0]=Create_Notebook(NULL,GTK_POS_TOP,
                     "Observator"         ,wg[1],
                     "Rotorconfig"        ,wg[2],
                     "Commands"           ,wg[3],
                     "Satellites"         ,wg[4],
                     NULL);
  return wg[0];
}

/*************************************
 * Open Preferences window
 *************************************/
void MenuPrefs(GtkWidget *widget)
{
  GtkWidget *pref_window;
  GtkWidget *main_window;
  GtkWidget *wn,*wa[3],*wb[7],*wc[7],*wd[2],*wd1[10],*wd2[10],*wd3[10],*we[5],*wf[3],*wg[6];
  GtkWidget *wt[3],*wo[4];
  GtkWidget *wz;

  main_window=Find_Parent_Window(widget);

/* Create new window or pop-up it if it already exist. */
  if (!(pref_window=Create_Window(main_window,0,0,Lab_Prefwnd,NULL))) return;

/* Add buttons etc. at top*/
  if (exist_file(PREF_FILE))
    wt[1]=Create_Entry(LAB_LOPREF,NULL,PREF_FILE); 
  else if (exist_file(PREF_FILE_OLD))
    wt[1]=Create_Entry(LAB_LOPREF,NULL,PREF_FILE_OLD); 
  else
    wt[1]=Create_Entry(LAB_LOPREF,NULL,"Not found."); 

  if (exist_file(prefs.used_preffile))
    wt[2]=Create_Entry(LAB_GLPREF,NULL,prefs.used_preffile); 
  else
    wt[2]=Create_Entry(LAB_GLPREF,NULL,"Not found"); 

  wt[0]=Pack("Preference file",'v',wt[1],5,wt[2],5,NULL);

/* ---------- Part Window ---------- */
  wa[1]=Create_Optionmenu(LAB_WNDPOS,pref_commands,(prefs.main_wnd_geom==abs_wndpos? 1 : 0),
                        LAB_WNDPOS_DFLT,
                        LAB_WNDPOS_SMART,
                        0
                       );

  wa[2]=Create_ButtonArray(LAB_WNDSIZE,pref_commands,2,
                         SPIN,LAB_WND_WIDTH,"%d%d%d",prefs.main_wnd_w,0,1600,
                         SPIN,LAB_WND_HEIGHT,"%d%d%d",prefs.main_wnd_h,0,1200,
                         0
                        );
  wa[0]=Pack("Window attributes",'h',wa[1],5,wa[2],5,NULL);

/* ---------- Part Interface ---------- */
//  wb[1]=Create_Check(LAB_ETH,pref_commands,prefs.port_type);
  wb[1]=Create_ButtonArray(LAB_INTERF,pref_commands,3,
#ifdef __ADD_PAR__
                        RADIO,LAB_PAR,
#endif
#ifdef __ADD_ETH__
                        RADIO,LAB_ETH,
#endif
#ifdef __ADD_USB__
                        RADIO,LAB_USB,
#endif
                        0);

  wb[2]=NULL;
#ifdef __ADD_PAR__
  wb[2]=Create_ButtonArray(LAB_PARPORT,pref_commands,2,
                        RADIO,LAB_P1,
                        RADIO,LAB_P2,
                        RADIO,LAB_PN,
                        ENTRY,LAB_PE,"%10s","",
                        CHECK,LAB_FASTPPACC,
                        0
                       );

  wb[3]=Create_ButtonArray(LAB_SEL_HWTYPE,pref_commands,2,
                         RADIO,LAB_SEL_DECGEN,
                         RADIO,LAB_SEL_DECTYPE,
                         0
                        );
  wb[2]=Pack(NULL,'v',wb[2],10,wb[3],2,NULL);
#else
  wb[2]=Create_Label("");
#endif

  wb[4]=NULL;
#ifdef __ADD_USB__
  wb[4]=Create_ButtonArray(LAB_USBCHOICE,usb_commands,2,
                        RADIOs,LAB_USB_USEFIRST,
                        LABEL,"",
                        RADIOn,LAB_USB_USEID,
                        ENTRY,LAB_USBID,"%30s","",
                        0
                       );
  wb[5]=Create_Text(LAB_USBINFO,FALSE,"Courier 10");
  wb[6]=Create_Button(LAB_TESTUSB,usb_commands);
  wb[5]=Pack("Found FTD devices",'v',wb[5],1,wb[6],1,NULL);
  wb[3]=Create_Check(LAB_USBFAST,usb_commands,prefs.fast_ftd);
  wb[4]=Pack(LAB_USBPORT,'v',wb[3],5,wb[4],5,wb[5],5,NULL);
#endif

  wb[0]=Pack("",'v',wb[1],10,wb[2],2,wb[4],2,NULL);

/* ---------- Part Dirs/files ---------- */
  wc[1]=Create_Entry(LAB_DIR_SAVE,pref_commands,"%-30s",prefs.dir_name); 
  wc[2]=Create_Entry(LAB_FNTMPL,pref_commands,"%-30s",prefs.base_name); 

  wc[3]=Create_Entry(LAB_DIR_KEP,pref_commands,"%-30s",prefs.kep_dirname); 
  wc[4]=Create_Entry(LAB_FILE_KEP,pref_commands,"%-30s",prefs.kep_filename); 
  wc[3]=Pack("Kepler file",'v',wc[3],3,wc[4],3,NULL);
  wc[4]=NULL;
  wc[0]=Pack("",'v',wc[1],10,wc[2],2,wc[3],2,wc[4],2,NULL);

/* ---------- Part Record ---------- */
  wd1[1]=Create_ButtonArray(LAB_HRPTRECTIMES,pref_commands,2,
          SPIN,LAB_HRPT_START,"%d%d%d",prefs.hrpt_start_after_in_sync,0,1023,
          SPIN,LAB_HRPT_STOP,"%d%d%d",prefs.hrpt_stop_after_out_sync,0,1023,
                         0
                        );
  wd1[2]=Create_ButtonArray(LAB_HRPTFILENAMES,pref_commands,3,
          LABEL,"HRPT",
          ENTRY,LAB_HRPTFILE_BASE,"%-5s",prefs.file_hrptbase,
          ENTRY,LAB_HRPTFILE_EXT ,"%-5s",prefs.file_hrptext,
          LABEL,"METEOR",
          ENTRY,LAB_MHRPTFILE_BASE,"%-5s",prefs.file_mhrptbase,
          ENTRY,LAB_MHRPTFILE_EXT ,"%-5s",prefs.file_mhrptext,
          LABEL,"METOP",
          ENTRY,LAB_METOPFILE_BASE,"%-5s",prefs.file_metopbase,
          ENTRY,LAB_METOPFILE_EXT ,"%-5s",prefs.file_metopext,
          LABEL,"Fengyun3",
          ENTRY,LAB_FY3FILE_BASE,"%-5s",prefs.file_fy3base,
          ENTRY,LAB_FY3FILE_EXT ,"%-5s",prefs.file_fy3ext,
          0
         );
  wd1[3]=Create_Check(LAB_METFY3X,pref_commands,prefs.metfy3x);
  wd1[2]=Pack("For metop and fengyun:",'v',wd1[2],1,wd1[3],1,NULL);

#if _HRI1_
  wd3[1]=Create_ButtonArray(LAB_HRIRECTIMES,pref_commands,2,
          SPIN,LAB_HRI_START,"%d%d%d",prefs.hri_start_after_in_sync,0,1023,
          SPIN,LAB_HRI_STOP,"%d%d%d",prefs.hri_stop_after_out_sync,0,1023,
                         0
                        );
  wd3[2]=Create_ButtonArray(LAB_HRIFILENAMES,pref_commands,2,
          ENTRY,LAB_HRIFILE_BASE,"%-5s",prefs.file_hribase,
          ENTRY,LAB_HRIFILE_EXT ,"%-5s",prefs.file_hriext,
                         0
                        );
#endif
  wd1[5]=Create_Spin(LAB_LPS,pref_commands,"%d%d%d",
                            prefs.hrpt_lines_per_screen,0,10000);
  wd1[6]=Create_Toggle(LAB_DIR_NSN,pref_commands,FALSE);
  wd1[7]=Create_Optionmenu(LAB_VCHR,pref_commands,prefs.sh_recchan-1,
                        LAB_CH1_V,LAB_CH2_V,LAB_CH3_V,LAB_CH4_V,LAB_CH5_V,
                        LAB_CH6_V,LAB_CH7_V,LAB_CH8_V,LAB_CH9_V,LAB_CH10_V,
                        0);

  wd1[6]=Pack(NULL,'v',wd1[6],5,wd1[7],5,NULL);
  wd1[5]=Pack("Show during record",'h',wd1[5],1,wd1[6],1,NULL);

  wd1[8]=Create_Check(LAB_SAVREC,pref_commands,prefs.save_flag);
  wd1[9]=Create_Check(LAB_KEPREC,pref_commands,prefs.auto_get_kepler);
  wd1[8]=Pack("Action after record",'h',wd1[8],1,wd1[9],1,NULL);

  wd1[0]=Pack("",'v',wd1[1],5,wd1[5],5,wd1[8],5,NULL);
  wd3[0]=NULL;
#if _HRI1_
  wd3[0]=Pack("HRI parameters",'v',wd3[1],5,wd3[2],5,NULL);
#endif
//  wd2[0]=Pack("METOP parameters",'v',wd2[1],5,wd2[2],5,NULL);
//  wd2[0]=Pack(NULL,'v',wd2[0],1,wd3[0],1,NULL);
wd2[0]=NULL;
  wd[0]=Pack(NULL,'h',wd1[0],10,wd1[2],10,NULL);

/* ---------- Part Algos ---------- */

//  we[1]=Create_Check(LAB_LINEARIZE,pref_commands,prefs.linearize);
  we[1]=Create_ButtonArray("Linearization",pref_commands,1,
                         CHECK,LAB_LINEARIZE,//prefs.linearize,
                         CHECK,LAB_XYREL,//prefs.linearize,//keep_xyrel,
                         0);
  we[2]=Create_Spin(LAB_FIXZOOM,pref_commands,"%f%f%f",
                                      prefs.fix_zoomfactor,1.2,10.);
  we[3]=Create_ButtonArray(LAB_DESPEC_LIM,pref_commands,2,
                         SPIN,LAB_SPECK_SUL,"%d%d%d",prefs.sur_ulim,0,1023,
                         SPIN,LAB_SPECK_SPL,"%d%d%d",prefs.dsp_llim,0,1023,
                         0
                        );

  we[1]=Pack("",'h',we[1],10,NULL);
  we[2]=Pack("",'h',we[2],10,NULL);
  we[0]=Pack(NULL,'h',we[1],10,we[2],10,NULL);
  we[0]=Pack(NULL,'v',we[0],10,we[3],10,NULL);

/* ---------- Part Colors ---------- */
  wf[1]=Create_Spin(LAB_FSIZE,pref_commands,"%d%d%d%d",
                            5,prefs.font_size,0,500);

  wf[2]=Create_ButtonArray(LAB_GRIDCLR,pref_commands,3,
                        SPIN,LAB_GRID_RED,"%8d%8d%8d",prefs.grid_clr.red,0,255,
                        SPIN,LAB_GRID_GREEN,"%8d%8d%8d",prefs.grid_clr.green,0,255,
                        SPIN,LAB_GRID_BLUE,"%8d%8d%8d",prefs.grid_clr.blue,0,255,
                         0
                        );
  wf[0]=Pack(NULL,'v',wf[1],10,wf[2],10,NULL);
  Set_Button(we[0],LAB_LINEARIZE,prefs.linearize);
  Set_Button(we[0],LAB_XYREL,prefs.keep_xyrel);

  wg[0]=trackbook();
/* The notebook */
  wn=Create_Notebook(NULL,GTK_POS_TOP,
                     "Window"         ,wa[0],
                     "Interface"      ,wb[0],
                     "Dirs/files"     ,wc[0],
                     "Record"         ,wd[0],
                     "Algos"          ,we[0],
                     "Colours & Fonts",wf[0],
                     "Tracker"        ,wg[0],
                     NULL);

/* ---------- Buttons bottom ---------- */
  wo[1]=Create_Button(LAB_Use,pref_commands);
  wo[2]=Create_Button(LAB_Save,pref_commands);
  wo[3]=Create_Button(LAB_Exit,pref_commands);

  wo[0]=Pack("",'h',wo[1],10,wo[2],10,wo[3],10,NULL);

  wz=Pack(NULL,'v',wt[0],5,wn,5,wo[0],5,NULL);
  gtk_container_add(GTK_CONTAINER(pref_window),wz);

  gtk_widget_show_all(pref_window);

/* Set buttons to current state */
  switch(prefs.port_type)
  {
    case ethernet : Set_Button(pref_window,LAB_ETH,TRUE); break;
    case usb      : Set_Button(pref_window,LAB_USB,TRUE); break;
    default       : Set_Button(pref_window,LAB_PAR,TRUE); break;
  }

  Sense_Button(pref_window,LAB_PE,FALSE);
  switch(prefs.parport_nr)
  {
    case 1 : Set_Button(pref_window,LAB_P1,TRUE); break;
    case 2 : Set_Button(pref_window,LAB_P2,TRUE); break;
    default: Set_Button(pref_window,LAB_PN,TRUE);
             Sense_Button(pref_window,LAB_PE,TRUE);
    break;
  }
  Set_Entry(pref_window,LAB_PE,"%x",prefs.parport_address);

  switch(prefs.port_type)
  {
    case usb:
      Set_Button(pref_window,LAB_USB,TRUE);
    break;
    case ethernet:
      Set_Button(pref_window,LAB_ETH,TRUE);
    break;
    default:
      Set_Button(pref_window,LAB_PAR,TRUE);
      Show_Button(pref_window,LAB_USBPORT,FALSE);
    break;
  }

  if (prefs.usb_use_id)
    Set_Button(pref_window,LAB_USB_USEID,TRUE);
  else
    Set_Button(pref_window,LAB_USB_USEFIRST,TRUE);

  Set_Entry(pref_window,LAB_USBID,"%s",prefs.usbport_id);

  Set_Button(pref_window,LAB_SEL_DECTYPE,prefs.hwsel_type);
  Set_Button(pref_window,LAB_VCHR,prefs.sh_recchan-1);

  if (prefs.fast_parportaccess)
    Set_Button(pref_window,LAB_FASTPPACC,TRUE);
  else
    Set_Button(pref_window,LAB_FASTPPACC,FALSE);

//  switch(prefs.main_wnd_geom)
//  {
//    case abs_wndpos: Set_Button(pref_window,LAB_WNDPOS,1);  break;
//    default        : Set_Button(pref_window,LAB_WNDPOS,0);  break;
//  }

  switch(prefs.hrpt_scan_ver)
  {
    case S_N: Set_Button(pref_window,LAB_DIR_NSN,TRUE);  break;
    default : Set_Button(pref_window,LAB_DIR_NSN,FALSE);  break;
  }

  Set_Button(pref_window,LAB_SAVREC,prefs.save_flag);
  Set_Button(pref_window,LAB_KEPREC,prefs.auto_get_kepler);

  Set_Button(pref_window,LAB_ROTORSYS,prefs.rotor.use_xy);
  Set_Button(pref_window,LAB_XY_DISH,prefs.rotor.x_at_disc);
  Set_Button(pref_window,LAB_SETLIM,1);
  Set_Button(pref_window,LAB_X_DIR,prefs.rotor.x_west_is_0);
  Set_Button(pref_window,LAB_Y_DIR,prefs.rotor.y_south_is_0);


  Sense_Button(pref_window,LAB_METOPFILE_BASE,!prefs.metfy3x);
  Sense_Button(pref_window,LAB_FY3FILE_BASE,!prefs.metfy3x);

}
