/**************************************************
 * RCSId: $Id: rawd_qpsk.c,v 1.5 2017/07/05 17:38:39 ralblas Exp $
 *
 * HRPT record functions.
 * Based on FPGA-decoder, connected to parallel port.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: rawd_qpsk.c,v $
 * Revision 1.5  2017/07/05 17:38:39  ralblas
 * _
 *
 * Revision 1.4  2017/01/04 15:45:13  ralblas
 * _
 *
 * Revision 1.3  2016/11/13 22:43:23  ralblas
 * _
 *
 * Revision 1.2  2016/09/30 06:55:40  ralblas
 * _
 *
 * Revision 1.1  2016/03/24 20:34:45  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include "wsat.h"
#include "menu_wsat.h"
#include "hrpt_funcs.h"
extern PREFS prefs;
#define LAB_IQTEST    "IQTEST"
#define LAB_OTHER    "(other)"
#define LAB_BASEFR "^VCO fr"
#define LAB_Start   "Start/Stop"
#define LAB_Stop    "Stop"
#define LAB_Save    "Save"
#define LAB_IGNSYNC "Ignore sync"
#define LAB_ADVANCED  "Advanced"
#define LAB_ADV_SET   "!advanced"
#define LAB_Exit      " Close "
#define LAB_IQTESTwnd   "Record IQ"
#define LAB_Recording "Recording"
#define LAB_RECLINES  "kbytes received"
#define LAB_MAXY "^Lines/screen"
#define LAB_GENDIV "!Div generator"
#define LAB_DECDIV "!Div decoder"
#define LAB_GENBPS "!bps generator"
#define LAB_DECBPS "!bps decoder"
#define LAB_METOP "2MSymb/s"
#define LAB_LRPT  "40kSymb/s"

static GtkWidget *hide_but;
static void get_sat_set_but_rawd(RECORDINFO *rdi,GtkWidget *iqtest_window)
{
#ifdef __ADD_USB__
  if (rdi->port_type==usb)
  {
    int channr=get_sat(&rdi->usb);

    switch(channr)
    {
      case 7 : Set_Button(iqtest_window,LAB_IQTEST,1); rdi->sat_type=IQTEST; rdi->file_type=FT_IQTEST; break;
      default: Set_Button(iqtest_window,LAB_OTHER,1);  rdi->sat_type=sat_unk;  break;
    }
  }
#endif
}

static int dump_IQ_line(PICINFO *pci,guint16 *frame)
{
  FILE *fp=pci->fp;
  char tmp;
  int i;
  for (i=0; i<pci->fri.words_per_frame; i++)
  {
    tmp=frame[i]&0xff;
    fwrite(&tmp,1,1,fp);
  }
  return 1;
}


#define BUF_SIZE 1024
#define MUXSELECTBIT 0x80
#define SWAPBITS 1
static int get_usb_data(RECORDINFO *rdi,int *word)
{
  static int nrbytes;
  static DWORD dwBytesRead;
  static unsigned char pcBufRead[BUF_SIZE+10],inp;
  int done;
  static int hword;
  static int single_byte=1;

  FT_STATUS ftStatus;
  FT_HANDLE ftHandle=rdi->usb.ftHandle;
  if (!word)
  {
    nrbytes=0;
    return 0;
  }

  done=0;
  while (!done)
  {
    if (!nrbytes)          // no bytes in buffer -> needs read
    {                      // check status
#if __GTK_WIN32__ == 1
      ftStatus = ft_Read(ftHandle, pcBufRead, BUF_SIZE, &dwBytesRead);
#else
      ftStatus = FT_Read(ftHandle, pcBufRead, BUF_SIZE, &dwBytesRead);
#endif
      if ((ftStatus != FT_OK) || (dwBytesRead<BUF_SIZE))
      { // buffer not full, so time to do 'something'
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
      }
      nrbytes=dwBytesRead;
    }
    if (nrbytes>0)
    { // data in USB-buffer -> read it
      // in HW: first bit(s) at LSB: Q3I3 Q2I2 Q1I1 Q0I0 
      inp=pcBufRead[dwBytesRead-nrbytes];
      if (single_byte)
      {
#if SWAPBITS == 1
        // swapping ==> I0Q0 I1Q1 I2Q2 I3Q3
        // fits better with GNURadio
        int i;
        *word=0;
        for (i=0; i<8; i++)
        {
          (*word)+=  ((inp>>i)&1)<<(7-i);
        }
#else
        *word=inp;
#endif
        done=1;
      }
      else
      {
        if (!(inp&MUXSELECTBIT))
        {
          hword=((inp&0x30)<<4) + (inp&0x0f);                // [9:8] [3:0]
	  *word=0;
        }
        else
        {
          *word=hword | ((inp&0x30)<<6) | ((inp&0x0f)<<4) ; // [11:10] [7:4]
          done=1;
        }
      }
      nrbytes--;  // 
    }
  }
  return 1;
}

// CHRPT: 130*10 kb/s=1.3 Mb/s = 162 kbyte/s
// overdracht: 2 bytes per woord dus 130*16/8=260 kbyte/s
#define STARTFRAME_USB(reg)  ((reg & 0x0400))
#define INSYNC_USB(reg) ((reg & 0x0800))
int usb_get_iqdata(guint16    *frame,RECORDINFO *rdi)
{
  int datstat;
  int words_remaining;
  gboolean start_frame_flag=FALSE;
  int cnt_nosig=0;
  words_remaining=rdi->fri.words_per_frame;
  while (words_remaining > 0)              /* Read all words in a frame */
  {
/* Get data from parallel port */
    if (!(get_usb_data(rdi,&datstat)))
    {
      return 0;                            /* Signal lost! */
    }

/* Check if received word contains a frame sync pulse from decoder.
   If sync pulse received, this is the first word of a frame.
   (Normally, this is the case each time getpp_hri_frame is entered.)
   NOTE This is NOT the decoder in-sync flag!
*/
    if (rdi->sat_type==IQTEST) start_frame_flag=TRUE;
    if (STARTFRAME_USB(datstat))
    {
      if (!rdi->ignore_sync)
        if (start_frame_flag) break;
/* Now, enable to save remaining words of frame. */
      start_frame_flag=TRUE;
    }

/* Save remaining words of frame. */
    if (rdi->ignore_sync)
      start_frame_flag=TRUE;

    if (start_frame_flag)
    {
      *frame=datstat & 0x3ff;
      frame++;
      words_remaining--;
      cnt_nosig=0;
    }
    else
    {
/* If no readyn: make sure that now and then gui is activated */
      cnt_nosig++;
      if (cnt_nosig>1000)
      {
        while (g_main_iteration(FALSE));
        if (!rdi->recording) break;
        cnt_nosig=0;
      }
    }
  }
  if (!INSYNC_USB(datstat)) return 2;        /* decoder not in-sync. */
//g_usleep(170000); // test buffer
  return 1;                                 /* decoder in-sync. */
}

/*************************************
 * Record one LRIT picture.
 *************************************/
void Recordpp_one_IQTest(GtkWidget  *iqtest_window,
                       GtkWidget  *main_window,
                       RECORDINFO *rdi,
                       PICINFO    *pci,
                       VIEWINFO   *vwi)
{
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  RGBI *rgbi=Get_RGBI(drawing_area);
  PICINFO pci_w;
  int xdrawmax=0;
  int ydrawmax=0;
  int ydraw=0,y=0;
  int nr_frames;
  guint16 *frame;
  int out_sync_counter=0;
  gboolean run_record=TRUE;
  int status,prev_status=-1;
  int i;
  int ypos;

  guint *xmap=NULL;
  guint *ymap=NULL;
  guint *zmap=NULL;
  memset(&pci_w,0,sizeof(pci_w));

  if (drawing_area)
  {
    xdrawmax=drawing_area->allocation.width;
    ydrawmax=drawing_area->allocation.height;
  }
  
/* If a file was open for read, close it */
  close_picfile(pci);


/* open file if the record has to be saved */
  if (rdi->save_flag)
  {
    if (!(open_new_recfile(rdi->file_prefix,rdi->file_suffix,rdi)))
    {
      Create_Message("Error","Can't open file %s!",rdi->file_name);
      return;
    }
    Set_Entry(iqtest_window,LAB_Recording,"%s",rdi->file_name);
    if (rdi->batch_mode) log_message("Start recording in %s.",rdi->file_name);
       
    pci_w.width=0;
    pci_w.height=0;
    pci_w.ox=0;
    pci_w.oy=0;
    pci_w.scan_ver=rdi->scan_ver;
    for (i=0; i<NR_CHANNELS_MAX; i++) 
      pci_w.channels_recorded[i]=rdi->channels_recording[i];
    pci_w.file_type=rdi->file_type;
    pci_w.fp=rdi->fp;
    strcpy(pci_w.sat_name,rdi->sat_name);

/* Get computer time, to be stored in file header */
    {
      time_t t;
      struct tm *tm;
      t=time(NULL);
      if (!(tm=gmtime(&t))) tm=localtime(&t);
      pci_w.orbit.start_tm_comp=*tm;
      pci_w.orbit.start_tm=*tm;
    }
    pci_w.orbit.use_sattime=FALSE;
    pci_w.fri=calc_hrptframeinfo(pci_w.file_type,pci_w.channels_recorded,pci_w.width);
  }
  else
  {
    Set_Entry(iqtest_window,LAB_Recording,"(no save)");
  }

/* allocate mem to hold 1 frame */
  frame=(guint16 *)malloc(rdi->fri.words_per_frame*sizeof(guint16));

/* Define some view characteristics for the channel to show real-time */
  vwi->zval=zoom_full(0,0,0,0);
  vwi->scan_hor=W_E;
  vwi->scan_ver=N_S;
  vwi->lum_min=0;
  vwi->lum_max=255;

/* Determine mapping of this channel.
   ymap from map_pic is translation pic_y -> screen_y, but here vice-versa 
   is needed. So, ymap can only be used to handle sat-direction.
   Actual sizing is done lateron.
*/
  pci->height=ydrawmax;             /* set 1:1 mapping */
  vwi->scan_ver=rdi->scan_ver;
  vwi->scan_hor=rdi->scan_hor;
  vwi->channel_selected[1]=TRUE;
  map_pic(main_window,pci,vwi,FALSE,&xmap,&ymap,&zmap);

/* Start the record loop. Each loop is 1 frame. */
  nr_frames=0;
  while ((run_record) && (rdi->recording))
  {
    // do only if window is resized
    if (drawing_area)
    {
      if ((xdrawmax!=drawing_area->allocation.width) || (ydrawmax!=drawing_area->allocation.height))
      {// size window did change -> adapt x and y mapping
        if (xmap) free(xmap); xmap=NULL;
        if (ymap) free(ymap); ymap=NULL;
        if (zmap) free(zmap); zmap=NULL;
        map_pic(main_window,pci,vwi,FALSE,&xmap,&ymap,&zmap);
        xdrawmax=drawing_area->allocation.width;
        ydrawmax=drawing_area->allocation.height;
      }
    }

/* Catch gtk-events (e.g. stop-signal, which makes rdi->recording=FALSE.) */
    if (rdi->batch_mode)
    {
    }
    else
    {
      while (g_main_iteration(FALSE));
    }

/* Get a 1k-block. Abort recording if failed. */
// status: 1=ok, 2=out-sync, 0=?? never happens?
//    status=get_hrpt_frame(frame,rdi);
    status=usb_get_iqdata(frame,rdi);

    if ((prev_status>=0) && (status!=prev_status))
    {
      if ((status) && (!prev_status))
        Set_Entry(iqtest_window,LAB_Recording,"Restored.");
      else if (!status)
        Set_Entry(iqtest_window,LAB_Recording,"No transm!");
    }
    prev_status=status;

/* Handle in-sync info decoder */
    if (status==2)                            /* out-sync */
      out_sync_counter++;
    else                                      /* in-sync */
      out_sync_counter=0;
    if (rdi->sat_type!=IQTEST)
    {
      if ((rdi->hrpt_stop_after_out_sync) && (out_sync_counter > rdi->hrpt_stop_after_out_sync))
        break;                                  /* abort by decoder */
    }

    if (main_window)
    {
      int height=MAX(rdi->height,drawing_area->allocation.height);
/* Do actual mapping */
      ypos=y*(ydrawmax-1)/height;

/* Restart at top if canvas is full */
      if (ypos>=ydrawmax) y=0;
      ydraw=*(ymap+ypos);

/* solve under/overflow (just in case; should never happen) */
      if (ydraw<0) ydraw=0;
      if (ydraw>=ydrawmax) ydraw=ydrawmax-1;

/* Draw the received line into the canvas. */
      if (rdi->sat_type==IQTEST)
      {
        draw_a_line_bw(frame,xdrawmax,ydraw,xmap,zmap,1023,rgbi,pci,NULL);
      }
      y++;
      Refresh_Rect(drawing_area,0,ydraw,
                   drawing_area->allocation.width,
                   1);

      if (!(nr_frames%10)) Set_Entry(iqtest_window,LAB_RECLINES,"%d",nr_frames);
    }
    else if (rdi->batch_mode)
    {   
      y++;
      if (!(y%10)) log_message("kbytes %d...",y);
    }

/* Save frame */
    if (rdi->save_flag)
    {
#if _IQTEST1_
      if (rdi->sat_type==IQTEST)
      {
        dump_IQ_line(&pci_w,frame);
      }
#endif
    }

    nr_frames++;

  }

/* End of recording. */
  if (rdi->save_flag)
  {
    close_recfile(rdi);
  }
  free(frame);
  Set_Entry(iqtest_window,LAB_Recording,"Stop");
  if (rdi->batch_mode) log_message("Stop recording after %d frames.",nr_frames);

  free(xmap);
  free(ymap);
  free(zmap);

  return;
}

static void set_rec_par(RECORDINFO *rdi,PICINFO *pci)
{
  pci->clrmap.size=1024;
  pci->width=1024;
  pci->height=500;
  pci->file_type=rdi->file_type;
  pci->sat_type=rdi->sat_type;
  // usb: always all channels recorded.
  // Set recording/recorded channels; needed after reading sattype from decoder
  if (rdi->port_type==usb)
  {
    rdi->channels_recording[0]=TRUE;
    rdi->channels_recording[1]=TRUE;
    pci->channels_recorded[0]=rdi->channels_recording[0];
    pci->channels_recorded[1]=rdi->channels_recording[1];
  }

/* calculate some frame charactersitics */
  pci->fri=calc_hrptframeinfo(pci->file_type,pci->channels_recorded,pci->width);
  rdi->fri=pci->fri;
  strcpy(rdi->file_prefix,"qpsk");
  strcpy(rdi->file_suffix,".dat");
}

/*************************************
 * Record LRIT.
 *************************************/
void Recordpp_IQTest(GtkWidget *iqtest_window,
                   GtkWidget *main_window,
                   RECORDINFO *rdi,
                   PICINFO *pci,
                   VIEWINFO *vwi)
{
  int i;
/* Check if it is allowed to access the selected port */
  if (!Open_HWConnect(rdi))
  {                              // err returned!
    switch(rdi->port_type)
    {
      case usb:
        if (main_window)
          Create_Message("Error","Can't open USB port %s",
            (rdi->usb.usb_use_id?rdi->usb.usbport_id : ""));
        else
          fprintf(stderr,"Can't open USB port.");
      break;
      case ethernet:
        if (main_window)
          Create_Message("Error","Can't open Ethermet port.");
        else
          fprintf(stderr,"Can't open Ethermet port.");
      break;
      default:
      break;
    }
    return;
  }

/* Copy recording channel info into picinfo struct */
  for (i=0; i<2; i++)
  {
    pci->channels_recorded[i]=rdi->channels_recording[i];
  }

/* Determine some pic-parameters */
  set_rec_par(rdi,pci);
/* Start the record loop; Each loop is 1 picture. */
  while (rdi->recording)
  {
/* Wait until the decoder is in-sync for some time */
    Set_Entry(iqtest_window,LAB_Recording,"Wait");
    if (rdi->batch_mode) log_message("Wait for in-sync.");
    get_sat_set_but_rawd(rdi,iqtest_window);
    while (g_main_iteration(FALSE));

#ifdef __ADD_USB__
    if (rdi->port_type==usb) ena_dat(&rdi->usb);
#endif

    while (g_main_iteration(FALSE));
    set_rec_par(rdi,pci);

    Recordpp_one_IQTest(iqtest_window,main_window,rdi,pci,vwi);

/* If just 1 record then close the program (record finished) */
    if (rdi->one_record)
    {
      rdi->recording=FALSE;
      Close_HWConnect(rdi);
      if (main_window) gtk_main_quit(); else return;
    }
  }
  Set_Entry(iqtest_window,LAB_Recording,"Off");
  Close_HWConnect(rdi);
  
/* Open file if recorded */
  if ((!rdi->batch_mode) && (rdi->save_flag))
  {
    strcpy(pci->file_name,rdi->file_name);
    open_file(pci,vwi,main_window,0);
  }
  if (rdi->batch_mode) log_message("End recording.");
}

#define SWTYPE_STOPSTARTn
/*************************************
 * Get button signals for Record functions
 *************************************/
static void Record_Raw_Commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *main_window=Find_Window(Find_Parent_Window(widget),Lab_Main);
  GtkWidget *iqtest_window=Find_Window(main_window,LAB_IQTESTwnd);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);

  char *name;
  int i;
  static int base_freq=20;
  if (!rdi) return;  /* create_options_menu activates this func before everything is settled! */

  name=(char *)data;

  if (!strcmp(name,LAB_MAXY))
  {
    rdi->height=GTK_ADJUSTMENT(widget)->value;
  }

  if (!strcmp(name,LAB_IQTEST))
  {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      rdi->file_type=FT_IQTEST;
      rdi->sat_type=IQTEST;
  #ifdef __ADD_USB__
      if (rdi->port_type==usb) set_sat(&rdi->usb,7); 
  #endif
    }
  }


/* channels_recording: only 1 (N.A. for LRIT)
*/
  for (i=1; i<NR_CHANNELS_MAX; i++)
    rdi->channels_recording[i]=FALSE;
  rdi->channels_recording[1]=TRUE;

/* Recording to save or not to save... */
  if (!strcmp(name,LAB_Save))
  {
    rdi->save_flag=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

  if (!strcmp(name,LAB_IGNSYNC))
  {
    rdi->ignore_sync=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }

#ifdef XXX
  if (!strcmp(name,LAB_ADVANCED))
  {
    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
    {
      hide_but=Find_Widget(iqtest_window,LAB_ADV_SET);
      if (hide_but) gtk_widget_hide(hide_but); 
    }
    else
    {
      if (hide_but) gtk_widget_show(hide_but);
      hide_but=NULL;
    }
  }
#endif

/* Start record */

  if ((!strcmp(name,LAB_Start)) && (Update_Togglelabel(widget)))
  {
    if (rdi->sat_type!=sat_unk)
    {
      rdi->recording=TRUE;
  #ifdef __ADD_USB__
      if (rdi->port_type==usb)
      {
        ena_dat(&rdi->usb);
      }
  #endif
      Sense_Button(iqtest_window,LAB_Exit,FALSE);
      Sense_Button(iqtest_window,LAB_Save,FALSE);
      Sense_Button(iqtest_window,LAB_IQTEST,FALSE);

      Recordpp_IQTest(iqtest_window,main_window,rdi,pci,vwi);

      Sense_Button(iqtest_window,LAB_Exit,TRUE);
      Sense_Button(iqtest_window,LAB_Save,TRUE);
      Sense_Button(iqtest_window,LAB_IQTEST,TRUE);
      rdi->recording=FALSE;
  #ifdef __ADD_USB__
      dis_dat(&rdi->usb);
  #endif
      Set_Button(iqtest_window,LAB_Start,FALSE);
    }
    else
    {
      Set_Button(iqtest_window,LAB_Start,FALSE);
      Create_Message("Error","No (correct) satellite choosen!\n");
    }
  }

  if ((!strcmp(name,LAB_Start)) && (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))))
  {
    rdi->recording=FALSE;
  }

  if (!strcmp(name,LAB_Stop))
  {
    rdi->recording=FALSE;
  }

  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(iqtest_window);
//    Sense_Button(main_window,MENU_HRI,TRUE);
    Sense_Button(main_window,MENU_HRPT,TRUE);
    Sense_Button(main_window,MENU_LRIT,TRUE);
  }

  if (!strcmp(name,LAB_BASEFR))
  {
    int f;
    f=GTK_ADJUSTMENT(widget)->value;
    if (f==30)
    {
      if (base_freq==20) 
        Set_Adjust(widget,LAB_BASEFR,"%d",40);
      else
        Set_Adjust(widget,LAB_BASEFR,"%d",20);
    }
    base_freq=f;
  }
  
  if (!strcmp(name,LAB_GENDIV))
  {
    int div;
    div=GTK_ADJUSTMENT(widget)->value;
    usb_write_reg(&rdi->usb,0x14,div&0xff);      // div generator LSB
    usb_write_reg(&rdi->usb,0x15,(div>>8)&0xff); // div generator MSB
    Set_Entry(iqtest_window,LAB_GENBPS,"%.3f",49.152/(float)div);
  }

  if (!strcmp(name,LAB_DECDIV))
  {
    int div;
    div=GTK_ADJUSTMENT(widget)->value;
    usb_write_reg(&rdi->usb,0x16,div&0xff); // div decoder LSB
    usb_write_reg(&rdi->usb,0x17,(div>>8)&0xff); // div decoder MSB
    Set_Entry(iqtest_window,LAB_DECBPS,"%.3f",21.292*base_freq/20./(float)div);
  }
  if (!strcmp(name,LAB_METOP))
  {
    Set_Adjust(iqtest_window,LAB_GENDIV,"%d",23);
    Set_Adjust(iqtest_window,LAB_DECDIV,"%d",10*base_freq/20);
  }
  if (!strcmp(name,LAB_LRPT))
  {
    Set_Adjust(iqtest_window,LAB_GENDIV,"%d",1228);
    Set_Adjust(iqtest_window,LAB_DECDIV,"%d",525*base_freq/20);
  }
}

static void close_wnd(GtkWidget *wnd)
{
  GtkWidget *mwnd=Find_Window(gtk_widget_get_toplevel(wnd),Lab_Main);;
  Sense_Button(mwnd,MENU_HRPT,TRUE);
//  Sense_Button(mwnd,MENU_HRI,TRUE);
  Sense_Button(mwnd,MENU_LRIT,TRUE);
//  Sense_Button(mwnd,MENU_RAWD,TRUE);
  destroy_window(wnd);
}

/*************************************
 * Record window. 
 * This function is started from the main menu.
 *************************************/
void Menu_Rec_IQTest(GtkWidget *widget)
{
  GtkWidget *w1,*w2,*w3;
  GtkWidget *wa,*wb,*wc,*wd,*we,*wf,*wg,*wh,*wq,*wx,*wy,*wz;
  GtkWidget *iqtest_window;
  GtkWidget *main_window=Find_Parent_Window(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  GdkColor startbut_clr,startbutsclr;
  startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
  startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;
  hide_but=NULL;
  if (!(iqtest_window=Create_Window(main_window,0,0,LAB_IQTESTwnd,close_wnd))) return;

/* Set policy to 'auto-resize' */
  gtk_window_set_policy((GtkWindow *)iqtest_window,FALSE,FALSE,TRUE);

//  rdi->sat_type=IQTEST;
//  rdi->file_type=FT_IQTEST;

  if (rdi->port_type==usb) usb_init_connect(&rdi->usb);
  if (rdi->port_type==usb)
  {
    int i;
    for (i=0; i<=10; i++) rdi->channels_recording[i]=TRUE;
  }
  else if (rdi->port_type==ethernet)
  {
    int i;
    for (i=0; i<=10; i++) rdi->channels_recording[i]=TRUE;
  }
  else
  {
    rdi->channels_recording[0]=TRUE;
  }
  if (!rdi->height) rdi->height=drawing_area->allocation.height;

/* ==================== Button groups ==================== */

/* -------------------- LRIT/FY3  ------------------- */
  w1=Create_Radio(LAB_IQTEST,TRUE,Record_Raw_Commands);
  w2=Create_Radio(LAB_OTHER,FALSE,Record_Raw_Commands);
  wa=Pack("Sat type",'v',w1,0,w2,0,NULL);
  wb=Create_Spin(LAB_BASEFR,Record_Raw_Commands,"%d%d%d%d",10,20,10,40);
  wa=SPack(NULL,"v",wa,"1",wb,"E1",NULL);
  w1=Create_ButtonArray("Rate",Record_Raw_Commands,3,
             LABEL,"",
             LABEL,"Div",
             LABEL,"MSymb/sec",
             LABEL,"Gen",
             SPIN,LAB_GENDIV,"%d%d%d",1,1,2047,
             ENTRY_NOFUNC,LAB_GENBPS,"%5s","",
             LABEL,"Dec",
             SPIN,LAB_DECDIV,"%d%d%d",1,1,2047,
             ENTRY_NOFUNC,LAB_DECBPS,"%5s","",
             NULL);

  w2=Create_ButtonArray("Set",Record_Raw_Commands,3,
             BUTTON,LAB_METOP,
             BUTTON,LAB_LRPT,
             NULL);


  wb=Pack("Freq",'v',w1,1,w2,1,NULL);
/* --------------------START/STOP----------------------- */
  w1=Create_Toggle(LAB_Start,Record_Raw_Commands,FALSE);
  Set_Color(w1,GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);
w2=NULL;
  wc=SPack("","hH",w1,"fe3",w2,"fe3",NULL);
/* --------------------- STATUS ------------------------- */
  w1=Create_Entry(LAB_Recording,NULL,"%-15s","Off");
  w2=Create_Entry(LAB_RECLINES,NULL,"%8d",0);
  wd=Pack("Status",'v',w1,2,w2,2,NULL);

/* ==================== Advanced Button groups ==================== */
/* --------------------- DEC/GEN ---------------------- */
  we=NULL;
  
  w1=Create_Spin(LAB_MAXY,Record_Raw_Commands,"%d%d%d",rdi->height,1,10000);
  wf=Pack("Show during record",'h',w1,5,NULL);


/* --------------------SAVE----------------------- */
  w1=Create_Check(LAB_Save,Record_Raw_Commands,rdi->save_flag);
  w3=Create_Check(LAB_IGNSYNC,Record_Raw_Commands,rdi->ignore_sync);
  wg=Pack("",'h',w1,1,w3,2,NULL);


/* --------------------Advanced / Exit----------------------- */
  wh=Create_Button(LAB_Exit,Record_Raw_Commands);
//  wq=Create_Toggle(LAB_ADVANCED,Record_Raw_Commands,FALSE);
wq=NULL;

/* --------------------PACK----------------------- */
  wx=SPack(NULL,"h",wa,"e1",wb,"1",NULL);
  wx=Pack(NULL,'v',wx,5,wc,5,wd,1,NULL);

  if (we)
  {
    wy=Pack(LAB_ADV_SET,'v',we,5,wf,5,wg,1,NULL);
  }
  else
  {
    wy=Pack(LAB_ADV_SET,'v',wf,5,wg,1,NULL);
  }

  if (wq)
    wz=SPack("","Hh",wq,"fe10",wh,"fe1",NULL);
  else
    wz=SPack("","Hh",wh,"fe1",NULL);

  wz=Pack(NULL,'v',wx,5,wy,5,wz,1,NULL);

  gtk_container_add(GTK_CONTAINER(iqtest_window),wz);

  gtk_widget_show_all(iqtest_window);
  place_window(iqtest_window,0,0,prefs.sub_wnd_geom);

  get_sat_set_but_rawd(rdi,iqtest_window);
  Sense_Button(main_window,MENU_HRPT,FALSE);
//  Sense_Button(main_window,MENU_HRI,FALSE);
  Sense_Button(main_window,MENU_LRIT,FALSE);
//  Sense_Button(main_window,MENU_IQTEST,FALSE);
  Sense_Button(iqtest_window,LAB_OTHER,FALSE);

  Record_Raw_Commands(wq, LAB_ADVANCED);

  {
    int n1,n2;
    usb_init_connect(&rdi->usb);
    usb_read_reg(&rdi->usb,0x14,&n1);      // div generator LSB
    usb_read_reg(&rdi->usb,0x15,&n2);      // div generator MSB
    n1=(n2<<8)+n1;
    Set_Adjust(iqtest_window,LAB_GENDIV,"%d",n1);
    Set_Entry(iqtest_window,LAB_GENBPS,"%.3f",49.152/(float)n1);

    usb_read_reg(&rdi->usb,0x16,&n1);      // div decoder LSB
    usb_read_reg(&rdi->usb,0x17,&n2);      // div decoder MSB
    n1=(n2<<8)+n1;
    Set_Adjust(iqtest_window,LAB_DECDIV,"%d",n1);
    Set_Entry(iqtest_window,LAB_DECBPS,"%.3f",21.292/(float)n1);
  }
}

