/**************************************************
 * RCSId: $Id: rawd_rec_pp.c,v 1.1 2016/03/24 20:34:45 ralblas Exp $
 *
 * Raw par. port record functions
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: rawd_rec_pp.c,v $
 * Revision 1.1  2016/03/24 20:34:45  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#ifdef __ADD_PAR__ 

#include <unistd.h>
#include <malloc.h>
#include "pp_dec_interf.h"
#include "wsat.h"
#include "menu_wsat.h"

#define LEN_RAWFRAME 2048

extern PREFS prefs;

/* ====== Define button text for RAW Record window ====== */
#define LAB_DEC       "Decoder"
#define LAB_GEN       "Generator"
#define LAB_RECALL    "Record all"
#define LAB_Start     "Start/Stop"
#define LAB_Save      "Save"
#define LAB_Exit      " Exit "
#define Lab_Rawdwnd   "Record RAW"
#define LAB_Recording "Recording    "
#define LAB_Cntrec    "Recorded"

/*************************************
 * Write routines
 *************************************/
int write_rawd_hdr(RECORDINFO *rdi,int height)
{
  return 1;
}

int write_rawd_line(RECORDINFO *rdi,guint16 *line)
{
  guchar tmp[6];
  FILE *fp=rdi->fp;
  int i;

  for (i=0; i<LEN_RAWFRAME; i++)
  {
    tmp[0]=*(line);
    fwrite(tmp,1,1,fp);
    line++;
  }
  return 1;
}


/*************************************
 * Calculate some hri-based frame numbers
 *************************************/
FRAMEINFO calc_rawdframeinfo()
{
  FRAMEINFO fri;
  fri.header_len=0;
  fri.leader_len=0;
  fri.inter_chan=1;
  fri.words_per_frame=LEN_RAWFRAME;
  fri.bytes_per_line=LEN_RAWFRAME;

  return fri;
}

/*************************************
 * Get a data block from parallel port.
 * Return:
 *   0 -> Abort by user
 *   1 -> OK
 *   2 -> OK, but out-of-sync (no valid data) message.
 *************************************/
int getpp_rawd_frame(guint16    *blk,        /* interf. w. rest of funcs */
                     RECORDINFO *rdi)
{
  int datstat;
  int words_remaining;
  int cnt_nosig=0;

  words_remaining=rdi->fri.words_per_frame;

  while (words_remaining > 0)              /* Read all words in a block */
  {
/* Get data from parallel port */
    if (!(get_pp_data(rdi,&datstat)))
    {

      return 0;
    }

    *blk=datstat&0xff;
    blk++;
    words_remaining--;
    cnt_nosig=0;
  }
  if (!INSYNC_PP(datstat)) return 2;        /* decoder not in-sync. */
  return 1;
}

/*************************************
 * Record one RAW (data block).
 *************************************/
void Recordpp_one_Rawd(GtkWidget  *rawd_window,
                      GtkWidget  *main_window,
                      RECORDINFO *rdi,
                      PICINFO    *pci,
                      VIEWINFO   *vwi)
{
  RGBI *rgbi=Get_RGBI(main_window);
  GtkWidget *drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  int xdrawmax=drawing_area->allocation.width;
  int ydrawmax=drawing_area->allocation.height;
  int ydraw=0;
  int nr_frames;
  guint16 *frame;

#ifdef RAW_FRM
  int out_sync_counter=0;
#endif
  gboolean run_record=TRUE;
  int status,prev_status=-1;

  guint *xmap=NULL;
  guint *ymap=NULL;
  guint *zmap=NULL;

/* If a file was open for read, close it */
  close_picfile(pci);
  
/* open file if the record has to be saved */
  if (rdi->save_flag)
  {
    if (!(open_new_recfile(rdi->file_prefix,rdi->file_suffix,rdi)))
    {
      Create_Message("Error","Can't open file %s!",rdi->file_name);
      return;
    }
    Set_Entry(rawd_window,LAB_Recording,"%s",rdi->file_name);

#if _RAWDT_
    write_rawd_hdr(rdi,0);
#endif
  }
  else
    Set_Entry(rawd_window,LAB_Recording,"(no save)");


/* allocate mem to hold 1 data block */
  frame=(guint16 *)malloc(rdi->fri.words_per_frame*sizeof(guint16));


/* Determine some pic-parameters for real-time show */
  pci->clrmap.size=256;
  pci->width=2048;
  pci->height=500;
  pci->fri=calc_rawdframeinfo();


/* Define some view characteristics for the channel to show real-time */
  vwi->zval=zoom_full(0,0,0,0);
  vwi->scan_hor=W_E;
  vwi->scan_ver=N_S;
  vwi->lum_min=0;
  vwi->lum_max=255;

/* Determine mapping of this channel */
  map_pic(main_window,pci,vwi,FALSE,&xmap,&ymap,&zmap);

/* Start the record loop. Each loop is 1 frame. */
  nr_frames=0;
  while ((run_record) && (rdi->recording))
  {
    while (g_main_iteration(FALSE));

/* Get a RAW-frame. Show if signal lost. */
    status=getpp_rawd_frame(frame,rdi);
    if ((prev_status>=0) && (status!=prev_status))
    {
      if ((status) && (!prev_status))
        Set_Entry(rawd_window,LAB_Recording,"Signal restored");
      else if (!status)
        Set_Entry(rawd_window,LAB_Recording,"!! No signal!!");
    }
    prev_status=status;

#ifdef RAW_FRM
/* Handle in-sync info decoder */
    if (status==2)                            /* out-sync */
      out_sync_counter++;
    else                                      /* in-sync */
      out_sync_counter=0;
    if (out_sync_counter > rdi->rawd_stop_after_out_sync)
      break;                                  /* abort by decoder */
#endif

    {
/* Draw the received line into the canvas. */
      draw_a_line_bw(frame,xdrawmax,ydraw,xmap,zmap,255,rgbi,NULL);
      Refresh_Rect(drawing_area,0,ydraw,
                   drawing_area->allocation.width,
                   1);
      ydraw++;

/* Restart at top if canvas is full */
      if (ydraw>=ydrawmax) ydraw=0;
    }

/* Save frame */
    if (rdi->save_flag)
#if _RAWDT_
      write_rawd_line(rdi,frame);
#endif

    nr_frames++;

  }

/* End of recording. */
  if (rdi->save_flag)
  {
    close_recfile(rdi);
  }

  free(frame);
  Set_Entry(rawd_window,LAB_Recording,"Stop");
  return;
}


/*************************************
 * Record RAW.
 *************************************/
void Recordpp_Rawd(GtkWidget *window,
                  RECORDINFO *rdi,
                  PICINFO *pci,
                  VIEWINFO *vwi)
{
  GtkWidget *mainwnd=Find_Window(window,Lab_Main);

  int addr_portd=rdi->parport_base;
  int count_rec=0;
/* Set counters */
  Set_Entry(window,LAB_Cntrec,"%d",count_rec);

/* calculate some frame charactersitics */
  rdi->fri.words_per_frame=LEN_RAWFRAME;
  rdi->fri.bytes_per_line=LEN_RAWFRAME;

/*
  strcpy(rdi->file_prefix,rdi->file_rawdbase);
  strcpy(rdi->file_suffix,rdi->file_rawdext);
*/
  strcpy(rdi->file_prefix,"raw");
  strcpy(rdi->file_suffix,".dat");

/* Check if we are allowed to access the parallel port */
  if (ioperm(addr_portd,3,1))
  {
    Create_Message("Error",PAR_ERR_MESSAGE,addr_portd);
    return;
  }

/* Start the record loop; Each loop is 1 picture. */
  while (rdi->recording)
  {
/* Wait until the decoder is in-sync for some time */
    Set_Entry(window,LAB_Recording,"Wait");
#ifdef RAW_FRM
    if (!(wait_for_insync(rdi,rdi->hri_start_after_in_sync))) break;
#endif

/* start record. */
    count_rec++;
    Set_Entry(window,LAB_Cntrec,"%d",count_rec);
    Recordpp_one_Rawd(window,mainwnd,rdi,pci,vwi);
  }
  Set_Entry(window,LAB_Recording,"Off");

/* Open file if recorded */
  if (rdi->save_flag)
  {
    strcpy(pci->file_name,rdi->file_name);
    open_file(pci,vwi,mainwnd,0);
  }
}



/*************************************
 * Get button signals for Record functions
 *************************************/
void Record_Rawd_Commands(GtkWidget *widget, gpointer data)
{
  GtkWidget *rawd_window=gtk_widget_get_toplevel(widget);
  GtkWidget *main_window=Find_Window(rawd_window,Lab_Main);
  PICINFO *pci=Get_Picinfo_db(widget);
  VIEWINFO *vwi=Get_Viewinfo_db(widget);
  RECORDINFO *rdi=Get_Recordinfo_db(widget);

  int addr_portd=rdi->parport_base;
  char *name;
  gboolean reprogramming_fpga=FALSE;

  name=(char *)data;

/* Decoder/generator choice */
  if (!strcmp(name,LAB_DEC))
  {
    rdi->gen1_dec0=(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget))? FALSE : TRUE);
    reprogramming_fpga=TRUE;
  }

  if (!strcmp(name,LAB_Save))
  {
    rdi->save_flag=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
  }


/* Start record */
  if ((!strcmp(name,LAB_Start)) && (Update_Togglelabel(widget)))
  {
    rdi->recording=TRUE;
    Sense_Button(rawd_window,LAB_Exit,FALSE);
    Sense_Button(rawd_window,LAB_Save,FALSE);
    Recordpp_Rawd(rawd_window,rdi,pci,vwi);
    Sense_Button(rawd_window,LAB_Exit,TRUE);
    Sense_Button(rawd_window,LAB_Save,TRUE);
    rdi->recording=FALSE;
    Set_Button(rawd_window,LAB_Start,FALSE);
  }

  if ((!strcmp(name,LAB_Start)) && (!(Update_Togglelabel(widget))))
  {
    rdi->recording=FALSE;
  }


  if (!strcmp(name,LAB_Exit))
  {
    Close_Window(rawd_window);
    Sense_Button(main_window,MENU_HRPT,TRUE);
    Sense_Button(main_window,MENU_HRI,TRUE);
  }
/* Write new select to decoder */
  if (reprogramming_fpga)
  {
/* Test if we have parallel port privilege */
    if (ioperm(addr_portd,3,1))
    {
      Create_Message("Error",PAR_ERR_MESSAGE,addr_portd);
      return;
    }

    if (!rdi->hwsel_type)
      rdi->sel_hw=rdi->gen1_dec0;

    select_channel_pp(NULL,rdi,FALSE);
    reprogramming_fpga=FALSE;
  }
}

static void close_wnd(GtkWidget *wnd)
{
  GtkWidget *mwnd=Find_Window(gtk_widget_get_toplevel(wnd),Lab_Main);;
  Sense_Button(mwnd,MENU_HRPT,TRUE);
  Sense_Button(mwnd,MENU_HRI,TRUE);
  destroy_window(wnd);
}

/*************************************
 * Record window
 * This function is started from the main menu.
 *************************************/
void Menu_Rec_Rawd(GtkWidget *widget)
{
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  GtkWidget *w1,*w2;
  GtkWidget *wa,*wb,*wc,*wz;
  GtkWidget *rawd_window;
  GtkWidget *main_window=Find_Parent_Window(widget);
  GdkColor startbut_clr,startbutsclr;
  startbut_clr.red=0xffff; startbut_clr.green=0x5555; startbut_clr.blue=0x5555;
  startbutsclr.red=0xffff; startbutsclr.green=0x8888; startbutsclr.blue=0x8888;

  if (!(rawd_window=Create_Window(main_window,0,0,Lab_Rawdwnd,close_wnd))) return;

  rdi->sat_type=0; /*RAW;*/

/* --------------------- START ------------------------- */
  w1=Create_Toggle(LAB_Start,Record_Rawd_Commands,FALSE);
  Set_Color(w1,GTK_RC_BG,NULL,&startbutsclr,&startbut_clr,&startbut_clr,NULL);
  wa=w1;


/* --------------------- INFO ------------------------- */
  w1=Create_Entry(LAB_Recording,NULL,"Off        ");
  w2=Create_Entry(LAB_Cntrec   ,NULL,"  ");
  wb=Pack("Status",'v',w1,0,w2,5,NULL);

/* --------------------- HW-sel ------------------------- */
  if (rdi->hwsel_type)
  {
    wc=NULL;
  }
  else
  {
    w1=Create_Radio(LAB_DEC,TRUE,Record_Rawd_Commands);
    w2=Create_Radio(LAB_GEN,FALSE,Record_Rawd_Commands);
    wc=Pack("Record mode",'h',w1,0,w2,0,NULL);
  }


  w1=Create_Check(LAB_Save,Record_Rawd_Commands,rdi->save_flag);
  w2=Create_Button(LAB_Exit,Record_Rawd_Commands);

  if (wc)
    wz=Pack(NULL,'v',wa,4,wb,4,wc,4,w1,4,w2,4,NULL);
  else
    wz=Pack(NULL,'v',wa,wb,4,w1,4,w2,4,NULL);

  gtk_container_add(GTK_CONTAINER(rawd_window),wz);

  gtk_widget_show(rawd_window);
  place_window(rawd_window,0,0,prefs.sub_wnd_geom);

  if (rdi->port_type==parport)
  {
    if (!ioperm(rdi->parport_base,3,1))
    {
      if (get_ppreg(rdi->parport_base+2) & PIN_17)
        rdi->sel_hw=FALSE;
      else
        rdi->sel_hw=TRUE;

      if (rdi->hwsel_type)
      {
        rdi->sel_hw=TRUE;
        select_channel_pp(NULL,rdi,FALSE);
      }
      else
        Set_Button(rawd_window,LAB_GEN,rdi->sel_hw);
    }
  }

  Sense_Button(main_window,MENU_HRPT,FALSE);
  Sense_Button(main_window,MENU_HRI,FALSE);
}
#endif

