/**************************************************
 * RCSId: $Id: wsat.c,v 1.9 2017/07/07 21:54:14 ralblas Exp $
 *
 * Main program.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: wsat.c,v $
 * Revision 1.9  2017/07/07 21:54:14  ralblas
 * _
 *
 * Revision 1.8  2017/05/30 19:37:22  ralblas
 * _
 *
 * Revision 1.7  2017/05/28 14:45:36  ralblas
 * _
 *
 * Revision 1.6  2017/05/08 21:45:48  ralblas
 * _
 *
 * Revision 1.5  2017/01/06 09:25:52  ralblas
 * _
 *
 * Revision 1.4  2016/12/03 22:53:09  ralblas
 * _
 *
 * Revision 1.3  2016/09/30 08:43:48  ralblas
 * _
 *
 * Revision 1.2  2016/05/21 19:59:00  ralblas
 * _
 *
 * Revision 1.1  2016/03/13 23:48:40  ralblas
 * Initial revision
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <string.h>

#include "wsat.h"
#include "menu_wsat.h"
#include "hrpt_funcs.h"

/* Preferences */
PREFS prefs;

/* Define version number using -DRELEASE=\"...\" in makefile. */
#ifndef RELEASE
#define RELEASE "Undefined"
#endif

#if ENABLE_DEBUG == 1
extern int __sgtkdebug__;
#endif


/*************************************
 * Activate recording directly
 *************************************/
static gboolean sched_recordnow(gpointer data)
{
#ifdef __ADD_PAR__
  GtkWidget *widget=(GtkWidget *)data;
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  char str[20],*p;
  strcpy(str,rdi->rec_chan);

  if ((p=strchr(str,':'))) { *p=0; p++; }

  if (!strcmp(str,"hri"))
  {
#if _HRI1_ == 1
    if ((p) && (strchr(p,'g')))
      Recordpp_Hri_Auto(widget,TRUE);
    else
      Recordpp_Hri_Auto(widget,FALSE);
#endif
  }
  else if (!strcmp(str,"hrpt"))
  {
    Recordpp_Hrpt_Auto(widget,p,HRPT);
  }
  else if (!strcmp(str,"chrpt"))
  {
    Recordpp_Hrpt_Auto(widget,p,CHRPT);
  }
#endif
  return FALSE;
}

/*************************************
 * Activate slide show directly
 *************************************/
gboolean sched_slide(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  do_slide(widget,TRUE);
  return FALSE;
}


#define CNVS_ID "Is_Canvas"
/*************************************
 * Update window; wait until finished
 *************************************/
void Update_Main_Wait(GtkWidget *widget)
{
  GtkWidget *window;
  GtkWidget *main_window,*canvas;
  GtkWidget *drawing_area;
  PICINFO *pci;
  VIEWINFO *vwi;
  window=Find_Parent_Window(widget);
  if (!window) return;
  main_window=Find_Window(window,Lab_Main);
  canvas=Find_Widget(main_window,CNVS_ID);
  drawing_area=Find_Widget(main_window,"GTK_DRAWING_AREA");
  pci=Get_Picinfo_db(widget);
  vwi=Get_Viewinfo_db(widget);
  correct_zoomval(drawing_area,vwi,pci);

  vwi->redraw=TRUE;      /* Signal that a redraw is requested. */
/* Only update if enabled. */
  if (Get_Enable_Update(canvas))
  {
    vwi->redraw=FALSE;   /* Redraw accepted, remove signal */

    Set_Enable_Update(canvas,FALSE); /* Prevent re-drawing while drawing */
    if (!draw_pic(pci,vwi,main_window))                /* draw picture */
    {
      if (vwi->grid) draw_lonlat(pci,vwi,main_window); /* draw grid */
    }

    Set_Enable_Update(canvas,TRUE);

/* If redraw was requested (during draw_pic) then honnor it now. */
    if (vwi->redraw) 
    {
      vwi->redraw=FALSE;
      Update_Main(widget);
    }
  }
  Update_Lum(widget,pci,vwi);
}


static gboolean sched_update(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  Update_Main_Wait(widget);
  return FALSE;
}



/*************************************
 * Update window (scheduled)
 *************************************/
void Update_Main(GtkWidget *widget)
{
  g_idle_add(sched_update,widget);
}

/*************************************
 * Executed if window size is changed 
 *   -> picture is re-generated
 *************************************/
static void configure_event_main(GtkWidget         *widget,
                                 GdkEventConfigure *event)
{
  Renew_RGBBuf(widget);
  Update_Main(widget);
}


RECORDINFO *grdi;

void Menu_Help(GtkWidget *widget,gpointer data)
{
  char *cpr="(C) copyright";
  char fw[8];
  char sfirmware[50];
#if __GTK_WIN32__ == 0
/*Doesn't work with some fonts! */
//  cpr=GDK_copyright;

#endif
#if __ADD_USB__
  if (!get_firmware(&grdi->usb,fw))
  {
    snprintf(sfirmware,49,"V%d_%d\n\n(date 20%02d.%02d)",
      fw[0],fw[1],fw[2],fw[3]);
  }
  else
  {
    snprintf(sfirmware,49,"(unknown)");
  }
#endif
  if (prefs.port_type==usb)
  {
#if __GTK_WIN32__ == 0
    Create_Message_cont("Program info","%s %s\n%s %s %s\n%s\n\n%s %s\n%s\n\n%s %s",
      "wsat release ",RELEASE,
      "Created ",__DATE__,__TIME__,
      "______________________________",
      "Firmware:",sfirmware,
      "______________________________",
      cpr,"R. Alblas\nwerkgroep Kunstmanen\nNederland");
#else
    Create_Message("Program info","%s %s\n%s %s %s\n%s\n\n%s %s\n%s\n\n%s %s",
      "wsat release ",RELEASE,
      "Created ",__DATE__,__TIME__,
      "______________________________",
      "Firmware:",sfirmware,
      "______________________________",
      cpr,"R. Alblas\nwerkgroep Kunstmanen\nNederland");
#endif
  }
  else
  {
#if __GTK_WIN32__ == 0
    Create_Message_cont("Program info","%s %s\n\n\n%s %s",
      "wsat release ",RELEASE,
      cpr,"R. Alblas\nwerkgroep Kunstmanen\nNederland");
#else
    Create_Message("Program info","%s %s\n\n\n%s %s",
      "wsat release ",RELEASE,
      cpr,"R. Alblas\nwerkgroep Kunstmanen\nNederland");
#endif
  }
}

void usage(char *prog)
{
  fprintf(stderr,"**************************************************************\n");
  fprintf(stderr,"* Weather satellite program for HRPT/CHRPT/HRI.\n");
  fprintf(stderr,"* Release: %s\n",RELEASE);
  fprintf(stderr,"* %c R. ALblas, werkgroep Kunstmanen\n",GDK_copyright);
  fprintf(stderr,"*-------------------------------------------------------------\n");
  fprintf(stderr,"* Usage:\n");
  fprintf(stderr,"*        %s <options>\n",prog);
  fprintf(stderr,"*        <options>:\n");
  fprintf(stderr,"*          -i <file>           read <file>\n");
  fprintf(stderr,"*          -sn                 south-north (flip hor and ver)\n");
  fprintf(stderr,"*          -w <width>          Window width (default: %d)\n",
                                                                DEFAULT_WIDTH);
  fprintf(stderr,"*          -h <height>         Window height (default: %d)\n",
                                                                DEFAULT_HEIGHT);
  
  fprintf(stderr,"*          -record <sat:chan>  Auto-record; exit program if done\n");
  fprintf(stderr,"*            <sat:chan>        specify satellite and channels\n");
  fprintf(stderr,"*            <sat>             hrpt | chrpt | hri\n");
  fprintf(stderr,"*            <chan>            1-5, a, g (hrpt)\n");
  fprintf(stderr,"*                              1-0, a, g (chrpt) (0=channel 10)\n");
  fprintf(stderr,"*                              g (hri)\n");
  fprintf(stderr,"*                              a=all channels\n");
  fprintf(stderr,"*                              g=generator mode (only usefull for test)\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -crecord <sat:chan> Same as -record, but record 'forever'\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -batch <sat:chan>   Batch-record; exit program if done.\n");
  fprintf(stderr,"*                              Same as -record, except no graphical mode.\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -cbatch <sat:chan>  Same as -batch, but record 'forever'\n");
  fprintf(stderr,"*\n");
  fprintf(stderr,"*          -log                batch messages in wsat.log in\n");
  fprintf(stderr,"*          -sim1 <file>        use <file> as input, mimick usb interface\n");
  fprintf(stderr,"*                                metop, Fengyun only\n");
  fprintf(stderr,"*          -sim2 <file>        use <file> as input, insert frames\n");
  fprintf(stderr,"*                                ???\n");
  fprintf(stderr,"*  \n");                            
  fprintf(stderr,"*Examples:\n");
  fprintf(stderr,"*         wsat                     start wsat\n");
  fprintf(stderr,"*         wsat ned.gif             show pic in file 'ned.gif'\n");
  fprintf(stderr,"*         wsat -record hrpt:124    record HRPT channels 1/2/4\n");
  fprintf(stderr,"*         wsat -record chrpt:1290  record CHRPT channels 1/2/9/10\n");
  fprintf(stderr,"*         wsat -record hrpt:a      record HRPT, all channels\n");
  fprintf(stderr,"*         wsat -record hri         record HRI\n");
  fprintf(stderr,"*         wsat -record hrpt:ag     record HRPT generator mode\n");
  fprintf(stderr,"*         wsat -batch hri          record HRI, batch-mode\n");
  fprintf(stderr,"**************************************************************\n");
}


void Menu_Test(GtkWidget *widget,gpointer data)
{
  RECORDINFO *rdi=Get_Recordinfo_db(widget);
  switch (rdi->port_type)
  {
#ifdef __ADD_PAR__
    case parport: Menu_Testpp(widget,data); break;
#endif
#ifdef __ADD_USB__
    case usb: Menu_Testusb(widget,data); break;
#endif
    default: break;
  }
}
#if _TRACKER_
GtkWidget *create_tracker();
GtkWidget *create_showtrack(GtkWidget *);
#endif

int check_arg(int i,int argc,char *argv)
{
  if (i==argc-1)
  {
    fprintf(stderr,"ERROR: Missing argument for %s.\n",argv);
    return -1;
  }
  return 0;
}

int main(int argc, char **argv)
{
  GtkWidget *main_window,*track_window,*menu,*info,*canvas;
  GtkWidget *position,*pixval,*gammaval,*satname,*wz[5],*wdbg;
  PICINFO *pci;
  VIEWINFO *vwi;
  RECORDINFO *rdi;
  FILE_TYPE ftype=0;
  int i;
  int w=0,h=0;
  int scan_ver=0;
  const gboolean enable_tracker=TRUE;

#if ENABLE_DEBUG == 1
  __sgtkdebug__=0;
#endif

/* Allocate mem for global structs */
  pci=(PICINFO *)calloc(sizeof(PICINFO),1);
  vwi=(VIEWINFO *)calloc(sizeof(VIEWINFO),1);
  rdi=(RECORDINFO *)calloc(sizeof(RECORDINFO),1);
  grdi=rdi;
  gtk_disable_setlocale();  // always use decimal point, never comma!
/* Get command line arguments */
  gtk_init(&argc, &argv);
  gdk_rgb_init();

  *rdi->rec_chan=0;

/* Read preferences */
  Read_Prefs(argv[0]);
  Load_Prefs(rdi,vwi,pci);

  set_fontsize(prefs.font_size);

  for (i=1; i<argc; i++)
  {
#if ENABLE_DEBUG == 1
    if (!strcmp(argv[i],"-dbg")) { __sgtkdebug__=1; continue; } else
#endif
    if (!strcmp(argv[i],"-i"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      strcpy(pci->file_name,argv[i]);
    }

    else if (!strcmp(argv[i],"-w"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      prefs.main_wnd_w=atoi(argv[i]);
    }
    else if (!strcmp(argv[i],"-h"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      prefs.main_wnd_h=atoi(argv[i]);
    }
    else if (!strcmp(argv[i],"-H"))
    {
      usage(argv[0]);
      exit(0);
    }
    else if (!strcmp(argv[i],"--help"))
    {
      usage(argv[0]);
      exit(0);
    }
    else if (!strcmp(argv[i],"-V"))
    {
      fprintf(stderr,"* Version: %s\n",RELEASE);
      exit(0);
    }

    else if (!strcmp(argv[i],"-sn"))
    {
      scan_ver=S_N;
    }
    else if (!strcmp(argv[i],"-t"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      if (!strcmp(argv[i],"m1"))       ftype=FT_MHRPT1;
      else if (!strcmp(argv[i],"m2"))  ftype=FT_MHRPT2;
      else if (!strcmp(argv[i],"mp"))  ftype=FT_METOP;
      else if (!strcmp(argv[i],"fy3")) ftype=FT_FY3;
      else
      {
        fprintf(stderr,"Error: wrong type '%s'\n",argv[i]);
        fprintf(stderr,"Allowed types: m1 (=meteror N1), m2 (=meteror N2), mp (=metop), fy3 (=Fengyun-3)\n");
        exit(1);
      }
    }

    
/* auto-rec */
    else if (!strcmp(argv[i],"-record"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=TRUE;
    }

    else if (!strcmp(argv[i],"-crecord"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=FALSE;
    }
    
/* batch-mode */
    else if (!strcmp(argv[i],"-batch"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=TRUE;
      rdi->batch_mode=TRUE;
    }

    else if (!strcmp(argv[i],"-cbatch"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      strcpy(rdi->rec_chan,argv[i]);
      rdi->one_record=FALSE;
      rdi->batch_mode=TRUE;
    }

    else if (!strcmp(argv[i],"-log"))
    {
      prefs.log=TRUE;
    }
/* slide show */
    else if (!strcmp(argv[i],"-slide"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      vwi->slide_mode=TRUE;
    }
    else if (!strcmp(argv[i],"-speed"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      vwi->slide_speed=atoi(argv[i]);
    }
    else if (!strcmp(argv[i],"-sim1"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      rdi->sim_file=argv[i];
      rdi->sim_record=1;
      if (!exist_file(rdi->sim_file))
      {
        fprintf(stderr,"'%s' doesn't exist.\n",rdi->sim_file);
        return -1;
      }
    }
    else if (!strcmp(argv[i],"-sim2"))
    {
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      rdi->sim_file=argv[i];
      rdi->sim_record=2;
      if (!exist_file(rdi->sim_file))
      {
        fprintf(stderr,"'%s' doesn't exist.\n",rdi->sim_file);
        return -1;
      }
    }
    
    else if (!strcmp(argv[i],"-pr_vcid"))
    {
      FILE *fp;
      if (check_arg(i,argc,argv[i])) return -1;
      i++;
      fp=fopen(argv[i],"rb");
      if (!fp) 
      {
        fprintf(stderr,"No file %s\n",argv[i]);
        return -1;
      }
      pri_vcid(fp);
      fclose(fp);
      return 0;
    }
    else if (argv[i][0]=='-')
    {
      fprintf(stderr,"Wrong argument '%s'\n",argv[i]);
      return -1;
    }
  }
                                                               
/* Do batch-mode */
  if (rdi->batch_mode)
  {
#ifdef __ADD_PAR__
    do_batch(rdi,pci,vwi);
#endif
    return 0;
  }

  if (!*(pci->file_name) && (argc>1) && (*argv[argc-2] !='-') && (*argv[argc-1] !='-'))
  {
    strcpy(pci->file_name,argv[argc-1]);
  }

  if (*(pci->file_name))
  {
    int err;
    if ((err=open_file(pci,vwi,NULL,ftype)))
    {
      switch(err)
      {
        case 'o': 
          fprintf(stderr,"ERROR: Can't open %s\n",pci->file_name);
        break;
        case 't': 
          fprintf(stderr,"ERROR: %s is unknown file type.\n",pci->file_name);
        break;
        default:
          fprintf(stderr,"ERROR: Problem with reading %s (code %c = %d).\n",
              pci->file_name,err,err);
        break;
      }
      free(pci);
      free(vwi);
      free(rdi);
      exit(1);
    }

  }
  if (scan_ver) vwi->scan_ver=scan_ver;
  w=prefs.main_wnd_w;
  h=prefs.main_wnd_h;

  if ((!w) && (!h))
  {
    w=-1;
    h=-1;
  }

/* Define the main window */

  main_window=Create_Window(NULL,
                            w,h,
                            Lab_Main,gtk_main_quit);
  if (!main_window) fatal_error();


/* Add database to main window */
  gtk_object_set_data(GTK_OBJECT(main_window),WSAT_PCI,(gpointer)pci);
  gtk_object_set_data(GTK_OBJECT(main_window),WSAT_VWI,(gpointer)vwi);
  gtk_object_set_data(GTK_OBJECT(main_window),WSAT_RDI,(gpointer)rdi);

/* Create the menu bar */
  menu=Create_Menu(main_window,MENU_FILE           ,NULL          ,BUTTON,
                                 MENU_OPEN         ,MenuSelectFile,BUTTON,
                                 MENU_CLOSE        ,close_file    ,BUTTON,
                                 MENU_SAVE         ,MenuSave      ,BUTTON,
                                 MENU_INFO         ,Menu_Info     ,BUTTON,
                                 MENU_LDKEPLER     ,Menu_LDKepler ,BUTTON,
                                 MENU_SEPAR        ,NULL          ,BUTTON,
                                 MENU_QUIT         ,Close_Window  ,BUTTON,

                               MENU_EDIT           ,NULL          ,BUTTON,
                                 MENU_FHDR         ,Menu_Hdredt   ,BUTTON,
                                 MENU_KEPLER       ,Menu_orbitedt ,BUTTON,
                                 MENU_SEPAR        ,NULL          ,BUTTON,
                                MENU_PREF          ,MenuPrefs     ,BUTTON,

                               MENU_VIEW           ,NULL          ,BUTTON,
                                 MENU_SATDIR       ,Menu_Viewbuts ,CHECK,
                                 MENU_GRIDADD      ,Menu_Viewbuts ,CHECK,
                                 MENU_POSXY        ,Menu_Viewbuts ,CHECK,
                                 MENU_INVERSE      ,Menu_Viewbuts ,CHECK,
                                 MENU_TEMP         ,Menu_Viewbuts ,CHECK,
                                 MENU_LIN          ,Menu_Viewbuts ,CHECK,
                                 MENU_DESPEC       ,Menu_Viewbuts ,CHECK,
                                 MENU_DESTRIP      ,Menu_Viewbuts ,CHECK,
                                 MENU_RS           ,Menu_Viewbuts ,CHECK,
                                 MENU_MARK_FLTS    ,Menu_Viewbuts ,CHECK,
                                 MENU_SKIP_FLTS    ,Menu_Viewbuts ,CHECK,
                                 MENU_SKIP_PP      ,NULL          ,BUTTON,
                                   MENU_SKIP_PRE   ,Menu_Viewbuts ,CHECK,
                                   MENU_SKIP_POST  ,Menu_Viewbuts ,CHECK,
                                 MENU_PALETTE      ,NULL          ,BUTTON,
                                   MENU_LD_PALETTE ,load_palette  ,RADIO,
                                   MENU_GR_PALETTE ,load_greyscale,RADIO,
                                 MENU_LUM          ,Menu_Lummap   ,BUTTON,
                                 MENU_CHMAP        ,Menu_Mapchan  ,BUTTON,
                                 MENU_FILM         ,Menu_Film     ,BUTTON,

                               MENU_ZOOM           ,NULL          ,BUTTON,
                                 MENU_FULL         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMIN         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMOUT        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMVAR        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMFIX        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMLAST       ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMDXY        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMDYX        ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMXY         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZMYX         ,Menu_Zoombuts ,BUTTON,
                                 MENU_ZVAL         ,Menu_Zoom     ,BUTTON,

//                               MENU_LUMTHR         ,Menu_Lumthr   ,BUTTON,

                               MENU_REC            ,NULL          ,BUTTON,
#if _APT_
                                 MENU_APT          ,NULL          ,BUTTON,
#endif
#if _HRPT1_
                                 MENU_HRPT         ,Menu_Rec_Hrpt ,BUTTON,
#endif
//#if _METOP1_
//                                 MENU_METOP        ,Menu_Rec_Metop,BUTTON,
//#endif
#if _LRIT1_
                                 MENU_LRIT         ,Menu_Rec_Lrit ,BUTTON,
#endif
#if _WEFAX_
                                 MENU_WEFAX        ,NULL          ,BUTTON,
#endif

// next old; only par.port support
#ifdef __ADD_PAR__
#if _HRI1_
                                 MENU_HRI          ,Menu_Rec_Hri  ,BUTTON,
#endif
#if _RAWDT_
                                 MENU_RAWD         ,Menu_Rec_Rawd ,BUTTON,
#endif
#endif

#if _IQTEST1_
                                 MENU_IQTEST       ,Menu_Rec_IQTest ,BUTTON,
#endif
                               MENU_CHAN           ,NULL          ,BUTTON,
                                 MENU_FC           ,Menu_Channel  ,RADIO,
                                 MENU_CH1          ,Menu_Channel  ,RADIO,
                                 MENU_CH2          ,Menu_Channel  ,RADIO,
                                 MENU_CH3          ,Menu_Channel  ,RADIO,
                                 MENU_CH4          ,Menu_Channel  ,RADIO,
                                 MENU_CH5          ,Menu_Channel  ,RADIO,
                                 MENU_CH6          ,Menu_Channel  ,RADIO,
                                 MENU_CH7          ,Menu_Channel  ,RADIO,
                                 MENU_CH8          ,Menu_Channel  ,RADIO,
                                 MENU_CH9          ,Menu_Channel  ,RADIO,
                                 MENU_CH10         ,Menu_Channel  ,RADIO,

                               MENU_HELP           ,NULL          ,BUTTON,
                                 MENU_PINFO        ,Menu_Help     ,BUTTON,
                                 MENU_TEST         ,Menu_Test     ,BUTTON,
                                 MENU_METOPTEST    ,Menu_MetopTest,BUTTON,

                               0);

// add button to menu bar
  {
    GtkWidget *tb=Create_Button(MENU_LUMTHR,Menu_Lumthr);
    menu=Pack(NULL,'h',menu,1,tb,1,NULL);
  }
/* Create info bar */
  position=Create_Entry(LAB_COORD,NULL,"x=%5.2f,y=%5.2f",0.,0.);
  pixval=Create_Entry(LAB_PIXVAL,NULL,"%8d",0);
  gammaval=Create_Spin(LAB_GAMMAVAL,NULL,"%8.1f%f%f",1.,0.2,5.);
  satname=Create_Entry(LAB_SATNM,NULL,"%10s","");
  info=Pack(NULL,'h',position,3,pixval,3,gammaval,3,satname,0,NULL);

/* Create the canvas */
  canvas=Create_Canvas(main_window,
                prefs.canvas_w,prefs.canvas_h,
                configure_event_main,
                key_actions,mouse_actions,mouse_actions);

gtk_object_set_data(GTK_OBJECT(canvas),WDGT_ID,CNVS_ID);

/* Add scrolbars */
  canvas=Add_Scrollbars(canvas,"hbar","vbar",scroll_actions);
/* Collect all parts */
  wz[1]=SPack(NULL,"v",info,"1",canvas,"ef1",NULL);

  wdbg=NULL;
#if DEBUG_KLAD==1
  wdbg=nb_debug();
#endif

#if _TRACKER_
  if (enable_tracker)
  {
    wz[2]=create_tracker();

    if (1)
    {
      wz[3]=create_showtrack(main_window);
      wz[4]=Create_Notebook(NULL,GTK_POS_TOP,
                       "Viewer"  , wz[1],
                       "Tracker" , wz[2],
                       "satpos"  , wz[3],
  #if DEBUG_KLAD==1
                       "Debug"   , wdbg,
  #endif
                  NULL);
    }
    else // poging tracker-window apart
    {
      wz[4]=Create_Notebook(NULL,GTK_POS_TOP,
                       "Viewer"  , wz[1],
                       "Tracker" , wz[2],
  #if DEBUG_KLAD==1
                       "Debug"   , wdbg,
  #endif
                  NULL);
      track_window=Create_Window(main_window,500,500,Lab_Track,NULL);
      wz[3]=create_showtrack(track_window);
      gtk_container_add(GTK_CONTAINER(track_window),wz[3]);
      gtk_widget_show(track_window);
    }
  }
  else
#endif
  {
    wz[4]=wz[1];
  }
  wz[0]=SPack(NULL,"v",menu,"1",wz[4],"ef1",NULL);
  gtk_container_add(GTK_CONTAINER (main_window),wz[0]);
  place_window(main_window,
               prefs.main_wnd_x,prefs.main_wnd_y,
               prefs.main_wnd_geom);

/* Load palette:
     If file specified: palette in file
     If no file specified: grey palette 
*/
  if (pci->fp==NULL)
    load_std_palette(main_window,NULL);
  else
    load_std_palette(main_window,pci);

  load_extra_palette(main_window);

/* Set menu items */
  Channel_Select(main_window,pci,vwi);

  init_menubuts(main_window,pci,vwi);
  Set_Entry(main_window,LAB_SATNM,"%s",pci->sat_name);
/*
  Enable window update. 
  It was disabled to prevent re-drawing picture several times because
  of channel select etc.
*/
  Set_Enable_Update(canvas,TRUE);
  gtk_widget_show_all(main_window);

/* Handle auto-record */
  if (*rdi->rec_chan)
  {
    g_idle_add(sched_recordnow,main_window);
  }

  if (vwi->slide_mode)
    g_idle_add(sched_slide,main_window);

  Set_Button(main_window,MENU_SKIP_POST,TRUE);
  gtk_widget_show_all(main_window);

#if _TRACKER_
  if (prefs.rotor.use_xy)
  {
    Show_Button(main_window,LAB_ROTOR_XY,TRUE);
    Show_Button(main_window,LAB_ROTOR_ELEAZIM,FALSE);
  }
  else
  {
    Show_Button(main_window,LAB_ROTOR_ELEAZIM,TRUE);
    Show_Button(main_window,LAB_ROTOR_XY,FALSE);
  }
#endif

  gtk_main();

  free(pci);
  free(vwi);
  free(rdi);
  return 0;
}

