/**************************************************
 * RCSId: $Id: wsat_track.c,v 1.3 2017/03/26 21:57:30 ralblas Exp $
 *
 * tracker.
 * Project: WSAT
 * Author: R. Alblas
 *
 * History: 
 * $Log: wsat_track.c,v $
 * Revision 1.3  2017/03/26 21:57:30  ralblas
 * _
 *
 * Revision 1.2  2017/02/16 20:14:27  ralblas
 * _
 *
 * Revision 1.1  2017/01/04 16:27:39  ralblas
 * Initial revision
 *
 * Revision 1.5  2017/01/04 16:27:39  ralblas
 * _
 *
 **************************************************/
/*******************************************************************
 * Copyright (C) 2000 R. Alblas. 
 *
 * This is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software. If not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
 * 02111-1307, USA.
 ********************************************************************/
#include <ctype.h>

#include "wsat.h"
#include "godil_regmap.h"
#if _TRACKER_
#include <stdlib.h>
#include "wsat_functions.h"
#define USE_GTK
#include "sattrack.h"
#include "xtrack_basefunc.h"
#include "hrpt_funcs.h"
#include "sattrack_funcs.h"
#define LAB_JAAR "^year"
#define LAB_MND "^month"
#define LAB_DAG "^day"
#define LAB_UUR "^hour"
#define LAB_MINT "^min"
#define LAB_SEC "^sec"
#define LAB_OFFSETHOUR "test_offsetmin"
#define LAB_NEXTPASS "!Next pass"
#define LAB_LON "^lon"
#define LAB_LAT "^lat"
#define LAB_ELE "^elev"
#define LAB_AZI "^azim"
#define LAB_ELE2 "^Elev"
#define LAB_AZI2 "^Azim"
#define LAB_X2 "^X "
#define LAB_Y2 "^Y "
#define LAB_XXX "^X"
#define LAB_YYY "^Y"
#define LAB_SATNAAM "Satellite "
#define LAB_START "Start /Stop "
#define LAB_NOODST "Emerg. stop"
#define LAB_READ "Make list"
#define LAB_SATLIST "list"
#define LAB_SELSAT "Selected"
#define LAB_LED "!In zicht"
#define LAB_KBL "!kleurenblind"
#define LAB_GETKEP "Download Kepler"
#define LAB_DWNLDRES "!Result kepler"
#define LAB_TRACKCANVAS "trackcanvas"

extern PREFS prefs;


typedef struct db
{
  gboolean redraw;
  gboolean redrawing;
//refpos
//sat_sel
} DB;


DB db;

static int test_min;
static void run_track(GtkWidget *widget,SAT *sat);
int pos2godil(USB_ITEMS *,ROTOR *,float d1,float d2,int showpos);

static char *tmstrsh(struct tm *tm,char *what)
{
  static char s[51];
  char frmt[20];
  if (!what)
  {
    strftime(s,50,"%d-%m    %H:%M:%S",tm);
  }
  else
  {
    strcpy(frmt,"");
    if (strchr(what,'m')) strcat(frmt,"%d-%m ");
    if (strchr(what,'H')) strcat(frmt,"%H");
    if (strchr(what,'M')) strcat(frmt,":%M");
    if (strchr(what,'S')) strcat(frmt,":%S");
    strftime(s,50,frmt,tm);
  }
  return s;
}

static SAT *prepare_track(SAT *sat,char *wname,char *sname)
{
  FILE *fp;
  while ((sat=Remove_Sat(sat)));
  if (!(fp=fopen(wname,"r"))) return NULL;
  sat=read_msat(fp,sname,sat);
  fclose(fp);
  while ((sat) && (sat->prev)) sat=sat->prev;
  return sat;
}

#if __ADD_USB__
extern RECORDINFO *grdi;
static int to_tracker(RECORDINFO *rdi,float xazim,float yelev)
{
  return pos2godil(&grdi->usb,&prefs.rotor,xazim,yelev,1);
}

static char *type_prcnt(char *p)
{
  while ((*p) && ((isdigit(*p)) || (*p=='.'))) p++;
  return p;
}

static void fillin_satpos(char *frmt,char *name,float elev,float azim,float x,float y)
{
  char *p;
  char *rslt=malloc(strlen(frmt)+strlen(name)+50);
  for (p=frmt; *p; p++) if (*p=='%')
  {
    if (*(p+1)=='%')
    {
      p+=2; continue;
    }

    *p=1;
    
  }
  while ((p=strchr(frmt,1)))
  {
    *p='%';
    if ((p=type_prcnt(p+1)))
    {
      switch(*p)
      {
        case 'n': 
          *p='s';
          sprintf(rslt,frmt,name);
          strcpy(frmt,rslt);
        break;
        case 'e': 
          *p='f';
          sprintf(rslt,frmt,elev);
          strcpy(frmt,rslt);
        break;
        case 'a': 
          *p='f';
          sprintf(rslt,frmt,azim);
          strcpy(frmt,rslt);
        break;
        case 'x': 
          *p='f';
          sprintf(rslt,frmt,x);
          strcpy(frmt,rslt);
        break;
        case 'y': 
          *p='f';
          sprintf(rslt,frmt,y);
          strcpy(frmt,rslt);
        break;
        default:
        break;
      }
    }
  }
  free(rslt);
}


static void ext_prog(int up,DIRECTION *satdir)
{
  char *prog=NULL;
  if (!prefs.ext_prog_on) return;
  if ((up) && (*prefs.prog_trackup))
  {
    if ((prog=malloc(strlen(prefs.prog_trackup)+100)))
    {
      strcpy(prog,prefs.prog_trackup);
      fillin_satpos(prog,"",
         R2D(satdir->elev),R2D(satdir->azim),R2D(satdir->x),R2D(satdir->y));
      system(prog);
      freed(&prog);
    }
  }
  if ((!up) && (*prefs.prog_trackdown))
  {
    if ((prog=malloc(strlen(prefs.prog_trackdown)+100)))
    {
      strcpy(prog,prefs.prog_trackdown);
      fillin_satpos(prog,"",
         R2D(satdir->elev),R2D(satdir->azim),R2D(satdir->x),R2D(satdir->y));
      system(prog);
      freed(&prog);
    }
  }
}

static void catch_pos(DIRECTION satdir,float *rot1,float *rot2)
{
  static int is_up=0;
  if (satdir.elev>=prefs.elev_horiz)
  {
    if (!is_up) ext_prog(1,&satdir);
    is_up=1;
  }
  else
  {
    if (is_up) ext_prog(0,&satdir);
    is_up=0;
  }

  if (prefs.rotor.use_xy)
  {
    *rot1=prefs.rotor.to_rotor.x;
    *rot2=prefs.rotor.to_rotor.y;
  }
  else
  {
    *rot1=prefs.rotor.to_rotor.azim;
    *rot2=prefs.rotor.to_rotor.elev;
  }
}


struct tm mom_loctm(float extra_offset)
{
  return mom_tm(prefs.tm_off.tz_offset+prefs.utc_offset+extra_offset,&prefs.tm_off);
}

static GdkColor rgb2clr(int r,int g,int b)
{
  GdkColor clr;
  clr.red=r;
  clr.green=g;
  clr.blue=b;
  return clr;
}

static RGBI rgb_bck;


static void refresh_wnd(GtkWidget *wnd)
{
  GdkColor clr;
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  int wnd_width;
  int wnd_height;
  clr=rgb2clr(0x88,0x88,0x88); // db->clrs.ref_vis;
  wnd_width=drawing_area->allocation.width;
  wnd_height=drawing_area->allocation.height;
  draw_ref(wnd,prefs.refpos.lon,prefs.refpos.lat);
//  draw_vis(wnd,db.sat_sel,&db->refpos,TRUE,clr);
  Refresh_Rect(drawing_area,0,0,wnd_width,wnd_height);
}


static void clear_wnd(GtkWidget *wnd)
{
  GtkWidget *drawing_area=Find_Widget(wnd,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  int i;
  if (!rgb_bck.rgbbuf) return;
  int x,y;
  for (y=0; y<rgbi->height; y++)   // y:== -90...+90
  {
    for (x=0; x<rgbi->width; x++)  // x:== -180...+180
    {
// hier juiste 'i' berekenen!
      i=(x+y*rgbi->width)*3;
      if (rgb_bck.bufsize > i+2)
      {
        *(rgbi->rgbbuf+i)=*(rgb_bck.rgbbuf+i);
        *(rgbi->rgbbuf+i+1)=*(rgb_bck.rgbbuf+i+1);
        *(rgbi->rgbbuf+i+2)=*(rgb_bck.rgbbuf+i+2);
      }
    }
  }
}



static GtkWidget *widgetxx;

#define HTTP_KEPLER   "http://celestrak.com/NORAD/elements"
#define HTTP_KEPLERSH        "celestrak.com"
#define COL_SNAM 0
#define COL_UP 1
#define COL_MELEV 2
#define COL_MELEVT 3
#define COL_REF 4
#define MAXCOL 4

static void add_to_list2(GtkWidget *w,TRACK *track)
{
  TRACK *tr;
  char *tmp[MAXCOL];
  char txt[MAXCOL][100];
  struct tm tm;
  int i;
  int row=0;
  GtkCList *lst=GTK_CLIST(w); 
  gtk_clist_clear(lst);
  memset(txt,0,MAXCOL*100);
  for (tr=track; tr; tr=tr->next)
  {
    SAT *sat=tr->sat;
    tm=offset_tm(tr->up_time,prefs.utc_offset);
    sprintf(txt[COL_UP],"%s",tmstrsh(&tm,NULL));

    sprintf(txt[COL_MELEV],"%.1f",R2D(tr->max_elev));

    tm=offset_tm(tr->maxelev_time,prefs.utc_offset);
    sprintf(txt[COL_MELEVT],"%s",tmstrsh(&tm,"HMS"));
    sprintf(txt[COL_SNAM],"%s",sat->satname);
//    tdiff=(start_tm.tm_year-sat->kepler.epoch_year)*366+(start_tm.tm_yday-(int)sat->kepler.epoch_day);
//    sprintf(txt[COL_REF],"Day %d year %d",(int)sat->kepler.epoch_day,sat->kepler.epoch_year+1900);
//    sprintf(txt[COL_REF],"%d days",tdiff);
    for (i=0; i<MAXCOL; i++) tmp[i]=txt[i];
    gtk_clist_append(lst, tmp);
    gtk_clist_set_selectable(lst,row,FALSE); // set not selectable
    row++;
  }

}

#define rewind(t) while ((t) && (t->prev)) t=t->prev; 

static TRACK *first_passes(SAT *sat,int nr)
{
  struct tm_ms cur_tmms;
  int n;
  SAT *sat1=NULL;
  TRACK *tr,*tro=NULL,*tr2=NULL;
  TRACK *trp,*trn;
  for (sat1=sat; sat1; sat1=sat1->next)
  {
    struct tm start_tm;
    struct tm stop_tm;
    float offset_time=(float)test_min/60.-0.1+prefs.tm_off.tz_offset;
    time_t t;
    Remove_Tracks(sat1->track);
    sat1->track=NULL;
    time(&t);
    cur_tmms=mom_tmms(offset_time,&prefs.tm_off);// start 6 minutes before current time
    start_tm=cur_tmms.tm;
//    start_tm=mom_utctm(-360.);  // start 6 minutes before current time
    for (n=0; n<nr; n++)
    {
      TRACK track;
      memset(&track,0,sizeof(track));
      if (predict(sat1,&start_tm,&stop_tm,&track,&prefs.refpos,NULL,
                                   prefs.elev_horiz,prefs.elev_det,TRUE))
      {
        tr=Create_Track(sat1->track);
        if (!sat1->track) sat1->track=tr;
        trp=tr->prev;
        trn=tr->next;
        *tr=track;
        tr->prev=trp;
        tr->next=trn;
        tr->sat=sat1;
      }
    }
  }

  for (n=0; n<nr; n++)
  {
    time_t t2=0;
    for (sat1=sat; sat1; sat1=sat1->next)
    {
      for (tr=sat1->track; tr; tr=tr->next)
      {
        if (tr->done) continue;
        time_t t1=mktime_ntz(&tr->up_time);
        if ((!t2) || (t1<t2))
        {
          t2=t1;
          tr2=tr;
        }
      }
    }
    if (tr2)
    {
      tro=Create_Track(tro);
      trp=tro->prev;
      trn=tro->next;
      *tro=*tr2;   // !!! link naar sat!!!
      tro->prev=trp;
      tro->next=trn;
      tr2->done=TRUE;
    }
  }
  rewind(tro);
  return tro;
}
static void get_newpasses(SAT *sat,GtkWidget *widget,gboolean activate)
{
  SAT *s1;
  static TRACK *next_tracks;
  Remove_Tracks(next_tracks);
  for (s1=sat; s1; s1=s1->next) s1->selected=FALSE;
  next_tracks=first_passes(sat,10);
  if (next_tracks) next_tracks->sat->selected=TRUE;
  add_to_list2(Find_Widget(widget,LAB_SATLIST),next_tracks);

  #if __ADD_USB__
  if (activate)
  {
    run_track(widget,NULL);
    run_track(widget,sat);
  }
  #endif
}


#define LAB_LEDABOVE 0x0f0
#define LAB_LEDBELOW 0xf00
#define LAB_LEDNOCON 0xf0f
#define LAB_LEDSTOPP 0x00f
static int do_track(gpointer pointer)
{
  SAT *sat=(SAT *)pointer;
  SAT *s1;
  EPOINT pos_subsat;
  EPOINT pos_earth;
  DIRECTION satdir;
//  time_t t;
  static float xazim=1000.;
  static float yelev=1000.;
  float elev=0.;
  struct tm cur_tm,show_tm;
  struct tm_ms cur_tmms;
  static int running; // dit helpt niet tegen langdurige do_track?
  if (xazim==1000.) xazim=prefs.rotor.storm.x;
  if (yelev==1000.) yelev=prefs.rotor.storm.y;

  if (running) return TRUE;
  running=1;
  cur_tmms=mom_tmms((float)test_min/60.,&prefs.tm_off);
  cur_tm=cur_tmms.tm;
  show_tm=mom_loctm(0.);                             // current time local
//  show_tm=offset_tm(cur_tm,prefs.utc_offset+prefs.tm_off.tz_offset+(float)(test_min/60.)); // current time to show
  for (s1=sat; s1; s1=s1->next)
  {
    calc_satposms(cur_tmms,&pos_subsat,&satdir,&prefs.refpos,s1,&prefs.rotor);
    s1->pos=pos_subsat;
    if (s1->selected)
    {
      calc_rotpos(satdir,&prefs.rotor,prefs.elev_horiz,prefs.reptime);
      catch_pos(satdir,&xazim,&yelev);
      elev=satdir.elev;
      if (!usb_init_connect(&grdi->usb))
      {
        if (to_tracker(grdi,xazim,yelev))
        {
          run_track(NULL,NULL);
          Set_Button(widgetxx,LAB_START,FALSE);
          Create_Message("Error","USB Timeout!");
          return FALSE;
        }
      }
//  cur_tm.tm_hour+=(int)prefs.tm_off.tz_offset;
//  cur_tm.tm_min+=(prefs.tm_off.tz_offset-(int)prefs.tm_off.tz_offset)*60;
//  mktime_ntz(&cur_tm);

      Set_Entry(widgetxx,LAB_LON,"%.1f",R2D(pos_subsat.lon));
      Set_Entry(widgetxx,LAB_LAT,"%.1f",R2D(pos_subsat.lat));
      Set_Entry(widgetxx,LAB_ELE,"%.1f",R2D(satdir.elev));
      Set_Entry(widgetxx,LAB_AZI,"%.1f",R2D(satdir.azim));
      Set_Entry(widgetxx,LAB_XXX,"%.1f",R2D(satdir.x));
      Set_Entry(widgetxx,LAB_YYY,"%.1f",R2D(satdir.y));

      Set_Entry(widgetxx,LAB_ELE2,"%.1f",yelev); // (float)(elev)/10.);
      Set_Entry(widgetxx,LAB_AZI2,"%.1f",xazim); // (float)(azim)/10.);
      Set_Entry(widgetxx,LAB_X2,"%.1f",xazim); // (float)(nx)/10.);
      Set_Entry(widgetxx,LAB_Y2,"%.1f",yelev); // (float)(ny)/10.);

      Set_Entry(widgetxx,LAB_JAAR,"%d",show_tm.tm_year+1900);
      Set_Entry(widgetxx,LAB_MND ,"%02d",show_tm.tm_mon+1);
      Set_Entry(widgetxx,LAB_DAG ,"%02d",show_tm.tm_mday);
      Set_Entry(widgetxx,LAB_UUR ,"%02d",show_tm.tm_hour);
      Set_Entry(widgetxx,LAB_MINT,"%02d",show_tm.tm_min);
      Set_Entry(widgetxx,LAB_SEC ,"%02d",show_tm.tm_sec);

      {
        double dt=difftime(mktime_ntz(&s1->track->up_time),mktime_ntz(&cur_tm));
        gboolean visible=FALSE;
        if (dt<0)
        {
          visible=TRUE;
          dt=difftime(mktime_ntz(&s1->track->down_time),mktime_ntz(&cur_tm));
          if (dt<0) dt=0;
        }
        int h=dt/3600;
        int m=(dt-(h*3600))/60;
        int s=dt-h*3600-m*60;
        if (visible)
          Set_Entry(widgetxx,LAB_NEXTPASS,"%s down in %d:%02d:%02d",s1->satname,h,m,s);
        else
          Set_Entry(widgetxx,LAB_NEXTPASS,"%s up in %d:%02d:%02d",s1->satname,h,m,s);
      }
    }
  }

  if (grdi->usb.ftHandle)
  {
    if (elev>=prefs.elev_horiz)
    {
      Set_Led(widgetxx,LAB_LED ,LAB_LEDABOVE);
      Set_Entry(widgetxx,LAB_KBL ,"^");
    }
    else
    {
      Set_Led(widgetxx,LAB_LED ,LAB_LEDBELOW);
      Set_Entry(widgetxx,LAB_KBL ,"v");
    }
  }
  else
  {
    Set_Led(widgetxx,LAB_LED ,LAB_LEDNOCON);
    Set_Entry(widgetxx,LAB_KBL ,"x");
  }
  clear_wnd(widgetxx); // achtegrond opnieuw schrijven

  for (s1=sat; s1; s1=s1->next)
  {
    draw_onepos(cur_tm,widgetxx,s1,pos_subsat,pos_earth);
  }
// draw_sat(widgetxx,sat,pos_subsat.lon,pos_subsat.lat);
  refresh_wnd(widgetxx);

  {
    static gboolean visible;
    if ((visible) && (elev<=0))
    {
      get_newpasses(sat,widgetxx,TRUE);
    }
    visible=(elev>0);
  }

  g_main_iteration(TRUE);
  running=0;

return TRUE;
}

static void run_track(GtkWidget *widget,SAT *sat)
{
  static int to;
  if (widget==NULL)
  {
    to=0;
    return;
  }
  widgetxx=widget;
  if (sat)
  {
    to=gtk_timeout_add(prefs.reptime*1000,do_track,sat);
  }
  else
  {
    if (to) gtk_timeout_remove(to); to=0;
    usb_write_reg(&grdi->usb,A_DSP,0);
  }
}
#endif

#ifdef NIETGEBRUIKT
static void add_to_list3(GtkWidget *w,SAT *sat)
{
  char *tmp[MAXCOL];
  char txt[MAXCOL][100];
  struct tm tm;
  int i;
  int tdiff;
  GtkCList *lst=GTK_CLIST(w); 
  gtk_clist_clear(lst);
  memset(txt,0,MAXCOL*100);
  for (; sat; sat=sat->next)
  {
    TRACK track;
    time_t t;
    struct tm start_tm;
    struct tm stop_tm;
    time(&t);
    start_tm=mom_utctm(-1800.);  // start 30 minutes before current time
    strcpy(txt[COL_UP],"??");
    if (predict(sat,&start_tm,&stop_tm,&track,&prefs.refpos,D2R(prefs.elev_horiz),D2R(prefs.elev_det)))
    {
      tm=offset_tm(track.up_time,prefs.utc_offset);
      sprintf(txt[COL_UP],"%s",tmstrsh(&tm,NULL));

      sprintf(txt[COL_MELEV],"%.1f",R2D(track.max_elev));

      tm=offset_tm(track.maxelev_time,prefs.utc_offset);
      sprintf(txt[COL_MELEVT],"%s",tmstrsh(&tm,"HMS"));
    }
    sprintf(txt[COL_SNAM],"%s",sat->satname);
    tdiff=(start_tm.tm_year-sat->kepler.epoch_year)*366+(start_tm.tm_yday-(int)sat->kepler.epoch_day);
//    sprintf(txt[COL_REF],"Day %d year %d",(int)sat->kepler.epoch_day,sat->kepler.epoch_year+1900);
    sprintf(txt[COL_REF],"%d days",tdiff);
    for (i=0; i<MAXCOL; i++) tmp[i]=txt[i];
    gtk_clist_append(lst, tmp);
  }
  gtk_clist_set_sort_column(lst,1);
  gtk_clist_sort(lst);
}
#endif

// download kepler
// ret: 0: OK
//     'p': download proggram problem
//     'c': connection problem
//     'd': download problem
static int download_kepler(char *gfile,char *ofile)
{
  char cmd[200];
  static int no_connection;
  if (!gfile)
  {
    no_connection=0;
    sprintf(cmd,"wget -V");
    if (system(cmd)) no_connection='p';

    if (!no_connection)
    {
      sprintf(cmd,"wget -q --spider %s",HTTP_KEPLER);
      if (system(cmd)) no_connection='c';
    }
  }
  else if (!no_connection)
  {
    sprintf(cmd,"wget -q -O %s %s/%s",ofile,HTTP_KEPLER,gfile);
    if (system(cmd)) no_connection='d';
    if (no_connection)
    {
//      remove(ofile);
    }
  }
  return no_connection;  // 0: OK, 'p': connection problem, 'd': download problem
}

#define incr_min 5
#define WEATHERFILENAME "weather.txt"
static void cmdfunc(GtkWidget *widget,gpointer pointer)
{
  char *name=(char *)pointer;
  static char *wname;
  char *sname=prefs.satlist; // "NOAA*"; // ;METOP*;METEOR*;FENGYUN 1D*";
  static SAT *sat;
  freed(&wname);
  if (prefs.cur_dir)
    sprintfd(&wname,"%s%c%s",prefs.cur_dir,DIR_SEPARATOR,WEATHERFILENAME);
  else
    strcpyd(&wname,WEATHERFILENAME);

  if (!strcmp(name,LAB_GETKEP))
  {
    int res;
    Set_Entry(widget,LAB_DWNLDRES,"Downloading...");
    while (g_main_iteration(FALSE));
    if ((res=download_kepler(NULL,NULL)))
    {
      if (res=='p')
        Set_Entry(widget,LAB_DWNLDRES,"%s","wget not installed?");
      else
        Set_Entry(widget,LAB_DWNLDRES,"%s%s","Can't reach ",HTTP_KEPLERSH);
    }
    else
    {
      if ((res=download_kepler(WEATHERFILENAME,wname)))
      {
        Set_Entry(widget,LAB_DWNLDRES,"%s %s","Can't download",WEATHERFILENAME);
      }
      else
      {
        Set_Entry(widget,LAB_DWNLDRES,"Download %s OK",WEATHERFILENAME);
//        Set_Button(widget,LAB_START);
      }
    }
  }

  if (!strcmp(name,LAB_START))
  {
    Set_Entry(widget,LAB_KBL,"s");
    Set_Led(widget,LAB_LED,LAB_LEDSTOPP);
    if (Update_Togglelabel(widget))
    {
      if (!exist_file(wname))
      {
        Set_Button(widget,LAB_START,FALSE);
        Create_Message("Error","Cant' find file '%s'",wname);
        return;
      }
      if (!(sat=prepare_track(sat,wname,sname)))
      {
        Set_Button(widget,LAB_START,FALSE);
        Create_Message("Error","%s doesn't contain satellite '%s'",WEATHERFILENAME,sname);
      }

      get_newpasses(sat,widget,TRUE);
      Set_Entry(widget,LAB_DWNLDRES,"%s",wname);
    }
    else
    {
#if __ADD_USB__
      run_track(widget,NULL);
#endif
    }
  }

  if (!strcmp(name,LAB_READ))
  {
    if (!(sat=prepare_track(sat,wname,sname)))
    {
      Set_Button(widget,LAB_START,FALSE);
      if ((sname) && (*sname))
        Create_Message("Error","%s doesn't contain satellite '%s'",WEATHERFILENAME,sname);
      else
        Create_Message("Error","No satelites defined. See Preferences, tab 'Tracker', 'Satellite list'.");
    }
    get_newpasses(sat,widget,FALSE);
  }
  if (!strcmp(name,LAB_NOODST))
  {
  }

  if (!strcmp(name,LAB_OFFSETHOUR))
  {
    test_min=Get_Adjust(widget,LAB_OFFSETHOUR);
    if (test_min%incr_min)
    {
      test_min=test_min-(test_min%incr_min);
      Set_Adjust(widget,LAB_OFFSETHOUR,"%d",test_min);
      test_min=Get_Adjust(widget,LAB_OFFSETHOUR);
    }
  }
}


// !! ENTRY: cmdfunc werkt niet want wordt door canvas afgepakt?? Zie key_actions.c
GtkWidget *create_tracker()
{
  GtkWidget *w[9];
  w[1]=Create_ButtonArray("",cmdfunc,1,
                          BUTTON,LAB_GETKEP,
                          ENTRY,LAB_DWNLDRES,"%20s","",
                          BUTTON,LAB_READ,
                          TOGGLE,LAB_START,0,
//                          ENTRY_NOFUNC,LAB_SELSAT,"%10s","",
//                          BUTTON,LAB_NOODST,
                          0
                         );
  w[2]=Create_Clist(LAB_SATLIST,NULL,NULL,NULL,MAXCOL,
      "Satellite",12,
      "Up time",20,
      "Max Elev",10,
      "at",10,
      //"Age",10,
      NULL);
  w[1]=Pack(NULL,'h',w[1],1,w[2],1,NULL);

/*
  w[3]=Create_ButtonArray("Time",cmdfunc,0,
                          SPIN,LAB_JAAR,"%03d%03d",1980,2040,
                          SPIN,LAB_MND,"%02d%02d",0,13,
                          SPIN,LAB_DAG,"%02d%02d",0,32,
                          SPIN,LAB_UUR,"%02d%02d",-1,24,
                          SPIN,LAB_MINT,"%02d%02d",-1,60,
                          SPIN,LAB_SEC,"%02d%02d",-1,60,
                          0
                         );
*/
  w[3]=Create_ButtonArray("Time",cmdfunc,0,
                          ENTRY,LAB_JAAR,"%4d",0,
                          ENTRY,LAB_MND,"%4d",0,
                          ENTRY,LAB_DAG,"%4d",0,
                          ENTRY,LAB_UUR,"%4d",0,
                          ENTRY,LAB_MINT,"%4d",0,
                          ENTRY,LAB_SEC,"%4d",0,
                          SPIN,LAB_OFFSETHOUR,"%04d%04d%d%d",incr_min,0,-1440,1440,
                          ENTRY,LAB_NEXTPASS,"%24s","",
                          0
                         );
  w[4]=Create_ButtonArray("Position",NULL,0,
                          ENTRY,LAB_LON,"%05.1f",0.,
                          ENTRY,LAB_LAT,"%05.1f",0.,
                          ENTRY,LAB_ELE,"%05.1f",0.,
                          ENTRY,LAB_AZI,"%05.1f",0.,
                          ENTRY,LAB_XXX,"%05.1f",0.,
                          ENTRY,LAB_YYY,"%05.1f",0.,
                          0
                         );
  w[5]=Create_ButtonArray(LAB_ROTOR_ELEAZIM,NULL,0,
                          ENTRY,LAB_ELE2,"%05.1f",prefs.rotor.storm.elev,
                          ENTRY,LAB_AZI2,"%05.1f",prefs.rotor.storm.azim,
                          0
                         );

  w[6]=Create_ButtonArray(LAB_ROTOR_XY,NULL,0,
                          ENTRY,LAB_X2,"%05.1f",prefs.rotor.storm.x,
                          ENTRY,LAB_Y2,"%05.1f",prefs.rotor.storm.y,
                          0
                         );
  w[7]=Create_Led(LAB_LED,0x00d);
  w[8]=Create_Entry(LAB_KBL,NULL,"%c",' ');
  w[7]=Pack(NULL,'v',w[7],1,w[8],1,NULL);
  w[7]=Pack("To rotor",'h',w[5],1,w[6],1,w[7],1,NULL);

  w[6]=Create_ButtonArray("",NULL,1,
                           LED," s=Stopped"   ,LAB_LEDSTOPP,
                           LED," x=No connect",LAB_LEDNOCON,
                           LED," v=Below hor.",LAB_LEDBELOW,
                           LED," ^=Above hor.",LAB_LEDABOVE,
                          0
                         );
  w[7]=Pack(NULL,'h',w[7],1,w[6],1,NULL);
  w[4]=Pack(NULL,'h',w[4],10,w[7],10,NULL);


  w[0]=SPack(NULL,"v",w[1],"ef1",w[3],"5",w[4],"5",NULL);
  return w[0];
}

static  void Update_Main_track_Wait(GtkWidget *widget);

static gboolean sched_update(gpointer data)
{
  GtkWidget *widget=(GtkWidget *)data;
  GtkWidget *window=Find_Parent_Window(widget);
  Update_Main_track_Wait(widget);
  refresh_wnd(window);
  return FALSE;
}



static void Update_Main_track(GtkWidget *widget)
{
  g_idle_add(sched_update,widget);
}

GtkWidget *get_trackerwindow(GtkWidget *w)
{
  GtkWidget *window=Find_Window(w,Lab_Track);
  if (!window) window=Find_Window(w,Lab_Main);
  return window;
}

static  void Update_Main_track_Wait(GtkWidget *widget)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *main_window=get_trackerwindow(window);
  GtkWidget *drawing_area=Find_Widget(window,LAB_TRACKCANVAS);
  RGBI *rgbi=Get_RGBI(drawing_area);
  int i;
  db.redraw=TRUE;
  db.redrawing=TRUE;
  if (Get_Enable_Update(drawing_area))
  {
    db.redraw=FALSE;
    Set_Enable_Update(drawing_area,FALSE); /* Prevent re-drawing while drawing */
    draw_back(main_window);
    if (!db.redraw)
    {
      draw_rast(main_window);
    }
    else
    {
      db.redraw=FALSE;
      Update_Main_track(widget);
    }
    if (rgbi)
    {
      if (rgb_bck.rgbbuf) free(rgb_bck.rgbbuf);
      rgb_bck.bufsize=rgbi->bufsize;
      rgb_bck.rgbbuf=malloc(rgb_bck.bufsize);
      rgb_bck.width =rgbi->width;
      rgb_bck.height=rgbi->height;
      for (i=0; i<rgb_bck.bufsize; i++) *(rgb_bck.rgbbuf+i)=*(rgbi->rgbbuf+i);
    }
    Set_Enable_Update(drawing_area,TRUE);
  }
  db.redrawing=FALSE;
}



#define RAND 0

static void configure_event_track(GtkWidget         *widget,
                                  GdkEventConfigure *event)
{
//  GtkWidget *window=Find_Parent_Window(widget);
//  GtkWidget *drawing_area=Find_Widget(window,"GTK_DRAWING_AREA");

  Renew_RGBBuf(widget);
  Update_Main_track(widget);
}


#ifdef NIETGEBRUIKT
static void scroll_actions_track(GtkWidget *widget, gpointer data)
{
  GtkWidget *window=Find_Parent_Window(widget);
  GtkWidget *drawing_area=Find_Widget(window,LAB_TRACKCANVAS);
//  char *name=(char *)data;
  float value,lower,upper;
//return;
  value=GTK_ADJUSTMENT(widget)->value;
  upper=GTK_ADJUSTMENT(widget)->upper;
  lower=GTK_ADJUSTMENT(widget)->lower;

#ifdef XXX
  if (!db->setscrolflag)       /* To prevent conflict ox/ox ch. in Set_Scrol */
  {
    if (!strcmp(name,"hbar"))
    {
      db->ox=value-lower;
    }
    if (!strcmp(name,"vbar"))
    {
      db->oy=value-lower;
    }
  }
#endif
  Update_Main_track(widget);
  expose_event(drawing_area,NULL);
}
#endif

GtkWidget *create_showtrack(GtkWidget *window)
{

  GtkWidget *w[2],*canvas;
  float fact=1.;
  int cvsw=(360+RAND)*fact;
  int cvsh=(180+RAND)*fact;
cvsw=0; cvsh=0;
  canvas=Create_Canvas1(window,LAB_TRACKCANVAS,
                cvsw,cvsh,
                configure_event_track,
                NULL,NULL,NULL);

//  canvas=Add_Scrollbars(canvas,"hbar","vbar",scroll_actions_track);
  w[0]=SPack(NULL,"v",canvas,"ef1",NULL);
Set_Enable_Update(canvas,TRUE);
  return w[0];
}

#endif

